//	AudioThread.h - Thread class used to parallelise audio for individual nodes.
//  ----------------------------------------------------------------------------
//	This file is part of 'Heilan Browser', an X3D browser specialised for audio
//	work.
//	Copyright (C) 2005-2009  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#ifndef AUDIOTHREAD_H_
#define AUDIOTHREAD_H_

#include "Thread.h"
#include "ThreeFloats.h"
#include "CriticalSection.h"
#include "Semaphore.h"

namespace HeilanX3D
{

class AbstractNode;
class ParentNode;

///	Thread class used to parallelise audio for individual nodes.
class AudioThread : public Thread
{
  public:
	///	Constructor.
	/*
		\param parallelNode The node whose audio is to be processed in parallel.
		\param parentNode The ParentNode which is controlling the parallel
		operation.

		Starts the thread running.
	 */
	AudioThread(AbstractNode *parallelNode, ParentNode *parentNode);
	///	Destructor.
	~AudioThread();

	///	The body of the thread.
	bool run();

	///	Signals the thread to get a block of audio from the associated node.
	void getAudioBlock(int numSamps, const ThreeFloats& pos);
  private:
	///	Pointer to this thread's associated node.
	AbstractNode *node;
	///	Pointer to this thread's associated ParentNode.
	ParentNode *parent;
	///	CriticalSection used to protect getAudioBlock().
	CriticalSection critSec;
	///	Sempahore used to wait in run() until we have something to do.
	Semaphore semaphore;

	///	Used to determine when it's time to stop the thread.
	bool timeToStop;

	///	The current number of samples to process.
	int numSamples;
	///	The current position of the audio object.
	ThreeFloats position;
};

}

#endif
