//	ConditionalWait.h - Simple class for waiting on a condition variable.
//  ----------------------------------------------------------------------------
//	This file is part of 'Heilan Browser', an X3D browser specialised for audio
//	work.
//	Copyright (C) 2005-2009  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#ifndef CONDITIONALWAIT_H_
#define CONDITIONALWAIT_H_

#include "DllStuff.h"

#include "SDL_thread.h"

///	Simple class for waiting on a condition variable.
class dllStuff ConditionalWait
{
  public:
	///	Constructor.
	ConditionalWait();
	///	Destructor.
	~ConditionalWait();

	///	Wait until signal() has been called.
	/*!
		\param timeout How long to wait before timing out - if 0 wait
		indefinitely.
	 */
	void wait(unsigned int timeout = 0);
	///	Signal wait() to return.
	void signal();
  private:
	///	The mutex used.
	SDL_mutex *mutex;
	///	The condition variable used.
	SDL_cond *condition;
};

#endif
