//	FourFloats.h - Simple struct containing four floats.
//  ----------------------------------------------------------------------------
//	This file is part of 'Heilan Browser', an X3D browser specialised for audio
//	work.
//	Copyright (C) 2005-2009  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#ifndef FOURFLOATS_H_
#define FOURFLOATS_H_

#include "DllStuff.h"

///	Simple struct containing four floats.
/*!
	\todo Override = operator to accept a std::string.
 */
struct dllStuff FourFloats
{
	float x;
	float y;
	float z;
	float r;

	///	Default constructor.
	FourFloats()
	{
		x = y = z = r = 0.0f;
	}

	///	Constructor, all members set to val.
	FourFloats(float val)
	{
		x = y = z = r = val;
	}

	///	Constructor.
	FourFloats(float xVal, float yVal, float zVal, float rVal)
	{
		x = xVal;
		y = yVal;
		z = zVal;
		r = rVal;
	}

	///	Assignment operator.
	FourFloats& operator=(const FourFloats& op2)
	{
		x = op2.x;
		y = op2.y;
		z = op2.z;
		r = op2.r;
		return *this;
	}

	///	Assignment operator (assigns all members the value of val).
	FourFloats& operator=(float val)
	{
		x = val;
		y = val;
		z = val;
		r = val;
		return *this;
	}

	///	Addition operator.
	FourFloats operator+(const FourFloats& op2)
	{
		FourFloats retval;

		retval.x = x + op2.x;
		retval.y = y + op2.y;
		retval.z = z + op2.z;
		retval.r = r + op2.r;

		return retval;
	}

	///	Assignment and Addition operator.
	FourFloats operator+=(const FourFloats& op2)
	{
		x += op2.x;
		y += op2.y;
		z += op2.z;
		r += op2.r;
		return *this;
	}

	///	Multiplication operator.
	FourFloats operator*(const FourFloats& op2)
	{
		FourFloats retval;

		retval.x = x * op2.x;
		retval.y = y * op2.y;
		retval.z = z * op2.z;
		retval.r = r * op2.r;

		return retval;
	}

	///	Multiplication operator (float).
	FourFloats operator*(float op2)
	{
		FourFloats retval;

		retval.x = x * op2;
		retval.y = y * op2;
		retval.z = z * op2;
		retval.r = r * op2;

		return retval;
	}

	///	Inequality operator.
	bool operator!=(const FourFloats& op2) const
	{
		return ((r!=op2.r)||(x!=op2.x)||(x!=op2.y)||(x!=op2.z));
	}
};

#endif
