//	TwoFloats.h - Simple struct containing three floats.
//  ----------------------------------------------------------------------------
//	This file is part of 'Heilan Browser', an X3D browser specialised for audio
//	work.
//	Copyright (C) 2005-2009  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#ifndef TWOFLOATS_H_
#define TWOFLOATS_H_

#include "DllStuff.h"

///	Simple struct containing two floats.
struct dllStuff TwoFloats
{
	float x;
	float y;

	///	Default constructor.
	TwoFloats()
	{
		x = y = 0.0f;
	}

	///	Constructor, all members set to val.
	TwoFloats(float val)
	{
		x = y = val;
	}

	///	Constructor.
	TwoFloats(float xVal, float yVal)
	{
		x = xVal;
		y = yVal;
	}

	///	Assignment operator.
	TwoFloats& operator=(const TwoFloats& op2)
	{
		x = op2.x;
		y = op2.y;
		return *this;
	}

	///	Assignment operator (assigns all members the value of val).
	TwoFloats& operator=(float val)
	{
		x = val;
		y = val;
		return *this;
	}

	///	Addition operator.
	TwoFloats operator+(const TwoFloats& op2)
	{
		TwoFloats retval;

		retval.x = x + op2.x;
		retval.y = y + op2.y;

		return retval;
	}

	///	Subtraction operator.
	TwoFloats operator-(const TwoFloats& op2)
	{
		TwoFloats retval;

		retval.x = x - op2.x;
		retval.y = y - op2.y;

		return retval;
	}

	///	Multiplication operator.
	TwoFloats operator*(const TwoFloats& op2)
	{
		TwoFloats retval;

		retval.x = x * op2.x;
		retval.y = y * op2.y;

		return retval;
	}

	///	Multiplication operator (float).
	TwoFloats operator*(float val)
	{
		TwoFloats retval;

		retval.x = x * val;
		retval.y = y * val;

		return retval;
	}

	///	Addition assignment operator.
	TwoFloats& operator+=(const TwoFloats& op2)
	{
		x += op2.x;
		y += op2.y;

		return *this;
	}

	///	Subtraction assignment operator.
	TwoFloats& operator-=(const TwoFloats& op2)
	{
		x -= op2.x;
		y -= op2.y;

		return *this;
	}

	///	Multiplication assignment operator.
	TwoFloats& operator*=(const TwoFloats& op2)
	{
		x *= op2.x;
		y *= op2.y;

		return *this;
	}

	///	Multiplication assignment operator.
	TwoFloats& operator*=(float op2)
	{
		x *= op2;
		y *= op2;

		return *this;
	}

	///	Equality operator.
	bool operator==(const TwoFloats& op2)
	{
		return (x==op2.x)&&(y==op2.y);
	}

	///	Inequality operator.
	bool operator!=(const TwoFloats& op2)
	{
		return (x!=op2.x)||(y!=op2.y);
	}

	friend TwoFloats dllStuff operator*(float op1, const TwoFloats& op2);
};

#endif
