//	PerlinTexture.h - Perlin noise texture.
//  ----------------------------------------------------------------------------
//	This file is part of 'Heilan Browser', an X3D browser specialised for audio
//	work.
//	Copyright (C) 2005-2009  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#ifndef PERLINTEXTURE_H_
#define PERLINTEXTURE_H_

#include "ProceduralTexture.h"
#include "RubbishNoiseTexture.h"

namespace HeilanX3D
{

///	Simple sine texture.
/*!
	\ingroup libheilanextras
 */
class PerlinTexture : public ProceduralTexture
{
  public:
	///	Constructor.
	PerlinTexture(int components = 1, int size = 256);
	///	Destructor.
	virtual ~PerlinTexture();

	///	Sets the persistence of the noise.
	/*!
		Note that this should only be done when the texture's not in use, since
		it regenerates pixels.
	 */
	void setPersistence(float val);
  protected:
	///	Helper method to generate the perlin noise.
	void generatePerlin();
  private:
	///	B-spline interpolation helper method.
	inline float bSplineInterp(float x0,
							   float x1,
							   float x2,
							   float x3,
							   float val)
	{
		// 4-point, 3rd-order B-spline (x-form)
		float ym1py1 = x0+x2;
		float c0 = (1.0f/6.0f) * ym1py1 + (2.0f/3.0f) * x1;
		float c1 = (1.0f/2.0f) * (x2-x0);
		float c2 = (1.0f/2.0f) * ym1py1 - x1;
		float c3 = (1.0f/2.0f) * (x1-x2) + (1.0f/6.0f) * (x3-x0);
		return ((c3*val+c2)*val+c1)*val+c0;
	};

	///	Number of components in the texture.
	int numComponents;
	///	The size of the texture.
	int texSize;

	///	The persistence value to use to generate the perlin noise.
	float persistence;

	///	The noise texture used to generate the perlin noise.
	RubbishNoiseTexture noiseTex;
};

}

#endif
