//	ProceduralTexture.cpp - Simple base class for procedural textures.
//  ----------------------------------------------------------------------------
//	This file is part of 'Heilan Browser', an X3D browser specialised for audio
//	work.
//	Copyright (C) 2005-2009  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#include "ProceduralTexture.h"

namespace HeilanX3D
{

//------------------------------------------------------------------------------
ProceduralTexture::ProceduralTexture(int numComponents, int texSize):
components(numComponents),
size(texSize)
{
	pixels = new float[components * (size*size)];
}

//------------------------------------------------------------------------------
ProceduralTexture::~ProceduralTexture()
{
	delete [] pixels;
}

//------------------------------------------------------------------------------
void ProceduralTexture::generateOpenGLTexture(GLuint &id)
{
	int pixelFormat;

	//Generate texture.
	glGenTextures(1, &id);

	//Write pixels to texture.
	glBindTexture(GL_TEXTURE_2D, id);

	switch(components)
	{
		case 1:
			pixelFormat = GL_LUMINANCE;
			break;
		case 3:
			pixelFormat = GL_RGB;
			break;
		case 4:
			pixelFormat = GL_RGBA;
			break;
		default:
			pixelFormat = GL_LUMINANCE;
	}

	glTexImage2D(GL_TEXTURE_2D,
				 0,
				 GL_RGBA,
				 size,
				 size,
				 0,
				 pixelFormat,
				 GL_FLOAT,
				 pixels);

	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
}

}
