//	ProceduralTexture.h - Simple base class for procedural textures.
//  ----------------------------------------------------------------------------
//	This file is part of 'Heilan Browser', an X3D browser specialised for audio
//	work.
//	Copyright (C) 2005-2009  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#ifndef PROCEDURALTEXTURE_H_
#define PROCEDURALTEXTURE_H_

#include "GLHeaders.h"

namespace HeilanX3D
{

///	Simple base class for procedural textures.
/*!
	\ingroup libheilanextras

	Textures are stored as arrays of floats.  This class will allocate and
	destroy the pixel data.  Subclasses should then generate the texture and
	write it to this memory location in their constructors.
 */
class ProceduralTexture
{
  public:
	///	Constructor.
	/*!
		\param numComponents How many colours there are (1 = luminance, 3 = RGB,
		4 = RGBA).
		\param texSize The size of the texture - all textures are square, so
		this represents the width and height.
	 */
	ProceduralTexture(int numComponents = 1, int texSize = 256);
	///	Destructor.
	virtual ~ProceduralTexture();

	///	Generates an OpenGL texture from pixels, and binds it to the ID passed in.
	void generateOpenGLTexture(GLuint &id);

	///	Returns the texture's pixel data.
	const float * const getPixels() const {return pixels;};
  protected:
	///	Pixel data, for subclasses to write to.
	float *pixels;
  private:
	///	The number of components in the texture.
	int components;
	///	Size of the texture.
	int size;
};

}

#endif
