//	AccelLine.cpp - Line segments drawn according to accelerometer data.
//  ----------------------------------------------------------------------------
//	This file is part of 'Heilan Browser', an X3D browser specialised for audio
//	work.
//	Copyright (C) 2005-2009  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#include "AccelLine.h"
#include "StringParser.h"
#include "Camera.h"

#include <cmath>
#include <iostream>

using namespace std;

namespace HeilanX3D
{

NodeConstructor<AccelLine> AccelLine::constructor;

//------------------------------------------------------------------------------
AccelLine::AccelLine():
AbstractNode(),
writePos(0),
needX(true),
needY(true),
needZ(true),
deltaTime(0.02f),
lineLength(1024),
drawAsQuads(false),
calibVal(0.05f),
filtLast(0.0f),
filtCoeff(0.05f),
aLast(0.25f),
dcFiltCoeff(0.99f),
dcFiltCoeff2(0.9973f)
{
	/*int x, y;
	float tempf;
	unsigned char *pixelPointer = pixels;*/

	setType("AccelLine");
	setVisual(true);

	//Generate texture.
	/*for(y=0;y<256;++y)
	{
		for(x=0;x<256;++x)
		{
			tempf = sinf(((float)x/256.0f)*3.1415926535897932384626433832795f);
			tempf *= sinf(((float)y/256.0f)*3.1415926535897932384626433832795f);
			tempf = (exp(tempf)-1.0f)/(exp(1.0f)-1.0f)*1.5f-0.1f;
			if(tempf < 0.0f)
				tempf = 0.0f;
			else if(tempf > 1.0f)
				tempf = 1.0f;
			*pixelPointer = 255;
			*(pixelPointer+1) = (unsigned char)(tempf * 255.0f);
			pixelPointer += 2;
		}
	}

	//Setup texture.
	glGenTextures(1, &texture);
	glBindTexture(GL_TEXTURE_2D, texture);
	glTexImage2D(GL_TEXTURE_2D,
				 0,
				 2,
				 256,
				 256,
				 0,
				 GL_LUMINANCE_ALPHA,
				 GL_UNSIGNED_BYTE,
				 pixels);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	glBindTexture(GL_TEXTURE_2D, 0);*/
};

//------------------------------------------------------------------------------
AccelLine::~AccelLine()
{
		
}

//------------------------------------------------------------------------------
void AccelLine::initialise()
{
	int x, y;
	float tempf;
	unsigned char *pixelPointer = pixels;

	//Generate texture.
	for(y=0;y<256;++y)
	{
		for(x=0;x<256;++x)
		{
			tempf = sinf(((float)x/256.0f)*3.1415926535897932384626433832795f);
			tempf *= sinf(((float)y/256.0f)*3.1415926535897932384626433832795f);
			tempf = (exp(tempf)-1.0f)/(exp(1.0f)-1.0f)*1.5f-0.1f;
			if(tempf < 0.0f)
				tempf = 0.0f;
			else if(tempf > 1.0f)
				tempf = 1.0f;
			*pixelPointer = 255;
			*(pixelPointer+1) = (unsigned char)(tempf * 255.0f);
			pixelPointer += 2;
		}
	}

	//Setup texture.
	glGenTextures(1, &texture);
	glBindTexture(GL_TEXTURE_2D, texture);
	glTexImage2D(GL_TEXTURE_2D,
				 0,
				 2,
				 256,
				 256,
				 0,
				 GL_LUMINANCE_ALPHA,
				 GL_UNSIGNED_BYTE,
				 pixels);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	glBindTexture(GL_TEXTURE_2D, 0);
}

//------------------------------------------------------------------------------
void AccelLine::graphicDoStuff()
{
	int i, j;
	ThreeFloats tempCam;
	int tempi;
	ThreeFloats temp3f;

	critSec.lock();
	{
	if((writePos-1) < 0)
		tempi = NumSegments-1;
	else
		tempi = writePos-1;

	glLineWidth(8.0f);
	glPointSize(8.0f);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	glEnable(GL_LINE_SMOOTH);
	glEnable(GL_BLEND);
	glDisable(GL_LIGHTING);
	glDisable(GL_DEPTH_TEST);

	/*tempCam = lines[tempi].endPos/300.0f;
	tempCam.x = -tempCam.x;
	tempCam.y = -tempCam.y;
	tempCam.z = -tempCam.z;
	glTranslatef(tempCam.x, tempCam.y, tempCam.z);*/

	if(!drawAsQuads)
	{
		glBindTexture(GL_TEXTURE_2D, texture);

		j = tempi;
		for(i=0;i<lineLength;++i)
		{
			temp3f = lines[j].endPos/300.0f;

			glBegin(GL_QUADS);
			{
				glColor4f(lines[j].colour.x,
						  lines[j].colour.y,
						  lines[j].colour.z,
						  (1.0f-((float)i/(float)lineLength)));

				glTexCoord2f(0.0f, 0.0f);
				glVertex3f(temp3f.x-0.5f,
						   temp3f.y-0.5f,
						   temp3f.z);
				glTexCoord2f(1.0f, 0.0f);
				glVertex3f(temp3f.x+0.5f,
						   temp3f.y-0.5f,
						   temp3f.z);
				glTexCoord2f(1.0f, 1.0f);
				glVertex3f(temp3f.x+0.5f,
						   temp3f.y+0.5f,
						   temp3f.z);
				glTexCoord2f(0.0f, 1.0f);
				glVertex3f(temp3f.x-0.5f,
						   temp3f.y+0.5f,
						   temp3f.z);
			}
			glEnd();

			--j;
			if(j < 0)
				j = NumSegments-1;
		}
	}
	else
	{
		j = tempi-1;
		if(j < 0)
			j = NumSegments-1;
		glBegin(GL_QUADS);
		{
			for(i=0;i<(lineLength-1);++i)
			{
				float tempf = 1.0f-((float)i/(float)lineLength);
				/*glColor4f(lines[j].colour.x,
						  lines[j].colour.y,
						  lines[j].colour.z,
						  tempf);*/
				glColor4f(1.0f,
						  0.0f,
						  0.0f,
						  tempf);

				glVertex3f(lines[j].endPos.x,
						   lines[j].endPos.y+(2.0f * tempf),
						   lines[j].endPos.z);
				glVertex3f(lines[j].endPos.x,
						   lines[j].endPos.y-(2.0f * tempf),
						   lines[j].endPos.z);
				glVertex3f(lines[j-1].endPos.x,
						   lines[j-1].endPos.y-(2.0f * tempf),
						   lines[j-1].endPos.z);
				glVertex3f(lines[j-1].endPos.x,
						   lines[j-1].endPos.y+(2.0f * tempf),
						   lines[j-1].endPos.z);

				--j;
				if(j < 1)
					j = NumSegments-1;
			}
		}
		glEnd();
	}

	glBindTexture(GL_TEXTURE_2D, 0);
	glEnable(GL_DEPTH_TEST);
	glDisable(GL_BLEND);
	glDisable(GL_LINE_SMOOTH);
	glEnable(GL_LIGHTING);

	/*tempCam = lines[tempi].endPos;
	tempCam.x = tempCam.x;
	tempCam.y = -tempCam.y;
	tempCam.z = tempCam.z;

	Camera::getInstance()->setGraphicPosition(tempCam);*/
	}
	critSec.unlock();
}

//------------------------------------------------------------------------------
void AccelLine::setAttribute(const string& attribute, EventValue& val)
{
	//int i;
	ThreeFloats vel;
	ThreeFloats pos;

	if(attribute == "xAccel")
	{
		int lastPos = writePos-1;
		float tempf = val;

		//Average filter on accel data input.
		/*for(i=15;i>0;--i)
			accelAv[i].x = accelAv[i-1].x;
		accelAv[0].x = tempf;
		tempf = 0;
		for(i=0;i<16;++i)
			tempf += accelAv[0].x;
		tempf /= 16.0f;*/

		//Threshold and lp filter.
		if(fabs(tempf) < calibVal)
			tempf = 0.0f;
		tempf = filtLast.x + filtCoeff.x * (tempf - filtLast.x);
		filtLast.x = tempf;

		if(lastPos < 0)
			lastPos = NumSegments-1;

		//Update velocity, dc filter.
		vel.x = tempf + lastVel.x;
		lines[writePos].velocity.x = (vel.x-lastVel.x) +
									 (dcFiltCoeff * lines[lastPos].velocity.x);
		lastVel.x = vel.x;
		//Update position.
		pos.x = lines[writePos].velocity.x + lastPos2.x;
		lines[writePos].endPos.x = (pos.x-lastPos2.x) +
									(dcFiltCoeff2 * lines[lastPos].endPos.x);
		lastPos2.x = pos.x;
		needX = false;

		if((!needY) && (!needZ))
		{
			critSec.lock();
			{
				lines[writePos].colour = getColour((lines[writePos].endPos -
													lines[lastPos].endPos)/3000.0f);

				++writePos;
				if(writePos >= NumSegments)
					writePos = 0;
				needX = true;
				needY = true;
				needZ = true;
			}
			critSec.unlock();
		}
	}
	else if(attribute == "yAccel")
	{
		int lastPos = writePos-1;
		float tempf = val;

		//Average filter on accel data input.
		/*for(i=15;i>0;--i)
			accelAv[i].y = accelAv[i-1].y;
		accelAv[0].y = tempf;
		tempf = 0;
		for(i=0;i<16;++i)
			tempf += accelAv[0].y;
		tempf /= 16.0f;*/

		//Threshold and lp filter.
		if(fabs(tempf) < calibVal)
			tempf = 0.0f;
		tempf = filtLast.y + filtCoeff.y * (tempf - filtLast.y);
		filtLast.y = tempf;

		if(lastPos < 0)
			lastPos = NumSegments-1;

		//Update velocity, dc filter.
		vel.y = tempf + lastVel.y;
		lines[writePos].velocity.y = (vel.y-lastVel.y) +
									 (dcFiltCoeff * lines[lastPos].velocity.y);
		lastVel.y = vel.y;
		//Update position.
		pos.y = lines[writePos].velocity.y + lastPos2.y;
		lines[writePos].endPos.y = (pos.y-lastPos2.y) +
									(dcFiltCoeff2 * lines[lastPos].endPos.y);
		lastPos2.y = pos.y;
		needY = false;

		if((!needX) && (!needZ))
		{
			critSec.lock();
			{
				lines[writePos].colour = getColour((lines[writePos].endPos -
												    lines[lastPos].endPos)/3000.0f);

				++writePos;
				if(writePos >= NumSegments)
					writePos = 0;
				needX = true;
				needY = true;
				needZ = true;
			}
			critSec.unlock();
		}
	}
	else if(attribute == "zAccel")
	{
		int lastPos = writePos-1;
		float tempf = val;

		//Average filter on accel data input.
		/*for(i=15;i>0;--i)
			accelAv[i].z = accelAv[i-1].z;
		accelAv[0].z = tempf;
		tempf = 0;
		for(i=0;i<16;++i)
			tempf += accelAv[0].z;
		tempf /= 16.0f;*/

		//Threshold and lp filter.
		if(fabs(tempf) < calibVal)
			tempf = 0.0f;
		tempf = filtLast.z + filtCoeff.z * (tempf - filtLast.z);
		filtLast.z = tempf;

		if(lastPos < 0)
			lastPos = NumSegments-1;

		//Update velocity, dc filter.
		vel.z = tempf + lastVel.z;
		lines[writePos].velocity.z = (vel.x-lastVel.z) +
									 (dcFiltCoeff * lines[lastPos].velocity.z);
		lastVel.z = vel.z;
		//Update position.
		pos.z = lines[writePos].velocity.z + lastPos2.z;
		lines[writePos].endPos.z = (pos.z-lastPos2.z) +
									(dcFiltCoeff2 * lines[lastPos].endPos.z);
		lastPos2.z = pos.z;
		needZ = false;

		if((!needX) && (!needY))
		{
			critSec.lock();
			{
				lines[writePos].colour = getColour((lines[writePos].endPos -
												    lines[lastPos].endPos)/3000.0f);

				++writePos;
				if(writePos >= NumSegments)
					writePos = 0;
				needX = true;
				needY = true;
				needZ = true;
			}
			critSec.unlock();
		}
	}
	else if(attribute == "lineLength")
		lineLength = val;
	else if(attribute == "drawAsQuads")
	{
		StringParser parse(val);

		if(parse.getNumBools() > 0)
			drawAsQuads = parse.getBool();
	}
}

//------------------------------------------------------------------------------
ThreeFloats AccelLine::getColour(const ThreeFloats& pos)
{
	float totalAccel;
	ThreeFloats retval;
	const ThreeFloats cold(0.0f, 0.0f, 0.5f);
	const ThreeFloats hot(1.0f, 0.5f, 0.0f);

	totalAccel = sqrtf((pos.x*pos.x) + (pos.y*pos.y) + (pos.z*pos.z));

	retval = ((1.0f-totalAccel) * cold) + (totalAccel * hot);

	return retval;
}

};
