//	Accel2Pos.cpp - Converts acceleration data into position data.
//  ----------------------------------------------------------------------------
//	This file is part of 'Heilan Browser', an X3D browser specialised for audio
//	work.
//	Copyright (C) 2005-2009  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#include "Accel2Pos.h"

#include <cmath>

//------------------------------------------------------------------------------
Accel2Pos::Accel2Pos():
Parameter(),
calibVal(0.05f),
filtCoeff(0.05f),
filtLast(0.0f),
lastVel(0.0f),
lastPos(0.0f),
lastVelFixed(0.0f),
lastPosFixed(0.0f),
dcFiltCoeff(0.99f),
dcFiltCoeff2(0.9973f)
{
	
}

//------------------------------------------------------------------------------
Accel2Pos::~Accel2Pos()
{
	
}

//------------------------------------------------------------------------------
void Accel2Pos::valueChanged(float val)
{
	float tempf = val;
	float pos, vel;
	float posFixed, velFixed;

	//Threshold and lp filter.
	if(fabs(tempf) < calibVal)
		tempf = 0.0f;
	tempf = filtLast + filtCoeff * (tempf - filtLast);
	filtLast = tempf;

	//Update velocity, dc filter.
	vel = tempf + lastVel;
	velFixed = (vel-lastVel) + (dcFiltCoeff * lastVelFixed);
	lastVel = vel;

	//Update position, filter so it always returns to 0.
	pos = velFixed + lastPos;
	posFixed = (pos-lastPos) + (dcFiltCoeff2 * lastPosFixed);
	lastPos = pos;

	Parameter::valueChanged(posFixed * 20.0f);
}
