//	Accel2Pos.h - Converts acceleration data into position data.
//  ----------------------------------------------------------------------------
//	This file is part of 'Heilan Browser', an X3D browser specialised for audio
//	work.
//	Copyright (C) 2005-2009  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#ifndef ACCEL2POS_H_
#define ACCEL2POS_H_

#include "Parameter.h"

///	Converts acceleration data into position data.
/*!
	\ingroup AshitakaParameters
	\ingroup libheilanextras
 */
class Accel2Pos : public Parameter
{
  public:
	///	Constructor.
	Accel2Pos();
	///	Destructor.
	~Accel2Pos();

	///	Converts the input acceleration value to a position value.
	void valueChanged(float val);
  private:
	///	Used to ignore too-small values.
	float calibVal;

	///	For the hi-pass filter.
	float filtCoeff;
	///	For the hi-pass filter.
	float filtLast;

	///	Previous unfixed velocity value.
	float lastVel;
	///	Previous unfixed position value.
	float lastPos;

	///	Previous fixed velocity value.
	float lastVelFixed;
	///	Previous fixed position value.
	float lastPosFixed;

	///	Velocity dc filter coefficient.
	float dcFiltCoeff;
	///	Position dc filter coefficient.
	float dcFiltCoeff2;
};

#endif
