//	AshitakaSparks.cpp - Simple particle class used to emphasize collisions
//					     between Quad2s.
//  ----------------------------------------------------------------------------
//	This file is part of 'Heilan Browser', an X3D browser specialised for audio
//	work.
//	Copyright (C) 2005-2009  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#include "AshitakaSparks.h"
#include "MiscHelper.h"

#include <cmath>

namespace AshitakaClasses
{

//------------------------------------------------------------------------------
AshitakaSparks::AshitakaSparks(GLuint tex):
lifeforce(0.0f),
fakeGravity(0.0f),
texture(tex)
{
	int i;
	float tempf;

	/*while(((colour.x < 0.35f) && (colour.y < 0.35f) && (colour.z < 0.35f)) ||
		  ((colour.x > 0.9f) && (colour.y > 0.9f) && (colour.z > 0.9f)))
	{
		colour.x = heilanRand(heilanRandSeed());
		colour.y = heilanRand(heilanRandSeed());
		colour.z = heilanRand(heilanRandSeed());
	}*/
	colour.x = 1.0f;
	colour.y = 1.0f;
	colour.z = 0.25f;

	for(i=0;i<NumSparks;++i)
	{
		sparks[i].position = 0.0f;
		sparks[i].direction.x = heilanRand(heilanRandSeed());
		sparks[i].direction.y = heilanRand(heilanRandSeed());
		sparks[i].direction.z = heilanRand(heilanRandSeed());

		tempf = sparks[i].direction.getDistance(ThreeFloats(0.0f));
		sparks[i].direction /= tempf;
	}
}

//------------------------------------------------------------------------------
AshitakaSparks::~AshitakaSparks()
{
	
}

//------------------------------------------------------------------------------
void AshitakaSparks::go(const ThreeFloats& sparkPos)
{
	int i;
	float tempf;

	for(i=0;i<NumSparks;++i)
	{
		sparks[i].position = sparkPos;
		sparks[i].direction.x = heilanRand(heilanRandSeed());
		sparks[i].direction.y = heilanRand(heilanRandSeed());
		sparks[i].direction.z = heilanRand(heilanRandSeed());

		tempf = sparks[i].direction.getDistance(ThreeFloats(0.0f));
		sparks[i].direction /= tempf;
	}

	lifeforce = 1.0f;
	fakeGravity = 0.0f;
}

//------------------------------------------------------------------------------
void AshitakaSparks::draw()
{
	int i;
	float tempf;
	float modelview[16];
	//const float speed = 0.25f;
	const float speed = 1.0f;
	const float speed2 = 1.0f;
	//const float particleSize = (1.0f/24.0f);
	const float particleSize = (1.0f/96.0f);
	const float particleSize2 = 0.0625f;
	const ThreeFloats downVector(0.0f, -1.0f, 0.0f);

	lifeforce -= 0.01f;
	/*if(fakeGravity < 1.0f)
		fakeGravity += 0.005f;*/

	glBlendFunc(GL_SRC_ALPHA, GL_ONE);

	for(i=0;i<NumSparks;++i)
	{
		if(i < (NumSparks/2))
		{
			//Move to new postion.
			sparks[i].position += (sparks[i].direction * lifeforce * (1.0f/64.0f));
			//Update direction.
			sparks[i].direction *= 1.0f-fakeGravity;
			sparks[i].direction += fakeGravity * downVector;
		}
		else
		{
			//Move to new postion.
			sparks[i].position += (sparks[i].direction * lifeforce * (1.0f/128.0f));
			//Update direction.
			sparks[i].direction *= 1.0f-fakeGravity;
			sparks[i].direction += fakeGravity * downVector;
		}

		//Draw to screen.
		glMatrixMode(GL_MODELVIEW);
		glPushMatrix();

		glBindTexture(GL_TEXTURE_2D, texture);

		if(i < (NumSparks/2))
		{
			glColor4f(colour.x + 0.25f,
					  colour.y + 0.25f,
					  colour.z + 0.25f,
					  lifeforce);
		}
		else
		{
			glColor4f(colour.x,
					  colour.y,
					  colour.z,
					  lifeforce*0.125f);
		}

		//angle = atanf(sparks[i].direction.y/sparks[i].direction.x);
		glTranslatef(sparks[i].position.x,
					 sparks[i].position.y,
					 sparks[i].position.z);

		//To billboard the sparks.
		glGetFloatv(GL_MODELVIEW_MATRIX, modelview);
		modelview[(0*4)+0] = 1.0f;
		modelview[(0*4)+1] = 0.0f;
		modelview[(0*4)+2] = 0.0f;

		modelview[(1*4)+0] = 0.0f;
		modelview[(1*4)+1] = 1.0f;
		modelview[(1*4)+2] = 0.0f;

		modelview[(2*4)+0] = 0.0f;
		modelview[(2*4)+1] = 0.0f;
		modelview[(2*4)+2] = 1.0f;
		glLoadMatrixf(modelview);

		//glRotatef((angle * (180.0f/3.141592653f)), 0.0f, 0.0f, 1.0f);

		if(i<(NumSparks/2))
		{
			tempf = particleSize*speed;
			glBegin(GL_QUADS);
			{
				glTexCoord2f(0.0f, 1.0f);
				glVertex3f(-particleSize, tempf, 0.0f);
				glTexCoord2f(1.0f, 1.0f);
				glVertex3f(particleSize, tempf, 0.0f);
				glTexCoord2f(1.0f, 0.0f);
				glVertex3f(particleSize, -tempf, 0.0f);
				glTexCoord2f(0.0f, 0.0f);
				glVertex3f(-particleSize, -tempf, 0.0f);
			}
			glEnd();
		}
		else
		{
			tempf = particleSize2*speed2;
			glBegin(GL_QUADS);
			{
				glTexCoord2f(0.0f, 1.0f);
				glVertex3f(-particleSize2, tempf, 0.0f);
				glTexCoord2f(1.0f, 1.0f);
				glVertex3f(particleSize2, tempf, 0.0f);
				glTexCoord2f(1.0f, 0.0f);
				glVertex3f(particleSize2, -tempf, 0.0f);
				glTexCoord2f(0.0f, 0.0f);
				glVertex3f(-particleSize2, -tempf, 0.0f);
			}
			glEnd();
		}

		glBindTexture(GL_TEXTURE_2D, 0);

		glPopMatrix();
	}

	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
}

//------------------------------------------------------------------------------
void AshitakaSparks::setColour(const ThreeFloats& col)
{
	colour = col;
}

}
