//	AshitakaSparks.h - Simple particle class used to emphasize collisions
//					   between Quad2s.
//  ----------------------------------------------------------------------------
//	This file is part of 'Heilan Browser', an X3D browser specialised for audio
//	work.
//	Copyright (C) 2005-2009  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#ifndef ASHITAKASPARKS_H_
#define ASHITAKASPARKS_H_

#include "ThreeFloats.h"

#include "GLHeaders.h"

namespace AshitakaClasses
{

///	Simple particle class used to emphasize collisions between Quad2s.
/*!
	AshitakaSparks have a set lifespan, and stop generating when it's up.
 */
class AshitakaSparks
{
  public:
	///	Constructor.
	/*!
		\param tex The texture to draw the sparks as.
	 */
	AshitakaSparks(GLuint tex);
	///	Destructor.
	~AshitakaSparks();

	///	Signals the generator to start generating sparks at sparkPos.
	void go(const ThreeFloats& sparkPos);
	///	True if we're currently active.
	bool isActive() const {return (lifeforce>0.0f);};

	///	Draws the sparks on screen, updates their positions etc.
	void draw();

	///	Sets the sparks' colour.
	void setColour(const ThreeFloats& col);
  private:
	///	How many sparks to use.
	enum
	{
		NumSparks = 16
	};

	///	Simple struct containing the data a spark needs.
	struct Spark
	{
		///	The current position.
		ThreeFloats position;
		///	The Spark's direction.
		ThreeFloats direction;
	};

	///	The colour of the particles.
	ThreeFloats colour;
	///	Vector holding our particles.
	Spark sparks[NumSparks];
	///	The sparks' lifeforce.
	float lifeforce;

	///	Test.
	float fakeGravity;

	///	The texture we use.
	GLuint texture;
};

}

#endif
