//	AudioSynthDelay.cpp - Delay effect for Ashitaka's audio synth.
//  ----------------------------------------------------------------------------
//	This file is part of 'Heilan Browser', an X3D browser specialised for audio
//	work.
//	Copyright (C) 2005-2009  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#include "AudioSynthDelay.h"

namespace AshitakaClasses
{

//------------------------------------------------------------------------------
AudioSynthDelay::AudioSynthDelay():
writePointer(0),
readPointer(0.0f)
{
	time = 0.5f;
	feedback = 0.0f;
	mix = 1.0f;
}

//------------------------------------------------------------------------------
AudioSynthDelay::~AudioSynthDelay()
{
	
}

//------------------------------------------------------------------------------
void AudioSynthDelay::processAudio(AmbisonicData *audio, int numSamples)
{
	int i;
	float tempf0;
	float readIndex;
	float readFrac;
	int indexInt;
	int readNext;
	float tempDelay;
	float bufferSizeF = static_cast<float>(BufferSize);
	float scrub = 0.0f;

	//Calculate audio.
	for(i=0;i<numSamples;++i)
	{
		//Update currentDelay.
		tempDelay = time * bufferSizeF;

		scrub = (tempDelay - readPointer) * (1.0f/1024.0f);
		if(scrub > 2.0f)
			scrub = 2.0f;
		else if(scrub < -2.0f)
			scrub = -2.0f;
		readPointer += scrub;

		//Get current sample.
		readIndex = static_cast<float>(writePointer) - readPointer;
		while(readIndex < 0.0f)
			readIndex += bufferSizeF;
		indexInt = static_cast<int>(readIndex);
		readNext = (indexInt+1) % BufferSize;
		readFrac = readIndex - static_cast<float>(indexInt);
		tempf0 = buffer[indexInt] * (1.0f-readFrac);
		tempf0 += buffer[readNext] * readFrac;

		//Update buffer.
		buffer[writePointer] = tanh(audio[i].W +
									(tempf0 * (feedback*0.35f)) *
									3.1415f);
		buffer[writePointer] += DenormalFix;
		buffer[writePointer] -= DenormalFix;
		++writePointer;
		writePointer %= BufferSize;

		//Write outputs.
		audio[i].W = ((audio[i].W * (1.0f-mix)) + (tempf0 * mix));
	}

	scrubDir = scrub;
}

}
