//	AudioSynthDelay.h - Delay effect for Ashitaka's audio synth.
//  ----------------------------------------------------------------------------
//	This file is part of 'Heilan Browser', an X3D browser specialised for audio
//	work.
//	Copyright (C) 2005-2009  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#ifndef AUDIOSYNTHDELAY_H_
#define AUDIOSYNTHDELAY_H_

#include "Parameter.h"
#include "AmbisonicData.h"

namespace AshitakaClasses
{

///	Delay effect for Ashitaka's audio synth.
class AudioSynthDelay
{
  public:
	///	Constructor.
	AudioSynthDelay();
	///	Destructor.
	~AudioSynthDelay();

	///	Applies the delay to the audio.
	/*!
		\param audio Audio buffer with monophonic audio as input in the W
		component - overwrite it to return.
		\param numSamples The size of buffer.
	 */
	void processAudio(AmbisonicData *audio, int numSamples);

	///	Delay time parameter.
	Parameter time;
	///	Feedback parameter.
	Parameter feedback;
	///	Mix parameter.
	Parameter mix;

	///	(Output) Scrub parameter.
	Parameter scrubDir;
  private:
	///	Size of the delay buffer.
	enum
	{
		BufferSize = 88200
	};

	///	The delay buffer.
	float buffer[BufferSize];

	///	The current write pointer.
	int writePointer;
	///	The current read pointer.
	float readPointer;
};

}

#endif
