//	CircleGO.h - Circle (multiple) GravityObject2.
//  ----------------------------------------------------------------------------
//	This file is part of 'Heilan Browser', an X3D browser specialised for audio
//	work.
//	Copyright (C) 2005-2009  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#ifndef CIRCLEGO_H_
#define CIRCLEGO_H_

#include "GravityObject2.h"

namespace AshitakaClasses
{

///	Circle (multiple) GravityObject2.
class CircleGO : public GravityObject2
{
  public:
	///	Constructor.
	CircleGO(int circleIndex);
	///	Destructor.
	~CircleGO();

	///	Used to update the positions of the primary Quad2s.
	void updatePrimaryQuads();
	///	Used to update the position of a secondary Quad2.
	void updateSecondaryQuad(Quad2 *quad);
	///	Used to make the GO die off after a while.
	void updateLifeForce();
	///	Not used?
	void updateExciter() {};

	///	Called from CircleExciter.
	void updateRadius(float val);
  private:
	///	The CircleGO's Exciter.
	class CircleExciter : public GravityObject2::Exciter
	{
	  public:
		///	Constructor.
		CircleExciter(CircleGO *go, int startingPoint);
		///	Destructor.
		~CircleExciter();

		///	Where the excitation is generated.
		std::vector<Buffer> *getAudio(int numSamples);
	  private:
		///	Struct holding the various coefficients etc. for the filter.
		struct Filter
		{
			///	Constructor.
			Filter():
			freq(50.0f),
			res(0.5f),
			cut(0.0f)
			{
				setCoeff(freq, res);

				b[0] = 0.0f;
				b[1] = 0.0f;
				b[2] = 0.0f;
				b[3] = 0.0f;
			};

			///	Sets the coefficients.
			void setCoeff(float frequency, float resonance)
			{
				freq = frequency;
				res = resonance;

				cut = (6.283185306f*freq)/44100.0f;
			};

			///	Cutoff frequency.
			float freq;
			///	Resonance.
			float res;

			///	Coefficients.
			float cut;
			///	Buffers.
			float b[4];
		};

		///	Our parent GO.
		CircleGO *parentGO;

		///	Where we are in the envelope sequence.
		int index;
		///	Counter to determine when to move to the next env section.
		float counter;
		///	Our envelope value.
		float env;

		///	The filter for the white noise.
		Filter filter;
	};
	///	Called when the number of primary Quads changes.
	void numQuadsChanged();

	///	Updates a single Quad.
	void updateQuad(Quad2 *quad, float weight);

	///	This GO's index in the wider circle.
	int index;
	///	The centre of the wider circle.
	ThreeFloats centrePos;
	///	The number of primary Quads we have.
	unsigned int quadsSize;
	///	The radius of the circle our Quads follow.
	float radius;
	///	The initial radius of the circle.
	float radBase;
	///	Used to make sure the GOs rotate around their centre.
	float rotVal;

	///	We use this to determine the circle's centre and our position.
	/*!
		Immediately after CircleGO is constructed, it will be assigned the
		centre position via setPosition(), so we work out where it's supposed
		to go the first time updatePrimaryQuads() is called.
	 */
	bool firstRun;

	///	Which pitch we're currently on.
	int pitchIndex;
	///	Used to determine when to update our pitch.
	bool pitchUpdate;
};

}

#endif
