//	DiamondPercGO.cpp - Percussive Gravity Object.
//  ----------------------------------------------------------------------------
//	This file is part of 'Heilan Browser', an X3D browser specialised for audio
//	work.
//	Copyright (C) 2005-2009  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#include "DiamondPercGO.h"
#include "Quad.h"
#include "MiscHelper.h"

#include <ctime>
#include <cmath>
#include <iostream>

using namespace std;

namespace AshitakaClasses
{

//------------------------------------------------------------------------------
DiamondPercGO::DiamondPercGO():
GravityObject(),
triggerCount(0),
loopLength(176400),
frequency(50.0f),
sineAmp(2.0f),
duration(0.18f),
filterRes(0.98f),
position(0.0f),
samplerate(44100),
envIndex(0.0f),
speed(0.0f),
pos(0.0f),
c(0.000004042f),
explodeVal(1.0f),
explodeAmount(1.0f),
explodeSize(0.25f)
{
	
}

//------------------------------------------------------------------------------
DiamondPercGO::~DiamondPercGO()
{
	
}

//------------------------------------------------------------------------------
void DiamondPercGO::updatePrimaryQuads()
{
	int i, j;
	ThreeFloats goPos = getPosition();
	vector<Quad *>& quads = getPrimaryQuads();
	vector<Quad *>::iterator it;
	ThreeFloats verts[8];
	ThreeFloats tempCol;

	//First take account of pitch (hi = yellow, lo = red).
	tempCol = ThreeFloats(1.0f, (1.0f-((explodeSize-0.25f)*4.0f)), 0.0f);
	//Now interpolate w/blue (trigger = yellow/red, no trigger = blue).
	tempCol *= 1.0f * explodeSize;
	tempCol += ThreeFloats(0.0f, 0.0f, explodeAmount);

	if(explodeVal < 1.0f)
	{
		if(explodeVal < 0.005f)
		{
			explodeAmount = 0.0f;
			explodeVal += 0.001f;
		}
		else
		{
			explodeAmount = logarithm(explodeVal);
			explodeVal += 0.01f;
		}
		explodeAmount *= 0.5f + (explodeSize * 1.5f);
	}

	for(it=quads.begin(), i=0;it!=quads.end();++it, ++i)
	{
		switch(i%4)
		{
			//Top left.
			case 0:
				verts[0].x = goPos.x - explodeSize;
				verts[1].x = goPos.x;
				verts[2].x = goPos.x;
				verts[3].x = verts[0].x;
				verts[4].x = goPos.x;
				verts[5].x = goPos.x;
				verts[6].x = goPos.x;
				verts[7].x = goPos.x;

				verts[0].y = goPos.y;
				verts[1].y = goPos.y;
				verts[2].y = goPos.y;
				verts[3].y = goPos.y;
				verts[4].y = goPos.y + 0.5f;
				verts[5].y = goPos.y + 0.5f;
				verts[6].y = goPos.y + 0.5f;
				verts[7].y = goPos.y + 0.5f;

				verts[0].z = goPos.z;
				verts[1].z = goPos.z + explodeSize;
				verts[2].z = goPos.z - explodeSize;
				verts[3].z = goPos.z;
				verts[4].z = goPos.z;
				verts[5].z = goPos.z;
				verts[6].z = goPos.z;
				verts[7].z = goPos.z;

				for(j=0;j<8;++j)
				{
					verts[j].x -= explodeAmount;
					verts[j].y += explodeAmount;
				}
				break;
			//Top right.
			case 1:
				verts[0].x = goPos.x + explodeSize;
				verts[1].x = goPos.x;
				verts[2].x = goPos.x;
				verts[3].x = verts[0].x;
				verts[4].x = goPos.x;
				verts[5].x = goPos.x;
				verts[6].x = goPos.x;
				verts[7].x = goPos.x;

				verts[0].y = goPos.y;
				verts[1].y = goPos.y;
				verts[2].y = goPos.y;
				verts[3].y = goPos.y;
				verts[4].y = goPos.y + 0.5f;
				verts[5].y = goPos.y + 0.5f;
				verts[6].y = goPos.y + 0.5f;
				verts[7].y = goPos.y + 0.5f;

				verts[0].z = goPos.z;
				verts[1].z = goPos.z - explodeSize;
				verts[2].z = goPos.z + explodeSize;
				verts[3].z = goPos.z;
				verts[4].z = goPos.z;
				verts[5].z = goPos.z;
				verts[6].z = goPos.z;
				verts[7].z = goPos.z;

				for(j=0;j<8;++j)
				{
					verts[j].x += explodeAmount;
					verts[j].y += explodeAmount;
				}
				break;
			//Bottom left.
			case 2:
				verts[0].x = goPos.x - explodeSize;
				verts[1].x = goPos.x;
				verts[2].x = goPos.x;
				verts[3].x = verts[0].x;
				verts[4].x = goPos.x;
				verts[5].x = goPos.x;
				verts[6].x = goPos.x;
				verts[7].x = goPos.x;

				verts[0].y = goPos.y;
				verts[1].y = goPos.y;
				verts[2].y = goPos.y;
				verts[3].y = goPos.y;
				verts[4].y = goPos.y - 0.5f;
				verts[5].y = goPos.y - 0.5f;
				verts[6].y = goPos.y - 0.5f;
				verts[7].y = goPos.y - 0.5f;

				verts[0].z = goPos.z;
				verts[1].z = goPos.z + explodeSize;
				verts[2].z = goPos.z - explodeSize;
				verts[3].z = goPos.z;
				verts[4].z = goPos.z;
				verts[5].z = goPos.z;
				verts[6].z = goPos.z;
				verts[7].z = goPos.z;

				for(j=0;j<8;++j)
				{
					verts[j].x -= explodeAmount;
					verts[j].y -= explodeAmount;
				}
				break;
			//Top right.
			case 3:
				verts[0].x = goPos.x + explodeSize;
				verts[1].x = goPos.x;
				verts[2].x = goPos.x;
				verts[3].x = verts[0].x;
				verts[4].x = goPos.x;
				verts[5].x = goPos.x;
				verts[6].x = goPos.x;
				verts[7].x = goPos.x;

				verts[0].y = goPos.y;
				verts[1].y = goPos.y;
				verts[2].y = goPos.y;
				verts[3].y = goPos.y;
				verts[4].y = goPos.y - 0.5f;
				verts[5].y = goPos.y - 0.5f;
				verts[6].y = goPos.y - 0.5f;
				verts[7].y = goPos.y - 0.5f;

				verts[0].z = goPos.z;
				verts[1].z = goPos.z - explodeSize;
				verts[2].z = goPos.z + explodeSize;
				verts[3].z = goPos.z;
				verts[4].z = goPos.z;
				verts[5].z = goPos.z;
				verts[6].z = goPos.z;
				verts[7].z = goPos.z;

				for(j=0;j<8;++j)
				{
					verts[j].x += explodeAmount;
					verts[j].y -= explodeAmount;
				}
				break;
		}
		(*it)->setVertices(verts);
		(*it)->setColour(tempCol);
	}
}

//------------------------------------------------------------------------------
void DiamondPercGO::updateSecondaryQuad(Quad *quad)
{
	
}

//------------------------------------------------------------------------------
void DiamondPercGO::updateLifeForce()
{
	
}

//------------------------------------------------------------------------------
void DiamondPercGO::getMonoAudio(AmbisonicData *buffer, int numSamples)
{
	int i;
	bool trigger = false;
	map<unsigned int, float>::iterator it;
	float inc = frequency/(float)samplerate;
	float tempf;

	//********************************
	//TODO:
	//+ Calculate loop length.
	//+ Visuals.
	//+ Lifeforce.
	//- Metronome?
	//********************************

	for(i=0;i<numSamples;++i)
	{
		//Check if it's time to trigger.
		it = triggerBuffer.find(triggerCount);
		if(it != triggerBuffer.end())
		{
			float energy = 1.0f - it->second;

			trigger = true;
			envIndex = 1.0f;
			position = 0.0f;

			frequency = (energy*200.0f) + 50.0f;
			sineAmp = ((1.0f-energy)*1.7f) + 0.3f;
			duration = ((1.0f-energy)*0.16f) + 0.02f;
			filterRes = ((1.0f-energy)*0.06f) + 0.92f;
			inc = frequency/(float)samplerate;

			//Add on random amplitude variations.
			sineAmp += heilanRand(time(NULL)) * 0.25f;

			explodeSize = ((1.0f-energy)* 0.25f) + 0.25f;
			explodeVal = 0.0f;
		}
		else
			trigger = false;

		++triggerCount;
		if(triggerCount > loopLength)
		{
			float life = getLifeForce();

			triggerCount = 0;

			//We play the loop 16 times before dying.
			life -= 0.0625f;
			setLifeForce(life);
		}

		//Update sound.
		if(envIndex > 0.0f)
		{
			//Generate sinewave, amplitude envelope.
			tempf = cosf(position * 2.0f * 3.1415926535897932384626433832795f);
			tempf *= exponential(envIndex) * sineAmp;

			//Apply lowpass filter.
			c = 2.0f-2.0f*cosf(2.0f*3.1415926535897932384626433832795f*(frequency*4.0f)/(float)samplerate);
			speed = speed + (tempf - pos) * c;
			pos = pos + speed;
			speed = speed * filterRes;
			tempf = pos;

			//Apply tanh distortion.
			buffer[i].W = tanh(tempf * 1.5708f) * 0.9f;
			buffer[i].W *= 0.25f;

			//Update pitch envelope.
			tempf = logarithm(envIndex);
			inc = (tempf * frequency)/(float)samplerate;

			position += inc;
			while(position > 1.0f)
				position -= 1.0f;

			envIndex -= (1.0f/duration)/(float)samplerate;
		}
		else
			buffer[i].W = 0.0f;
	}
}

//------------------------------------------------------------------------------
void DiamondPercGO::setTriggerData(const map<unsigned int, float>& data,
								   unsigned int writePoint)
{
	map<unsigned int, float>::const_iterator it;
	map<unsigned int, float>::iterator it2;
	unsigned int startOffset = static_cast<unsigned int>(-1);
	bool secondHalf = true;

	//Set triggerBuffer.
	triggerBuffer.clear();

	if(data.size() > 0)
	{
		//1st half of the buffer (to the right of writePoint).
		for(it=data.lower_bound(writePoint);it!=data.end();++it)
		{
			if(startOffset == static_cast<unsigned int>(-1))
			{
				startOffset = it->first;
				cout << "1st half startOffset: " << startOffset << endl;
				secondHalf = false;
			}
	
			triggerBuffer.insert(make_pair((it->first - startOffset), it->second));
			cout << (it->first - startOffset) << endl;
		}

		//2nd half of the buffer (to the left of writePoint).
		if(startOffset != static_cast<unsigned int>(-1))
			startOffset = (--(triggerBuffer.end()))->first;
		cout << "New startOffset: " << startOffset << endl;
		for(it=data.begin();
			it!=data.lower_bound(writePoint);
			++it)
		{
			if(startOffset == static_cast<unsigned int>(-1))
			{
				startOffset = it->first;
				cout << "2nd half startOffset: " << startOffset << endl;
			}

			if(secondHalf)
				triggerBuffer.insert(make_pair((it->first - startOffset),
										   it->second));
			else
				triggerBuffer.insert(make_pair((it->first + startOffset),
											   it->second));
			cout << (it->first + startOffset) << endl;
		}

		//Make sure we end before the final trigger - we use it to measure the
		//correct loop length.
		it2 = triggerBuffer.end();
		--it2;
		loopLength = it2->first - 1;

		cout << "Loop length: " << (float)loopLength/44100.0f << endl;
	}
}

}
