//	DiamondPercGO.h - Percussive Gravity Object.
//  ----------------------------------------------------------------------------
//	This file is part of 'Heilan Browser', an X3D browser specialised for audio
//	work.
//	Copyright (C) 2005-2009  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#ifndef DIAMONDPERCGO_H_
#define DIAMONDPERCGO_H_

#include "GravityObject.h"

namespace AshitakaClasses
{

///	Percussive Gravity Object.
class DiamondPercGO : public GravityObject
{
  public:
	///	Constructor.
	DiamondPercGO();
	///	Destructor.
	~DiamondPercGO();

	///	Updates all this GO's primary Quads.
	void updatePrimaryQuads();
	///	Updates the position of the passed-in secondary Quad.
	void updateSecondaryQuad(Quad *quad);
	///	Updates this GO's lifeForce.
	void updateLifeForce();

	///	Returns the audio for this GO.
	void getMonoAudio(AmbisonicData *buffer, int numSamples);
	///	Sets the trigger data we use.
	void setTriggerData(const std::map<unsigned int, float>& data,
						unsigned int writePoint);
  private:
	///	Helper method to generate an exp curve.
	inline float exponential(float in)
	{
		const float a = (1.0f-1.0f/0.1f);

		return in/(in+a*(in-1.0f));
	};
	///	Helper method to generate a log curve.
	inline float logarithm(float in)
	{
		const float a = (1.0f-1.0f/0.9f);

		return in/(in+a*(in-1.0f));
	};

	///	When to trigger the percussive sounds.
	std::map<unsigned int, float> triggerBuffer;
	///	Current position within triggerBuffer.
	unsigned int triggerCount;
	///	The start offset of the loop.
	//unsigned int startOffset;
	///	The length of our loop.
	unsigned int loopLength;

	///	Starting frequency.
	float frequency;
	///	Starting amplitude.
	float sineAmp;
	///	Duration.
	float duration;
	///	Filter resonance.
	float filterRes;

	///	Position within the sine wave.
	float position;
	///	Samplerate.
	unsigned int samplerate;

	///	Envelope index.
	float envIndex;

	///	For the lowpass filter.
	float speed;
	///	For the lowpass filter.
	float pos;
	///	For the lowpass filter.
	float c;

	///	Governs how far the primary quads should be from the GO's centre.
	float explodeVal;
	///	Governs how far the primary quads should be from the GO's centre.
	float explodeAmount;
	///	Size of the Quads.
	float explodeSize;
};

}

#endif
