//	FlockingGO.cpp - Flocking-type GravityObject2.
//  ----------------------------------------------------------------------------
//	This file is part of 'Heilan Browser', an X3D browser specialised for audio
//	work.
//	Copyright (C) 2005-2009  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#include "FlockingGO.h"
#include "MiscHelper.h"

#include <cassert>
#include <iostream>

using namespace std;

namespace AshitakaClasses
{

//------------------------------------------------------------------------------
FlockingGO::FlockingGO():
GravityObject2(),
currentWaypoint(0),
waypointCounter(30),
waypointSteps(30),
brandNew(true),
startCounter(15),
lastNumQuads(0),
pitchIndex(0)
{
	FlockingExciter *excite = new FlockingExciter(this);
	setExciter(excite);
	setStringLength(128);
	setInfluenceRadius(1.0f);
	//setMainColour(ThreeFloats(0.25f, 0.5f, 0.25f));
	setMainColour(ThreeFloats(0.0f, 0.0f, 1.0f));
	decay = 0.45f;

	waypoints[0] = ThreeFloats(-4.0f, 0.0f, 0.0f);
	waypoints[1] = ThreeFloats(-2.0f, -2.0f, 0.0f);
	waypoints[2] = ThreeFloats(0.0f, -4.0f, 0.0f);
	waypoints[3] = ThreeFloats(2.0f, -2.0f, 0.0f);
	waypoints[4] = ThreeFloats(4.0f, 0.0f, 0.0f);
	waypoints[5] = ThreeFloats(0.0f, 0.0f, 4.0f);
	waypoints[6] = ThreeFloats(0.0f, 0.0f, -4.0f);
	waypoints[7] = ThreeFloats(0.0f, -4.0f, -4.0f);
	waypoints[8] = ThreeFloats(-4.0f, 0.0f, -4.0f);
	waypoints[9] = ThreeFloats(0.0f, 4.0f, -4.0f);

	//setPosition(waypoints[0]);
}

//------------------------------------------------------------------------------
FlockingGO::~FlockingGO()
{
	
}

//------------------------------------------------------------------------------
void FlockingGO::updatePrimaryQuads()
{
	float tempf;
	unsigned int i;
	ThreeFloats temp3f;
	const ThreeFloats origin;
	vector<Quad2 *>& quads = getPrimaryQuads();
	unsigned int quadsSize = quads.size();
	const ThreeFloats * const gesture = getGesture();
	FlockingExciter *excite = static_cast<FlockingExciter *>(getExciter());

	assert(quadsSize > 0);

	if(brandNew)
	{
		for(i=0;i<quadsSize;++i)
		{
			quads[i]->setDirection(ThreeFloats(0.0f, -1.0f, 0.0f));
			//quads[i]->moveTo(ThreeFloats(0.0f, 0.0f, 0.0f), 1.0f);
		}
		brandNew = false;
		//return;
	}

	//Start by moving all Quads towards the 1st waypoint.
	if(startCounter > 0)
	{
		for(i=0;i<quadsSize;++i)
		{
			temp3f.x = heilanRand(heilanRandSeed()) * 0.1f;
			temp3f.y = heilanRand(heilanRandSeed()) * 0.1f;
			temp3f.z = heilanRand(heilanRandSeed()) * 0.1f;
			temp3f += getPosition();
			quads[i]->moveTo(temp3f, 1.0f);
		}
		--startCounter;
		return;
	}

	//Get average position of flock.
	temp3f = quads[0]->getPosition();
	for(i=1;i<quadsSize;++i)
		temp3f += quads[i]->getPosition();
	averagePos = temp3f/(float)quadsSize;

	//Make the GravityObject's position the centre of the flock.
	setPosition(averagePos);

	//Get average direction of flock.
	temp3f = quads[0]->getDirection();
	for(i=1;i<quadsSize;++i)
		temp3f += quads[i]->getDirection();
	temp3f /= (float)quadsSize;
	tempf = temp3f.getDistance(origin);
	if(tempf != 0.0f)
		averageDir = temp3f/tempf;
	else
		averageDir = temp3f;

	//Move quads.
	for(i=0;i<quadsSize;++i)
		updateQuad(quads[i], 1.0f);

	//Update the exciter.
	lockCritSec();
	{
		for(i=0;i<quadsSize;++i)
			excite->setQuadPosition(i, quads[i]->getPosition(), averagePos);
	}
	unlockCritSec();

	//Update currentWaypoint.
	--waypointCounter;
	excite->setWaypointCounter(waypointCounter, waypointSteps);
	if(waypointCounter <= 0)
	{
		if(currentWaypoint < 9)
			tempf = gesture[currentWaypoint].getDistance(gesture[currentWaypoint+1]);
		else
			tempf = gesture[currentWaypoint].getDistance(gesture[0]);
		waypointSteps = (int)((tempf*10.0f) + 20.0f);
		waypointCounter = waypointSteps;
		++currentWaypoint;
		if(currentWaypoint > 9)
			currentWaypoint = 0;

		++pitchIndex;
		if(pitchIndex > 9)
			pitchIndex = 0;
		setPitch(getPitchContour()[pitchIndex]);
		//setPosition(gesture[currentWaypoint]);
	}
}

//------------------------------------------------------------------------------
void FlockingGO::updateSecondaryQuad(Quad2 *quad)
{
	updateQuad(quad, 0.25f);
}

//------------------------------------------------------------------------------
void FlockingGO::updateLifeForce()
{
	float life = getLifeForce();

	if(life > 0.0f)
		setLifeForce(life-0.001f);
}

//------------------------------------------------------------------------------
void FlockingGO::numQuadsChanged()
{
	FlockingExciter *excite = static_cast<FlockingExciter *>(getExciter());
	vector<Quad2 *>& quads = getPrimaryQuads();
	unsigned int quadsSize = quads.size();
	unsigned int i;

	//Update the exciter so it has the correct number of pulse trains.
	if(lastNumQuads < quadsSize)
	{
		for(i=lastNumQuads;i<quadsSize;++i)
			excite->addQuad();
	}
	else if(lastNumQuads > quads.size())
	{
		for(i=quadsSize;i<lastNumQuads;++i)
			excite->removeQuad();
	}

	lastNumQuads = quadsSize;
}

//------------------------------------------------------------------------------
void FlockingGO::gestureUpdated()
{
	int i;
	int count = 0;
	ThreeFloats temp3f;
	const ThreeFloats * const gesture = getGesture();

	for(i=0;i<10;++i)
	{
		if(gesture[i].getDistance(temp3f) > 0.25f)
		{
			temp3f = gesture[i];
			++count;
		}
	}

	//Follow a random path if the main GO's been static.
	if(count < 8)
	{
		ThreeFloats newGesture[10];

		for(i=0;i<10;++i)
		{
			newGesture[i].x = heilanRand(heilanRandSeed()) * 4.0f;
			newGesture[i].y = heilanRand(heilanRandSeed()) * 4.0f;
			newGesture[i].z = heilanRand(heilanRandSeed()) * 4.0f;
		}
		setGesture(newGesture);
	}
}

//------------------------------------------------------------------------------
void FlockingGO::updateQuad(Quad2 *quad, float weight)
{
	float tempf;
	//unsigned int i;
	ThreeFloats temp3f, temp3f2;
	ThreeFloats newDir;
	//vector<Quad2 *>& quads = getPrimaryQuads();
	//unsigned int quadsSize = quads.size();
	const ThreeFloats origin;
	//int crowdingCount = 0;
	ThreeFloats quadPos = quad->getPosition();
	const ThreeFloats * const gesture = getGesture();

	//Set direction to point to the next waypoint.
	temp3f = gesture[currentWaypoint] - quadPos;
	tempf = temp3f.getDistance(origin);
	if(tempf != 0.0f)
		temp3f /= tempf;
	if(waypointCounter > (int)(2.0f*((float)waypointSteps*0.333333f)))
		tempf = (float)(waypointSteps-waypointCounter)/((float)waypointSteps*0.333333f);
	else if(waypointCounter < (int)((float)waypointSteps*0.333333f))
		tempf = waypointCounter/((float)waypointSteps*0.333333f);
	else
		tempf = 1.0f;
	newDir = (temp3f * tempf) + (quad->getDirection() * (1.0f-tempf));

	//Set direction to avoid crowding nearby flockmates.
	//This makes things far too jittery (because of the collision detection?),
	//so I'm leaving it out for now.
	/*temp3f = 0.0f;
	for(i=0;i<quadsSize;++i)
	{
		if(quads[i] != quad)
		{
			tempf = quadPos.getDistance(quads[i]->getPosition());
			if(tempf < 0.25f)
			{
				temp3f2 = quads[i]->getPosition() - quadPos;
				if(tempf != 0.0f)
					temp3f2 /= tempf;
				temp3f += temp3f2;
				++crowdingCount;
			}
		}
	}
	if(crowdingCount > 0)
	{
		temp3f /= (float)crowdingCount;
		newDir += temp3f * 0.25f;
		//newDir += temp3f;
	}*/

	//Set direction to point to the centre of the flock.
	temp3f = averagePos - quadPos;
	tempf = quadPos.getDistance(averagePos);
	if(tempf != 0.0f)
		temp3f /= tempf;
	newDir += temp3f * 0.5f;

	//Update position accordingly.
	tempf = newDir.getDistance(origin);
	if(tempf != 0.0f)
		newDir /= tempf;
	tempf = averagePos.getDistance(gesture[currentWaypoint]);
	//tempf = quadPos.getDistance(gesture[currentWaypoint]);
	tempf *= 0.125f;
	temp3f = quadPos + (newDir * tempf);
    quad->moveTo(temp3f, weight);

	quad->setColour(ThreeFloats(0.0f, 0.0f, 1.0f), weight);
}

//------------------------------------------------------------------------------
//------------------------------------------------------------------------------
FlockingGO::FlockingExciter::FlockingExciter(FlockingGO *go):
Exciter(),
parentGO(go),
waypointCounter(0),
waypointSteps(30)
{
	addBuffer();
}

//------------------------------------------------------------------------------
FlockingGO::FlockingExciter::~FlockingExciter()
{
	
}

//------------------------------------------------------------------------------
vector<GravityObject2::Exciter::Buffer> *FlockingGO::FlockingExciter::getAudio(int numSamples)
{
	int i;
	unsigned int j, k;
	vector<Buffer>& buffers = getBuffers();
	float amp;
	float trainSize = 1.0f/(float)pulseTrains.size();

	if(waypointCounter > (int)(2.0f * ((float)waypointSteps*0.333333f)))
		amp = (float)(waypointSteps-waypointCounter)/((float)waypointSteps*0.333333f);
	else if(waypointCounter < ((float)waypointSteps*0.333333f))
		amp = waypointCounter/((float)waypointSteps*0.333333f);
	else
		amp = 1.0f;
	amp *= 2.0f;

	for(j=0;j<buffers.size();++j)
		buffers[j].position = (float)waypointCounter/(float)waypointSteps;

	for(i=0;i<numSamples;++i)
	{
		for(j=0;j<buffers.size();++j)
		{
			buffers[j].audioData[i] = 0.0f;
			for(k=0;k<pulseTrains.size();++k)
			{
				pulseTrains[k].index += pulseTrains[k].increment;
				if(pulseTrains[k].index >= 1.0f)
				{
					buffers[j].audioData[i] += pulseTrains[k].upOrDown;
					pulseTrains[k].upOrDown = -pulseTrains[k].upOrDown;
					pulseTrains[k].index -= 1.0f;
				}
			}
			buffers[j].audioData[i] *= trainSize;
			buffers[j].audioData[i] *= amp * 0.5f;
		}
	}

	return &buffers;
}

//------------------------------------------------------------------------------
void FlockingGO::FlockingExciter::addQuad()
{
	PulseTrain train;

	pulseTrains.push_back(train);
}

//------------------------------------------------------------------------------
void FlockingGO::FlockingExciter::removeQuad()
{
	pulseTrains.pop_back();
}

//------------------------------------------------------------------------------
void FlockingGO::FlockingExciter::setQuadPosition(unsigned int index,
												  const ThreeFloats& pos,
												  const ThreeFloats& centre)
{
	float tempf;
	ThreeFloats temp3f;

	if(index < pulseTrains.size())
	{
		//Set amplitude.
		tempf = centre.getDistance(pos);
		tempf *= 0.125f;
		if(tempf > 1.0f)
			tempf = 1.0f;
		pulseTrains[index].amplitude = 1.0f-tempf;

		//Set pitch (this is a stupid, hacky way of doing things).
		temp3f = centre - pos;
		if((temp3f.x < 0.0f) || (temp3f.y < 0.0f) || (temp3f.z < 0.0f))
			tempf = -tempf;
		tempf *= 8.0f;
		tempf += 130.8127826503f;
		pulseTrains[index].increment = tempf/44100.0f;
	}
}

//------------------------------------------------------------------------------
void FlockingGO::FlockingExciter::setWaypointCounter(int val, int steps)
{
	waypointCounter = val;
	waypointSteps = steps;
}

}
