//	FlockingGO.h - Flocking-type GravityObject2.
//  ----------------------------------------------------------------------------
//	This file is part of 'Heilan Browser', an X3D browser specialised for audio
//	work.
//	Copyright (C) 2005-2009  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#ifndef FLOCKINGGO_H_
#define FLOCKINGGO_H_

#include "GravityObject2.h"

namespace AshitakaClasses
{

///	Flocking-type GravityObject2.
class FlockingGO : public GravityObject2
{
  public:
	///	Constructor.
	FlockingGO();
	///	Destructor.
	~FlockingGO();

	///	Used to update the positions of the primary Quad2s.
	void updatePrimaryQuads();
	///	Used to update the position of a secondary Quad2.
	void updateSecondaryQuad(Quad2 *quad);
	///	Used to make the GO die off after a while.
	void updateLifeForce();
	///	Not used?
	void updateExciter() {};
  private:
	///	The FlockingGO's Exciter.
	class FlockingExciter : public GravityObject2::Exciter
	{
	  public:
		///	Constructor.
		FlockingExciter(FlockingGO *go);
		///	Destructor.
		~FlockingExciter();

		///	Where the excitation is generated.
		std::vector<Buffer> *getAudio(int numSamples);

		///	Adds another pulse train.
		void addQuad();
		///	Removes a pulse train.
		void removeQuad();
		///	Sets the indexed pulse train's amplitude and pitch.
		void setQuadPosition(unsigned int index,
							 const ThreeFloats& pos,
							 const ThreeFloats& centre);
		///	Updates waypointCounter.
		void setWaypointCounter(int val, int steps);
	  private:
		///	Data used to generate a pulse train.
		struct PulseTrain
		{
			///Constructor.
			PulseTrain():
			amplitude(0.0f),
			increment(1.0f),
			index(0.0f),
			upOrDown(1.0f)
			{
				
			};

			///	Amplitude of the output.
			float amplitude;
			///	How much to increment index by for each sample.
			float increment;
			///	Our position within the train.
			float index;
			///	We alternative between a +ve and a -ve pulse.
			float upOrDown;
		};

		///	Our parent GO.
		FlockingGO *parentGO;

		///	Our pulse trains.
		std::vector<PulseTrain> pulseTrains;

		///	A copy of FlockingGO's waypointCounter.
		int waypointCounter;
		///	The number of steps to the next waypoint.
		int waypointSteps;
	};
	///	Called when the number of primary Quads changes.
	void numQuadsChanged();
	///	Called when gesture is updated.
	void gestureUpdated();

	///	Helper method to update the position of the Quad2 in question.
	void updateQuad(Quad2 *quad, float weight);

	///	The average position of the flock.
	ThreeFloats averagePos;
	///	The average direction of the flock.
	ThreeFloats averageDir;

	///	Our list of waypoints.
	ThreeFloats waypoints[10];
	///	The waypoint we're currently at.
	int currentWaypoint;
	///	Used to determine when to move to the next waypoint.
	int waypointCounter;
	///	The number of steps to the next waypoint.
	int waypointSteps;

	///	True when we've just been constructed.
	bool brandNew;
	///	Counter used to ensure the Quads start at roughly the initial waypoint.
	int startCounter;
	///	The previous number of Quads we had.
	unsigned int lastNumQuads;

	///	Which pitch we're currently on.
	int pitchIndex;
};

}

#endif
