//	JellyfishGO.cpp - Jellyfish-type GravityObject.
//  ----------------------------------------------------------------------------
//	This file is part of 'Heilan Browser', an X3D browser specialised for audio
//	work.
//	Copyright (C) 2005-2009  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#include "JellyfishGO.h"
#include "Quad.h"

#include <iostream>

using namespace std;

namespace AshitakaClasses
{

//------------------------------------------------------------------------------
JellyfishGO::JellyfishGO():
GravityObject(),
height(0.0f),
state(StretchTop),
movementIndex(0.0f),
movementInc(0.01f)
{
	
}

//------------------------------------------------------------------------------
JellyfishGO::~JellyfishGO()
{
	
}

//------------------------------------------------------------------------------
void JellyfishGO::updatePrimaryQuads()
{
	int i;
	float extraQuads;
	ThreeFloats goPos = getPosition();
	vector<Quad *>& quads = getPrimaryQuads();
	vector<Quad *>::iterator it;
	ThreeFloats verts[8];

	//Update height and y position.
	switch(state)
	{
		case StretchTop:
			height = 0.5f + movementIndex;
			break;
		case MoveBottom:
			height = 1.5f - movementIndex;
			goPos.y += (movementInc * 1.25f);
			setPosition(goPos);
			break;
		case MoveAll:
			goPos.y += movementInc;
			setPosition(goPos);
			break;
	}

	//Update state.
	if(state == MoveAll)
		movementInc *= 0.99f;
	movementIndex += movementInc;
	if((movementIndex >= 1.0f) && (state != MoveAll))
	{
		++state;
		movementIndex = 0.0f;
	}
	else if((state == MoveAll) && (movementIndex > 0.9f))
	{
		state = StretchTop;
		movementIndex = 0.0f;
		movementInc = 0.01f;
	}

	//Set Quad vertices.
	i = 0;
	extraQuads = 0.0f;
	for(it=quads.begin();it!=quads.end();++it)
	{
		switch(i)
		{
			//Cap Quad.
			case 0:
				verts[0].y = goPos.y + (height * 0.5f) + extraQuads;
				verts[1].y = verts[0].y;
				verts[2].y = verts[0].y;
				verts[3].y = verts[0].y;
				verts[4].y = goPos.y + height + extraQuads;
				verts[5].y = verts[4].y;
				verts[6].y = verts[4].y;
				verts[7].y = verts[4].y;

				verts[0].x = goPos.x - 0.108f;
				verts[1].x = goPos.x + 0.108f;
				verts[2].x = goPos.x;
				verts[3].x = goPos.x;
				verts[4].x = goPos.x - 0.125f;
				verts[5].x = goPos.x + 0.125f;
				verts[6].x = goPos.x;
				verts[7].x = goPos.x;

				verts[0].z = goPos.z - 0.0625f;
				verts[1].z = goPos.z - 0.0625f;
				verts[2].z = goPos.z + 0.125f;
				verts[3].z = goPos.z + 0.125f;
				verts[4].z = goPos.z - 0.125f;
				verts[5].z = goPos.z - 0.125f;
				verts[6].z = goPos.z + 0.25f;
				verts[7].z = goPos.z + 0.25f;
				break;
			//Front Quad.
			case 1:
				verts[0].y = goPos.y + extraQuads;
				verts[1].y = verts[0].y;
				verts[2].y = verts[0].y;
				verts[3].y = verts[0].y;
				verts[4].y = goPos.y + height + extraQuads;
				verts[5].y = verts[4].y;
				verts[6].y = verts[4].y;
				verts[7].y = verts[4].y;

				verts[0].x = goPos.x - 0.433f;
				verts[1].x = goPos.x + 0.433f;
				verts[2].x = goPos.x + 0.325f;
				verts[3].x = goPos.x - 0.325f;
				verts[4].x = goPos.x - 0.125f;
				verts[5].x = goPos.x + 0.125f;
				verts[6].x = goPos.x + 0.108f;
				verts[7].x = goPos.x - 0.108f;

				verts[0].z = goPos.z - 0.25f;
				verts[1].z = goPos.z - 0.25f;
				verts[2].z = goPos.z - 0.1875f;
				verts[3].z = goPos.z - 0.1875f;
				verts[4].z = goPos.z - 0.125f;
				verts[5].z = goPos.z - 0.125f;
				verts[6].z = goPos.z - 0.0625f;
				verts[7].z = goPos.z - 0.0625f;
				break;
			//Left Quad.
			case 2:
				verts[0].y = goPos.y + extraQuads;
				verts[1].y = verts[0].y;
				verts[2].y = verts[0].y;
				verts[3].y = verts[0].y;
				verts[4].y = goPos.y + height + extraQuads;
				verts[5].y = verts[4].y;
				verts[6].y = verts[4].y;
				verts[7].y = verts[4].y;

				verts[0].x = goPos.x;
				verts[1].x = goPos.x - 0.433f;
				verts[2].x = goPos.x - 0.325f;
				verts[3].x = goPos.x;
				verts[4].x = goPos.x;
				verts[5].x = goPos.x - 0.125f;
				verts[6].x = goPos.x - 0.108f;
				verts[7].x = goPos.x;

				verts[0].z = goPos.z + 0.5f;
				verts[1].z = goPos.z - 0.25f;
				verts[2].z = goPos.z - 0.1875f;
				verts[3].z = goPos.z + 0.375f;
				verts[4].z = goPos.z + 0.25f;
				verts[5].z = goPos.z - 0.125f;
				verts[6].z = goPos.z - 0.0625f;
				verts[7].z = goPos.z + 0.125f;
				break;
			//Right Quad.
			case 3:
				verts[0].y = goPos.y + extraQuads;
				verts[1].y = verts[0].y;
				verts[2].y = verts[0].y;
				verts[3].y = verts[0].y;
				verts[4].y = goPos.y + height + extraQuads;
				verts[5].y = verts[4].y;
				verts[6].y = verts[4].y;
				verts[7].y = verts[4].y;

				verts[0].x = goPos.x;
				verts[1].x = goPos.x + 0.433f;
				verts[2].x = goPos.x + 0.325f;
				verts[3].x = goPos.x;
				verts[4].x = goPos.x;
				verts[5].x = goPos.x + 0.125f;
				verts[6].x = goPos.x + 0.108f;
				verts[7].x = goPos.x;

				verts[0].z = goPos.z + 0.5f;
				verts[1].z = goPos.z - 0.25f;
				verts[2].z = goPos.z - 0.1875f;
				verts[3].z = goPos.z + 0.375f;
				verts[4].z = goPos.z + 0.25f;
				verts[5].z = goPos.z - 0.125f;
				verts[6].z = goPos.z - 0.0625f;
				verts[7].z = goPos.z + 0.125f;
				break;
		}
		(*it)->setVertices(verts);

		++i;
		if(i == 4)
		{
			i = 0;
			extraQuads += -0.5f;
		}
	}
}

//------------------------------------------------------------------------------
void JellyfishGO::updateSecondaryQuad(Quad *quad)
{
	
}

//------------------------------------------------------------------------------
void JellyfishGO::updateLifeForce()
{
	float life = getLifeForce();

	life -= 0.0005f;
	if(life >= 0.0f)
		setLifeForce(life);
	else
		setLifeForce(0.0f);
}

//------------------------------------------------------------------------------
void JellyfishGO::getMonoAudio(AmbisonicData *buffer, int numSamples)
{
	int i;

	for(i=0;i<numSamples;++i)
		buffer[i] = 0.0f;
}

}
