//	JellyfishGO.h - Jellyfish-type GravityObject.
//  ----------------------------------------------------------------------------
//	This file is part of 'Heilan Browser', an X3D browser specialised for audio
//	work.
//	Copyright (C) 2005-2009  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#ifndef JELLYFISHGO_H_
#define JELLYFISHGO_H_

#include "GravityObject.h"

namespace AshitakaClasses
{

///	Jellyfish-type GravityObject.
class JellyfishGO : public GravityObject
{
  public:
	///	Constructor.
	JellyfishGO();
	///	Destructor.
	~JellyfishGO();

	///	Updates all this GO's primary Quads.
	void updatePrimaryQuads();
	///	Updates the position of the passed-in secondary Quad.
	void updateSecondaryQuad(Quad *quad);
	///	Updates this GO's lifeForce.
	void updateLifeForce();

	///	Returns the audio for this GO.
	void getMonoAudio(AmbisonicData *buffer, int numSamples);
  private:
	///	The various movement states we might be in.
	enum
	{
		StretchTop,
		MoveBottom,
		MoveAll
	};

	///	The height of the jellyfish.
	float height;
	///	Whether we're moving the cap, the skirt, or staying still.
	int state;
	///	Index used in the movement.
	float movementIndex;
	///	How much to increment movementIndex by each time.
	float movementInc;
};

}

#endif
