//	Map1InParamFilter.cpp - Simple low pass filter for smoothing parameter
//						    changes.
//  ----------------------------------------------------------------------------
//	This file is part of 'Heilan Browser', an X3D browser specialised for audio
//	work.
//	Copyright (C) 2005-2009  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#include "Map1InParamFilter.h"

#include <cmath>

//------------------------------------------------------------------------------
Map1InParamFilter::Map1InParamFilter():
Parameter()
{
	int i;

	const double scale = 100;
	// 0 < Smoothness < 1. High is better, but may cause precision problems
	const double smoothness = 0.999;
	double a = 1.0-(2.4/scale);
	double b = smoothness;

	coeff[0] = a;
	coeff[1] = a*b;
	coeff[2] = a*b*b;
	double mastergain = 1.0 / (-1.0/(log(a)+2.0*log(b))+2.0/
						(log(a)+log(b))-1.0/log(a));
	gain[0] = mastergain;
	gain[1] = mastergain * (log(a*b*b)*(log(a)-log(a*b)) /
							((log(a*b*b)-log(a*b))*log(a*b))
							- log(a)/log(a*b));
	gain[2] = mastergain * (-(log(a)-log(a*b)) / (log(a*b*b)-log(a*b)));

	for(i=0;i<3;++i)
		reg[i] = 0.0f;
}

//------------------------------------------------------------------------------
Map1InParamFilter::~Map1InParamFilter()
{
	
}

//------------------------------------------------------------------------------
void Map1InParamFilter::valueChanged(float val)
{
	float tempf;

	reg[0] = coeff[0] * reg[0] + val;
	reg[1] = coeff[1] * reg[1] + val;
	reg[2] = coeff[2] * reg[2] + val;

	tempf = (gain[0] * reg[0]) + (gain[1] * reg[1]) + (gain[2] * reg[2]);

	if(tempf > 1.0f)
		tempf = 1.0f;
	else if(tempf < 0.0f)
		tempf = 0.0f;

	Parameter::valueChanged(tempf);
}
