//	Map3InSum.h - Takes 3 parameters as inputs, sums them together for the
//				  output.
//  ----------------------------------------------------------------------------
//	This file is part of 'Heilan Browser', an X3D browser specialised for audio
//	work.
//	Copyright (C) 2005-2009  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#ifndef MAP3INSUM_H_
#define MAP3INSUM_H_

#include "Parameter.h"

///	Takes 3 parameters as inputs, sums them together for the output.
/*!
	\ingroup AshitakaParameters
	\ingroup libheilanextras
 */
class Map3InSum : public Parameter
{
  public:
	///	Constructor.
	Map3InSum();
	///	Destructor.
	~Map3InSum();

	///	Sets the 3 inputs for this mapping.
	/*!
		You *MUST* call this with the required input Parameters before you
		attempt to use this class, or you'll get no output from it.
	 */
	void setInputs(Parameter *p1, Parameter *p2, Parameter *p3);

	///	Sums the 3 inputs together, outputs them to the registered callbacks.
	void valueChanged(float val);
  private:
	///	Input parameter 1.
	Parameter *param1;
	///	Input parameter 2.
	Parameter *param2;
	///	Input parameter 3.
	Parameter *param3;
};

#endif
