//	PolyhedronGO.cpp - Polyhedron-based GravityObject.
//  ----------------------------------------------------------------------------
//	This file is part of 'Heilan Browser', an X3D browser specialised for audio
//	work.
//	Copyright (C) 2005-2009  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#include "PolyhedronGO.h"

#include "SDL.h"
#include <iostream>

using namespace std;

namespace AshitakaClasses
{

//------------------------------------------------------------------------------
PolyhedronGO::PolyhedronGO():
GravityObject(),
numRows(8),
numColumns(8),
numSecondary(0),
streamCounter(0)
{
	int i, j;

	//setInfluenceRadius(1.0f);
	size = 1.0f;

	for(j=0;j<3;++j)
	{
		for(i=0;i<4;++i)
			streamVals[j][i] = 0.0f;
	}
}

//------------------------------------------------------------------------------
PolyhedronGO::~PolyhedronGO()
{
	
}

//------------------------------------------------------------------------------
void PolyhedronGO::updateStart()
{
	int i;
	float tempf, tempDistance, maxDistance;
	ThreeFloats temp3f, temp3f2;
	vector<Quad *>& quads = getPrimaryQuads();
	int tempNumQuads = quads.size();

	//Update position.
	temp3f.x = xPos;
	temp3f.y = yPos;
	temp3f.z = zPos;
	setPosition(temp3f);

	//Update streamVals.
	if(streamCounter == 0)
	{
		streamVals[0][3] = streamVals[0][2];
		streamVals[0][2] = streamVals[0][1];
		streamVals[0][1] = streamVals[0][0];
		streamVals[0][0] = stream1;
		streamVals[1][3] = streamVals[1][2];
		streamVals[1][2] = streamVals[1][1];
		streamVals[1][1] = streamVals[1][0];
		streamVals[1][0] = stream2;
		streamVals[2][3] = streamVals[2][2];
		streamVals[2][2] = streamVals[2][1];
		streamVals[2][1] = streamVals[2][0];
		streamVals[2][0] = stream3;
		streamCounter = 4;
	}
	else
		--streamCounter;

	numQuads = (float)tempNumQuads/64.0f;
	consistency = explode; //Update this to be a bit more complicated later...
	perceivedSize = 1.0f;
	tempf = 0.0f;
	maxDistance = 0.0f;
	for(i=0;i<tempNumQuads;++i)
	{
		temp3f2 = quads[i]->getCentre();
		temp3f2 -= temp3f;
		tempDistance = sqrtf((temp3f2.x*temp3f2.x) +
							 (temp3f2.y*temp3f2.y) +
							 (temp3f2.z*temp3f2.z));
		if(tempDistance > maxDistance)
			maxDistance = tempDistance;

		tempf += quads[i]->getVelocity();
	}

	perceivedSize = maxDistance * (1.0f/64.0f);

	tempf /= (float)tempNumQuads;
	avgVelocity = tempf * (1.0f/32.0f);
}

//------------------------------------------------------------------------------
void PolyhedronGO::updatePrimaryQuads()
{
	int x, y;
	unsigned int counter = 0;
	vector<Quad *>quads = getPrimaryQuads();

	for(y=0;y<numColumns;++y)
	{
		for(x=0;x<numRows;++x)
		{
			setQuad(quads[counter],
					(float)x/(float)numRows,
					((float)y/((float)numColumns)*0.5f)+0.25f);
			++counter;
		}
	}

	while(counter < quads.size())
	{
		for(y=0;y<numColumns;++y)
		{
			for(x=0;x<numRows;++x)
			{
				setQuad(quads[counter],
						(float)x/(float)numRows,
						((float)y/((float)numColumns)*0.5f)+0.25f);
				++counter;
				if(counter >= quads.size())
					break;
			}
			if(counter >= quads.size())
				break;
		}
	}
}

//------------------------------------------------------------------------------
void PolyhedronGO::updateSecondaryQuad(Quad *quad)
{
	/*int x, y;
	ThreeFloats goPos = getPosition();
	ThreeFloats qPos = goPos - quad->getCentre();
	float distance;

	distance = sqrtf((qPos.x*qPos.x) + (qPos.y*qPos.y) + (qPos.z*qPos.z));
	if(distance < 1.0f)
		distance = 0.0f;
	else
		distance = 1.0f/512.0f;

	numRows = getSquareRoot(getPrimaryQuads().size() + numSecondary);
	numColumns = numRows; //These don't really need to be separate variables...

	y = numSecondary/numColumns;
	x = numSecondary-y;
	setQuad(quad,
			(float)x/(float)numRows,
			((float)y/((float)numColumns)*0.5f)+0.25f,
			distance);

	++numSecondary;*/

	//Act as a simple attractor.
	/*{
		float tempf;
		float distance;
		ThreeFloats force;
		ThreeFloats direction;
		ThreeFloats tempAccel;
		ThreeFloats tempVel = quad->getVelocity3d();
		ThreeFloats qPos = quad->getCentre();
		ThreeFloats verts[8];
		ThreeFloats goPos = getPosition();

		//Calc direction and distance.
		direction = goPos - qPos;

		distance = sqrtf((direction.x * direction.x) +
						 (direction.y * direction.y) +
						 (direction.z * direction.z));

		//Normalise direction.
		direction = direction/distance;

		if(distance < 5.0f)
			distance = 5.0f;
		else if(distance > 25.0f)
			distance = 25.0f;

		//Calc force (F=(G*goMass*qMass)/(distance *distance)).
		tempf = (0.001f * (10.0f * size) * 10.0f)/(distance * distance);
		force = direction * tempf;

		//Update centre position.
		tempAccel = force/10.0f;
		//tempAccel *= -1.0f; //Repulsion
		tempVel += tempAccel;

		tempf = sqrtf((tempVel.x*tempVel.x) + (tempVel.y*tempVel.y) + (tempVel.z*tempVel.z));
		if(tempf > 1.0f)
		{
			tempVel = tempVel/tempf;
			tempVel *= 10.0f;
		}
		qPos += tempVel;

		//Update vertices.
		positionVertices(verts, qPos, tempVel*4.0f);
		quad->setVertices(verts);
	}*/
}

//------------------------------------------------------------------------------
void PolyhedronGO::updateLifeForce()
{
	
}

//------------------------------------------------------------------------------
void PolyhedronGO::getMonoAudio(AmbisonicData *buffer, int numSamples)
{
	int i;

	for(i=0;i<numSamples;++i)
		buffer[i] = 0.0f;
}

//------------------------------------------------------------------------------
void PolyhedronGO::numQuadsChanged()
{
	numRows = getSquareRoot(getPrimaryQuads().size());
	numColumns = numRows; //These don't really need to be separate variables...
}

}
