//	PulsatingGO.cpp - Pulsating-type GravityObject2.
//  ----------------------------------------------------------------------------
//	This file is part of 'Heilan Browser', an X3D browser specialised for audio
//	work.
//	Copyright (C) 2005-2009  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#include "PulsatingGO.h"
#include "MiscHelper.h"

using namespace std;

namespace AshitakaClasses
{

//------------------------------------------------------------------------------
PulsatingGO::PulsatingGO():
GravityObject2(),
justStarted(true),
startedCount(30),
pulseSection(true),
pulseIndex(0.0f),
pulseEnv(0.0f),
pitchIndex(0)
{
	PulsatingExciter *excite = new PulsatingExciter(this);
	setExciter(excite);
	setStringLength(128);
	setInfluenceRadius(1.0f);
	setMainColour(ThreeFloats(1.0f, 0.1f, 1.0f));
	decay = 0.6f;
}

//------------------------------------------------------------------------------
PulsatingGO::~PulsatingGO()
{
	
}

//------------------------------------------------------------------------------
void PulsatingGO::updatePrimaryQuads()
{
	unsigned int i;
	ThreeFloats temp3f;
	vector<Quad2 *>& quads = getPrimaryQuads();
	unsigned int quadsSize = quads.size();

	if(justStarted)
	{
		--startedCount;
		if(!startedCount)
			justStarted = false;
	}
	else
	{
		//We're on our way up.
		if(pulseSection)
		{
			pulseIndex += 0.01f;
			if(pulseIndex >= 1.0f)
			{
				pulseIndex = 1.0f;
				pulseSection = false;
				temp3f.x = heilanRand(heilanRandSeed()) * 4.0f;
				temp3f.y = heilanRand(heilanRandSeed()) * 4.0f;
				temp3f.z = heilanRand(heilanRandSeed()) * 4.0f;
				setPosition(temp3f);
				((PulsatingExciter *)getExciter())->changePosition();
			}
			pulseEnv = heilanExp(pulseIndex);
		}
		else //We're on our way down.
		{
			pulseIndex -= 0.075f;
			if(pulseIndex <= 0.0f)
			{
				pulseIndex = 0.0f;
				pulseSection = true;

				++pitchIndex;
				if(pitchIndex > 9)
					pitchIndex = 0;
				cout << "PGO: " << getPitchContour()[pitchIndex] << " : " << pitchIndex << endl;
				setPitch(getPitchContour()[pitchIndex]);
				cout << "  Pitch Counter set." << endl;
			}
			pulseEnv = heilanLog(pulseIndex);
		}
	}

	for(i=0;i<quadsSize;++i)
		updateQuad(quads[i], 1.0f);
}

//------------------------------------------------------------------------------
void PulsatingGO::updateSecondaryQuad(Quad2 *quad)
{
	updateQuad(quad, 0.25f);
}

//------------------------------------------------------------------------------
void PulsatingGO::updateLifeForce()
{
	float life = getLifeForce();

	if(life > 0.0f)
		setLifeForce(life-0.001f);
}

//------------------------------------------------------------------------------
void PulsatingGO::updateQuad(Quad2 *quad, float weight)
{
	float tempf;
	ThreeFloats temp3f;
	ThreeFloats origin;
	const ThreeFloats& goPos = getPosition();

	//quad->moveTo(quad->getPosition()+quad->getDirection(), weight);

	tempf = quad->getDirection().getDistance(origin);
	if((tempf > 1.1f) || (tempf < 0.9f))
	{
		temp3f.x = heilanRand(heilanRandSeed());
		temp3f.y = heilanRand(heilanRandSeed());
		temp3f.z = heilanRand(heilanRandSeed());
		tempf = temp3f.getDistance(origin);
		temp3f /= tempf;
		quad->setDirection(temp3f);
	}
	temp3f = goPos + (quad->getDirection() * ((pulseEnv * 1.65f)+0.35f));
	/*temp3f -= quad->getNewPosition();
	temp3f /= temp3f.getDistance(origin);
	quad->setDirection(temp3f);*/
	quad->moveTo(temp3f, weight);

	//Update the quad's colour.
	temp3f = ThreeFloats(1.0f, 0.25f, 0.25f);
	temp3f *= 1.0f-pulseEnv;
	temp3f += ThreeFloats(1.0f, 0.0f, 1.0f) * pulseEnv;
	quad->setColour(temp3f, weight);
}

//------------------------------------------------------------------------------
//------------------------------------------------------------------------------
PulsatingGO::PulsatingExciter::PulsatingExciter(PulsatingGO *go):
Exciter(),
parentGO(go),
freq(184.997211356f),
saw(0.0f)
{
	addBuffer();
	changePosition();
}

//------------------------------------------------------------------------------
PulsatingGO::PulsatingExciter::~PulsatingExciter()
{
	
}

//------------------------------------------------------------------------------
vector<GravityObject2::Exciter::Buffer> *PulsatingGO::PulsatingExciter::getAudio(int numSamples)
{
	int i;
	float tempAmp;
	unsigned int j;
	float inc = freq/44100.0f;
	vector<Buffer>& buffers = getBuffers();

	for(i=0;i<numSamples;++i)
	{
		saw -= inc;
		if(saw < -1.0f)
			saw += 2.0f;

		tempAmp = parentGO->getPulseEnv();

		for(j=0;j<buffers.size();++j)
		{
			buffers[j].position = 0.5f;
			buffers[j].audioData[i] = saw * tempAmp * 0.5f;
		}
	}

	return &buffers;
}

//------------------------------------------------------------------------------
void PulsatingGO::PulsatingExciter::changePosition()
{
	unsigned int i;
	vector<Buffer>& buffers = getBuffers();

	for(i=0;i<buffers.size();++i)
		buffers[i].position = (heilanRand(heilanRandSeed())+1.0f)*0.5f;
}

}
