//	PulsatingGO.h - Pulsating-type GravityObject2.
//  ----------------------------------------------------------------------------
//	This file is part of 'Heilan Browser', an X3D browser specialised for audio
//	work.
//	Copyright (C) 2005-2009  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#ifndef PULSATINGGO_H_
#define PULSATINGGO_H_

#include "GravityObject2.h"

namespace AshitakaClasses
{

///	Pulsating-type GravityObject2.
class PulsatingGO : public GravityObject2
{
  public:
	///	Constructor.
	PulsatingGO();
	///	Destructor.
	~PulsatingGO();

	///	Used to update the positions of the primary Quad2s.
	void updatePrimaryQuads();
	///	Used to update the position of a secondary Quad2.
	void updateSecondaryQuad(Quad2 *quad);
	///	Used to make the GO die off after a while.
	void updateLifeForce();
	///	Not used?
	void updateExciter() {};

	///	So our Exciter can get a hold of our current pulse envelope.
	float getPulseEnv() const {return pulseEnv;};
  private:
	//--------------------------------------------------------------------------
	///	Class representing how PulsatingGO excites its string model.
	class PulsatingExciter : public GravityObject2::Exciter
	{
	  public:
		///	Constructor.
		PulsatingExciter(PulsatingGO *go);
		///	Destructor.
		~PulsatingExciter();

		///	Where the excitation is generated.
		std::vector<Buffer> *getAudio(int numSamples);

		///	Randomises our position on the string.
		void changePosition();
	  private:
		///	Our parent PulsatingGO.
		PulsatingGO *parentGO;

		///	The frequency of the sawtooth wave.
		float freq;
		///	The current output of the sawtooth wave.
		float saw;
	};

	///	Helper method to update the position of the Quad2 in question.
	void updateQuad(Quad2 *quad, float weight);

	///	When the GO's first constructed, do nothing while the Quads move to it.
	bool justStarted;
	///	Counter to determine when to switch off justStarted.
	int startedCount;
	///	Whether we're on the up(true) or down(false) section of the pulse.
	bool pulseSection;
	///	Our index to the envelope.
	float pulseIndex;
	///	Our pulse's envelope value.
	float pulseEnv;

	///	Which pitch we're currently on.
	int pitchIndex;
};

}

#endif
