//	Quad.cpp - The element which is controlled by GravityObjects.
//  ----------------------------------------------------------------------------
//	This file is part of 'Heilan Browser', an X3D browser specialised for audio
//	work.
//	Copyright (C) 2005-2009  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#include "Quad.h"

#include "SDL.h"
#include "SDL_image.h"
#include <cmath>
#include <cstdlib>
#include <iostream>

using namespace std;

namespace AshitakaClasses
{

//------------------------------------------------------------------------------
Quad::Quad():
velocity(0.0f),
acceleration(0.0f),
goSize(1.0f),
drawTrail(0.0f),
trailCount(0),
trailWritePos(0),
texture(0)
{
	centre.x = (float)rand()/((float)RAND_MAX * 0.25f);
	centre.x -= 2.0f;
	centre.y = (float)rand()/((float)RAND_MAX * 0.25f);
	centre.y -= 2.0f;
	centre.z = (float)rand()/((float)RAND_MAX * 0.25f);
	centre.z -= 2.0f;

	velocity = 1.0f;
	direction.y = 1.0f;

	brightness = (((float)rand()/(float)RAND_MAX) * 0.75f) + 0.2f;
#ifdef INVERT_COLOURS
	setColour(ThreeFloats(0.0f, 0.0f, 0.0f));
#else
	setColour(ThreeFloats(0.0f, 0.0f, 1.0f));
#endif

	vertices[0].x = -0.5f;
	vertices[0].y = -0.5f;
	vertices[0].z = -0.5f;

	vertices[1].x = 0.5f;
	vertices[1].y = -0.5f;
	vertices[1].z = -0.5f;

	vertices[2].x = 0.5f;
	vertices[2].y = -0.5f;
	vertices[2].z = 0.5f;

	vertices[3].x = -0.5f;
	vertices[3].y = -0.5f;
	vertices[3].z = 0.5f;

	vertices[4].x = -0.5f;
	vertices[4].y = 0.5f;
	vertices[4].z = -0.5f;

	vertices[5].x = 0.5f;
	vertices[5].y = 0.5f;
	vertices[5].z = -0.5f;

	vertices[6].x = 0.5f;
	vertices[6].y = 0.5f;
	vertices[6].z = 0.5f;

	vertices[7].x = -0.5f;
	vertices[7].y = 0.5f;
	vertices[7].z = 0.5f;
}

//------------------------------------------------------------------------------
Quad::~Quad()
{
	
}

//------------------------------------------------------------------------------
void Quad::draw()
{
	//float transVal = 0.05f-((centre.z + 8.0f)/320.0f); //(0.05->0)

	/*if(texture == 0)
		loadTexture();*/

	glMatrixMode(GL_MODELVIEW);
	glPushMatrix();

	glDisable(GL_LIGHTING);
	glDisable(GL_DEPTH_TEST);
	glEnable(GL_BLEND);

#ifdef INVERT_COLOURS
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
#else
	glBlendFunc(GL_SRC_ALPHA, GL_ONE);
#endif
	drawQuad(0.05f, true);
	//drawBlob(0.25f);

	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	glEnable(GL_DEPTH_TEST);
	glEnable(GL_LIGHTING);

	glPopMatrix();
}

//------------------------------------------------------------------------------
void Quad::setVertices(ThreeFloats *verts)
{
	int i;
	ThreeFloats lastCentre = centre;
	float lastVelocity = velocity;

	for(i=0;i<8;++i)
		vertices[i] = verts[i];

	//I'm assuming this is reasonable to calculate the centre?
	centre = 0.0f;
	for(i=0;i<8;++i)
	{
		centre.x += vertices[i].x;
		centre.y += vertices[i].y;
		centre.z += vertices[i].z;
	}
	centre.x /= 8.0f;
	centre.y /= 8.0f;
	centre.z /= 8.0f;

	//Calculate velocity.
	velocity2.x = centre.x - lastCentre.x;
	velocity2.y = centre.y - lastCentre.y;
	velocity2.z = centre.z - lastCentre.z;
	velocity = sqrtf((velocity2.x * velocity2.x) +
		  			 (velocity2.y * velocity2.y) +
		  			 (velocity2.z * velocity2.z));

	//Calculate acceleration.
	acceleration = velocity - lastVelocity;
	lastVelocity = velocity;

	//Calculate the direction.
	if(velocity != 0.0f)
	{
		direction.x = centre.x/velocity;
		direction.y = centre.y/velocity;
		direction.z = centre.z/velocity;
	}

	/*if((velocity > 0.01f) && (drawTrail == 0.0f))
	{
		//++trailCount;
		//if(trailCount > 15)
		{
			trailEnd = centre;
			drawTrail = 1.0f;
			trailCount = 0;
		}
		//else if(trailCount == 1)
			trailStart = lastCentre;
	}
	else if((velocity > 0.01f) && (drawTrail > 0.0f))
	{
		drawTrail += 0.03f;
	}
	else
		trailCount = 0;*/

	++trailWritePos;
	if(trailWritePos >= TrailLength)
		trailWritePos = 0;
	trailPos[trailWritePos] = centre;

	if(velocity > 0.01f)
	{
		drawTrail = 1.0f;
	}
}

//------------------------------------------------------------------------------
void Quad::setColour(const ThreeFloats& col)
{
	baseColour = col;
	colour = baseColour;
	if(brightness < 0.5f)
		colour *= brightness * 2.0f;
	else
	{
		float tempf = brightness;

		tempf -= 0.5f;
		tempf *= 2.0f;
		colour = (colour * (1.0f-tempf)) + (ThreeFloats(1.0f) * tempf);
	}
}

//------------------------------------------------------------------------------
void Quad::setPrimaryGravityObject(GravityObject *obj)
{
	primary = obj;
}

//------------------------------------------------------------------------------
void Quad::setSize(float val)
{
	goSize = val;
}

//------------------------------------------------------------------------------
void Quad::drawQuad(float alpha, bool trail)
{
	int i, j, k;
	float tempf, tempf2;
	float tempCol, tempCol2;

	//First draw trail, if necessary.
	if((drawTrail > 0.0f) && trail)
	{
		for(i=trailWritePos,j=0;j<TrailLength-1;++i,++j)
		{
			if((i >= TrailLength) || (i == 0))
			{
				i = 0;
				k = TrailLength - 1;
			}
			else
				k = i - 1;

			tempf = (float)j/(float)TrailLength;
			if(tempf > 0.0f)
				tempf2 = (float)(j-1)/(float)TrailLength;
			else
				tempf2 = 0.0f;

			tempCol = tempf;
			tempCol2 = tempf2;

			if(tempf > 0.5f)
				tempf = 1.0f-tempf;
			tempf *= 0.25f;
			if(tempf2 > 0.5f)
				tempf2 = 1.0f-tempf2;
			tempf2 *= 0.25f;
			glBegin(GL_QUADS);
			{
#ifdef INVERT_COLOURS
				glColor4f(colour.x*0.25f,
						  colour.y*0.25f,
						  colour.z*0.25f,
						  tempCol2 * drawTrail * 0.125f);
#else
				glColor4f(colour.x*0.25f,
						  colour.y*0.25f,
						  colour.z*0.25f,
						  tempCol2 * drawTrail * 0.5f);
#endif
				glVertex3f(trailPos[k].x, trailPos[k].y + tempf2, trailPos[k].z);
				glVertex3f(trailPos[k].x, trailPos[k].y - tempf2, trailPos[k].z);
				glVertex3f(trailPos[i].x, trailPos[i].y - tempf, trailPos[i].z);
				glVertex3f(trailPos[i].x, trailPos[i].y + tempf, trailPos[i].z);
			}
			glEnd();
		}
		drawTrail -= 0.02f;
	}

	//Now draw quad.
	glColor4f(colour.x, colour.y, colour.z, alpha);
	glBegin(GL_QUADS);
	{
		//Front.
		glVertex3f(vertices[0].x, vertices[0].y, vertices[0].z);
		glVertex3f(vertices[1].x, vertices[1].y, vertices[1].z);
		glVertex3f(vertices[5].x, vertices[5].y, vertices[5].z);
		glVertex3f(vertices[4].x, vertices[4].y, vertices[4].z);

		//Right.
		glColor4f(colour.x * 0.2f, colour.y * 0.2f, colour.z * 0.2f, 0.25f);
		glVertex3f(vertices[1].x, vertices[1].y, vertices[1].z);
		glVertex3f(vertices[2].x, vertices[2].y, vertices[2].z);
		glVertex3f(vertices[6].x, vertices[6].y, vertices[6].z);
		glVertex3f(vertices[5].x, vertices[5].y, vertices[5].z);

		//Back.
		glColor4f(colour.x * 0.4f, colour.y * 0.4f, colour.z * 0.4f, 0.25f);
		glVertex3f(vertices[2].x, vertices[2].y, vertices[2].z);
		glVertex3f(vertices[3].x, vertices[3].y, vertices[3].z);
		glVertex3f(vertices[7].x, vertices[7].y, vertices[7].z);
		glVertex3f(vertices[6].x, vertices[6].y, vertices[6].z);

		//Left.
		glColor4f(colour.x * 0.6f, colour.y * 0.6f, colour.z * 0.6f, 0.25f);
		glVertex3f(vertices[0].x, vertices[0].y, vertices[0].z);
		glVertex3f(vertices[3].x, vertices[3].y, vertices[3].z);
		glVertex3f(vertices[7].x, vertices[7].y, vertices[7].z);
		glVertex3f(vertices[4].x, vertices[4].y, vertices[4].z);

		//Top.
		glColor4f(colour.x * 0.8f, colour.y * 0.8f, colour.z * 0.8f, 0.25f);
		glVertex3f(vertices[4].x, vertices[4].y, vertices[4].z);
		glVertex3f(vertices[5].x, vertices[5].y, vertices[5].z);
		glVertex3f(vertices[6].x, vertices[6].y, vertices[6].z);
		glVertex3f(vertices[7].x, vertices[7].y, vertices[7].z);

		//Bottom.
		glColor4f(colour.x * 0.1f, colour.y * 0.1f, colour.z * 0.1f, 0.25f);
		glVertex3f(vertices[0].x, vertices[0].y, vertices[0].z);
		glVertex3f(vertices[1].x, vertices[1].y, vertices[1].z);
		glVertex3f(vertices[2].x, vertices[2].y, vertices[2].z);
		glVertex3f(vertices[3].x, vertices[3].y, vertices[3].z);
	}
	glEnd();
}

//------------------------------------------------------------------------------
void Quad::drawBlob(float alpha, bool trail)
{
	int i, j, k;
	float tempf, tempf2;
	float tempCol, tempCol2;

	//First draw trail, if necessary.
	if((drawTrail > 0.0f) && trail)
	{
		for(i=trailWritePos,j=0;j<TrailLength-1;++i,++j)
		{
			if((i >= TrailLength) || (i == 0))
			{
				i = 0;
				k = TrailLength - 1;
			}
			else
				k = i - 1;

			tempf = (float)j/(float)TrailLength;
			if(tempf > 0.0f)
				tempf2 = (float)(j-1)/(float)TrailLength;
			else
				tempf2 = 0.0f;

			tempCol = tempf;
			tempCol2 = tempf2;

			if(tempf > 0.5f)
				tempf = 1.0f-tempf;
			tempf *= 0.25f;
			if(tempf2 > 0.5f)
				tempf2 = 1.0f-tempf2;
			tempf2 *= 0.25f;
			glBegin(GL_QUADS);
			{
				glColor4f(colour.x*0.25f,
						  colour.y*0.25f,
						  colour.z*0.25f,
						  tempCol2 * drawTrail * 0.5f);
				glVertex3f(trailPos[k].x, trailPos[k].y + tempf2, trailPos[k].z);
				glVertex3f(trailPos[k].x, trailPos[k].y - tempf2, trailPos[k].z);
				glVertex3f(trailPos[i].x, trailPos[i].y - tempf, trailPos[i].z);
				glVertex3f(trailPos[i].x, trailPos[i].y + tempf, trailPos[i].z);
			}
			glEnd();
		}
		drawTrail -= 0.02f;
	}

	//Now draw quad.
	glColor4f(colour.x, colour.y, colour.z, alpha);
	glBindTexture(GL_TEXTURE_2D, texture);
	glBegin(GL_QUADS);
	{
		glTexCoord2f(0.0f, 0.0f);
		glVertex3f((vertices[0].x-(0.2f*goSize)),
				   (vertices[0].y-(0.2f*goSize)),
				   vertices[0].z);
		glTexCoord2f(1.0f, 0.0f);
		glVertex3f((vertices[0].x+(0.2f*goSize)),
				   (vertices[0].y-(0.2f*goSize)),
				   vertices[0].z);
		glTexCoord2f(1.0f, 1.0f);
		glVertex3f((vertices[0].x+(0.2f*goSize)),
				   (vertices[0].y+(0.2f*goSize)),
				   vertices[0].z);
		glTexCoord2f(0.0f, 1.0f);
		glVertex3f((vertices[0].x-(0.2f*goSize)),
				   (vertices[0].y+(0.2f*goSize)),
				   vertices[0].z);
	}
	glEnd();
	glBindTexture(GL_TEXTURE_2D, 0);
}

//------------------------------------------------------------------------------
void Quad::loadTexture()
{
	SDL_Surface *tempTex = 0;
	SDL_Surface *tempTex2 = 0;
	SDL_Surface *tempTex3 = 0;

	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();

	//This is the easiest way I could find to create a sensible SDL_PixelFormat
	//for SDL_ConvertSurface().
	tempTex3 = SDL_CreateRGBSurface(SDL_SWSURFACE,
									1,
									1,
									32,
									0x000000ff,
									0x0000ff00,
									0x00ff0000,
									0xff000000);

	//if((tempTex = IMG_Load("/home/niall/Projects/HeilanBrowser/src/flare02.png")))
	if((tempTex = IMG_Load("/home/niall/downloads/LensFlares/Niall01.png")))
	{
		//We first convert the image into a specific format, before we call
		//glTexImage2D.
		if((tempTex2 = SDL_ConvertSurface(tempTex,
			tempTex3->format,
			SDL_SWSURFACE)))
		{
			glGenTextures(1, &texture);

			glBindTexture(GL_TEXTURE_2D, texture);

			glTexImage2D(GL_TEXTURE_2D,
						 0,
						 GL_RGBA,
						 tempTex2->w,
						 tempTex2->h,
						 0,
						 GL_RGBA,
						 GL_UNSIGNED_BYTE,
						 tempTex2->pixels);

			glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
			glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
			glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP);
			glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP);

			SDL_FreeSurface(tempTex2);
			SDL_FreeSurface(tempTex);
			glBindTexture(GL_TEXTURE_2D, 0);
		}
		else
		{
			SDL_FreeSurface(tempTex);
			cout << "Quad: Could not load image." << endl;
		}
	}
	else
		cout << "Quad: Could not load image." << endl;

	SDL_FreeSurface(tempTex3);
}

}
