//	Quad2.cpp - Second version of the Ashitaka Quad class.
//  ----------------------------------------------------------------------------
//	This file is part of 'Heilan Browser', an X3D browser specialised for audio
//	work.
//	Copyright (C) 2005-2009  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#include "Quad2.h"
#include "GravityObject2.h"
#include "MiscHelper.h"

#include <ctime>
#include <cmath>
#include <iostream>

using namespace std;

namespace AshitakaClasses
{

//------------------------------------------------------------------------------
Quad2::Quad2():
velocity(0.0f),
radius(0.03f),
primaryGO(0),
trailPos(0)
{
	float tempf;

	//Positions.
	position.x = heilanRand(heilanRandSeed()) * 4.0f;
	position.y = heilanRand(heilanRandSeed()) * 4.0f;
	position.z = heilanRand(heilanRandSeed()) * 4.0f;
	newPosition = position;

	//Direction.
	direction.x = heilanRand(heilanRandSeed());
	direction.y = heilanRand(heilanRandSeed());
	direction.z = heilanRand(heilanRandSeed());
	//Need to normalise it.
	tempf = direction.x * direction.x;
	tempf += direction.y * direction.y;
	tempf += direction.z * direction.z;
	tempf = sqrtf(tempf);
	direction /= tempf;

	//Colour.
	brightness = (heilanRand(heilanRandSeed())+1.0f) * 0.5f;
}

//------------------------------------------------------------------------------
Quad2::~Quad2()
{
	
}

//------------------------------------------------------------------------------
void Quad2::draw()
{
	int i;
	int tempint;
	float inc;
	GLfloat tempCol2[4];

	ThreeFloats tempCol(colour.x+brightness,
						colour.y+brightness,
						colour.z+brightness);

	glMatrixMode(GL_MODELVIEW);
	glPushMatrix();

	//Draw the trail.
	tempint = trailPos-1;
	if(tempint < 0)
		tempint = TrailSize-1;
	//glBlendFunc(GL_SRC_ALPHA, GL_ONE);
	glEnable(GL_LINE_SMOOTH);
	glBegin(GL_LINES);
	{
		inc = 1.0f/(float)(TrailSize/2);
		inc *= 0.25f;
		tempCol2[0] = tempCol.x * 2.0f;
		tempCol2[1] = tempCol.y * 2.0f;
		tempCol2[2] = tempCol.z * 2.0f;
		tempCol2[3] = 0.25f;
		glColor4fv(tempCol2);
		//for(i=0;i<(TrailSize-1);++i)
		for(i=0;i<((TrailSize/2)-1);++i)
		{
			glVertex3f(trail[tempint].x,
					   trail[tempint].y,
					   trail[tempint].z);
			--tempint;
			if(tempint < 0)
				tempint = TrailSize-1;
			tempCol2[3] -= inc;
			glColor4fv(tempCol2);
			glVertex3f(trail[tempint].x,
					   trail[tempint].y,
					   trail[tempint].z);
		}
	}
	glEnd();
	glDisable(GL_LINE_SMOOTH);
	//glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);

	glTranslatef(position.x, position.y, position.z);
	//glRotatef(90.0f, direction.x, direction.y, direction.z);
	glRotatef(90.0f, 1.0f, direction.x, 1.0f);
	glRotatef(90.0f, direction.y, 1.0f, 1.0f);
	//glScalef(0.5f, velocity*4.0f, velocity*4.0f);
	//glScalef(0.5f, (velocity * 128.0f), 0.5f);
	glScalef(0.5f, 0.5f, 0.5f);

	//Temporary.
	glBegin(GL_QUADS);
	{
		//Back.
		glColor4f(0.375f*tempCol.x, 0.375f*tempCol.y, 0.375f*tempCol.z, 0.35f);
		glVertex3f(-0.125f, -0.125f, -0.125f);
		glVertex3f(0.125f, -0.125f, -0.125f);
		glVertex3f(0.125f, 0.125f, -0.125f);
		glVertex3f(-0.125f, 0.125f, -0.125f);

		//Bottom.
		glColor4f(0.5f*tempCol.x, 0.5f*tempCol.y, 0.5f*tempCol.z, 0.35f);
		glVertex3f(-0.125f, -0.125f, 0.125f);
		glVertex3f(0.125f, -0.125f, 0.125f);
		glVertex3f(0.125f, -0.125f, -0.125f);
		glVertex3f(-0.125f, -0.125f, -0.125f);

		//Left.
		glColor4f(0.625f*tempCol.x, 0.625f*tempCol.y, 0.625f*tempCol.z, 0.35f);
		glVertex3f(-0.125f, -0.125f, -0.125f);
		glVertex3f(-0.125f, -0.125f, 0.125f);
		glVertex3f(-0.125f, 0.125f, 0.125f);
		glVertex3f(-0.125f, 0.125f, -0.125f);

		//Right.
		glColor4f(0.75f*tempCol.x, 0.75f*tempCol.y, 0.75f*tempCol.z, 0.35f);
		glVertex3f(0.125f, -0.125f, -0.125f);
		glVertex3f(0.125f, -0.125f, 0.125f);
		glVertex3f(0.125f, 0.125f, 0.125f);
		glVertex3f(0.125f, 0.125f, -0.125f);

		//Top.
		glColor4f(1.0f*tempCol.x, 1.0f*tempCol.y, 1.0f*tempCol.z, 0.35f);
		glVertex3f(-0.125f, 0.125f, 0.125f);
		glVertex3f(0.125f, 0.125f, 0.125f);
		glVertex3f(0.125f, 0.125f, -0.125f);
		glVertex3f(-0.125f, 0.125f, -0.125f);

		//Front.
		glColor4f(0.875f*tempCol.x, 0.875f*tempCol.y, 0.875f*tempCol.z, 0.35f);
		glVertex3f(-0.125f, -0.125f, 0.125f);
		glVertex3f(0.125f, -0.125f, 0.125f);
		glVertex3f(0.125f, 0.125f, 0.125f);
		glVertex3f(-0.125f, 0.125f, 0.125f);
	}
	glEnd();

	/*glColor3f(0.0f, 0.0f, 0.0f);
	//Back
	glBegin(GL_LINE_STRIP);
	{
		glVertex3f(-0.125f, -0.125f, -0.125f);
		glVertex3f(0.125f, -0.125f, -0.125f);
		glVertex3f(0.125f, 0.125f, -0.125f);
		glVertex3f(-0.125f, 0.125f, -0.125f);
		glVertex3f(-0.125f, -0.125f, -0.125f);
	}
	glEnd();

	//Front
	glBegin(GL_LINE_STRIP);
	{
		glVertex3f(-0.125f, -0.125f, 0.125f);
		glVertex3f(0.125f, -0.125f, 0.125f);
		glVertex3f(0.125f, 0.125f, 0.125f);
		glVertex3f(-0.125f, 0.125f, 0.125f);
		glVertex3f(-0.125f, -0.125f, 0.125f);
	}
	glEnd();

	//Back-Front lines.
	glBegin(GL_LINES);
	{
		glVertex3f(-0.125f, -0.125f, -0.125f);
		glVertex3f(-0.125f, -0.125f, 0.125f);

		glVertex3f(0.125f, -0.125f, -0.125f);
		glVertex3f(0.125f, -0.125f, 0.125f);

		glVertex3f(-0.125f, 0.125f, -0.125f);
		glVertex3f(-0.125f, 0.125f, 0.125f);

		glVertex3f(0.125f, 0.125f, -0.125f);
		glVertex3f(0.125f, 0.125f, 0.125f);
	}
	glEnd();*/

	glPopMatrix();
}

//------------------------------------------------------------------------------
void Quad2::moveTo(const ThreeFloats& newPos, float weight)
{
	newPosition *= (1.0f-weight);
	newPosition += weight * newPos;
}

//------------------------------------------------------------------------------
void Quad2::setDirection(const ThreeFloats& dir)
{
	direction = dir;
}

//------------------------------------------------------------------------------
void Quad2::interpolate()
{
	if(position != newPosition)
	{
		ThreeFloats tempDir;
		float distance = position.getDistance(newPosition);

		if(distance > 0.01f)
		{
			tempDir = newPosition - position;
			position += (tempDir*0.5f);
			velocity = distance;
		}
		else
		{
			position = newPosition;
			velocity = 0.0f;
		}

		trail[trailPos] = position;
		++trailPos;
		if(trailPos >= TrailSize)
			trailPos = 0;
	}
}

//------------------------------------------------------------------------------
void Quad2::setColour(const ThreeFloats& col, float weight)
{
	colour *= (1.0f-weight);
	colour += col * weight;
}

//------------------------------------------------------------------------------
void Quad2::setPrimaryGravityObject(GravityObject2 *obj)
{
	primaryGO = obj;
}

}
