//	Quad2.h - Second version of the Ashitaka Quad class.
//  ----------------------------------------------------------------------------
//	This file is part of 'Heilan Browser', an X3D browser specialised for audio
//	work.
//	Copyright (C) 2005-2009  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#ifndef QUAD2_H_
#define QUAD2_H_

#include "ThreeFloats.h"
#include "GLHeaders.h"

namespace AshitakaClasses
{

class GravityObject2;

///	Second version of the Ashitaka Quad class.
class Quad2
{
  public:
	///	Constructor.
	Quad2();
	///	Destructor.
	~Quad2();

	///	Draws the Quad.
	void draw();

	///	Informs the Quad that it should move to the passed-in position.
	/*!
		\param newPos The position to move to.
		\param weight How much this call counts (i.e. the primary GO would
		supply a weight of 1.0f, but others would supply a smaller value).  The
		Quad will move further towards newPos when weight is 1.0f, than if it
		were 0.25f.
	 */
	void moveTo(const ThreeFloats& newPos, float weight);
	///	Sets the Quad's direction.
	/*!
		This may be used by GOs trying to determine where to moveTo() the Quad.
	 */
	void setDirection(const ThreeFloats& dir);
	///	Moves the Quad to newPosition, if newPosition is different to position.
	void interpolate();

	///	Sets the Quad's colour (but not its brightness).
	void setColour(const ThreeFloats& col, float weight);

	///	Returns the Quad's actual position.
	const ThreeFloats& getPosition() const {return position;};
	///	Returns the Quad's intended position.
	const ThreeFloats& getNewPosition() const {return newPosition;};
	///	Returns the Quad's direction.
	const ThreeFloats& getDirection() const {return direction;};
	///	The Quad's velocity.
	float getVelocity() const {return velocity;};
	///	The Quad's radius.
	float getRadius() const {return radius;};

	///	Sets the Quad's primary GravityObject.
	void setPrimaryGravityObject(GravityObject2 *obj);
	///	Returns the Quad's primary GravityObject.
	GravityObject2 *getPrimaryGravityObject() const {return primaryGO;};
  private:
	///	The Quad's actual position.
	ThreeFloats position;
	///	The Quad's intended position.
	ThreeFloats newPosition;
	///	The Quad's direction.
	ThreeFloats direction;
	///	The Quad's velocity.
	float velocity;
	///	The Quad's radius.
	float radius;
	///	The Quad's colour.
	ThreeFloats colour;
	///	The Quad's brightness.
	float brightness;

	///	The Quad's primary GO.
	GravityObject2 *primaryGO;

	///	The size of trail.
	enum
	{
		TrailSize = 64
	};
	///	A trail of our previous TrailSize positions.
	ThreeFloats trail[TrailSize];
	///	Our current position in trail.
	int trailPos;
};

}

#endif
