//	RandomLineGO.cpp - Random line-type GravityObject.
//  ----------------------------------------------------------------------------
//	This file is part of 'Heilan Browser', an X3D browser specialised for audio
//	work.
//	Copyright (C) 2005-2009  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#include "RandomLineGO.h"
#include "Quad.h"
#include "MiscHelper.h"

#include <ctime>
#include <cmath>
#include <iostream>

using namespace std;

namespace AshitakaClasses
{

//------------------------------------------------------------------------------
RandomLineGO::RandomLineGO():
GravityObject(),
waitFrames(0),
timeToMove(false),
newPos(0.0f),
audioData(0),
adPos(0.0f),
envState(Attack),
envVal(0.0f),
colourCount(0.0f),
transState(Off),
transCount(0)
{
	int i;

	for(i=0;i<8;++i)
		transVals[i] = 0.0f;
}

//------------------------------------------------------------------------------
RandomLineGO::~RandomLineGO()
{
	if(audioData)
		delete [] audioData;
}

//------------------------------------------------------------------------------
void RandomLineGO::updatePrimaryQuads()
{
	int i;
	ThreeFloats goPos = getPosition();
	vector<Quad *>& quads = getPrimaryQuads();
	vector<Quad *>::iterator it;
	ThreeFloats verts[8];
	ThreeFloats pos1, pos2;
	static time_t seed = time(NULL);
	ThreeFloats tempCol;
	bool setColour = false;

	//Only need to update their positions when waitFrames gets to 0.
	if(waitFrames <= 0)
	{
		waitFrames = (int)((heilanRand(seed) + 1.0f) * 60.0f);
		++seed;

		for(it=quads.begin();it!=quads.end();++it)
		{
			pos1.x = heilanRand(seed);
			++seed;
			pos1.y = heilanRand(seed);
			++seed;
			pos1.z = heilanRand(seed);
			++seed;
			pos2.x = heilanRand(seed);
			++seed;
			pos2.y = heilanRand(seed);
			++seed;
			pos2.z = heilanRand(seed);
			++seed;

			verts[0].x = goPos.x + pos1.x - 0.05f;
			verts[0].y = goPos.y + pos1.y - 0.05f;
			verts[0].z = goPos.z + pos1.z + 0.05f;

			verts[1].x = goPos.x + pos2.x - 0.05f;
			verts[1].y = goPos.y + pos2.y - 0.05f;
			verts[1].z = goPos.z + pos2.z + 0.05f;

			verts[2].x = goPos.x + pos2.x - 0.05f;
			verts[2].y = goPos.y + pos2.y - 0.05f;
			verts[2].z = goPos.z + pos2.z - 0.05f;

			verts[3].x = goPos.x + pos1.x - 0.05f;
			verts[3].y = goPos.y + pos1.y - 0.05f;
			verts[3].z = goPos.z + pos1.z - 0.05f;

			verts[4].x = goPos.x + pos1.x + 0.05f;
			verts[4].y = goPos.y + pos1.y + 0.05f;
			verts[4].z = goPos.z + pos1.z + 0.05f;

			verts[5].x = goPos.x + pos2.x + 0.05f;
			verts[5].y = goPos.y + pos2.y + 0.05f;
			verts[5].z = goPos.z + pos2.z + 0.05f;

			verts[6].x = goPos.x + pos2.x + 0.05f;
			verts[6].y = goPos.y + pos2.y + 0.05f;
			verts[6].z = goPos.z + pos2.z - 0.05f;

			verts[7].x = goPos.x + pos1.x + 0.05f;
			verts[7].y = goPos.y + pos1.y + 0.05f;
			verts[7].z = goPos.z + pos1.z - 0.05f;

			(*it)->setVertices(verts);
		}
		timeToMove = true;
		colourCount = 1.0f;
	}
	else
		--waitFrames;

	if(colourCount > 0.0f)
	{
		tempCol = ThreeFloats(colourCount, 0.0f, 0.0f);
		tempCol += ThreeFloats(0.0f, 0.0f, (1.0f-colourCount));
		colourCount -= 0.01f;
		setColour = true;
	}

	//Add in transient jumping.
	switch(transState)
	{
		case Up:
			for(it=quads.begin();it!=quads.end();++it)
			{
				const ThreeFloats * const verts2 = (*it)->getVertices();

				for(i=0;i<8;++i)
				{
					verts[i] = verts2[i];
					verts[i].y -= ((float)transCount * 0.05f);
					verts[i].y += 1.0f;
				}

				(*it)->setVertices(verts);
			}
			transCount = 20;
			transState = Down;

			tempCol += ThreeFloats(0.0f, 1.0f, 0.0f);
			setColour = true;
			break;
		case Down:
			for(it=quads.begin();it!=quads.end();++it)
			{
				const ThreeFloats * const verts2 = (*it)->getVertices();

				for(i=0;i<8;++i)
				{
					verts[i] = verts2[i];
					verts[i].y -= 0.05f;
				}

				(*it)->setVertices(verts);
			}
			--transCount;
			if(transCount == 0)
				transState = Off;

			tempCol += ThreeFloats(0.0f, ((float)transCount/20.0f), 0.0f);
			setColour = true;
			break;
		case Off:
			break;
	}

	if(setColour)
	{
		for(it=quads.begin();it!=quads.end();++it)
			(*it)->setColour(tempCol);
	}
}

//------------------------------------------------------------------------------
void RandomLineGO::updateSecondaryQuad(Quad *quad)
{
	
}

//------------------------------------------------------------------------------
void RandomLineGO::updateLifeForce()
{
	float life = getLifeForce();

	life -= 0.00075f;
	setLifeForce(life);

	if(life <= 0.0f)
		setLifeForce(0.0f);
}

//------------------------------------------------------------------------------
void RandomLineGO::getMonoAudio(AmbisonicData *buffer, int numSamples)
{
	int i, j;
	float inc = 1.0f;
	float min;
	float max;

	for(i=0;i<numSamples;++i)
	{
		min = 1.0f;
		max = -1.0f;

		//Clear X, Y and Z just in case.
		buffer[i] = 0.0f;

		if(timeToMove)
		{
			envState = Attack;
			envVal = 0.5f;

			newPos = adPos += 44100.0f;
			if(newPos >= 88200.0f)
				newPos = fmod(newPos, 88200.0f);
			timeToMove = false;
		}
		else
			newPos += 1.0f;

		//Update adPos.
		if(adPos != newPos)
		{
			inc = (newPos - adPos) * (1.0f/1024.0f);
			if(inc > 2.0f)
				inc = 2.0f;
			else if(inc < -2.0f)
				inc = -2.0f;
		}
		adPos += inc;
		if(adPos >= 88200.0f)
			adPos = fmod(adPos, 88200.0f);

		//Update envelope.
		switch(envState)
		{
			case Attack:
				envVal += 0.01f;
				if(envVal >= 1.0f)
					envState = Decay;
				break;
			case Decay:
				envVal -= 0.01f;
				if(envVal < 0.5f)
					envState = Sustain;
				break;
			case Sustain:
				envVal = 0.5f;
				break;
		}

		//Detect transients.
		for(j=7;j>0;--j)
			transVals[j] = transVals[j-1];
		transVals[0] = audioData[static_cast<int>(adPos)];

		for(j=0;j<8;++j)
		{
			if(transVals[j] < min)
				min = transVals[j];
			if(transVals[j] > max)
				max = transVals[j];
		}

		if((max-min) > 0.05f)
			transState = Up;

		// * 0.5f so the main instrument doesn't get too drowned out.
		buffer[i].W = audioData[static_cast<int>(adPos)] * envVal * 0.5f;
	}
}

//------------------------------------------------------------------------------
void RandomLineGO::setAudioData(float *data)
{
	audioData = data;
}

}
