//	RandomLineGO.h - Random line-type GravityObject.
//  ----------------------------------------------------------------------------
//	This file is part of 'Heilan Browser', an X3D browser specialised for audio
//	work.
//	Copyright (C) 2005-2009  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#ifndef RANDOMLINEGO_H_
#define RANDOMLINEGO_H_

#include "GravityObject.h"

namespace AshitakaClasses
{

///	Random line-type GravityObject.
class RandomLineGO : public GravityObject
{
  public:
	///	Constructor.
	RandomLineGO();
	///	Destructor.
	~RandomLineGO();

	///	Updates all this GO's primary Quads.
	void updatePrimaryQuads();
	///	Updates the position of the passed-in secondary Quad.
	void updateSecondaryQuad(Quad *quad);
	///	Updates this GO's lifeForce.
	void updateLifeForce();

	///	Returns the audio for this GO.
	void getMonoAudio(AmbisonicData *buffer, int numSamples);
	///	Sets our copy of the main instrument's audio.
	void setAudioData(float *data);
  private:
	///	The three envelope states.
	typedef enum
	{
		Attack,
		Decay,
		Sustain
	} EnvState;
	///	The three transient states.
	typedef enum
	{
		Up,
		Down,
		Off
	} TransState;

	///	Number of frames to wait before jumping to a new position.
	int waitFrames;

	///	True when it's time to move to a new position in the audio.
	bool timeToMove;
	///	New position to move to.
	float newPos;
	///	Our copy of the main instrument's audio.
	float *audioData;
	///	Current position in audioData.
	float adPos;
	///	Current envelope state.
	EnvState envState;
	///	Current envelope value.
	float envVal;
	///	Colour counter for fading from red to blue.
	float colourCount;

	///	Used to detect transients in audioData.
	float transVals[8];
	///	What state the visual transient detection is in.
	TransState transState;
	///	Counter for the down part of transState.
	int transCount;
};

}

#endif
