//	SimpleAttractorGO.cpp - Simple attractor GravityObject.
//  ----------------------------------------------------------------------------
//	This file is part of 'Heilan Browser', an X3D browser specialised for audio
//	work.
//	Copyright (C) 2005-2009  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#include "SimpleAttractorGO.h"
#include "Quad.h"

#include "SDL.h"
#include <iostream>
#include <cmath>

using namespace std;

namespace AshitakaClasses
{

//------------------------------------------------------------------------------
SimpleAttractorGO::SimpleAttractorGO():
GravityObject()
{
	
}

//------------------------------------------------------------------------------
SimpleAttractorGO::~SimpleAttractorGO()
{
	
}

//------------------------------------------------------------------------------
void SimpleAttractorGO::updatePrimaryQuads()
{
	ThreeFloats goPos = getPosition();
	vector<Quad *>& quads = getPrimaryQuads();
	vector<Quad *>::iterator it;

	for(it=quads.begin();it!=quads.end();++it)
	{
		float tempf;
		float distance;
		ThreeFloats force;
		ThreeFloats direction;
		ThreeFloats tempAccel;
		ThreeFloats tempVel = (*it)->getVelocity3d();
		ThreeFloats qPos = (*it)->getCentre();
		ThreeFloats verts[8];

		//Calc direction and distance.
		direction = goPos - qPos;

		distance = sqrtf((direction.x * direction.x) +
						 (direction.y * direction.y) +
						 (direction.z * direction.z));

		//Normalise direction.
		direction = direction/distance;

		if(distance < 5.0f)
			distance = 5.0f;
		else if(distance > 25.0f)
			distance = 25.0f;

		//Calc force (F=(G*goMass*qMass)/(distance *distance)).
		tempf = (0.001f * 20.0f * 10.0f)/(distance * distance);
		force = direction * tempf;

		//Update centre position.
		tempAccel = force/10.0f;
		//tempAccel *= -1.0f; //Repulsion
		tempVel += tempAccel;

		tempf = sqrtf((tempVel.x*tempVel.x) + (tempVel.y*tempVel.y) + (tempVel.z*tempVel.z));
		if(tempf > 1.0f)
		{
			tempVel = tempVel/tempf;
			tempVel *= 10.0f;
		}
		qPos += tempVel;

		//Update vertices.
		positionVertices(verts, qPos, tempVel*4.0f);
		(*it)->setVertices(verts);
	}
}

//------------------------------------------------------------------------------
void SimpleAttractorGO::updateSecondaryQuad(Quad *quad)
{
	{
		float tempf;
		float distance;
		ThreeFloats force;
		ThreeFloats direction;
		ThreeFloats tempAccel;
		ThreeFloats tempVel = quad->getVelocity3d();
		ThreeFloats qPos = quad->getCentre();
		ThreeFloats verts[8];
		ThreeFloats goPos = getPosition();

		//Calc direction and distance.
		direction = goPos - qPos;

		distance = sqrtf((direction.x * direction.x) +
						 (direction.y * direction.y) +
						 (direction.z * direction.z));

		//Normalise direction.
		direction = direction/distance;

		if(distance < 5.0f)
			distance = 5.0f;
		else if(distance > 25.0f)
			distance = 25.0f;

		//Calc force (F=(G*goMass*qMass)/(distance *distance)).
		tempf = (0.001f * 20.0f * 10.0f)/(distance * distance);
		force = direction * tempf;

		//Update centre position.
		tempAccel = force/10.0f;
		//tempAccel *= -1.0f; //Repulsion
		tempVel += tempAccel;

		tempf = sqrtf((tempVel.x*tempVel.x) + (tempVel.y*tempVel.y) + (tempVel.z*tempVel.z));
		if(tempf > 1.0f)
		{
			tempVel = tempVel/tempf;
			tempVel *= 10.0f;
		}
		qPos += tempVel;

		//Update vertices.
		positionVertices(verts, qPos, tempVel*4.0f);
		quad->setVertices(verts);
	}
}

//------------------------------------------------------------------------------
void SimpleAttractorGO::updateLifeForce()
{
	float life = getLifeForce();

	life -= 0.001f;
	setLifeForce(life);

	if(life <= 0.0f)
		setLifeForce(0.0f);
}

//------------------------------------------------------------------------------
void SimpleAttractorGO::getMonoAudio(AmbisonicData *buffer, int numSamples)
{
	int i;

	for(i=0;i<numSamples;++i)
		buffer[i] = 0.0f;
}

}
