//	SimpleAttractorGO.h - Simple attractor GravityObject.
//  ----------------------------------------------------------------------------
//	This file is part of 'Heilan Browser', an X3D browser specialised for audio
//	work.
//	Copyright (C) 2005-2009  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#ifndef SIMPLEATTRACTORGO_H_
#define SIMPLEATTRACTORGO_H_

#include "GravityObject.h"

namespace AshitakaClasses
{

///	Simple attractor GravityObject.
/*!
	Based on the processing code here:
	http://www.shiffman.net/itp/classes/nature/week03_s06/attraction/
 */
class SimpleAttractorGO : public GravityObject
{
  public:
	///	Constructor.
	SimpleAttractorGO();
	///	Destructor.
	~SimpleAttractorGO();

	///	Updates all this GO's primary Quads.
	void updatePrimaryQuads();
	///	Updates the position of the passed-in secondary Quad.
	void updateSecondaryQuad(Quad *quad);
	///	Updates this GO's lifeForce.
	void updateLifeForce();

	///	Returns the audio for this GO.
	void getMonoAudio(AmbisonicData *buffer, int numSamples);
  private:
	///	Helper method to position vertices around a centre point.
	inline void positionVertices(ThreeFloats *verts,
								 const ThreeFloats& centre,
								 const ThreeFloats& direction)
	{
		verts[0].x = centre.x - 0.5f + direction.x;
		verts[0].y = centre.y - 0.5f + direction.y;
		verts[0].z = centre.z + 0.5f + direction.z;

		verts[1].x = centre.x + 0.5f + direction.x;
		verts[1].y = centre.y - 0.5f + direction.y;
		verts[1].z = centre.z + 0.5f + direction.z;

		verts[2].x = centre.x + 0.5f + direction.x;
		verts[2].y = centre.y - 0.5f + direction.y;
		verts[2].z = centre.z - 0.5f + direction.z;

		verts[3].x = centre.x - 0.5f + direction.x;
		verts[3].y = centre.y - 0.5f + direction.y;
		verts[3].z = centre.z - 0.5f + direction.z;

		verts[4].x = centre.x - 0.5f - direction.x;
		verts[4].y = centre.y + 0.5f - direction.y;
		verts[4].z = centre.z + 0.5f - direction.z;

		verts[5].x = centre.x + 0.5f - direction.x;
		verts[5].y = centre.y + 0.5f - direction.y;
		verts[5].z = centre.z + 0.5f - direction.z;

		verts[6].x = centre.x + 0.5f - direction.x;
		verts[6].y = centre.y + 0.5f - direction.y;
		verts[6].z = centre.z - 0.5f - direction.z;

		verts[7].x = centre.x - 0.5f - direction.x;
		verts[7].y = centre.y + 0.5f - direction.y;
		verts[7].z = centre.z - 0.5f - direction.z;
	};
};

}

#endif
