//	SnakeGO.cpp - Snake-type GravityObject.
//  ----------------------------------------------------------------------------
//	This file is part of 'Heilan Browser', an X3D browser specialised for audio
//	work.
//	Copyright (C) 2005-2009  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#include "SnakeGO.h"

using namespace std;

namespace AshitakaClasses
{

//------------------------------------------------------------------------------
SnakeGO::SnakeGO():
GravityObject(),
vertSine(0.0f),
horizSine(0.0f)
{
	
}

//------------------------------------------------------------------------------
SnakeGO::~SnakeGO()
{
	
}

//------------------------------------------------------------------------------
void SnakeGO::updatePrimaryQuads()
{
	int i, j;
	float tempf;
	float tempDec;
	ThreeFloats goPos = getPosition();
	vector<Quad *>& quads = getPrimaryQuads();
	vector<Quad *>::iterator it;
	ThreeFloats temp3f;
	float dist;
	ThreeFloats verts[8];

	//First update positions.
	for(i=(QuadManager::NumQuads/2)-1;i>0;--i)
		positions[i] = positions[i-1];
	positions[0].x = (sinf(horizSine * 6.283185306f) * 5.0f) + goPos.x;
	positions[0].y = sinf(vertSine * 6.283185306f) + goPos.y;
	positions[0].z = (cosf(horizSine * 6.283185306f) * 5.0f) + goPos.z;
	vertSine += 0.01f;
	while(vertSine > 1.0f)
		vertSine -= 1.0f;
	horizSine += 0.001f;
	while(horizSine > 1.0f)
		horizSine -= 1.0f;

	//Next orientate Quads correctly, with each 'end' of a Quad matching the
	//correspondingly-indexed position.
	i = 0;
	j = 16;
	temp3f.x = positions[4].x-positions[0].x;
	temp3f.y = positions[4].y-positions[0].y;
	temp3f.z = positions[4].z-positions[0].z;
	dist = sqrtf((temp3f.x * temp3f.x) +
				 (temp3f.y * temp3f.y) +
				 (temp3f.z * temp3f.z));
	tempf = dist;
	//tempf = 1.0f;
	tempDec = (1.0f/(float)quads.size()) * dist;
	//for(it=(quads.end()-1);it!=(quads.begin()-1);--it)
	for(it=quads.begin();it!=quads.end();++it)
	{
		float theta;
		float sinTheta, cosTheta;
		float tempf2 = tempf * 0.25f;
		//float tempDec2 = tempDec * 0.25f;

		//Y-axis is easy because we just keep the vertices vertical, w/no
		//rotation.
		verts[0].y = positions[i].y - tempf;
		verts[1].y = positions[i].y - tempf;
		verts[4].y = positions[i].y + tempf;
		verts[5].y = positions[i].y + tempf;
		verts[2].y = positions[i+4].y - (tempf-tempDec);
		verts[3].y = positions[i+4].y - (tempf-tempDec);
		verts[6].y = positions[i+4].y + (tempf-tempDec);
		verts[7].y = positions[i+4].y + (tempf-tempDec);

		//Do 'front' side of the Quad.
		if(i == 0)
		{
			theta = atanf((positions[4].z-positions[0].z)/
						  (positions[4].x-positions[0].x));
			sinTheta = sinf(theta) * tempf2;
			cosTheta = cosf(theta) * tempf2;

			verts[0].x = positions[0].x - sinTheta;
			verts[0].z = positions[0].z + cosTheta;
			verts[1].x = positions[0].x + sinTheta;
			verts[1].z = positions[0].z - cosTheta;
			verts[4].x = positions[0].x - sinTheta;
			verts[4].z = positions[0].z + cosTheta;
			verts[5].x = positions[0].x + sinTheta;
			verts[5].z = positions[0].z - cosTheta;
		}
		else
		{
			verts[0].x = verts[3].x;
			verts[0].z = verts[3].z;
			verts[1].x = verts[2].x;
			verts[1].z = verts[2].z;
			verts[4].x = verts[7].x;
			verts[4].z = verts[7].z;
			verts[5].x = verts[6].x;
			verts[5].z = verts[6].z;
		}

		//Do 'back' side of the Quad.
		theta = atanf((positions[i+8].z-positions[i].z)/
					  (positions[i+8].x-positions[i].x));
		sinTheta = sinf(theta) * tempf2;
		cosTheta = cosf(theta) * tempf2;

		verts[2].x = positions[i+4].x - sinTheta;
		verts[2].z = positions[i+4].z + cosTheta;
		verts[3].x = positions[i+4].x + sinTheta;
		verts[3].z = positions[i+4].z - cosTheta;
		verts[6].x = positions[i+4].x - sinTheta;
		verts[6].z = positions[i+4].z + cosTheta;
		verts[7].x = positions[i+4].x + sinTheta;
		verts[7].z = positions[i+4].z - cosTheta;

		tempf -= tempDec;
		i += 4;
		j -= 4;
		if(i >= ((QuadManager::NumQuads/2)-2))
			i = (QuadManager::NumQuads/2)-3;

		(*it)->setVertices(verts);
	}
}

//------------------------------------------------------------------------------
void SnakeGO::updateSecondaryQuad(Quad *quad)
{
	
}

//------------------------------------------------------------------------------
void SnakeGO::updateLifeForce()
{
	float life = getLifeForce();

	life -= 0.0005f;
	setLifeForce(life);

	if(life <= 0.0f)
		setLifeForce(0.0f);
}

//------------------------------------------------------------------------------
void SnakeGO::getMonoAudio(AmbisonicData *buffer, int numSamples)
{
	int i;

	for(i=0;i<numSamples;++i)
		buffer[i] = 0.0f;
}

}
