//	SnakeGO.h - Snake-type GravityObject.
//  ----------------------------------------------------------------------------
//	This file is part of 'Heilan Browser', an X3D browser specialised for audio
//	work.
//	Copyright (C) 2005-2009  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#ifndef SNAKEGO_H_
#define SNAKEGO_H_

#include "GravityObject.h"
#include "QuadManager.h"

namespace AshitakaClasses
{

///	Snake-type GravityObject.
class SnakeGO : public GravityObject
{
  public:
	///	Constructor.
	SnakeGO();
	///	Destructor.
	~SnakeGO();

	///	Updates all this GO's primary Quads.
	void updatePrimaryQuads();
	///	Updates the position of the passed-in secondary Quad.
	void updateSecondaryQuad(Quad *quad);
	///	Updates this GO's lifeForce.
	void updateLifeForce();

	///	Returns the audio for this GO.
	void getMonoAudio(AmbisonicData *buffer, int numSamples);
  private:
	///	Positions of the sections of the snake.
	ThreeFloats positions[QuadManager::NumQuads/2];
	///	Vertical motion sine wave.
	float vertSine;
	///	Horizontal motion sine/cos wave.
	float horizSine;
};
	
}

#endif
