//	AudioInput.cpp - Object allowing you to stream audio into a scene from a
//				     soundcard.
//  ----------------------------------------------------------------------------
//	This file is part of 'Heilan Browser', an X3D browser specialised for audio
//	work.
//	Copyright (C) 2005-2009  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#include "AudioInput.h"
#include "AmbisonicData.h"
#include "StringParser.h"

#include <iostream>
#include <cassert>

using namespace std;

namespace HeilanX3D
{

NodeConstructor<AudioInput> AudioInput::constructor;

//------------------------------------------------------------------------------
AudioInput::AudioInput():
ChildNode(),
audioBuffer(0),
bufferSize(0),
channel(0)
{
	setType("AudioInput");
	setVisual(false);
	setAural(true);
}

//------------------------------------------------------------------------------
AudioInput::~AudioInput()
{
	if(audioBuffer)
		delete [] audioBuffer;
}

//------------------------------------------------------------------------------
AmbisonicData *AudioInput::getMonoAudio(int numSamples)
{
	int i, j;
	const int numChannels = AbstractNode::getNumInputAudioChannels();
	const float * const inputAudio = AbstractNode::getInputAudio();

	//This should NEVER happen, unless there's an error somewhere else...
	assert(numSamples <= bufferSize);

	//Check the user's given us a sensible channel number.
	if((numChannels > 0) && (channel >= numChannels))
	{
		channel = numChannels-1;

		cout << "AudioInput: Channel out of range, resetting to:";
		cout << channel << endl;
	}

	//Only stream audio if there's audio to stream.
	if((numChannels < 1) || (inputAudio == 0))
	{
		for(i=0;i<numSamples;++i)
			audioBuffer[i].W = 0.0f;
	}
	else
	{
		j = channel;
		for(i=0;i<numSamples;++i)
		{
			audioBuffer[i].W = inputAudio[j];
			j += numChannels;
		}
	}

	return audioBuffer;
}

//------------------------------------------------------------------------------
void AudioInput::setBlockSize(int newSize)
{
	if(audioBuffer)
		delete [] audioBuffer;

	bufferSize = newSize;
	audioBuffer = new AmbisonicData[bufferSize];
}

//------------------------------------------------------------------------------
void AudioInput::setAttribute(const string& attribute, EventValue& val)
{
	if(attribute == "channel")
		channel = val;
}

}
