//	CylinderBackground.h - Special background node.
//  ----------------------------------------------------------------------------
//	This file is part of 'Heilan Browser', an X3D browser specialised for audio
//	work.
//	Copyright (C) 2005-2009  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#ifndef CYLINDERBACKGROUND_H_
#define CYLINDERBACKGROUND_H_

#include "BindableNode.h"

//#include "AvailableExtensions.h"
#include <cmath>

namespace HeilanX3D
{

///	Special background node.
class CylinderBackground : public BindableNode
{
  public:
	///	Constructor.
	CylinderBackground();
	///	Destructor.
	~CylinderBackground();

	///	Initialises various bits and pieces.
	void initialise();

	///	Overridden to draw the background.
	void graphicDoStuff();
	///	Overridden to do nothing.
	/*AmbisonicData *audioDoStuff(int numSamples,
								const ThreeFloats& pos) {return 0;};*/

	///	Overridden to do nothing.
	void setBlockSize(int newSize) {};

	///	Called to set the value of the named attribute.
	void setAttribute(const std::string& attribute, EventValue& val);

	static NodeConstructor<CylinderBackground> constructor;
  private:
	///	Enum containing constants for the number of segments used.
	enum
	{
		NumSegments = 32
	};

	///	Current top colour.
	ThreeFloats topColour;
	///	Current middle colour (-1 means interpolate between top & bottom).
	ThreeFloats middleColour;
	///	Current bottom colour.
	ThreeFloats bottomColour;

	///	Precalculated coefficients.
	float xVerts[NumSegments];
	///	Precalculated coefficients.
	float yVerts[NumSegments];
};

}

#endif
