//	GestureDetector.cpp - A simple visualisation of gestures detected from a 3D
//						  accelerometer.
//  ----------------------------------------------------------------------------
//	This file is part of 'Heilan Browser', an X3D browser specialised for audio
//	work.
//	Copyright (C) 2005-2009  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#include "ParticleGenerator.h"
#include "GestureDetector.h"
#include "StringParser.h"
#include "MiscHelper.h"

#include "SDL.h"
#include "SDL_image.h"
#include <iostream>
#include <sstream>

using namespace std;

namespace HeilanX3D
{

NodeConstructor<GestureDetector> GestureDetector::constructor;

//------------------------------------------------------------------------------
GestureDetector::GestureDetector():
ChildNode(),
showDebugVisuals(false),
useParticles(false),
match(NoMatch),
matchedGesture(0),
jerk(0),
gestureCount(0),
matchCount(0),
jerkCount(0),
matchFade(0.0f),
whichTexture(0),
jerkFade(0.0f),
whichJerk(1),
particles(0)
{
	int i;

	setType("GestureDetector");
	setVisual(true);

	for(i=0;i<10;++i)
	{
		//0 == anti-clockwise circle.
		testGesture[0][i].x = sinf(6.283185306f * ((float)i/10.0f)) * 0.5f;
		testGesture[0][i].y = cosf(6.283185306f * ((float)i/10.0f)) * 0.5f;
		//1 == clockwise circle.
		testGesture[1][i].x = sinf(6.283185306f * ((float)(10-i)/10.0f)) * 0.5f;
		testGesture[1][i].y = cosf(6.283185306f * ((float)(10-i)/10.0f)) * 0.5f;
		//2 == left-right shaking.
		//testGesture[2][i].x = (i%2) ? -0.5f : 0.5f;
		/*if((i%3) == 0)
		{
			testGesture[2][i].x = 0.5f;
			testGesture[2][i].y = 0.1f;
		}
		else if((i%2) == 1)
			testGesture[2][i].x = 0.0f;
		else
		{
			testGesture[2][i].x = -0.5f;
			testGesture[2][i].y = 0.1f;
		}*/
		//2 == upside down v.
		if(i < 5)
		{
			testGesture[2][i].x = -0.5f * ((float)(4-i)/4.0f);
			testGesture[2][i].y = ((float)i/4.0f)-0.5f;
		}
		else
		{
			testGesture[2][i].x = 0.5f * ((float)(i-5)/4.0f);
			testGesture[2][i].y = ((float)(9-i)/4.0f)-0.5f;
		}
		//2 == anticlockwise gesture along z plane.
		/*testGesture[0][i].x = sinf(6.283185306f * ((float)i/10.0f)) * 0.5f;
		testGesture[0][i].z = cosf(6.283185306f * ((float)i/10.0f)) * 0.5f;*/
	}

	//Upside down u-shape.
	/*testGesture[2][0].x = -0.2f;
	testGesture[2][0].y = -0.5f;

	testGesture[2][1].x = -0.2f;
	testGesture[2][1].y = -0.25f;

	testGesture[2][2].x = -0.175f;
	testGesture[2][2].y = 0.0f;

	testGesture[2][3].x = -0.15f;
	testGesture[2][3].y = 0.25f;

	testGesture[2][4].x = -0.075f;
	testGesture[2][4].y = 0.5f;

	testGesture[2][5].x = 0.075f;
	testGesture[2][5].y = 0.5f;

	testGesture[2][6].x = 0.15f;
	testGesture[2][6].y = 0.25f;

	testGesture[2][7].x = 0.175f;
	testGesture[2][7].y = 0.0f;

	testGesture[2][8].x = 0.2f;
	testGesture[2][8].y = -0.25f;

	testGesture[2][9].x = 0.2f;
	testGesture[2][9].y = -0.5f;*/
}

//------------------------------------------------------------------------------
GestureDetector::~GestureDetector()
{
	if(particles)
		delete particles;
}

//------------------------------------------------------------------------------
void GestureDetector::initialise()
{
	EventValue ev;
	SDL_Surface *tempIm;

	if(!particles)
	{
		particles = new ParticleGenerator();
		particles->initialise();
		ev = "0 0 0";
		particles->setAttribute("gravityDirection", ev);
		ev = "0.5";
		particles->setAttribute("gravity", ev);
		ev = "0.5";
		particles->setAttribute("avgLifespan", ev);
		ev = "256";
		particles->setAttribute("numParticles", ev);
		ev = "0.25";
		particles->setAttribute("particleSize", ev);
		ev = "1 1 0";
		particles->setAttribute("colour", ev);
		ev = "1 1 1";
		particles->setAttribute("endColour", ev);
		ev = "true";
		particles->setAttribute("shrink", ev);
	}

	if((tempIm = IMG_Load_RW(SDL_RWFromConstMem(GestureTextures::anticlockwisegesture_png,
											    GestureTextures::anticlockwisegesture_pngSize), 1)) != 0)
	{
		glGenTextures(1, &matchTextures[0]);

		glBindTexture(GL_TEXTURE_2D, matchTextures[0]);

		glTexImage2D(GL_TEXTURE_2D,
					 0,
					 GL_RGBA,
					 tempIm->w,
					 tempIm->h,
					 0,
					 GL_RGBA,
					 GL_UNSIGNED_BYTE,
					 tempIm->pixels);

		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);

		glBindTexture(GL_TEXTURE_2D, 0);

		SDL_FreeSurface(tempIm);
	}
	if((tempIm = IMG_Load_RW(SDL_RWFromConstMem(GestureTextures::clockwisegesture_png,
											    GestureTextures::clockwisegesture_pngSize), 1)) != 0)
	{
		glGenTextures(1, &matchTextures[1]);

		glBindTexture(GL_TEXTURE_2D, matchTextures[1]);

		glTexImage2D(GL_TEXTURE_2D,
					 0,
					 GL_RGBA,
					 tempIm->w,
					 tempIm->h,
					 0,
					 GL_RGBA,
					 GL_UNSIGNED_BYTE,
					 tempIm->pixels);

		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);

		glBindTexture(GL_TEXTURE_2D, 0);

		SDL_FreeSurface(tempIm);
	}
	if((tempIm = IMG_Load_RW(SDL_RWFromConstMem(GestureTextures::ngesture_png,
											    GestureTextures::ngesture_pngSize), 1)) != 0)
	{
		glGenTextures(1, &matchTextures[2]);

		glBindTexture(GL_TEXTURE_2D, matchTextures[2]);

		glTexImage2D(GL_TEXTURE_2D,
					 0,
					 GL_RGBA,
					 tempIm->w,
					 tempIm->h,
					 0,
					 GL_RGBA,
					 GL_UNSIGNED_BYTE,
					 tempIm->pixels);

		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);

		glBindTexture(GL_TEXTURE_2D, 0);

		SDL_FreeSurface(tempIm);
	}
	if((tempIm = IMG_Load_RW(SDL_RWFromConstMem(GestureTextures::leftarrow_png,
											    GestureTextures::leftarrow_pngSize), 1)) != 0)
	{
		glGenTextures(1, &jerkTextures[0]);

		glBindTexture(GL_TEXTURE_2D, jerkTextures[0]);

		glTexImage2D(GL_TEXTURE_2D,
					 0,
					 GL_RGBA,
					 tempIm->w,
					 tempIm->h,
					 0,
					 GL_RGBA,
					 GL_UNSIGNED_BYTE,
					 tempIm->pixels);

		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);

		glBindTexture(GL_TEXTURE_2D, 0);

		SDL_FreeSurface(tempIm);
	}
	if((tempIm = IMG_Load_RW(SDL_RWFromConstMem(GestureTextures::rightarrow_png,
											    GestureTextures::rightarrow_pngSize), 1)) != 0)
	{
		glGenTextures(1, &jerkTextures[1]);

		glBindTexture(GL_TEXTURE_2D, jerkTextures[1]);

		glTexImage2D(GL_TEXTURE_2D,
					 0,
					 GL_RGBA,
					 tempIm->w,
					 tempIm->h,
					 0,
					 GL_RGBA,
					 GL_UNSIGNED_BYTE,
					 tempIm->pixels);

		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);

		glBindTexture(GL_TEXTURE_2D, 0);

		SDL_FreeSurface(tempIm);
	}
}

//------------------------------------------------------------------------------
void GestureDetector::graphicDoStuff()
{
	int i;
	int tempint;
	EventValue ev;
	float tempf = 0.0f;
	ThreeFloats temp3f;
	ThreeFloats normal;
	float matched = -10.0f;
	float leftVal, rightVal;
	bool displayNow = false;
	ThreeFloats perp1, perp2;
	const ThreeFloats origin;

	//Update gesture if necessary.
	if(gestureCount <= 0)
	{
		for(i=9;i>0;--i)
			gesture[i] = gesture[i-1];
		gesture[0] = ThreeFloats(accelX, accelY, accelZ) * 0.25f;
		gestureCount = 5;

		//Test for match gestures.
		recGest = gesture;
		if(recGest.normalise() < 100.0f) //So we ignore small gestures.
		{
			recGest.spacePoints();
			recGest.centre();
			//Test against stored gestures.
			for(i=0;i<3;++i)
			{
				tempf = recGest.dot(testGesture[i]);
				if(tempf > matched)
				{
					matched = tempf;
					matchedGesture = i;
				}
			}
			if(matched <= 0.0f)
				match = NoMatch;
			else
			{
				tempf /= sqrtf(recGest.dot(recGest) *
							   testGesture[matchedGesture].dot(testGesture[matchedGesture]));
				if(matched > 1.75f)
					match = FullMatch;
				else if(matched > 0.8f)
					match = SomeMatch;
				else
					match = NoMatch;
			}

			if((match == FullMatch) && (matchCount == 0))
			{
				switch(matchedGesture)
				{
					case 0:
						ev = "true";
						sendEvent("match_1", ev);
						break;
					case 1:
						ev = "true";
						sendEvent("match_2", ev);
						break;
					case 2:
						ev = "true";
						sendEvent("match_3", ev);
						break;
				}
				matchCount = 10;
				matchFade = 1.0f;
			}
			else if(matchCount > 0)
				--matchCount;
		}

		//Test for jerks.
		if(jerkCount == 0)
		{
			leftVal = 0.0f;
			rightVal = 0.0f;
			for(i=1;i<9;++i)
			{
				if(gesture[i].x > gesture[i-1].x)
					rightVal += (gesture[i].x - gesture[i-1].x);
				else if(gesture[i].x < gesture[i-1].x)
					leftVal += (gesture[i-1].x - gesture[i].x);
			}
			if((rightVal > leftVal) && (rightVal > 2.0f))
			{
				jerk = 2;
				jerkFade = 1.0f;
				jerkCount = 10;
				ev = "true";
				sendEvent("leftJerk", ev);
			}
			else if((rightVal < leftVal) && (leftVal > 2.0f))
			{
				jerk = 1;
				jerkFade = 1.0f;
				jerkCount = 10;
				ev = "true";
				sendEvent("rightJerk", ev);
			}
			else
				jerk = 0;
		}
		else if(jerkCount > 0)
			--jerkCount;
	}
	else
		--gestureCount;

	if(showDebugVisuals)
	{
		//Check if we've got something to display.
		temp3f = gesture[0];
		for(i=1;i<10;++i)
		{
			if(temp3f != gesture[i])
			{
				displayNow = true;
				break;
			}
		}

		//Display the gesture if it exists.
		if(displayNow)
		{
			perp1 = gesture[0]/gesture[0].getDistance(origin);
			perp1.x = -perp1.x;
			perp1 *= gesture[0].getDistance(gesture[1]) * 0.1f;
			for(i=0;i<9;++i)
			{
				perp2 = gesture[i+1]/gesture[i+1].getDistance(origin);
				perp2.x = -perp2.x;
				perp2 *= gesture[i].getDistance(gesture[i+1]) * 0.1f;
				normal = heilanGetNormal(gesture[i]+perp1,
										 gesture[i]-perp1,
										 gesture[i+1]-perp2);
				glBegin(GL_QUADS);
				{
					glNormal3f(normal.x, normal.y, normal.z);
					glVertex3f(gesture[i].x+perp1.x,
							   gesture[i].y+perp1.y,
							   gesture[i].z+perp1.z);
					glVertex3f(gesture[i].x-perp1.x,
							   gesture[i].y-perp1.y,
							   gesture[i].z-perp1.z);
					glVertex3f(gesture[i+1].x-perp2.x,
							   gesture[i+1].y-perp2.y,
							   gesture[i+1].z-perp2.z);
					glVertex3f(gesture[i+1].x+perp2.x,
							   gesture[i+1].y+perp2.y,
							   gesture[i+1].z+perp2.z);
				}
				glEnd();
				perp1 = perp2;
			}

			for(i=0;i<9;++i)
			{
				glDisable(GL_LIGHTING);

				glColor3f(1.0f, 0.0f, 0.0f);
				glLineWidth(4.0f);

				glBegin(GL_LINES);
				{
					glVertex3f(recGest[i].x,
							   recGest[i].y,
							   recGest[i].z);
					glVertex3f(recGest[i+1].x,
							   recGest[i+1].y,
							   recGest[i+1].z);
				}
				glEnd();

				if(match == SomeMatch)
				{
					glColor3f(0.0f, 0.0f, 1.0f);
					glBegin(GL_QUADS);
					{
						glVertex3f(-1.0f, -1.0f, -4.0f);
						glVertex3f(1.0f, -1.0f, -4.0f);
						glVertex3f(1.0f, 1.0f, -4.0f);
						glVertex3f(-1.0f, 1.0f, -4.0f);
					}
					glEnd();

					glColor4f(0.0f, 0.0f, 0.0f, 0.5f);
				}
				else if(match == FullMatch)
				{
					glColor3f(1.0f, 1.0f, 0.0f);
					glBegin(GL_QUADS);
					{
						glVertex3f(-1.0f, -1.0f, -4.0f);
						glVertex3f(1.0f, -1.0f, -4.0f);
						glVertex3f(1.0f, 1.0f, -4.0f);
						glVertex3f(-1.0f, 1.0f, -4.0f);
					}
					glEnd();

					glColor4f(0.0f, 0.0f, 0.0f, 1.0f);
				}
				if(match != NoMatch)
				{
					glLineWidth(6.0f);
					glBegin(GL_LINES);
					{
						for(i=0;i<9;++i)
						{
							glVertex3f(testGesture[matchedGesture][i].x,
									   testGesture[matchedGesture][i].y,
									   testGesture[matchedGesture][i].z);
							glVertex3f(testGesture[matchedGesture][i+1].x,
									   testGesture[matchedGesture][i+1].y,
									   testGesture[matchedGesture][i+1].z);
						}
					}
					glEnd();
				}

				glLineWidth(1.0f);
				glEnable(GL_LIGHTING);
			}
		}
	}

	//Display a match texture if necessary.
	if(!useParticles)
	{
		if(matchFade > 0.0f)
		{
			if((matchFade == 1.0f) && (matchCount == 10))
				whichTexture = matchedGesture;

			glDisable(GL_DEPTH_TEST);
			glDisable(GL_LIGHTING);
			glEnable(GL_BLEND);
			glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);

			glBindTexture(GL_TEXTURE_2D, matchTextures[whichTexture]);
			glColor4f(1.0f, 1.0f, 1.0f, matchFade);
			glBegin(GL_QUADS);
			{
				glTexCoord2f(0.0f, 1.0f);
				glVertex3f(-2.0f, -2.0f, 0.0f);

				glTexCoord2f(1.0f, 1.0f);
				glVertex3f(2.0f, -2.0f, 0.0f);

				glTexCoord2f(1.0f, 0.0f);
				glVertex3f(2.0f, 2.0f, 0.0f);
	
				glTexCoord2f(0.0f, 0.0f);
				glVertex3f(-2.0f, 2.0f, 0.0f);
			}
			glEnd();
			glBindTexture(GL_TEXTURE_2D, 0);

			glDisable(GL_BLEND);
			glEnable(GL_DEPTH_TEST);
			glEnable(GL_LIGHTING);

			if(matchCount == 0)
				matchFade -= 0.01f;
		}

		//Display a jerk texture if necessary.
		if(jerkFade > 0.0f)
		{
			if((jerkFade == 1.0f) && (jerkCount == 10))
				whichJerk = jerk;

			glDisable(GL_DEPTH_TEST);
			glDisable(GL_LIGHTING);
			glEnable(GL_BLEND);
			glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);

			if(whichJerk != 0)
			{
				glBindTexture(GL_TEXTURE_2D, jerkTextures[whichJerk-1]);
				glColor4f(1.0f, 1.0f, 1.0f, jerkFade);
				glBegin(GL_QUADS);
				{
					glTexCoord2f(0.0f, 1.0f);
					glVertex3f(-2.0f, -2.0f, 0.0f);

					glTexCoord2f(1.0f, 1.0f);
					glVertex3f(2.0f, -2.0f, 0.0f);

					glTexCoord2f(1.0f, 0.0f);
					glVertex3f(2.0f, 2.0f, 0.0f);

					glTexCoord2f(0.0f, 0.0f);
					glVertex3f(-2.0f, 2.0f, 0.0f);
				}
				glEnd();
				glBindTexture(GL_TEXTURE_2D, 0);
			}

			glDisable(GL_BLEND);
			glEnable(GL_DEPTH_TEST);
			glEnable(GL_LIGHTING);

			if(jerkCount == 0)
				jerkFade -= 0.01f;
		}
	}
	else
	{
		stringstream tempConverter;

		if(matchFade > 0.0f)
		{
			if((matchFade == 1.0f) && (matchCount == 10))
				whichTexture = matchedGesture;

			matchFade -= 0.01f;

			//Set the position.
			tempf = matchFade;
			tempf *= 9.0f;
			tempint = (int)tempf;
			tempf -= (float)tempint;
			temp3f = testGesture[whichTexture][tempint] * (1.0f-tempf);
			temp3f += testGesture[whichTexture][tempint+1] * tempf;
			temp3f *= 3.0f;
			tempConverter << temp3f.x << " ";
			tempConverter << temp3f.y << " ";
			tempConverter << temp3f.z;

			if(particles)
			{
				ev = "false";
				particles->setAttribute("stop", ev);
				ev = tempConverter.str();
				particles->setAttribute("position", ev);
			}
		}
		else if(particles)
		{
			ev = "true";
			particles->setAttribute("stop", ev);
		}

		if(particles)
			particles->graphicDoStuff();
	}
}

//------------------------------------------------------------------------------
void GestureDetector::setAttribute(const string& attribute, EventValue& val)
{
	if(attribute == "debugVisuals")
	{
		StringParser parser(val);

		if(parser.getNumBools() > 0)
			showDebugVisuals = parser.getBool();
	}
	else if(attribute == "useParticles")
	{
		StringParser parser(val);

		if(parser.getNumBools() > 0)
			useParticles = parser.getBool();
	}
	else if(attribute == "accelX")
	{
		accelX.valueChanged(val);
		sendEvent("accelX", val);
	}
	else if(attribute == "accelY")
	{
		accelY.valueChanged(val);
		sendEvent("accelY", val);
	}
	else if(attribute == "accelZ")
	{
		accelZ.valueChanged(val);
		sendEvent("accelZ", val);
	}
}

}
