//	MassSpringDamper.cpp - Simple physical model to route to other nodes.
//  ----------------------------------------------------------------------------
//	This file is part of 'Heilan Browser', an X3D browser specialised for audio
//	work.
//	Copyright (C) 2005-2009  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#include "MassSpringDamper.h"

#include <sstream>
#include <iostream>

using namespace std;

namespace HeilanX3D
{

NodeConstructor<MassSpringDamper> MassSpringDamper::constructor;
//------------------------------------------------------------------------------
MassSpringDamper::MassSpringDamper():
ChildNode(),
samplerate(44100.0f),
intendedPos(0.0f),
stiffness(10.0f),
mass(10.0f),
damping(1.0f),
amplitude(1.0f),
stiffnessAndMass(1.0f),
samplePeriod(1.0f/44100.0f),
contCounter(0)
{
	setType("MassSpringDamper");
	setVisual(false);
	setAural(true);
}

//------------------------------------------------------------------------------
MassSpringDamper::~MassSpringDamper()
{
	
}

//------------------------------------------------------------------------------
AmbisonicData *MassSpringDamper::getMonoAudio(int numSamples)
{
	int i;

	for(i=0;i<numSamples;++i)
	{
		//Update position calculation.
		integrate(samplePeriod);

		//Update the counter, so we know when to send the current mass position.
		++contCounter;

		//If it's time, send the envelope value as an event.
		if(contCounter > (int)(samplerate*0.01f))
		{
			stringstream tempConverter;

			contCounter = 0;

			tempConverter << (position.x * amplitude) + intendedPos;
			sendEvent("massPosition", tempConverter.str());
		}
	}

	return 0;
}

//------------------------------------------------------------------------------
void MassSpringDamper::setSamplerate(float val)
{
	samplerate = val;
	samplePeriod = 1.0f/samplerate;
}

//------------------------------------------------------------------------------
void MassSpringDamper::setAttribute(const string& attribute, EventValue& val)
{
	if((attribute == "set_position") || (attribute == "position"))
	{
		float lastPos = intendedPos;

		intendedPos = val;

		position.x += lastPos-intendedPos;
	}
	else if(attribute == "stiffness")
	{
		stiffness = val;

		stiffnessAndMass = stiffness/mass;
	}
	else if(attribute == "mass")
	{
		mass = val;

		stiffnessAndMass = stiffness/mass;
	}
	else if(attribute == "damping")
		damping = val;
	else if(attribute == "amplitude")
		amplitude = val;
}

}
