//	OscPlayback.cpp - Node which plays back a file of saved OSC data.
//  ----------------------------------------------------------------------------
//	This file is part of 'Heilan Browser', an X3D browser specialised for audio
//	work.
//	Copyright (C) 2005-2009  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#include "OscPlayback.h"
#include "OpenSoundController.h"
#include "OSCMessage.h"
#include "FileFinder.h"

#include <fstream>
#include <sstream>
#include <iostream>

using namespace std;

namespace HeilanX3D
{

NodeConstructor<OscPlayback> OscPlayback::constructor;

//------------------------------------------------------------------------------
OscPlayback::OscPlayback():
ChildNode(),
samples(0)
{
	setType("OscPlayback");
	setAural(true);
}

//------------------------------------------------------------------------------
OscPlayback::~OscPlayback()
{
	
}

//------------------------------------------------------------------------------
AmbisonicData *OscPlayback::getMonoAudio(int numSamples)
{
	int i;
	multimap<unsigned int, Message>::iterator endIt;
	multimap<unsigned int, Message>::iterator tempIt;
	multimap<unsigned int, Message>::iterator startIt;

	for(i=0;i<numSamples;++i)
	{
		++samples;

		startIt = messages.lower_bound(samples);
		endIt = messages.upper_bound(samples);
		for(tempIt=startIt;tempIt!=endIt;++tempIt)
		{
			OSC::Message tempMessage;

			tempMessage.setAddress(tempIt->second.address);
			tempMessage.addOscString(tempIt->second.value);

			OpenSoundController::getInstance()->handleOSCMessage(&tempMessage);
		}
	}

	return 0;
}

//------------------------------------------------------------------------------
void OscPlayback::setAttribute(const string& attribute, EventValue& val)
{
	if((attribute == "url") && ((string)val != ""))
	{
		string tempPath;

		tempPath = FileFinder::getInstance().findFile(val);
		if(tempPath != "")
		{
			ifstream inFile(tempPath.c_str());
			stringstream tempConverter;
			//-1 = new line,
			//0 = timestamp,
			//1 = address,
			//2 = parameter,
			//3 = value
			int currentSection = -1;
			unsigned int timestamp;
			string address;
			string parameter;
			string value;
	
			if(inFile.fail())
			{
				cout << "OscPlayback: Could not load " << (string)val << endl;
				return;
			}
	
			while(!inFile.eof())
			{
				char tempch;
	
				tempch = inFile.get();
	
				if((tempch != ' ') && (tempch != '\n') && (tempch != '\r'))
				{
					if(currentSection == -1)
						currentSection = 0;
	
					tempConverter << tempch;
				}
				else if(currentSection == 0)
				{
					tempConverter >> timestamp;
					tempConverter.clear();
					tempConverter.str("");
					++currentSection;
				}
				else if(currentSection == 1)
				{
					address = tempConverter.str();
					tempConverter.clear();
					tempConverter.str("");
					++currentSection;
				}
				else if(currentSection == 2)
				{
					parameter = tempConverter.str();
					tempConverter.clear();
					tempConverter.str("");
					++currentSection;
				}
				else if(currentSection == 3)
				{
					Message tempMessage;
	
					value = tempConverter.str();
					tempConverter.clear();
					tempConverter.str("");
	
					tempConverter << "/heilan/" << address;
					tempConverter << "/" << parameter;
	
					tempMessage.address = tempConverter.str();
					tempMessage.value = value;
					messages.insert(make_pair(timestamp, tempMessage));
	
					tempConverter.clear();
					tempConverter.str("");
					currentSection = -1;
					timestamp = 0;
				}
			}
			cout << "OscPlayback: Done loading." << endl;
		}
		else
			cout << "OscPlayback: " << (string)val << " does not seem to exist?" << endl;
	}
}

}
