//	PhysicalSurface.cpp - A simple visual plane based on Tao's physical
//						  modelling techniques.
//  ----------------------------------------------------------------------------
//	This file is part of 'Heilan Browser', an X3D browser specialised for audio
//	work.
//	Copyright (C) 2005-2009  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#include "PhysicalSurface.h"
#include "StringParser.h"

#include <iostream>

using namespace std;

namespace HeilanX3D
{

NodeConstructor<PhysicalSurface> PhysicalSurface::constructor;

//------------------------------------------------------------------------------
PhysicalSurface::PhysicalSurface():
ChildNode(),
force1(0.0f),
force2(0.0f),
force3(0.0f),
force4(0.0f),
drawLines(true)
{
	setType("PhysicalSurface");
	setVisual(true);
}

//------------------------------------------------------------------------------
PhysicalSurface::~PhysicalSurface()
{
	
}

//------------------------------------------------------------------------------
void PhysicalSurface::initialise()
{
	int i;

	//Calculate audio stuff.
	for(i=0;i<(SurfaceHeight*SurfaceWidth);++i)
	{
		positions[i] = 0.0f;
		forces[i] = 0.0f;
		velocities[i] = 0.0f;
	}

	forces[SurfaceWidth+1] = 1.0f;

	//Calculate cells' mass.
	float defaultVelocityMultiplier;
	float decay = (1.0f/16.0f);
	float intendedFreq = 110.0f;
	float actualFreq;
	float compensationFactor;
	defaultVelocityMultiplier = (1.0-((.00012f*30.0f)/30.0f)/decay);
	actualFreq = (24000.0f*30.0f/30.0f)/(SurfaceWidth+1);
	compensationFactor=powf(4.0f, log10f(actualFreq/intendedFreq)/log10f(2.0f));
	mass = 3.5f * compensationFactor;
	invMass = 1.0f/mass;

	velocityMultiplier = defaultVelocityMultiplier;
}

//------------------------------------------------------------------------------
void PhysicalSurface::graphicDoStuff()
{
	int i, j;
	int tempint;

	//Update surface.

	//Calculate forces.
	{
		//Propagate forces.
		j = 0;
		for(i=1;i<(SurfaceWidth-1);++i)
		{
			tempint = (j*SurfaceHeight)+i;
			forces[tempint] = (positions[tempint-1] +
							   positions[tempint+1] +
							   positions[tempint+SurfaceHeight-1] +
							   positions[tempint+SurfaceHeight] +
							   positions[tempint+SurfaceHeight+1] +
							   positions[(SurfaceWidth*(SurfaceHeight-1))+tempint-1] +
							   positions[(SurfaceWidth*(SurfaceHeight-1))+tempint] +
							   positions[(SurfaceWidth*(SurfaceHeight-1))+tempint+1]) -
							  (8.0f * positions[tempint]);
		}
		tempint = (j*SurfaceHeight);
		forces[tempint] = (forces[tempint+1] +
						   forces[tempint+SurfaceHeight] +
						   forces[tempint+SurfaceHeight+1] +
						   forces[(SurfaceWidth*(SurfaceHeight-1))+tempint] +
						   forces[(SurfaceWidth*(SurfaceHeight-1))+tempint+1]) -
						  (5.0f * forces[tempint]);

		tempint = (j*SurfaceHeight)+(SurfaceWidth-1);
		forces[tempint] = (forces[tempint-1] +
						   forces[tempint+SurfaceHeight] +
						   forces[tempint+SurfaceHeight-1] +
						   forces[(SurfaceWidth*(SurfaceHeight-1))+tempint] +
						   forces[(SurfaceWidth*(SurfaceHeight-1))+tempint-1]) -
						  (5.0f * forces[tempint]);

		for(j=1;j<(SurfaceHeight-1);++j)
		{
			for(i=1;i<(SurfaceWidth-1);++i)
			{
				tempint = (j*SurfaceHeight)+i;
				forces[tempint] = (positions[tempint-1] +
								   positions[tempint+1] +
								   positions[tempint-SurfaceHeight-1] +
								   positions[tempint-SurfaceHeight] +
								   positions[tempint-SurfaceHeight+1] +
								   positions[tempint+SurfaceHeight-1] +
								   positions[tempint+SurfaceHeight] +
								   positions[tempint+SurfaceHeight+1]) -
								  (8.0f * positions[tempint]);
			}
			tempint = (j*SurfaceHeight);
			forces[tempint] = (forces[tempint+1] +
							   forces[tempint-SurfaceHeight] +
							   forces[tempint-SurfaceHeight+1] +
							   forces[tempint+SurfaceHeight] +
							   forces[tempint+SurfaceHeight+1]) -
							  (5.0f * forces[tempint]);

			tempint = (j*SurfaceHeight)+(SurfaceWidth-1);
			forces[tempint] = (forces[tempint-1] +
							   forces[tempint-SurfaceHeight] +
							   forces[tempint-SurfaceHeight-1] +
							   forces[tempint+SurfaceHeight] +
							   forces[tempint+SurfaceHeight-1]) -
							  (5.0f * forces[tempint]);
		}

		for(i=1;i<(SurfaceWidth-1);++i)
		{
			tempint = (j*SurfaceHeight)+i;
			forces[tempint] = (positions[tempint-1] +
							   positions[tempint+1] +
							   positions[tempint-SurfaceHeight-1] +
							   positions[tempint-SurfaceHeight] +
							   positions[tempint-SurfaceHeight+1] +
							   positions[tempint-(j*SurfaceHeight)-1] +
							   positions[tempint-(j*SurfaceHeight)] +
							   positions[tempint-(j*SurfaceHeight)+1]) -
							  (8.0f * positions[tempint]);
		}
		tempint = (j*SurfaceHeight);
		forces[tempint] = (forces[tempint+1] +
						   forces[tempint-SurfaceHeight] +
						   forces[tempint-SurfaceHeight+1] +
						   forces[tempint-(j*SurfaceHeight)] +
						   forces[tempint-(j*SurfaceHeight)+1]) -
						  (5.0f * forces[tempint]);

		tempint = (j*SurfaceHeight)+(SurfaceWidth-1);
		forces[tempint] = (forces[tempint-1] +
						   forces[tempint-(j*SurfaceHeight)] +
						   forces[tempint-(j*SurfaceHeight)-1]) -
						  (3.0f * forces[tempint]);

		//Update force sensor values.
		forces[SurfaceWidth+1] += force1;
		forces[(3*SurfaceWidth)+1] += force2;
		forces[SurfaceWidth+3] += force3;
		forces[(3*SurfaceWidth)+3] += force4;
	}

	//Calculate positions.
	//for(i=1;i<(SurfaceHeight-1);++i)
	for(i=0;i<SurfaceHeight;++i)
	{
		for(j=1;j<(SurfaceWidth-1);++j)
		{
			tempint = (i*SurfaceWidth)+j;

			velocities[tempint] += forces[tempint] * invMass;
			velocities[tempint] *= velocityMultiplier;
			positions[tempint] += velocities[tempint];
		}
	}

	//Draw surface.
	glDisable(GL_LIGHTING);
	glLineWidth(2.0f);

	glScalef(4.0f, 4.0f, 4.0f);

	//Draw polys.
	glColor3f(1.0f, 0.0f, 0.0f);
	glPointSize(4.0f);
	glBegin(GL_QUADS);
	{
		for(i=0;i<SurfaceHeight;++i)
		{
			for(j=0;j<(SurfaceWidth-1);++j)
			{
				float tempi = (float)i/(float)SurfaceHeight;
				float tempj = (float)j/(float)(SurfaceWidth-1);

				glColor3f(1.0f, tempi, tempj);

				bezierSubdivide(j, i);
			}
		}
	}
	glEnd();

	//Now draw lines.
	if(drawLines)
	{
		glColor3f(0.0f, 0.0f, 0.0f);
		glDisable(GL_DEPTH_TEST);
		for(i=0;i<SurfaceHeight;++i)
		{
			glBegin(GL_LINE_STRIP);
			{
				for(j=0;j<SurfaceWidth;++j)
				{
					glVertex3f(((float)j/(float)SurfaceWidth)-0.5f,
							   positions[(i*SurfaceWidth)+j],
							   ((float)i/(float)SurfaceHeight)-0.5f);
				}
			}
			glEnd();
		}
		glBegin(GL_LINE_STRIP);
		{
			for(j=0;j<SurfaceWidth;++j)
			{
				glVertex3f(((float)j/(float)SurfaceWidth)-0.5f,
						   positions[j],
						   ((float)i/(float)SurfaceHeight)-0.5f);
			}
		}
		glEnd();
		for(j=0;j<SurfaceWidth;++j)
		{
			glBegin(GL_LINE_STRIP);
			{
				for(i=0;i<SurfaceHeight;++i)
				{
					glVertex3f(((float)j/(float)SurfaceWidth)-0.5f,
							   positions[(i*SurfaceWidth)+j],
							   ((float)i/(float)SurfaceHeight)-0.5f);
				}
				glVertex3f(((float)j/(float)SurfaceWidth)-0.5f,
						   positions[j],
						   ((float)i/(float)SurfaceHeight)-0.5f);
			}
			glEnd();
		}
		glEnable(GL_DEPTH_TEST);
	}

	glEnable(GL_LIGHTING);
}

//------------------------------------------------------------------------------
void PhysicalSurface::setAttribute(const string& attribute, EventValue& val)
{
	if(attribute == "force1")
		force1 = val;
	else if(attribute == "force2")
		force2 = val;
	else if(attribute == "force3")
		force3 = val;
	else if(attribute == "force4")
		force4 = val;
	else if(attribute == "drawLines")
		drawLines = ((float)val > 0.5f);
}
	
}
