//	SineTest.cpp - Simple sine tone test node.
//  ----------------------------------------------------------------------------
//	This file is part of 'Heilan Browser', an X3D browser specialised for audio
//	work.
//	Copyright (C) 2005-2009  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#include "SineTest.h"
#include "AmbisonicData.h"
#include "Camera.h"
#include "StringParser.h"

#include "GLHeaders.h"
#include <cmath>
#include <cassert>

using namespace std;

#define Pi 3.1415926535897932384626433832795f

namespace HeilanX3D
{

NodeConstructor<SineTest> SineTest::constructor;

//------------------------------------------------------------------------------
SineTest::SineTest():
ChildNode(),
audioBuffer(0),
bufferSize(0),
sinFreq(440.0f),
lastAtten(0.0f)
{
	setType("SineTest");
	setVisual(true);
	setAural(true);

	sinCoeff = 2.0f * sinf(Pi*(sinFreq/44100.0f));
	sinX0 = 0.0f;
	sinX1 = 1.0f;
}

//------------------------------------------------------------------------------
SineTest::~SineTest()
{
	if(audioBuffer)
		delete [] audioBuffer;
}

//------------------------------------------------------------------------------
void SineTest::graphicDoStuff()
{
	glColor3f(1.0f, 1.0f, 1.0f);
	glBegin(GL_LINES);
		glVertex3f(-0.5f, 0.0f, 0.0f);
		glVertex3f(0.5f, 0.0f, 0.0f);

		glVertex3f(0.0f, -0.5f, 0.0f);
		glVertex3f(0.0f, 0.5f, 0.0f);

		glVertex3f(0.0f, 0.0f, -0.5f);
		glVertex3f(0.0f, 0.0f, 0.5f);
	glEnd();
}

//------------------------------------------------------------------------------
AmbisonicData *SineTest::getMonoAudio(int numSamples)
{
	int i;

	//This should NEVER happen, unless there's an error somewhere else...
	assert(numSamples <= bufferSize);

	//Calculate audio, put in audioBuffer.
	for(i=0;i<numSamples;++i)
	{
		//Calculate monophonic audio.
		sinX0 = sinX0 - (sinX1 * sinCoeff);
		sinX1 = sinX1 + (sinX0 * sinCoeff);

		//Convert float sine tone to Ambisonic, put it in the buffer.
		audioBuffer[i].W = sinX0;
	}

	return audioBuffer;
}

//------------------------------------------------------------------------------
void SineTest::setBlockSize(int newSize)
{
	if(audioBuffer)
		delete [] audioBuffer;

	bufferSize = newSize;
	audioBuffer = new AmbisonicData[bufferSize];
}

//------------------------------------------------------------------------------
void SineTest::setAttribute(const string& attribute, EventValue& val)
{
	if(attribute == "freq")
	{
		sinFreq = val;
		sinCoeff = 2.0f * sinf(Pi*(sinFreq/44100.0f));
	}
}

}
