//	WebcamTexture.cpp - A texture that displays the output of a webcam.
//  ----------------------------------------------------------------------------
//	This file is part of 'Heilan Browser', an X3D browser specialised for audio
//	work.
//	Copyright (C) 2005-2009  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#include "WebcamTexture.h"

#include <iostream>

using namespace std;

namespace HeilanX3D
{

NodeConstructor<WebcamTexture> WebcamTexture::constructor;

//------------------------------------------------------------------------------
WebcamTexture::WebcamTexture():
webcam(0)
{
	setType("WebcamTexture");
	setVisual(true);
}

//------------------------------------------------------------------------------
WebcamTexture::~WebcamTexture()
{
	if(webcam)
	{
		webcam->stopCapture();
		delete webcam;
	}
}

//------------------------------------------------------------------------------
void WebcamTexture::callback(unsigned char *data)
{
	int x, y;

	critSec.lock();
	{
#ifdef WIN32
		//We invert the y axis because OpenGL's coordinates are the opposite
		//way round to Microsoft's.
		for(y=0;y<Height;++y)
		{
			for(x=0;x<(Width*3);++x)
				pixels[((Height-1-y)*Width*3)+x] = data[(y*Width*3)+x];
		}
#elif defined(LINUX)
		for(y=0;y<Height;++y)
		{
			for(x=0;x<(Width*3);++x)
				pixels[(y*Width*3)+x] = data[(y*Width*3)+x];
		}
#endif
	}
	critSec.unlock();
}

//------------------------------------------------------------------------------
void WebcamTexture::initialise()
{
	webcam = new Webcam();

	glGenTextures(1, &texture);

	glBindTexture(GL_TEXTURE_2D, texture);

	glTexImage2D(GL_TEXTURE_2D,
				 0,
				 GL_RGB,
				 TexSize,
				 TexSize,
				 0,
				 GL_BGR,
				 GL_UNSIGNED_BYTE,
				 pixels);

	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);

	glBindTexture(GL_TEXTURE_2D, 0);

	if(!webcam->isRunning())
	{
		webcam->addCallback(this);

		if(!webcam->startCapture())
			cout << "Could not start webcam capture." << endl;
	}
}

//------------------------------------------------------------------------------
void WebcamTexture::graphicDoStuff()
{
	critSec.lock();
	{
		glBindTexture(GL_TEXTURE_2D, texture);
		glTexSubImage2D(GL_TEXTURE_2D,
						0,
						0,
						0,
						Width,
						Height,
						GL_BGR,
						GL_UNSIGNED_BYTE,
						pixels);
	}
	critSec.unlock();
}

}
