//	FPSNavigation.cpp - FPS-style navigation type.
//  ----------------------------------------------------------------------------
//	This file is part of 'Heilan Browser', an X3D browser specialised for audio
//	work.
//	Copyright (C) 2005-2009  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#include "FPSNavigation.h"
#include "Camera.h"

#include "SDL.h"
#include <cmath>
#include <iostream>

using namespace std;

namespace HeilanX3D
{
	
NavigationConstructor<FPSNavigation> FPSNavigation::constructor;

//------------------------------------------------------------------------------
FPSNavigation::FPSNavigation():
Navigation(),
movingForward(false),
movingBackward(false),
movingLeft(false),
movingRight(false),
xRotation(0.0f),
xRot(0.0f),
yRot(0.0f),
lastXRot(0.0f),
lastYRot(0.0f)
{
	SDL_WM_GrabInput(SDL_GRAB_ON);
	SDL_WarpMouse(Camera::getInstance()->getViewWidth()/2,
				  Camera::getInstance()->getViewHeight()/2);
}

//------------------------------------------------------------------------------
FPSNavigation::~FPSNavigation()
{
	SDL_WM_GrabInput(SDL_GRAB_OFF);
}

//------------------------------------------------------------------------------
void FPSNavigation::mouseMove(int x, int y)
{
	const int screenWidth = Camera::getInstance()->getViewWidth();
	//const int screenHeight = Camera::getInstance()->getViewHeight();

	if(fabs((float)x-lastXRot) < 20.0f)
		rotation.x += ((float)x - lastXRot) * 0.1f;
	else if(((float)x-lastXRot) > 0.0f)
		rotation.x += 1.0f;
	else
		rotation.x -= 1.0f;
	lastXRot = (float)x;

	//This is so we get continuous motion.
	//(We don't use WarpMouse() all the time because it generates mouseMove
	// events, and we'd get caught in an infinite loop.)
	if(lastXRot <= 0)
	{
		lastXRot = screenWidth/2;
		SDL_WarpMouse(screenWidth/2,
					  (int)lastYRot);
	}
	else if(lastXRot >= (screenWidth-1))
	{
		lastXRot = screenWidth/2;
		SDL_WarpMouse(screenWidth/2,
					  (int)lastYRot);
	}


	/*
	//Up-down motion's a bit funny as it stands - the scene gets rotated as if
	//the camera's still pointed straight ahead.

	rotation.y += (y - lastYRot) * 0.1f;
	lastYRot = y;

	if(lastYRot <= 0)
	{
		lastYRot = screenHeight/2;
		SDL_WarpMouse(lastXRot,
					  screenHeight/2);
	}
	else if(lastYRot >= (screenHeight-1))
	{
		lastYRot = screenHeight/2;
		SDL_WarpMouse(lastXRot,
					  screenHeight/2);
	}*/
}

//------------------------------------------------------------------------------
void FPSNavigation::keyDown(SDLKey key)
{
	switch(key)
	{
		case SDLK_w:
			movingForward = true;
			break;
		case SDLK_a:
			movingLeft = true;
			break;
		case SDLK_s:
			movingBackward = true;
			break;
		case SDLK_d:
			movingRight = true;
			break;
		default:
			break;
	}
}

//------------------------------------------------------------------------------
void FPSNavigation::keyUp(SDLKey key)
{
	switch(key)
	{
		case SDLK_w:
			movingForward = false;
			break;
		case SDLK_a:
			movingLeft = false;
			break;
		case SDLK_s:
			movingBackward = false;
			break;
		case SDLK_d:
			movingRight = false;
			break;
		default:
			break;
	}
}

//------------------------------------------------------------------------------
void FPSNavigation::screenRefresh()
{
	if(movingForward)
	{
		//position.z -= 0.15f;
		position.z -= cosf((rotation.x/360.0f) * 6.283185306f) * 0.1f;
		position.x -= -sinf((rotation.x/360.0f) * 6.283185306f) * 0.1f;
	}
	if(movingBackward)
	{
		//position.z += 0.15f;
		position.z += cosf((rotation.x/360.0f) * 6.283185306f) * 0.1f;
		position.x += -sinf((rotation.x/360.0f) * 6.283185306f) * 0.1f;
	}
	if(movingLeft)
	{
		//position.x -= 0.15f;
		position.x -= cosf((rotation.x/360.0f) * 6.283185306f) * 0.1f;
		position.z -= sinf((rotation.x/360.0f) * 6.283185306f) * 0.1f;
	}
	if(movingRight)
	{
		//position.x += 0.15f;
		position.x += cosf((rotation.x/360.0f) * 6.283185306f) * 0.1f;
		position.z += sinf((rotation.x/360.0f) * 6.283185306f) * 0.1f;
	}

	/*if(xRotation != 0.0f)
		rotation.x += (xRotation * 50.0f);*/

	/*Camera::getInstance()->setGraphicRotation(rotation);
	Camera::getInstance()->setGraphicPosition(position);*/
}

}
