//	SimpleViewpointNavigation.cpp - 'Moving the mouse interpolates between
//									Viewpoints.
//  ----------------------------------------------------------------------------
//	This file is part of 'Heilan Browser', an X3D browser specialised for audio
//	work.
//	Copyright (C) 2005-2009  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#include "SimpleViewpointNavigation.h"
#include "Camera.h"
#include "Viewpoint.h"
#include "ThreeFloats.h"
#include "FourFloats.h"

#include "SDL_mouse.h"
#include <cmath>
#include <iostream>

using namespace std;

namespace HeilanX3D
{

NavigationConstructor<SimpleViewpointNavigation> SimpleViewpointNavigation::constructor;

//------------------------------------------------------------------------------
SimpleViewpointNavigation::SimpleViewpointNavigation():
Navigation(),
leftIsDown(false),
middleIsDown(false),
rightIsDown(false),
lastY(0),
viewpointIndex(0.0f)
{
	
}

//------------------------------------------------------------------------------
SimpleViewpointNavigation::~SimpleViewpointNavigation()
{
	
}

//------------------------------------------------------------------------------
void SimpleViewpointNavigation::mouseDown(int button, int x, int y)
{
	switch(button)
	{
		case SDL_BUTTON_LEFT:
			leftIsDown = true;
			break;
		case SDL_BUTTON_MIDDLE:
			middleIsDown = true;
			break;
		case SDL_BUTTON_RIGHT:
			rightIsDown = true;
			break;
	}
	lastY = y;
}

//------------------------------------------------------------------------------
void SimpleViewpointNavigation::mouseMove(int x, int y)
{
	float fracIndex;
	float numNodes;
	ThreeFloats tempPos;
	FourFloats tempRot;
	ThreeFloats startPos;
	ThreeFloats endPos;
	FourFloats startRot;
	FourFloats endRot;

	Viewpoint *startVP;
	Viewpoint *endVP;

	if(!leftIsDown)
		return;

	//Calculate new viewpointIndex.
	viewpointIndex += (float)(lastY-y) * (1.0f/64.0f);
	numNodes = (float)BindableNodeSingleton::getInstance()->getNumNodes("Viewpoint");
	if(viewpointIndex >= numNodes)
		viewpointIndex = fmod(viewpointIndex, numNodes);
	else if(viewpointIndex < 0.0f)
		viewpointIndex = numNodes - fmod(fabs(viewpointIndex), numNodes);

	//Calculate start and end positions of the interpolation.
	//Okay, I should have probably given some of these classes shorter names...
	startVP = (Viewpoint *)BindableNodeSingleton::getInstance()->getIndexedNode("Viewpoint",
																   (int)viewpointIndex);
	if(((int)viewpointIndex+1) >= (int)numNodes)
	{
		endVP = (Viewpoint *)BindableNodeSingleton::getInstance()->getIndexedNode("Viewpoint",
																     0);
	}
	else
	{
		endVP = (Viewpoint *)BindableNodeSingleton::getInstance()->getIndexedNode("Viewpoint",
																     (int)viewpointIndex+1);
	}
	startPos = startVP->getPosition();
	//startRot = startVP->getRotation();
	endPos = endVP->getPosition();
	//endRot = endVP->getRotation();

	//Calculate current position between the start and end points.
	fracIndex = viewpointIndex - (float)((int)viewpointIndex);
	tempPos = (startPos * (1.0f-fracIndex)) + (endPos * fracIndex);
	//tempRot = (startRot * (1.0f-fracIndex)) + (endRot * fracIndex);

	//Update the camera's position and rotation accordingly.
	/*Camera::getInstance()->setGraphicPosition(tempPos);
	Camera::getInstance()->setGraphicRotation(tempRot);*/

	lastY = y;
}

//------------------------------------------------------------------------------
void SimpleViewpointNavigation::mouseUp(int button, int x, int y)
{
	switch(button)
	{
		case SDL_BUTTON_LEFT:
			leftIsDown = false;
			break;
		case SDL_BUTTON_MIDDLE:
			middleIsDown = false;
			break;
		case SDL_BUTTON_RIGHT:
			rightIsDown = false;
			break;
	}
}

}
