//	ApplicationInitFactory.cpp - Holds Application::initFactory().
//	----------------------------------------------------------------------------
//	This file is part of 'Heilan Browser', an X3D browser specialised for audio
//	work.
//	Copyright (C) 2005-2009  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#include "Application.h"
#include "NodeFactory.h"
#include "Navigation.h"

#include "Scene.h"
#include "Shape.h"
#include "Box.h"
#include "Appearance.h"
#include "Material.h"
#include "Transform.h"
#include "ImageTexture.h"
#include "MetadataString.h"
#include "NavigationInfo.h"
#include "Viewpoint.h"
#include "AudioClip.h"
#include "Sound.h"
#include "TimeSensor.h"
#include "ColorInterpolator.h"
#include "Background.h"
#include "NurbsCurve.h"
#include "Coordinate.h"
#include "NurbsPatchSurface.h"
#include "MetadataDouble.h"
#include "MetadataFloat.h"
#include "MetadataInteger.h"
#include "Color.h"
#include "Cone.h"
#include "Cylinder.h"
#include "Group.h"
#include "WorldInfo.h"
#include "ColorRGBA.h"
#include "CoordinateInterpolator.h"
#include "DirectionalLight.h"
#include "Sphere.h"
#include "Normal.h"
#include "TextureCoordinate.h"
#include "IndexedFaceSet.h"
#include "IndexedLineSet.h"
#include "LineSet.h"
#include "PointSet.h"
#include "IndexedTriangleSet.h"
#include "IndexedTriangleStripSet.h"
#include "IndexedTriangleFanSet.h"
#include "TriangleSet.h"
#include "TriangleStripSet.h"
#include "TriangleFanSet.h"
#include "MetadataSet.h"
#include "PixelTexture.h"
#include "TextureTransform.h"
#include "PositionInterpolator.h"
#include "ScalarInterpolator.h"
#include "OrientationInterpolator.h"
#include "NormalInterpolator.h"
#include "TextureCoordinateGenerator.h"
#include "MultiTexture.h"
#include "MultiTextureCoordinate.h"
#include "MultiTextureTransform.h"
#include "ProgramShader.h"
#include "ShaderProgram.h"
#include "Switch.h"
#include "Fog.h"
#include "LineProperties.h"
#include "Billboard.h"
#include "PointLight.h"
#include "BooleanFilter.h"
#include "BooleanToggle.h"
#include "BooleanTrigger.h"
#include "IntegerTrigger.h"
#include "TimeTrigger.h"
#include "BooleanSequencer.h"
#include "IntegerSequencer.h"
#include "SpotLight.h"
#include "ElevationGrid.h"
#include "KeySensor.h"
#include "Inline.h"
#include "StringSensor.h"
#include "ProximitySensor.h"
#include "MovieTexture.h"
#include "Viewport.h"

#include "ExamineNavigation.h"
#include "NoneNavigation.h"

#include "LibSndFileLoader.h"
#include "VorbisFileLoader.h"

using namespace HeilanX3D;

//------------------------------------------------------------------------------
void Application::initFactory()
{
	//Nodes.
	NodeFactory::getInstance().registerNode(&Scene::constructor);
	NodeFactory::getInstance().registerNode(&Shape::constructor);
	NodeFactory::getInstance().registerNode(&Box::constructor);
	NodeFactory::getInstance().registerNode(&Appearance::constructor);
	NodeFactory::getInstance().registerNode(&Material::constructor);
	NodeFactory::getInstance().registerNode(&Transform::constructor);
	NodeFactory::getInstance().registerNode(&ImageTexture::constructor);
	NodeFactory::getInstance().registerNode(&MetadataString::constructor);
	NodeFactory::getInstance().registerNode(&NavigationInfo::constructor);
	NodeFactory::getInstance().registerNode(&Viewpoint::constructor);
	NodeFactory::getInstance().registerNode(&AudioClip::constructor);
	NodeFactory::getInstance().registerNode(&Sound::constructor);
	NodeFactory::getInstance().registerNode(&TimeSensor::constructor);
	NodeFactory::getInstance().registerNode(&ColorInterpolator::constructor);
	NodeFactory::getInstance().registerNode(&Background::constructor);
	NodeFactory::getInstance().registerNode(&NurbsCurve::constructor);
	NodeFactory::getInstance().registerNode(&Coordinate::constructor);
	NodeFactory::getInstance().registerNode(&NurbsPatchSurface::constructor);
	NodeFactory::getInstance().registerNode(&MetadataDouble::constructor);
	NodeFactory::getInstance().registerNode(&MetadataFloat::constructor);
	NodeFactory::getInstance().registerNode(&MetadataInteger::constructor);
	NodeFactory::getInstance().registerNode(&Color::constructor);
	NodeFactory::getInstance().registerNode(&Cone::constructor);
	NodeFactory::getInstance().registerNode(&Cylinder::constructor);
	NodeFactory::getInstance().registerNode(&Group::constructor);
	NodeFactory::getInstance().registerNode(&WorldInfo::constructor);
	NodeFactory::getInstance().registerNode(&ColorRGBA::constructor);
	NodeFactory::getInstance().registerNode(&CoordinateInterpolator::constructor);
	NodeFactory::getInstance().registerNode(&DirectionalLight::constructor);
	NodeFactory::getInstance().registerNode(&Sphere::constructor);
	NodeFactory::getInstance().registerNode(&Normal::constructor);
	NodeFactory::getInstance().registerNode(&TextureCoordinate::constructor);
	NodeFactory::getInstance().registerNode(&IndexedFaceSet::constructor);
	NodeFactory::getInstance().registerNode(&IndexedLineSet::constructor);
	NodeFactory::getInstance().registerNode(&LineSet::constructor);
	NodeFactory::getInstance().registerNode(&PointSet::constructor);
	NodeFactory::getInstance().registerNode(&IndexedTriangleSet::constructor);
	NodeFactory::getInstance().registerNode(&IndexedTriangleStripSet::constructor);
	NodeFactory::getInstance().registerNode(&IndexedTriangleFanSet::constructor);
	NodeFactory::getInstance().registerNode(&TriangleSet::constructor);
	NodeFactory::getInstance().registerNode(&TriangleStripSet::constructor);
	NodeFactory::getInstance().registerNode(&TriangleFanSet::constructor);
	NodeFactory::getInstance().registerNode(&MetadataSet::constructor);
	NodeFactory::getInstance().registerNode(&PixelTexture::constructor);
	NodeFactory::getInstance().registerNode(&TextureTransform::constructor);
	NodeFactory::getInstance().registerNode(&PositionInterpolator::constructor);
	NodeFactory::getInstance().registerNode(&ScalarInterpolator::constructor);
	NodeFactory::getInstance().registerNode(&OrientationInterpolator::constructor);
	NodeFactory::getInstance().registerNode(&NormalInterpolator::constructor);
	NodeFactory::getInstance().registerNode(&TextureCoordinateGenerator::constructor);
	NodeFactory::getInstance().registerNode(&MultiTexture::constructor);
	NodeFactory::getInstance().registerNode(&MultiTextureCoordinate::constructor);
	NodeFactory::getInstance().registerNode(&MultiTextureTransform::constructor);
	NodeFactory::getInstance().registerNode(&ProgramShader::constructor);
	NodeFactory::getInstance().registerNode(&ShaderProgram::constructor);
	NodeFactory::getInstance().registerNode(&Switch::constructor);
	NodeFactory::getInstance().registerNode(&Fog::constructor);
	NodeFactory::getInstance().registerNode(&LineProperties::constructor);
	NodeFactory::getInstance().registerNode(&Billboard::constructor);
	NodeFactory::getInstance().registerNode(&PointLight::constructor);
	NodeFactory::getInstance().registerNode(&BooleanFilter::constructor);
	NodeFactory::getInstance().registerNode(&BooleanToggle::constructor);
	NodeFactory::getInstance().registerNode(&BooleanTrigger::constructor);
	NodeFactory::getInstance().registerNode(&IntegerTrigger::constructor);
	NodeFactory::getInstance().registerNode(&TimeTrigger::constructor);
	NodeFactory::getInstance().registerNode(&BooleanSequencer::constructor);
	NodeFactory::getInstance().registerNode(&IntegerSequencer::constructor);
	NodeFactory::getInstance().registerNode(&SpotLight::constructor);
	NodeFactory::getInstance().registerNode(&ElevationGrid::constructor);
	NodeFactory::getInstance().registerNode(&KeySensor::constructor);
	NodeFactory::getInstance().registerNode(&Inline::constructor);
	NodeFactory::getInstance().registerNode(&StringSensor::constructor);
	NodeFactory::getInstance().registerNode(&ProximitySensor::constructor);
	NodeFactory::getInstance().registerNode(&MovieTexture::constructor);
	NodeFactory::getInstance().registerNode(&Viewport::constructor);

	//NavigationTypes.
	NavigationFactory::getInstance()->registerNavigation("EXAMINE",
														 &ExamineNavigation::constructor);
	NavigationFactory::getInstance()->registerNavigation("NONE",
														 &NoneNavigation::constructor);

	//SoundFileLoaders.
	SFLoaderFactory::getInstance().registerSFLoader(&LibSndFileLoader::constructor,
													"wav aiff aifc aif raw paf iff svx sf voc w64 mat4 mat5 pvf xi htk caf flac");
#ifdef VORBISFILE
	SFLoaderFactory::getInstance().registerSFLoader(&VorbisFileLoader::constructor,
													"ogg");
#endif
}
