//	AttributeHelper.cpp - Classes used to convert single string values into the
//						  correct multiple value attribute.
//  ----------------------------------------------------------------------------
//	This file is part of 'Heilan Browser', an X3D browser specialised for audio
//	work.
//	Copyright (C) 2005-2009  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#include "AttributeHelper.h"
#include "StringParser.h"

#include <sstream>

using namespace std;

//------------------------------------------------------------------------------
//------------------------------------------------------------------------------
bool AttributeHelperBase::isValid(const string& val)
{
	if(val.find_last_of('.') != string::npos)
		return true;
	else
		return false;
}

//------------------------------------------------------------------------------
string AttributeHelperBase::getAddress(const string& address)
{
	string retval;
	unsigned int tempint = address.find_last_of('.');

	if(tempint != string::npos)
		retval = address.substr(0, address.find_first_of('.'));
	else
		retval = address;

	return retval;
}

//------------------------------------------------------------------------------
int AttributeHelperBase::getIndex(const string& address)
{
	unsigned int tempPos;
	string tempstr;
	int retval = -1;

	tempPos = address.find_last_of('.');
	if(tempPos != string::npos)
	{
		tempstr = address.substr(tempPos+1);

		if((tempstr == "x") || (tempstr == "r"))
			retval = 0;
		else if((tempstr == "y") || (tempstr == "g"))
			retval = 1;
		else if((tempstr == "z") || (tempstr == "b"))
			retval = 2;
		else if((tempstr == "a") || (tempstr == "w"))
			retval = 3;
		//Anything else, assume it's a number.
		else if(tempstr.size() > 0)
		{
			stringstream tempStream(tempstr);

			tempStream >> retval;
		}
	}

	return retval;
}

//------------------------------------------------------------------------------
int AttributeHelperBase::getSecondaryIndex(const string& address)
{
	unsigned int tempPos, tempPos2;
	string tempstr, tempstr2;
	int retval = -1;

	tempPos = address.find_last_of('.');
	if(tempPos != string::npos)
	{
		tempstr2 = address.substr(0, tempPos);

		tempPos2 = tempstr2.find_last_of('.');
		if(tempPos2 != string::npos)
		{
			tempstr = tempstr2.substr(tempPos2+1);

			if(tempstr.size() > 0)
			{
				stringstream tempStream(tempstr);

				tempStream >> retval;
			}
		}
	}

	return retval;
}

//------------------------------------------------------------------------------
//------------------------------------------------------------------------------
template <class T>
AttributeHelper<T>::AttributeHelper(T *attr):
AttributeHelperBase()
{
	
}

//------------------------------------------------------------------------------
template <class T>
AttributeHelper<T>::~AttributeHelper()
{
	
}

//------------------------------------------------------------------------------
template <class T>
void AttributeHelper<T>::convert(const string& address,
							     const string& val)
{
	
}

//------------------------------------------------------------------------------
//------------------------------------------------------------------------------
AttributeHelper<TwoFloats>::AttributeHelper(TwoFloats *attr):
AttributeHelperBase(),
attribute(attr)
{
	
}

//------------------------------------------------------------------------------
void AttributeHelper<TwoFloats>::convert(const string& address,
							  			 const string& val)
{
	StringParser parse(val);

	switch(getIndex(address))
	{
		case 0:
			if(parse.getNumNumbers() > 0)
				attribute->x = parse.getFloat();
			break;
		case 1:
			if(parse.getNumNumbers() > 0)
				attribute->y = parse.getFloat();
			break;
		case -1:
			if(parse.getNumTwoFloats() > 0)
				*attribute = parse.getTwoFloats();
			break;
	}
}

//------------------------------------------------------------------------------
//------------------------------------------------------------------------------
AttributeHelper<ThreeFloats>::AttributeHelper(ThreeFloats *attr):
AttributeHelperBase(),
attribute(attr)
{
	
}

//------------------------------------------------------------------------------
void AttributeHelper<ThreeFloats>::convert(const string& address,
							  			   const string& val)
{
	StringParser parse(val);

	switch(getIndex(address))
	{
		case 0:
			if(parse.getNumNumbers() > 0)
				attribute->x = parse.getFloat();
			break;
		case 1:
			if(parse.getNumNumbers() > 0)
				attribute->y = parse.getFloat();
			break;
		case 2:
			if(parse.getNumNumbers() > 0)
				attribute->z = parse.getFloat();
			break;
		case -1:
			if(parse.getNumThreeFloats() > 0)
				*attribute = parse.getThreeFloats();
			break;
	}
}

//------------------------------------------------------------------------------
//------------------------------------------------------------------------------
AttributeHelper<FourFloats>::AttributeHelper(FourFloats *attr):
AttributeHelperBase(),
attribute(attr)
{
	
}

//------------------------------------------------------------------------------
void AttributeHelper<FourFloats>::convert(const string& address,
							  			  const string& val)
{
	StringParser parse(val);

	switch(getIndex(address))
	{
		case 0:
			if(parse.getNumNumbers() > 0)
				attribute->x = parse.getFloat();
			break;
		case 1:
			if(parse.getNumNumbers() > 0)
				attribute->y = parse.getFloat();
			break;
		case 2:
			if(parse.getNumNumbers() > 0)
				attribute->z = parse.getFloat();
			break;
		case 3:
			if(parse.getNumNumbers() > 0)
				attribute->r = parse.getFloat();
			break;
		case -1:
			if(parse.getNumFourFloats() > 0)
				*attribute = parse.getFourFloats();
			break;
	}
}

//------------------------------------------------------------------------------
//------------------------------------------------------------------------------
AttributeHelper<Quaternion>::AttributeHelper(Quaternion *attr):
AttributeHelperBase(),
attribute(attr)
{
	
}

//------------------------------------------------------------------------------
void AttributeHelper<Quaternion>::convert(const string& address,
							  			  const string& val)
{
	StringParser parse(val);

	switch(getIndex(address))
	{
		case 0:
			if(parse.getNumNumbers() > 0)
				attribute->x = parse.getFloat();
			break;
		case 1:
			if(parse.getNumNumbers() > 0)
				attribute->y = parse.getFloat();
			break;
		case 2:
			if(parse.getNumNumbers() > 0)
				attribute->z = parse.getFloat();
			break;
		case 3:
			if(parse.getNumNumbers() > 0)
				attribute->w = parse.getFloat();
			break;
		case -1:
			if(parse.getNumQuaternions() > 0)
				*attribute = parse.getQuaternion();
			break;
	}
}

//------------------------------------------------------------------------------
//------------------------------------------------------------------------------
AttributeHelper< vector<double> >::AttributeHelper(vector<double> *attr):
AttributeHelperBase(),
attribute(attr)
{
	
}

//------------------------------------------------------------------------------
void AttributeHelper< vector<double> >::convert(const string& address,
							  			   	  	const string& val)
{
	int tempint;
	StringParser parse(val);

	tempint = getIndex(address);
	if((tempint > -1) && (parse.getNumNumbers() > 0))
		(*attribute)[tempint] = parse.getDouble();
}

//------------------------------------------------------------------------------
//------------------------------------------------------------------------------
AttributeHelper< vector<float> >::AttributeHelper(vector<float> *attr):
AttributeHelperBase(),
attribute(attr)
{
	
}

//------------------------------------------------------------------------------
void AttributeHelper< vector<float> >::convert(const string& address,
							  			   	   const string& val)
{
	int tempint;
	StringParser parse(val);

	tempint = getIndex(address);
	if((tempint > -1) && (parse.getNumNumbers() > 0))
		(*attribute)[tempint] = parse.getFloat();
}

//------------------------------------------------------------------------------
//------------------------------------------------------------------------------
AttributeHelper< vector<int> >::AttributeHelper(vector<int> *attr):
AttributeHelperBase(),
attribute(attr)
{
	
}

//------------------------------------------------------------------------------
void AttributeHelper< vector<int> >::convert(const string& address,
							  			   	   const string& val)
{
	int tempint;
	StringParser parse(val);

	tempint = getIndex(address);
	if((tempint > -1) && (parse.getNumNumbers() > 0))
		(*attribute)[tempint] = parse.getInt();
}

//------------------------------------------------------------------------------
//------------------------------------------------------------------------------
AttributeHelper< vector<ThreeFloats> >::AttributeHelper(vector<ThreeFloats> *attr):
AttributeHelperBase(),
attribute(attr)
{
	
}

//------------------------------------------------------------------------------
void AttributeHelper< vector<ThreeFloats> >::convert(const string& address,
							  			   	   		 const string& val)
{
	int tempint, tempint2;
	StringParser parse(val);

	tempint = getIndex(address);
	if(tempint > -1)
	{
		tempint2 = getSecondaryIndex(address);
		if((tempint2 > -1) && (parse.getNumNumbers() > 0))
		{
			switch(tempint)
			{
				case 0:
					(*attribute)[tempint2].x = parse.getFloat();
					break;
				case 1:
					(*attribute)[tempint2].y = parse.getFloat();
					break;
				case 2:
					(*attribute)[tempint2].z = parse.getFloat();
					break;
			}
		}
		else if((tempint > -1) && (parse.getNumThreeFloats() > 0))
			(*attribute)[tempint] = parse.getThreeFloats();
	}
}

//------------------------------------------------------------------------------
//------------------------------------------------------------------------------
AttributeHelper< vector<FourFloats> >::AttributeHelper(vector<FourFloats> *attr):
AttributeHelperBase(),
attribute(attr)
{
	
}

//------------------------------------------------------------------------------
void AttributeHelper< vector<FourFloats> >::convert(const string& address,
							  			   	   		 const string& val)
{
	int tempint, tempint2;
	StringParser parse(val);

	tempint = getIndex(address);
	if(tempint > -1)
	{
		tempint2 = getSecondaryIndex(address);
		if((tempint2 > -1) && (parse.getNumNumbers() > 0))
		{
			switch(tempint)
			{
				case 0:
					(*attribute)[tempint2].x = parse.getFloat();
					break;
				case 1:
					(*attribute)[tempint2].y = parse.getFloat();
					break;
				case 2:
					(*attribute)[tempint2].z = parse.getFloat();
					break;
				case 3:
					(*attribute)[tempint2].r = parse.getFloat();
					break;
			}
		}
		else if((tempint > -1) && (parse.getNumFourFloats() > 0))
			(*attribute)[tempint] = parse.getFourFloats();
	}
}

//------------------------------------------------------------------------------
//------------------------------------------------------------------------------
AttributeHelper< vector<TwoFloats> >::AttributeHelper(vector<TwoFloats> *attr):
AttributeHelperBase(),
attribute(attr)
{
	
}

//------------------------------------------------------------------------------
void AttributeHelper< vector<TwoFloats> >::convert(const string& address,
							  			   	   	   const string& val)
{
	int tempint, tempint2;
	StringParser parse(val);

	tempint = getIndex(address);
	if(tempint > -1)
	{
		tempint2 = getSecondaryIndex(address);
		if((tempint2 > -1) && (parse.getNumNumbers() > 0))
		{
			switch(tempint)
			{
				case 0:
					(*attribute)[tempint2].x = parse.getFloat();
					break;
				case 1:
					(*attribute)[tempint2].y = parse.getFloat();
					break;
			}
		}
		else if((tempint > -1) && (parse.getNumTwoFloats() > 0))
			(*attribute)[tempint] = parse.getTwoFloats();
	}
}
