//	ConditionalWait.cpp - Simple class for waiting on a condition variable.
//  ----------------------------------------------------------------------------
//	This file is part of 'Heilan Browser', an X3D browser specialised for audio
//	work.
//	Copyright (C) 2005-2009  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#include "ConditionalWait.h"
#include "FatalError.h"
#include "SDL.h"
#include <iostream>
#include <sstream>

using namespace std;

//------------------------------------------------------------------------------
ConditionalWait::ConditionalWait()
{
	//Set up mutex.
	mutex = SDL_CreateMutex();
	//SDL_mutexP(mutex);

	//Set up condition.
	condition = SDL_CreateCond();
}

//------------------------------------------------------------------------------
ConditionalWait::~ConditionalWait()
{
	SDL_DestroyCond(condition);
	SDL_DestroyMutex(mutex);
}

//------------------------------------------------------------------------------
void ConditionalWait::wait(unsigned int timeout)
{
	SDL_mutexP(mutex);
	if(!timeout)
		SDL_CondWait(condition, mutex);
	else
		SDL_CondWaitTimeout(condition, mutex, timeout);
	SDL_mutexV(mutex);
}

//------------------------------------------------------------------------------
void ConditionalWait::signal()
{
	SDL_mutexP(mutex);
	if(SDL_CondSignal(condition) == -1)
	{
		ostringstream err;
		err << "Could not signal ConditionalWait.  Assuming this is a fatal error...";
		throw FatalError(err.str());
	}
	SDL_mutexV(mutex);
}
