//	GLHeaders.cpp - Wraps our OpenGL headers to (hopefully) make dependencies a
//				    bit less of a hassle.
//  ----------------------------------------------------------------------------
//	This file is part of 'Heilan Browser', an X3D browser specialised for audio
//	work.
//	Copyright (C) 2005-2009  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#include "GLHeaders.h"

#include <iostream>
#include <sstream>
#include <fstream>
#include <cstring>

using namespace std;

namespace HeilanGL
{

//------------------------------------------------------------------------------
bool setupShader(GLhandleARB handle, const string& file)
{
	bool retval = false;

#ifndef GL_ARB_shader_objects
	if(glShaderSourceARB)
#endif
	{
		stringstream tempstr;
		ifstream phil(file.c_str());

		if(phil.is_open())
		{
			phil >> tempstr.rdbuf();

			char *tempch = new char[tempstr.str().length()+1];
			strcpy(tempch, tempstr.str().c_str());

			cout << endl << endl << file << " contents:" << endl << endl;
			cout << tempch << endl;

			glShaderSourceARB(handle, 1, (const GLcharARB **)(&tempch), NULL);
			delete tempch;

			retval = true;
		}
	}

	return retval;
}

//------------------------------------------------------------------------------
void displayInfoLog(GLhandleARB handle)
{
	GLint infologLength = 0;
	GLsizei charsWritten  = 0;
    char *infoLog;

	glGetObjectParameterivARB(handle,
							  GL_OBJECT_INFO_LOG_LENGTH_ARB,
                              &infologLength);

    if (infologLength > 0)
    {
        infoLog = new char[infologLength];
        glGetInfoLogARB(handle, infologLength, &charsWritten, infoLog);
		cout << endl << "InfoLog (" << handle << "):" << endl << endl;
		cout << infoLog;
		delete [] infoLog;
    }
}

}
