//	SFImage.cpp - Class representing an X3D SFImage type.
//  ----------------------------------------------------------------------------
//	This file is part of 'Heilan Browser', an X3D browser specialised for audio
//	work.
//	Copyright (C) 2005-2009  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#include "SFImage.h"

#include <iostream>

using namespace std;

namespace HeilanX3D
{

//------------------------------------------------------------------------------
SFImage::SFImage():
numComponents(0),
width(0),
height(0),
pixels(0)
{
	
}

//------------------------------------------------------------------------------
SFImage::~SFImage()
{
	if(pixels)
		delete [] pixels;
}

//------------------------------------------------------------------------------
bool SFImage::loadImage(const vector<int>& val)
{
	unsigned int i;
	unsigned int j;
	bool retval = false;

	if(val.size() > 3)
	{
		width = val[0];
		cout << "SFImage: width = " << width << endl;
		height = val[1];
		cout << "SFImage: height = " << height << endl;
		numComponents = val[2];
		cout << "SFImage: numComponents = " << numComponents << endl;

		if((val.size()-3) != (unsigned)(width*height))
		{
			numComponents = 0;
			width = 0;
			height = 0;

			cout << "This SFImage does not have enough pixel data for the ";
			cout << "specified width x height." << endl;
		}
		else
		{
			pixels = new unsigned char[width*height*numComponents];

			for(i=3,j=0;i<val.size();++i,++j)
			{
				cout << hex << val[i] << ":\t";
				switch(numComponents)
				{
					case 1:
						pixels[j] = val[i]; //Luminance only.
						cout << hex << (int)pixels[j] << " ";
						break;
					case 3:
						pixels[j] = (val[i]&0xFF0000)>>16; //Red.
						cout << hex << (int)pixels[j] << " ";
						++j;

						pixels[j] = (val[i]&0x00FF00)>>8; //Green.
						cout << hex << (int)pixels[j] << " ";
						++j;

						pixels[j] = val[i]&0x0000FF; //Blue.
						cout << hex << (int)pixels[j] << " ";
						break;
					case 4:
						pixels[j] = (val[i]&0xFF000000)>>24; //Red.
						cout << hex << (int)pixels[j] << " ";
						++j;
						pixels[j] = (val[i]&0x00FF0000)>>16; //Green.
						cout << hex << (int)pixels[j] << " ";
						++j;
						pixels[j] = (val[i]&0x0000FF00)>>8; //Blue.
						cout << hex << (int)pixels[j] << " ";
						++j;
						pixels[j] = val[i]&0x000000FF; //Alpha.
						cout << hex << (int)pixels[j] << " ";
						break;
				}
				cout << endl;
			}
			retval = true;
		}
	}
	else
	{
		cout << "This SFImage does not have enough data to be ";
		cout << "instantiated." << endl;
	}

	return retval;
}

}
