//	SFImage.h - Class representing an X3D SFImage type.
//  ----------------------------------------------------------------------------
//	This file is part of 'Heilan Browser', an X3D browser specialised for audio
//	work.
//	Copyright (C) 2005-2009  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#ifndef SFIMAGE_H_
#define SFIMAGE_H_

#include <vector>

namespace HeilanX3D
{

///	Class representing an X3D SFImage type.
/*!
	\todo Fix to work with big endian systems.
 */
class SFImage
{
  public:
	///	Constructor.
	SFImage();
	///	Destructor.
	~SFImage();

	///	Loads the image from a vector of integers.
	/*!
		\param val Vector containing integers for all the relevent values in the
		image.
		\return True if successful, false otherwise.

		What this does is read the image's width from the 1st entry in the
		vector, then its height from the 2nd, then the number of colour
		components from the 3rd, and finally the actually pixel values from the
		remaining entries.
	 */
	bool loadImage(const std::vector<int>& val);

	///	Returns the number of colour components used by the image.
	int getNumComponents() const {return numComponents;};
	///	Returns the width of the image.
	int getWidth() const {return width;};
	///	Returns the height of the image.
	int getHeight() const {return height;};
	///	Returns the pixels of the image as an unsigned char array/pointer.
	const unsigned char * const getPixels() {return pixels;};
  private:
	///	Number of colour components in the image.
	int numComponents;
	///	Width of the image.
	int width;
	///	Height of the image.
	int height;
	///	The image's pixels.
	unsigned char *pixels;
};

}

#endif
