//	ScopedMutex.h - Simple mutex class designed to be created on the stack.
//  ----------------------------------------------------------------------------
//	This file is part of 'Heilan Browser', an X3D browser specialised for audio
//	work.
//	Copyright (C) 2005-2009  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#ifndef SCOPEDMUTEX_H_
#define SCOPEDMUTEX_H_

#include "DllStuff.h"

struct SDL_mutex;

///	Simple mutex class designed to be created on the stack.
/*!
	i.e. to use it:
	\code
	void someFunction()
	{
		//Create mutex, lock it.
		ScopedMutex m;

		//Do stuff.
	}	//Mutex unlocked in it's destructor.
	\endcode
 */
class dllStuff ScopedMutex
{
  public:
	///	Constructor.
	/*!
		Creates the mutex and attempts to lock it.
	 */
	ScopedMutex();
	///	Destructor.
	/*!
		Unlocks the mutex and deletes it.
	 */
	~ScopedMutex();
  private:
	///	Pointer to the actual mutex.
	SDL_mutex *mutex;
};

#endif
