//	StringParser.cpp - Simple parser class to read from a string and output the
//					   digits from it.
//  ----------------------------------------------------------------------------
//	This file is part of 'Heilan Browser', an X3D browser specialised for audio
//	work.
//	Copyright (C) 2005-2009  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#include "StringParser.h"

#include <sstream>
#include <iostream>

using namespace std;

//------------------------------------------------------------------------------
StringParser::StringParser(const string& input)
{
	fillNumbers(input);
}

//------------------------------------------------------------------------------
StringParser::~StringParser()
{
	
}

//------------------------------------------------------------------------------
int StringParser::getNumTwoFloats() const
{
	int retval = 0;

	if((numbers.size() % 2) == 0)
		retval = numbers.size()/2;

	return retval;
}

//------------------------------------------------------------------------------
int StringParser::getNumThreeFloats() const
{
	int retval = 0;

	if((numbers.size() % 3) == 0)
		retval = numbers.size()/3;

	return retval;
}

//------------------------------------------------------------------------------
int StringParser::getNumFourFloats() const
{
	int retval = 0;

	if((numbers.size() % 4) == 0)
		retval = numbers.size()/4;

	return retval;
}

//------------------------------------------------------------------------------
int StringParser::getNumQuaternions() const
{
	int retval = 0;

	if((numbers.size() % 4) == 0)
		retval = numbers.size()/4;

	return retval;
}

//------------------------------------------------------------------------------
double StringParser::getDouble(int index) const
{
	double retval = 0.0;

	if(static_cast<unsigned int>(index) < numbers.size())
		retval = numbers[index];

	return retval;
}

//------------------------------------------------------------------------------
float StringParser::getFloat(int index) const
{
	float retval = 0.0f;

	if(static_cast<unsigned int>(index) < numbers.size())
		retval = static_cast<float>(numbers[index]);

	return retval;
}

//------------------------------------------------------------------------------
int StringParser::getInt(int index) const
{
	int retval = 0;

	if(static_cast<unsigned int>(index) < numbers.size())
		retval = static_cast<int>(numbers[index]);

	return retval;
}

//------------------------------------------------------------------------------
TwoFloats StringParser::getTwoFloats(int index) const
{
	TwoFloats retval;

	//Check that the string could have held a multiple of ThreeFloats first.
	if((numbers.size() % 2) == 0)
	{
		retval.x = numbers[index*2];
		retval.y = numbers[(index*2)+1];
	}

	return retval;
}

//------------------------------------------------------------------------------
ThreeFloats StringParser::getThreeFloats(int index) const
{
	ThreeFloats retval;

	//Check that the string could have held a multiple of ThreeFloats first.
	if((numbers.size() % 3) == 0)
	{
		retval.x = numbers[index*3];
		retval.y = numbers[(index*3)+1];
		retval.z = numbers[(index*3)+2];
	}

	return retval;
}

//------------------------------------------------------------------------------
FourFloats StringParser::getFourFloats(int index) const
{
	FourFloats retval;

	//Check that the string could have held a multiple of FourFloats first.
	if((numbers.size() % 4) == 0)
	{
		retval.r = numbers[index*4];
		retval.x = numbers[(index*4)+1];
		retval.y = numbers[(index*4)+2];
		retval.z = numbers[(index*4)+3];
	}

	return retval;
}

//------------------------------------------------------------------------------
Quaternion StringParser::getQuaternion(int index) const
{
	Quaternion retval;

	//Check that the string could have held a multiple of Quaternions first.
	if((numbers.size() % 4) == 0)
	{
		retval.x = numbers[index*4];
		retval.y = numbers[(index*4)+1];
		retval.z = numbers[(index*4)+2];
		retval.w = numbers[(index*4)+3];
	}

	return retval;
}

//------------------------------------------------------------------------------
bool StringParser::getBool(int index) const
{
	bool retval = false;

	if((unsigned)index < bools.size())
		retval = bools[index];

	return retval;
}

//------------------------------------------------------------------------------
void StringParser::fillNumbers(const string& input)
{
	double tempd;
	string tempstr;
	string tempstr2;
	stringstream tempConverter;
	int tempint = 0;
	int tempHex = -1;

	//First check if the string is empty.
	if(input.empty())
	{
		cout << "StringParser: string is empty." << endl;
		return;
	}
	else if((input.find_first_of(" \t\r\n") == 0) && (input.size() == 1))
	{
		cout << "StringParser: string is effectively empty." << endl;
		return;
	}

	//Check if it's a stream of bools rather than numbers.
	if((input.find_first_of("true") != string::npos) ||
	   (input.find_first_of("false") != string::npos) ||
	   (input.find_first_of("TRUE") != string::npos) ||
	   (input.find_first_of("FALSE") != string::npos))
	{
		fillBools(input);
		return;
	}

	//Next check if the string holds a single number.
	if(input.find_first_of(" \t\r\n") == string::npos) 
	{
		tempHex = getHex(input);

		if(tempHex == -1)
		{
			tempConverter << input;
			tempConverter >> tempd;
		}
		else
			tempd = tempHex;

		numbers.push_back(tempd);
	}
	else //There's a stream of numbers to be parsed.
	{
		if(input.find_first_of(" \t\r\n") != 0)
		{
			tempstr = input.substr(tempint, (input.find_first_of(" \t\r\n")));
			tempstr2 = input.substr(input.find_first_of(" \t\r\n")+1);
		}
		else
		{
			tempstr2 = input.substr(1);
			tempstr = tempstr2.substr(tempint, (tempstr2.find_first_of(" \t\r\n")));
			tempstr2 = tempstr2.substr(tempstr2.find_first_of(" \t\r\n")+1);
		}

		while(static_cast<unsigned int>(tempint) < input.size())
		{
			//To watch for multiple whitespace characters.
			while((tempstr.size() == 0)&&(static_cast<unsigned int>(tempint) < input.size()))
			{
				tempint += tempstr.size()+1;

				tempstr = tempstr2.substr(0, (tempstr2.find_first_of(" \t\r\n")));
				tempstr2 = tempstr2.substr(tempstr2.find_first_of(" \t\r\n")+1);
			}

			tempConverter.str("");
			tempHex = getHex(tempstr);
			if(tempHex == -1)
			{
				stringstream tC2;

				tC2 << tempstr;
				tC2 >> tempd;
			}
			else
				tempd = static_cast<double>(tempHex);

			numbers.push_back(tempd);

			tempint += tempstr.size()+1;

			tempstr = tempstr2.substr(0, (tempstr2.find_first_of(" \t\r\n")));
			tempstr2 = tempstr2.substr(tempstr2.find_first_of(" \t\r\n")+1);

			tempd = -1.0;
		}
	}
}

//------------------------------------------------------------------------------
void StringParser::fillBools(const std::string& input)
{
	string tempstr;
	string tempstr2;
	int tempint = 0;

	//Next check if the string holds a single bool.
	if(input.find_first_of(" \t\r\n") == string::npos) 
	{
		if((input == "true") || (input == "TRUE"))
			bools.push_back(true);
		else if((input == "false") || (input == "FALSE"))
			bools.push_back(false);
	}
	else //There's a stream of bools to be parsed.
	{
		if(input.find_first_of(" \t\r\n") != 0)
		{
			tempstr = input.substr(tempint, (input.find_first_of(" \t\r\n")));
			tempstr2 = input.substr(input.find_first_of(" \t\r\n")+1);
		}
		else
		{
			tempstr2 = input.substr(1);
			tempstr = tempstr2.substr(tempint, (tempstr2.find_first_of(" \t\r\n")));
			tempstr2 = tempstr2.substr(tempstr2.find_first_of(" \t\r\n")+1);
		}

		while(static_cast<unsigned int>(tempint) < input.size())
		{
			//To watch for multiple whitespace characters.
			while((tempstr.size() == 0)&&(static_cast<unsigned int>(tempint) < input.size()))
			{
				tempint += tempstr.size()+1;

				tempstr = tempstr2.substr(0, (tempstr2.find_first_of(" \t\r\n")));
				tempstr2 = tempstr2.substr(tempstr2.find_first_of(" \t\r\n")+1);
			}

			if((tempstr == "true") || (tempstr == "TRUE"))
				bools.push_back(true);
			else if((tempstr == "false") || (tempstr == "FALSE"))
				bools.push_back(false);

			tempint += tempstr.size()+1;

			tempstr = tempstr2.substr(0, (tempstr2.find_first_of(" \t\r\n")));
			tempstr2 = tempstr2.substr(tempstr2.find_first_of(" \t\r\n")+1);
		}
	}
}

//------------------------------------------------------------------------------
int StringParser::getHex(const std::string& val)
{
	int retval = -1;

	if(val.substr(0, 2) == "0x")
	{
		stringstream tempConverter;

		tempConverter << val;
		tempConverter >> hex >> retval;
	}

	return retval;
}
