//	Billboard.cpp - Class representing the Billboard parent node.
//  ----------------------------------------------------------------------------
//	This file is part of 'Heilan Browser', an X3D browser specialised for audio
//	work.
//	Copyright (C) 2005-2009  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#include "Billboard.h"
#include "StringParser.h"
#include "AttributeHelper.h"

#include "GLHeaders.h"
#include <iostream>

using namespace std;

namespace HeilanX3D
{

NodeConstructor<Billboard> Billboard::constructor;

//------------------------------------------------------------------------------
Billboard::Billboard():
ParentNode(),
axisOfRotation(0.0f, 1.0f, 0.0f)
{
	setType("Billboard");
	setVisual(true);
	setAural(true); //Just in case.
}

//------------------------------------------------------------------------------
Billboard::~Billboard()
{
	
}

//------------------------------------------------------------------------------
void Billboard::setAttribute(const string& attribute, EventValue& val)
{
	string realAtt = AttributeHelperBase::getAddress(attribute);

	if(realAtt == "axisOfRotation")
	{
		StringParser parser(val);
		AttributeHelper<ThreeFloats> helper(&axisOfRotation);

		if(helper.isValid(attribute))
			helper.convert(attribute, val);
		else if(parser.getNumThreeFloats() > 0)
			axisOfRotation = parser.getThreeFloats();
	}
}

//------------------------------------------------------------------------------
void Billboard::preChildDraw()
{
	float modelview[16];
	int i,j;

	//Setup matrix.
	glMatrixMode(GL_MODELVIEW);
	glPushMatrix();

	//Perform the rotation (set the modelview w/no rotations or scaling to fake
	//the billboarding).
	glGetFloatv(GL_MODELVIEW_MATRIX , modelview);
	for(i=0;i<3;++i) 
	{
		for(j=0;j<3;++j)
		{
			if(i==j)
				modelview[i*4+j] = 1.0f;
			else
				modelview[i*4+j] = 0.0f;
		}
	}
	glLoadMatrixf(modelview);
}

//------------------------------------------------------------------------------
void Billboard::postChildDraw()
{
	GLint tempint;

	//Reset matrix.
	glMatrixMode(GL_MODELVIEW);
	glPopMatrix();

	//Check if we should pop the texture matrix.
	//This is far from ideal, but I can't think of anywhere else to put it,
	//since TextureTransform is a child node...
	glGetIntegerv(GL_TEXTURE_STACK_DEPTH, &tempint);
	if(tempint > 1)
	{
		glMatrixMode(GL_TEXTURE);
		glPopMatrix();

		//Just in case...
		glMatrixMode(GL_MODELVIEW);
	}
}

}
