//	BooleanSequencer.h - Class representing the BooleanSequencer node.
//  ----------------------------------------------------------------------------
//	This file is part of 'Heilan Browser', an X3D browser specialised for audio
//	work.
//	Copyright (C) 2005-2009  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#ifndef BOOLEANSEQUENCER_H_
#define BOOLEANSEQUENCER_H_

#include "AbstractNode.h"

namespace HeilanX3D
{

///	Class representing the BooleanSequencer node.
class BooleanSequencer : public ChildNode
{
  public:
	///	Constructor.
	BooleanSequencer();
	///	Destructor.
	~BooleanSequencer();

	///	Over-ridden to do nothing.
	void graphicDoStuff() {};
	///	Over-ridden to do nothing.
	void setBlockSize(int newSize) {};

	///	Where the main work this class does happens.
	void setAttribute(const std::string& attribute, EventValue& val);

	static NodeConstructor<BooleanSequencer> constructor;
  private:
	///	Simple struct representing a key-keyValue pair for the sequencing.
	struct BooleanKey
	{
		///	The key value for this key.
		float key;
		///	The value for this key.
		bool value;
	};

	///	All the keys associated with this interpolator.
	std::vector<BooleanKey> keys;

	///	Our current position within the sequence.
	int position;
};

}

#endif
