//	Box.cpp - Class representing an X3D box node.
//  ----------------------------------------------------------------------------
//	This file is part of 'Heilan Browser', an X3D browser specialised for audio
//	work.
//	Copyright (C) 2005-2009  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#include "Box.h"
#include "StringParser.h"
#include "AttributeHelper.h"

#include <iostream>

using namespace std;

namespace HeilanX3D
{

NodeConstructor<Box> Box::constructor;

//------------------------------------------------------------------------------
Box::Box():
ChildNode(),
size(2.0f),
solid(true)
{
	setType("Box");
	setVisual(true);
}

//------------------------------------------------------------------------------
Box::~Box()
{
	
}

//------------------------------------------------------------------------------
void Box::initialise()
{
	/*if(!glMultiTexCoord2fARB)
		cout << "Box: No glMultiTexCoord2fARB extension!" << endl;
	else*/
	{
		//Setup displayList.
		displayList = glGenLists(1);
		glNewList(displayList, GL_COMPILE);
		{
			glBegin(GL_QUADS);
			{
				//Rear face.
				glNormal3f(0.0f, 0.0f, -1.0f);
				glMultiTexCoord2fARB(GL_TEXTURE0_ARB, 1.0f, 1.0f);
				glMultiTexCoord2fARB(GL_TEXTURE1_ARB, 1.0f, 1.0f);
				glVertex3f(-0.5f, -0.5f, -0.5f);
				glMultiTexCoord2fARB(GL_TEXTURE0_ARB, 1.0f, 0.0f);
				glMultiTexCoord2fARB(GL_TEXTURE1_ARB, 1.0f, 0.0f);
				glVertex3f(-0.5f,  0.5f, -0.5f);
				glMultiTexCoord2fARB(GL_TEXTURE0_ARB, 0.0f, 0.0f);
				glMultiTexCoord2fARB(GL_TEXTURE1_ARB, 0.0f, 0.0f);
				glVertex3f(0.5f,  0.5f, -0.5f);
				glMultiTexCoord2fARB(GL_TEXTURE0_ARB, 0.0f, 1.0f);
				glMultiTexCoord2fARB(GL_TEXTURE1_ARB, 0.0f, 1.0f);
				glVertex3f(0.5f, -0.5f, -0.5f);

				// Bottom Face
				glNormal3f(0.0f, -1.0f, 0.0f);
				glMultiTexCoord2fARB(GL_TEXTURE0_ARB, 1.0f, 0.0f);
				glMultiTexCoord2fARB(GL_TEXTURE1_ARB, 1.0f, 0.0f);
				glVertex3f(-0.5f, -0.5f, -0.5f);
				glMultiTexCoord2fARB(GL_TEXTURE0_ARB, 0.0f, 0.0f);
				glMultiTexCoord2fARB(GL_TEXTURE1_ARB, 0.0f, 0.0f);
				glVertex3f(0.5f, -0.5f, -0.5f);
				glMultiTexCoord2fARB(GL_TEXTURE0_ARB, 0.0f, 1.0f);
				glMultiTexCoord2fARB(GL_TEXTURE1_ARB, 0.0f, 1.0f);
				glVertex3f(0.5f, -0.5f, 0.5f);
				glMultiTexCoord2fARB(GL_TEXTURE0_ARB, 1.0f, 1.0f);
				glMultiTexCoord2fARB(GL_TEXTURE1_ARB, 1.0f, 1.0f);
				glVertex3f(-0.5f, -0.5f, 0.5f);

				// Top Face
				glNormal3f(0.0f, 1.0f, 0.0f);
				glMultiTexCoord2fARB(GL_TEXTURE0_ARB, 0.0f, 0.0f);
				glMultiTexCoord2fARB(GL_TEXTURE1_ARB, 0.0f, 0.0f);
				glVertex3f(-0.5f, 0.5f, -0.5f);
				glMultiTexCoord2fARB(GL_TEXTURE0_ARB, 0.0f, 1.0f);
				glMultiTexCoord2fARB(GL_TEXTURE1_ARB, 0.0f, 1.0f);
				glVertex3f(-0.5f, 0.5f, 0.5f);
				glMultiTexCoord2fARB(GL_TEXTURE0_ARB, 1.0f, 1.0f);
				glMultiTexCoord2fARB(GL_TEXTURE1_ARB, 1.0f, 1.0f);
				glVertex3f(0.5f, 0.5f, 0.5f);
				glMultiTexCoord2fARB(GL_TEXTURE0_ARB, 1.0f, 0.0f);
				glMultiTexCoord2fARB(GL_TEXTURE1_ARB, 1.0f, 0.0f);
				glVertex3f(0.5f, 0.5f, -0.5f);

				// Right face
				glNormal3f(1.0f, 0.0f, 0.0f);
				glMultiTexCoord2fARB(GL_TEXTURE0_ARB, 1.0f, 1.0f);
				glMultiTexCoord2fARB(GL_TEXTURE1_ARB, 1.0f, 1.0f);
				glVertex3f(0.5f, -0.5f, -0.5f);
				glMultiTexCoord2fARB(GL_TEXTURE0_ARB, 1.0f, 0.0f);
				glMultiTexCoord2fARB(GL_TEXTURE1_ARB, 1.0f, 0.0f);
				glVertex3f(0.5f, 0.5f, -0.5f);
				glMultiTexCoord2fARB(GL_TEXTURE0_ARB, 0.0f, 0.0f);
				glMultiTexCoord2fARB(GL_TEXTURE1_ARB, 0.0f, 0.0f);
				glVertex3f(0.5f, 0.5f, 0.5f);
				glMultiTexCoord2fARB(GL_TEXTURE0_ARB, 0.0f, 1.0f);
				glMultiTexCoord2fARB(GL_TEXTURE1_ARB, 0.0f, 1.0f);
				glVertex3f(0.5f, -0.5f, 0.5f);

				// Left Face
				glNormal3f(-1.0f, 0.0f, 0.0f);
				glMultiTexCoord2fARB(GL_TEXTURE0_ARB, 0.0f, 1.0f);
				glMultiTexCoord2fARB(GL_TEXTURE1_ARB, 0.0f, 1.0f);
				glVertex3f(-0.5f, -0.5f, -0.5f);
				glMultiTexCoord2fARB(GL_TEXTURE0_ARB, 1.0f, 1.0f);
				glMultiTexCoord2fARB(GL_TEXTURE1_ARB, 1.0f, 1.0f);
				glVertex3f(-0.5f, -0.5f, 0.5f);
				glMultiTexCoord2fARB(GL_TEXTURE0_ARB, 1.0f, 0.0f);
				glMultiTexCoord2fARB(GL_TEXTURE1_ARB, 1.0f, 0.0f);
				glVertex3f(-0.5f, 0.5f, 0.5f);
				glMultiTexCoord2fARB(GL_TEXTURE0_ARB, 0.0f, 0.0f);
				glMultiTexCoord2fARB(GL_TEXTURE1_ARB, 0.0f, 0.0f);
				glVertex3f(-0.5f, 0.5f, -0.5f);

				//Front face.
				glNormal3f(0.0f, 0.0f, 1.0f);
				glMultiTexCoord2fARB(GL_TEXTURE0_ARB, 0.0f, 1.0f);
				glMultiTexCoord2fARB(GL_TEXTURE1_ARB, 0.0f, 1.0f);
				glVertex3f(-0.5f, -0.5f, 0.5f);
				glMultiTexCoord2fARB(GL_TEXTURE0_ARB, 1.0f, 1.0f);
				glMultiTexCoord2fARB(GL_TEXTURE1_ARB, 1.0f, 1.0f);
				glVertex3f(0.5f, -0.5f, 0.5f);
				glMultiTexCoord2fARB(GL_TEXTURE0_ARB, 1.0f, 0.0f);
				glMultiTexCoord2fARB(GL_TEXTURE1_ARB, 1.0f, 0.0f);
				glVertex3f(0.5f, 0.5f, 0.5f);
				glMultiTexCoord2fARB(GL_TEXTURE0_ARB, 0.0f, 0.0f);
				glMultiTexCoord2fARB(GL_TEXTURE1_ARB, 0.0f, 0.0f);
				glVertex3f(-0.5f, 0.5f, 0.5f);
			}
			glEnd();
		}
		glEndList();
	}
}

//------------------------------------------------------------------------------
void Box::graphicDoStuff()
{
	glMatrixMode(GL_MODELVIEW);
	glPushMatrix();

	//glRotatef(thing, 1.0f, 1.0f, 1.0f);
	//thing += 0.1f;
	glScalef(size.x, size.y, size.z);

	glCallList(displayList);

	glPopMatrix();
}

//------------------------------------------------------------------------------
void Box::setAttribute(const std::string& attribute, EventValue& val)
{
	string realAtt = AttributeHelperBase::getAddress(attribute);

	if(realAtt == "size")
	{
		StringParser parser(val);
		AttributeHelper<ThreeFloats> helper(&size);

		if(helper.isValid(attribute))
			helper.convert(attribute, val);
		else if(parser.getNumNumbers() > 0)
			size = parser.getThreeFloats();
	}
	else if(attribute == "solid")
	{
		StringParser parse(val);

		if(parse.getNumBools() > 0)
			solid = parse.getBool();
	}
}

}
