//	Box.h - Class representing an X3D box node.
//  ----------------------------------------------------------------------------
//	This file is part of 'Heilan Browser', an X3D browser specialised for audio
//	work.
//	Copyright (C) 2005-2009  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#ifndef BOX_H_
#define BOX_H_

#include "AbstractNode.h"

#include "GLHeaders.h"

namespace HeilanX3D
{

///	Class representing an X3D box node.
class Box : public ChildNode
{
  public:
	///	Constructor.
	Box();
	///	Destructor.
	~Box();

	///	Initialises various bits and pieces.
	void initialise();

	///	Draws the Box.
	void graphicDoStuff();
	///	Not used, overridden to do nothing.
	/*AmbisonicData *audioDoStuff(int numSamples,
								const ThreeFloats& pos) {return 0;};*/

	///	Not used, overridden to do nothing.
	void setBlockSize(int newSize) {};

	///	Used to set the size and solidity of the box.
	void setAttribute(const std::string& attribute, EventValue& val);

	static NodeConstructor<Box> constructor;
  private:
	///	The size of the box.
	ThreeFloats size;
	///	The solidity of the box.
	bool solid;

	///	Display list used to simplify graphicDoStuff().
	GLuint displayList;
};

}

#endif
