//	ColorRGBA.cpp - Class representing a colour (RGBA) node.
//  ----------------------------------------------------------------------------
//	This file is part of 'Heilan Browser', an X3D browser specialised for audio
//	work.
//	Copyright (C) 2005-2009  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#include "ColorRGBA.h"
#include "StringParser.h"
#include "AttributeHelper.h"

#include <iostream>

using namespace std;

namespace HeilanX3D
{
	
NodeConstructor<ColorRGBA> ColorRGBA::constructor;

//------------------------------------------------------------------------------
ColorRGBA::ColorRGBA():
ChildNode(),
valueChanged(false)
{
	setType("ColorRGBA");
}

//------------------------------------------------------------------------------
ColorRGBA::~ColorRGBA()
{
	
}

//------------------------------------------------------------------------------
void ColorRGBA::setAttribute(const string& attribute, EventValue& val)
{
	int i;
	FourFloats tempCol;
	string realAtt = AttributeHelperBase::getAddress(attribute);

	if(realAtt == "color")
	{
		StringParser parser(val);
		AttributeHelper< vector<FourFloats> > helper(&colours);

		if(helper.isValid(attribute))
		{
			helper.convert(attribute, val);
			valueChanged = true;
		}
		else
		{
			if(colours.size() > 0)
				colours.clear();

			for(i=0;i<parser.getNumFourFloats();++i)
			{
				//Because otherwise they'll be in the wrong order.
				tempCol.x = parser.getFourFloats(i).r;
				tempCol.y = parser.getFourFloats(i).x;
				tempCol.z = parser.getFourFloats(i).y;
				tempCol.r = parser.getFourFloats(i).z;

				colours.push_back(tempCol);
			}

			valueChanged = true;
		}
	}
}

//------------------------------------------------------------------------------
bool ColorRGBA::getHasChanged()
{
	bool retval = valueChanged;

	if(valueChanged)
		valueChanged = false;

	return retval;
}

}
