//	Coordinate.cpp - Class representing a collection of 3d points.
//  ----------------------------------------------------------------------------
//	This file is part of 'Heilan Browser', an X3D browser specialised for audio
//	work.
//	Copyright (C) 2005-2009  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#include "Coordinate.h"
#include "StringParser.h"
#include "AttributeHelper.h"

#include <iostream>

using namespace std;

namespace HeilanX3D
{
	
NodeConstructor<Coordinate> Coordinate::constructor;

//------------------------------------------------------------------------------
Coordinate::Coordinate():
ChildNode(),
valueChanged(false)
{
	setType("Coordinate");
}

//------------------------------------------------------------------------------
Coordinate::~Coordinate()
{
	
}

//------------------------------------------------------------------------------
void Coordinate::setAttribute(const string& attribute, EventValue& val)
{
	int i;
	string realAtt = AttributeHelperBase::getAddress(attribute);

	//cout << "setAttribute(" << attribute << ", " << val << ");" << endl;

	if(realAtt == "point")
	{
		StringParser parser(val);
		AttributeHelper< vector<ThreeFloats> > helper(&points);

		if(helper.isValid(attribute))
		{
			helper.convert(attribute, val);
			valueChanged = true;
		}
		else
		{
			if(points.size() > 0)
				points.clear();
	
			for(i=0;i<parser.getNumThreeFloats();++i)
			{
				points.push_back(parser.getThreeFloats(i));
				/*cout << points[points.size()-1].x;
				cout << "\t" << points[points.size()-1].y;
				cout << "\t" << points[points.size()-1].z << endl;*/
			}
	
			valueChanged = true;
		}
	}
}

//------------------------------------------------------------------------------
bool Coordinate::getHasChanged()
{
	bool retval = valueChanged;

	if(valueChanged)
		valueChanged = false;

	return retval;
}

}
