//	Coordinate.h - Class representing a collection of 3d points.
//  ----------------------------------------------------------------------------
//	This file is part of 'Heilan Browser', an X3D browser specialised for audio
//	work.
//	Copyright (C) 2005-2009  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#ifndef COORDINATE_H_
#define COORDINATE_H_

#include "AbstractNode.h"

#include <vector>

namespace HeilanX3D
{

///	Class representing a collection of 3d points.
class dllStuff Coordinate : public AbstractNode
{
  public:
	///	Constructor.
	Coordinate();
	///	Destructor.
	~Coordinate();

	///	Overridden to do nothing.
	void graphicDoStuff() {};
	///	Overridden to do nothing.
	//AmbisonicData *audioDoStuff(int numSamples, const ThreeFloats& pos) {return 0;};
	///	Overridden to do nothing.
	void setBlockSize(int newSize) {};

	///	Called to set the Coordinate's points.
	void setAttribute(const std::string& attribute, EventValue& val);

	///	Returns true if setAttribute has been called prior to this call.
	/*!
		When this is called, the current value of valueChanged is returned, and
		if it is true, valueChanged is reset to false.
	 */
	bool getHasChanged();
	///	Returns the points of this Coordinate.
	const std::vector<ThreeFloats>& getPoints() const {return points;};

	static NodeConstructor<Coordinate> constructor;
  private:
	///	Whether or not the ParentNode needs to update it's points.
	bool valueChanged;

	///	This Coordinate's points.
	std::vector<ThreeFloats> points;
};

}

#endif
