//	CoordinateInterpolator.cpp - Node used to interpolate between sets of
//								 coordinates.
//  ----------------------------------------------------------------------------
//	This file is part of 'Heilan Browser', an X3D browser specialised for audio
//	work.
//	Copyright (C) 2005-2009  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#include "CoordinateInterpolator.h"
#include "StringParser.h"

#include <iostream>
#include <sstream>

using namespace std;

namespace HeilanX3D
{

NodeConstructor<CoordinateInterpolator> CoordinateInterpolator::constructor;

//------------------------------------------------------------------------------
CoordinateInterpolator::CoordinateInterpolator():
ChildNode()
{
	setType("CoordinateInterpolator");
}

//------------------------------------------------------------------------------
CoordinateInterpolator::~CoordinateInterpolator()
{
	
}

//------------------------------------------------------------------------------
void CoordinateInterpolator::setAttribute(const string& attribute,
										  EventValue& val)
{
	unsigned int i, j;
	float tempf, tempf2;
	CoordinateKey tempKey;
	ThreeFloats tempCoord;
	stringstream tempConverter;

	if(attribute == "key")
	{
		StringParser parser(val);

		if(keys.size() <= 0)
		{
			for(i=0;i<(unsigned int)parser.getNumNumbers();++i)
			{
				tempKey.key = parser.getFloat(i);
				keys.push_back(tempKey);
			}
		}
		else
		{
			for(i=0;((i<keys.size()) && (i<(unsigned)parser.getNumNumbers()));++i)
				keys[i].key = parser.getFloat(i);
		}
	}
	else if(attribute == "keyValue")
	{
		StringParser parser(val);

		if(keys.size() <= 0)
		{
			cout << "CoordinateInterpolator Error: Trying to load key before ";
			cout << "keyValue.  Ignoring all values." << endl;
		}
		else
		{
			unsigned int numKeys = keys.size();

			if((parser.getNumThreeFloats() % numKeys) == 0)
			{
				unsigned int numKeyValues = parser.getNumThreeFloats()/numKeys;

				for(i=0;i<numKeys;++i)
				{
					for(j=0;j<numKeyValues;++j)
						keys[i].coord.push_back(parser.getThreeFloats((i*numKeyValues)+j));
				}
			}
			else
			{
				cout <<	"CoordinateInterpolator Error: Number of keyValues do ";
				cout << "not match number of keys.  Ignoring values." << endl;
			}
		}
	}
	else if(attribute == "set_fraction")
	{
		tempf = val;

		//Evaluate the interpolator at fraction tempf.
		for(i=0;i<(keys.size()-1);++i)
		{
			if((tempf >= keys[i].key) && (tempf < keys[i+1].key))
			{
				tempf2 = keys[i+1].key - keys[i].key;
				tempf2 = (tempf-keys[i].key)/tempf2;

				for(j=0;j<keys[i].coord.size();++j)
				{
					tempCoord.x = (keys[i].coord[j].x * (1.0f-tempf2)) +
								  (keys[i+1].coord[j].x * tempf2);
					tempCoord.y = (keys[i].coord[j].y * (1.0f-tempf2)) +
								  (keys[i+1].coord[j].y * tempf2);
					tempCoord.z = (keys[i].coord[j].z * (1.0f-tempf2)) +
								  (keys[i+1].coord[j].z * tempf2);
	
					tempConverter << tempCoord.x << " ";
					tempConverter << tempCoord.y << " ";
					tempConverter << tempCoord.z << " ";
				}
				sendEvent("value_changed", tempConverter.str());
				break;
			}
		}
	}
}

}
