//	CoordinateInterpolator.h - Node used to interpolate between sets of
//							   coordinates.
//  ----------------------------------------------------------------------------
//	This file is part of 'Heilan Browser', an X3D browser specialised for audio
//	work.
//	Copyright (C) 2005-2009  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#ifndef COORDINATEINTERPOLATOR_H_
#define COORDINATEINTERPOLATOR_H_

#include "AbstractNode.h"

#include <vector>

namespace HeilanX3D
{

///	Node used to interpolate between sets of coordinates.
class CoordinateInterpolator : public ChildNode
{
  public:
	///	Constructor.
	CoordinateInterpolator();
	///	Destructor.
	~CoordinateInterpolator();

	///	Overridden to do nothing.
	void graphicDoStuff() {};
	///	Overridden to do nothing.
	/*AmbisonicData *audioDoStuff(int numSamples, const
								ThreeFloats& pos) {return 0;}*/

	///	Overridden to do nothing.
	void setBlockSize(int newSize) {};

	///	Called to set the value of the named attribute.
	/*!
		This is where all of this node's operations occur - it exists solely
		to send Events on receipt of other Events.
	 */
	void setAttribute(const std::string& attribute, EventValue& val);

	static NodeConstructor<CoordinateInterpolator> constructor;
  private:
	///	Simple struct representing a key-keyValue pair for the interpolation.
	struct CoordinateKey
	{
		///	The key value for this key.
		float key;
		///	The coordinates for this key.
		std::vector<ThreeFloats> coord;
	};

	///	All the coordinate keys associated with this interpolator.
	std::vector<CoordinateKey> keys;
};

}

#endif
