//	DirectionalLight.cpp - Class representing an X3D box node.
//  ----------------------------------------------------------------------------
//	This file is part of 'Heilan Browser', an X3D browser specialised for audio
//	work.
//	Copyright (C) 2005-2009  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#include "DirectionalLight.h"
#include "StringParser.h"
#include "AttributeHelper.h"
#include "Lighting.h"

using namespace std;

namespace HeilanX3D
{

NodeConstructor<DirectionalLight> DirectionalLight::constructor;

//------------------------------------------------------------------------------
DirectionalLight::DirectionalLight():
ambientIntensity(0.0f),
color(1.0f, 1.0f, 1.0f),
global(false),
intensity(1.0f),
on(true)
{
	setType("DirectionalLight");
	setVisual(true);
	setShouldBeStart(true);

	direction.z = 1.0f;
}

//------------------------------------------------------------------------------
DirectionalLight::~DirectionalLight()
{
	
}

//------------------------------------------------------------------------------
void DirectionalLight::graphicDoStuff()
{
	if(on && !global)
	{
		ThreeFloats colour = color * intensity;

		Lighting::getInstance().addDirectionalLight(direction,
													colour,
													ambientIntensity);
	}
}

//------------------------------------------------------------------------------
void DirectionalLight::setAttribute(const string& attribute, EventValue& val)
{
	string realAtt = AttributeHelperBase::getAddress(attribute);

	if(realAtt == "ambientIntensity")
		ambientIntensity = val;
	else if(realAtt == "color")
	{
		StringParser parser(val);
		AttributeHelper<ThreeFloats> helper(&color);

		if(helper.isValid(attribute))
			helper.convert(attribute, val);
		else if(parser.getNumNumbers() > 0)
			color = parser.getThreeFloats();
	}
	else if(realAtt == "direction")
	{
		StringParser parser(val);
		AttributeHelper<ThreeFloats> helper(&direction);

		if(helper.isValid(attribute))
			helper.convert(attribute, val);
		else if(parser.getNumNumbers() > 0)
			direction = parser.getThreeFloats();

		//Because opengl's coordinate system seems to have the z-axis going in
		//the opposite direction X3Ds (unless I've misunderstood something).
		direction.z = -direction.z;
	}
	else if(realAtt == "global")
	{
		StringParser parse(val);

		if(parse.getNumBools() > 0)
		{
			global = parse.getBool();
	
			if(global)
			{
				ThreeFloats colour = color * intensity;
	
				Lighting::getInstance().addGlobalDirectionalLight(direction,
																  colour,
																  ambientIntensity);
			}
		}
	}
	else if(realAtt == "intensity")
		intensity = val;
	else if(realAtt == "on")
	{
		StringParser parse(val);

		if(parse.getNumBools() > 0)
			on = parse.getBool();
	}
}
	
}
