//	Fog.cpp - Node providing global fog.
//  ----------------------------------------------------------------------------
//	This file is part of 'Heilan Browser', an X3D browser specialised for audio
//	work.
//	Copyright (C) 2005-2009  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#include "Fog.h"
#include "StringParser.h"

#include "GLHeaders.h"

using namespace std;

namespace HeilanX3D
{

NodeConstructor<Fog> Fog::constructor;

//------------------------------------------------------------------------------
Fog::Fog():
BindableNode(),
color(1.0f, 1.0f, 1.0f),
fogType("LINEAR"),
visibilityRange(1.0f)
{
	setType("Fog");
	setVisual(true);
}

//------------------------------------------------------------------------------
Fog::~Fog()
{
	BindableNodeSingleton::getInstance()->unregisterNode(this);
}

//------------------------------------------------------------------------------
void Fog::initialise()
{
	//Register with the BindableNodeSingleton.
	BindableNodeSingleton::getInstance()->registerNode(this);
}

//------------------------------------------------------------------------------
void Fog::graphicDoStuff()
{
	float tempCol[4];
	float thing;

	if(fogType == "LINEAR")
		glFogi(GL_FOG_MODE, GL_LINEAR);
	else
		glFogi(GL_FOG_MODE, GL_EXP);
	tempCol[0] = color.x;
	tempCol[1] = color.y;
	tempCol[2] = color.z;
	tempCol[3] = 1.0f;
	glFogfv(GL_FOG_COLOR, tempCol);
	glFogf(GL_FOG_START, visibilityRange);
	if(visibilityRange < 10.0f)
		thing = 10.0f;
	else
		thing = visibilityRange * 0.5f;
	glFogf(GL_FOG_END, visibilityRange + thing);

	if(visibilityRange > 0.0f)
		glEnable(GL_FOG);
	else
		glDisable(GL_FOG);
}

//------------------------------------------------------------------------------
void Fog::setAttribute(const string& attribute, EventValue& val)
{
	if(attribute == "color")
	{
		StringParser parse(val);

		if(parse.getNumThreeFloats() > 0)
			color = parse.getThreeFloats();
	}
	else if(attribute == "fogType")
		fogType = (string)val;
	else if(attribute == "visibilityRange")
		visibilityRange = val;
}

}
