//	Group.cpp - Class representing the Group parent node.
//  ----------------------------------------------------------------------------
//	This file is part of 'Heilan Browser', an X3D browser specialised for audio
//	work.
//	Copyright (C) 2005-2009  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#include "Group.h"

#include "GLHeaders.h"

namespace HeilanX3D
{

NodeConstructor<Group> Group::constructor;

//------------------------------------------------------------------------------
Group::Group():
ParentNode()
{
	setType("Group");
	setVisual(true);
	setAural(true);
}

//------------------------------------------------------------------------------
Group::~Group()
{
	
}

//------------------------------------------------------------------------------
void Group::preChildDraw()
{
	glMatrixMode(GL_MODELVIEW);
	glPushMatrix();
}

//------------------------------------------------------------------------------
void Group::postChildDraw()
{
	GLint tempint;

	//Reset matrix.
	glMatrixMode(GL_MODELVIEW);
	glPopMatrix();

	//Check if we should pop the texture matrix.
	//This is far from ideal, but I can't think of anywhere else to put it,
	//since TextureTransform is a child node...
	glGetIntegerv(GL_TEXTURE_STACK_DEPTH, &tempint);
	if(tempint > 1)
	{
		glMatrixMode(GL_TEXTURE);
		glPopMatrix();

		//Just in case...
		glMatrixMode(GL_MODELVIEW);
	}
}

}
