//	ImageTexture.cpp - Class representing a basic texture.
//  ----------------------------------------------------------------------------
//	This file is part of 'Heilan Browser', an X3D browser specialised for audio
//	work.
//	Copyright (C) 2005-2009  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#include "ImageTexture.h"
#include "FileFinder.h"
#include "StringParser.h"
#include "MFString.h"

#include "SDL_image.h"
#include <iostream>

using namespace std;

namespace HeilanX3D
{

NodeConstructor<ImageTexture> ImageTexture::constructor;

//------------------------------------------------------------------------------
ImageTexture::ImageTexture():
ChildNode(),
repeatS(true),
repeatT(true)
{
	setType("ImageTexture");
	setVisual(true);
}

//------------------------------------------------------------------------------
ImageTexture::~ImageTexture()
{
	
}

//------------------------------------------------------------------------------
void ImageTexture::graphicDoStuff()
{
	glBindTexture(GL_TEXTURE_2D, texture);
}

//------------------------------------------------------------------------------
void ImageTexture::setAttribute(const string& attribute, EventValue& val)
{
	if(attribute == "url")
	{
		url = (string)val;
		loadTexture();
	}
	else if(attribute == "repeatS")
	{
		StringParser parse(val);

		if(parse.getNumBools() > 0)
		{
			repeatS = parse.getBool();

			if(repeatS)
			{
				glBindTexture(GL_TEXTURE_2D, texture);
				if(repeatS)
					glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
				else
					glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
				glBindTexture(GL_TEXTURE_2D, 0);
			}
		}
	}
	else if(attribute == "repeatT")
	{
		StringParser parse(val);

		if(parse.getNumBools() > 0)
		{
			repeatT = parse.getBool();

			if(repeatT)
			{
				glBindTexture(GL_TEXTURE_2D, texture);
				if(repeatT)
					glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
				else
					glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
				glBindTexture(GL_TEXTURE_2D, 0);
			}
		}
	}
}

//------------------------------------------------------------------------------
void ImageTexture::loadTexture()
{
	string tempPath;
	SDL_Surface *tempTex = 0;
	SDL_Surface *tempTex2 = 0;
	SDL_Surface *tempTex3 = 0;

	tempPath = FileFinder::getInstance().findFile(url);
	if(tempPath != "")
	{
		glMatrixMode(GL_MODELVIEW);
		glLoadIdentity();

		//This is the easiest way I could find to create a sensible SDL_PixelFormat
		//for SDL_ConvertSurface().
#ifdef __BIG_ENDIAN__
		tempTex3 = SDL_CreateRGBSurface(SDL_SWSURFACE,
										1,
										1,
										32,
										0xff000000,
										0x00ff0000,
										0x0000ff00,
										0x000000ff);
#else
		tempTex3 = SDL_CreateRGBSurface(SDL_SWSURFACE,
										1,
										1,
										32,
										0x000000ff,
										0x0000ff00,
										0x00ff0000,
										0xff000000);
#endif

		if((tempTex = IMG_Load(tempPath.c_str())))
		{
			//We first convert the image into a specific format, before we call
			//glTexImage2D.
			if((tempTex2 = SDL_ConvertSurface(tempTex,
				tempTex3->format,
				SDL_SWSURFACE)))
			{
				glGenTextures(1, &texture);

				glBindTexture(GL_TEXTURE_2D, texture);

				glTexImage2D(GL_TEXTURE_2D,
							 0,
							 GL_RGBA,
							 tempTex2->w,
							 tempTex2->h,
							 0,
							 GL_RGBA,
							 GL_UNSIGNED_BYTE,
							 tempTex2->pixels);

				glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
				glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
				if(repeatS)
					glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
				else
					glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
				if(repeatT)
					glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
				else
					glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);

				SDL_FreeSurface(tempTex2);
				SDL_FreeSurface(tempTex);
				glBindTexture(GL_TEXTURE_2D, 0);
			}
			else
			{
				SDL_FreeSurface(tempTex);
				cout << "ImageTexture: Could not load image: " << tempPath << endl;
			}
		}
		else
			cout << "ImageTexture: Could not load image: " << tempPath << endl;

		SDL_FreeSurface(tempTex3);
	}
	else
		cout << "ImageTexture: " << url << " does not seem to exist?" << endl;
}

}
