//	ImageTexture.h - Class representing a basic texture.
//  ----------------------------------------------------------------------------
//	This file is part of 'Heilan Browser', an X3D browser specialised for audio
//	work.
//	Copyright (C) 2005-2009  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#ifndef IMAGETEXTURE_H_
#define IMAGETEXTURE_H_

#include "AbstractNode.h"

#include "GLHeaders.h"

namespace HeilanX3D
{

///	Class representing a basic texture.
class ImageTexture : public ChildNode
{
  public:
	///	Constructor.
	ImageTexture();
	///	Destructor.
	~ImageTexture();

	///	Draws the ImageTexture.
	void graphicDoStuff();
	///	Not used, overridden to do nothing.
	/*AmbisonicData *audioDoStuff(int numSamples,
								const ThreeFloats& pos) {return 0;};*/

	///	Not used, overridden to do nothing.
	void setBlockSize(int newSize) {};

	///	Used to set the material's attributes.
	void setAttribute(const std::string& attribute, EventValue& val);

	static NodeConstructor<ImageTexture> constructor;
  private:
	///	Tries to load the texture in url.
	void loadTexture();

	///	The url to the image file.
	std::string url;
	///	Whether the texture should repeat in the S direction.
	bool repeatS;
	///	Whether the texture should repeat in the T direction.
	bool repeatT;

	///	The texture we use.
	GLuint texture;
};

}

#endif
