//	IndexedLineSet.h - Class representing an X3D indexed line set.
//  ----------------------------------------------------------------------------
//	This file is part of 'Heilan Browser', an X3D browser specialised for audio
//	work.
//	Copyright (C) 2005-2009  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#ifndef INDEXEDLINESET_H_
#define INDEXEDLINESET_H_

#include "ParentNode.h"
#include "Coordinate.h"
#include "Color.h"
#include "ColorRGBA.h"
#include "CriticalSection.h"

#include "GLHeaders.h"

namespace HeilanX3D
{

///	Class representing an X3D indexed line set.
class IndexedLineSet : public ParentNode
{
  public:
	///	Constructor.
	IndexedLineSet();
	///	Destructor.
	~IndexedLineSet();

	///	Sets the lines' attributes.
	void setAttribute(const std::string& attribute, EventValue& val);

	static NodeConstructor<IndexedLineSet> constructor;
  private:
	///	Draws the curve.
	void postChildDraw();

	///	Overridden to do nothing.
	void audioBlockSizeChanged(int newSize) {};

	///	Used to get the coordinates of the set from a child Coordinate node.
	void childrenLoaded();

	///	The indices of the colours used from colors/colorsRGBA.
	std::vector<int> colorIndex;
	///	Whether colour should be applied per vertex, or per face.
	bool colorPerVertex;
	///	The indices of the vertices used from coordinates.
	std::vector<int> coordIndex;

	///	Pointer to the Coordinate node we use to draw the object.
	Coordinate *coordinates;
	///	Pointer to the Color node we may use to colour the object.
	Color *colors;
	///	Pointer to the ColorRGBA node may we use to colour the object.
	ColorRGBA *colorsRGBA;

	///	A stored copy of coordinates' coords used to draw the object.
	std::vector<ThreeFloats> drawingCoords;
	///	A stored copy of colors' or colorsRGBA's colours used to draw the object.
	std::vector<FourFloats> drawingColours;

	///	Critical section used to guard coordinates while we're drawing.
	CriticalSection critSec;
};

}

#endif
