//	Inline.cpp - Class representing the Inline node.
//  ----------------------------------------------------------------------------
//	This file is part of 'Heilan Browser', an X3D browser specialised for audio
//	work.
//	Copyright (C) 2005-2009  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#include "Inline.h"
#include "Scene.h"
#include "X3DFileLoader.h"

#include <iostream>

using namespace std;

namespace HeilanX3D
{

NodeConstructor<Inline> Inline::constructor;

//------------------------------------------------------------------------------
Inline::Inline():
ChildNode()
{
	setType("Inline");
}

//------------------------------------------------------------------------------
Inline::~Inline()
{
	
}

//------------------------------------------------------------------------------
void Inline::setAttribute(const string& attribute, EventValue& val)
{
	if((attribute == "url") && ((string)val != ""))
	{
		bool reload = false;
		X3DFileLoader phil(val);

		if(phil.isValidX3DFile())
		{
			Scene *tempScene = 0;
			AbstractNode *rootNode = phil.loadScene();

			//Need to remove old children before we load the new ones.
			if(children.size() > 0)
			{
				//How to do this in a thread-safe way?
				reload = true;
			}

			tempScene = dynamic_cast<Scene *>(rootNode);

			if(tempScene)
			{
				unsigned int i;
				vector<ParentNode *> &parents = getParent();

				//Setup EXPORTs and optionally IMPORTS.
				phil.setupEXPORTNames(getDefName());
				if(reload)
					phil.handleIMPORTS();

				//Store children, remove tempScene from their list of parents.
				for(i=0;i<(unsigned)tempScene->getNumChildren();++i)
				{
					children.push_back(tempScene->getChild(i));
					tempScene->getChild(i)->removeParent(tempScene);
				}

				//Clear tempScene's children, don't delete them.
				tempScene->clear(true);
				//delete tempScene;
				tempScene->decRefCount(); //?

				//Add children to our ParentNode.
				if(parents.size() > 0)
				{
					for(i=0;i<children.size();++i)
					{
						parents[0]->addNode(children[i]);
						children[i]->addParent(parents[0]);
					}
				}

				cout << "Inline: Loaded " << (string)val << " file." << endl;
			}
		}
		else
			cout << "Inline: " << (string)val << " is not a valid X3D file." << endl;
	}
}

}
