//	Inline.h - Class representing the Inline node.
//  ----------------------------------------------------------------------------
//	This file is part of 'Heilan Browser', an X3D browser specialised for audio
//	work.
//	Copyright (C) 2005-2009  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#ifndef INLINE_H_
#define INLINE_H_

#include "AbstractNode.h"

namespace HeilanX3D
{

///	Class representing the Inline base node in all X3D files.
/*!
	\todo Implement IMPORT/EXPORT support.
	\todo Implement dynamic X3D file loading.

	\todo Implement loading from http/ftp files.
	\todo Implement load attribute when we implement Interactive Profile.
 */
class Inline : public ChildNode
{
  public:
	///	Constructor.
	Inline();
	///	Destructor.
	~Inline();

	///	Overridden to do nothing.
	void graphicDoStuff() {};

	///	Overridden to do nothing.
	void setBlockSize(int newSize) {};

	///	Where we get the url of the file to inline.
	void setAttribute(const std::string& attribute, EventValue& val);

	static NodeConstructor<Inline> constructor;
  private:
	///	The url to load.
	std::string url;

	///	Child nodes loaded from url.
	/*!
		We need to keep track of them because url might be changed dynamically,
		and if that happens, we need to delete the old children and load the
		new ones from the new url.
	 */
	std::vector<AbstractNode *> children;
};

}

#endif
