//	IntegerSequencer.cpp - Class representing the IntegerSequencer node.
//  ----------------------------------------------------------------------------
//	This file is part of 'Heilan Browser', an X3D browser specialised for audio
//	work.
//	Copyright (C) 2005-2009  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#include "IntegerSequencer.h"
#include "StringParser.h"

#include <sstream>
#include <iostream>

using namespace std;

namespace HeilanX3D
{

NodeConstructor<IntegerSequencer> IntegerSequencer::constructor;

//------------------------------------------------------------------------------
IntegerSequencer::IntegerSequencer():
position(0)
{
	setType("IntegerSequencer");
}

//------------------------------------------------------------------------------
IntegerSequencer::~IntegerSequencer()
{
	
}

//------------------------------------------------------------------------------
void IntegerSequencer::setAttribute(const string& attribute, EventValue& val)
{
	unsigned int i;
	float tempf;
	int tempint;
	IntegerKey tempKey;

	if(attribute == "key")
	{
		StringParser parser(val);

		//This is to handle the (hopefully) rare case that X3DFileLoader would
		//read keyValue before it reads key.
		if(keys.size() <= 0)
		{
			for(i=0;i<(unsigned int)parser.getNumNumbers();++i)
			{
				tempKey.key = parser.getFloat(i);
				tempKey.value = 0;

				keys.push_back(tempKey);
			}
		}
		else //keys is already populated with PositionKeys.
		{
			for(i=0;i<keys.size();++i)
				keys[i].key = parser.getFloat(i);
		}
	}
	else if(attribute == "keyValue")
	{
		StringParser parser(val);

		//This is to handle the (hopefully) rare case that X3DFileLoader would
		//read keyValue before it reads key.
		if(keys.size() <= 0)
		{
			for(i=0;i<(unsigned int)parser.getNumNumbers();++i)
			{
				tempKey.key = 0.0f;
				tempKey.value = parser.getInt(i);

				keys.push_back(tempKey);
			}
		}
		else //keys is already populated with PositionKeys.
		{
			//Just to make sure...
			if((unsigned int)parser.getNumNumbers() == keys.size())
			{
				for(i=0;i<keys.size();++i)
					keys[i].value = parser.getInt(i);
			}
			else
			{
				cout << parser.getNumBools() << endl;
				cout << "Error: Trying to intialise IntegerSequencer with ";
				cout << "a different number of key-keyValue pairs." << endl;
			}
		}
	}
	else if(attribute == "set_fraction")
	{
		tempf = val;

		//Find the key which matches the value passed in.
		for(i=0;i<(keys.size()-1);++i)
		{
			if((keys[i].key <= tempf) &&
			   ((tempf < keys[i+1].key) || (i == (keys.size()-2))))
			{
				//Only send a message if we haven't already sent one for
				//this index.
				tempint = i;
				if(tempint != position)
				{
					position = tempint;

					sendEvent("value_changed", keys[tempint].value);
				}
				break;
			}
		}
	}
	else if(attribute == "next")
	{
		StringParser parse(val);

		if(parse.getNumBools() > 0)
		{
			if(parse.getBool())
			{
				++position;
				if((unsigned)position >= keys.size())
					position = 0;

				sendEvent("value_changed", keys[position].value);
			}
		}
	}
	else if(attribute == "previous")
	{
		StringParser parse(val);

		if(parse.getNumBools() > 0)
		{
			if(parse.getBool())
			{
				--position;
				if(position < 0)
					position = keys.size()-1;

				sendEvent("value_changed", keys[position].value);
			}
		}
	}
}
	
}
