//	IntegerSequencer.h - Class representing the IntegerSequencer node.
//  ----------------------------------------------------------------------------
//	This file is part of 'Heilan Browser', an X3D browser specialised for audio
//	work.
//	Copyright (C) 2005-2009  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#ifndef INTEGERSEQUENCER_H_
#define INTEGERSEQUENCER_H_

#include "AbstractNode.h"

namespace HeilanX3D
{

///	Class representing the IntegerSequencer node.
class IntegerSequencer : public ChildNode
{
  public:
	///	Constructor.
	IntegerSequencer();
	///	Destructor.
	~IntegerSequencer();

	///	Over-ridden to do nothing.
	void graphicDoStuff() {};
	///	Over-ridden to do nothing.
	void setBlockSize(int newSize) {};

	///	Where the main work this class does happens.
	void setAttribute(const std::string& attribute, EventValue& val);

	static NodeConstructor<IntegerSequencer> constructor;
  private:
	///	Simple struct representing a key-keyValue pair for the sequencing.
	struct IntegerKey
	{
		///	The key value for this key.
		float key;
		///	The value for this key.
		int value;
	};

	///	All the keys associated with this interpolator.
	std::vector<IntegerKey> keys;

	///	Our current position within the sequence.
	int position;
};

}

#endif
