//	KeySensor.cpp - Class representing the KeySensor node.
//  ----------------------------------------------------------------------------
//	This file is part of 'Heilan Browser', an X3D browser specialised for audio
//	work.
//	Copyright (C) 2005-2009  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#include "KeySensor.h"
#include "StringParser.h"

#include <sstream>
#include <iostream>

using namespace std;

namespace HeilanX3D
{

NodeConstructor<KeySensor> KeySensor::constructor;

//------------------------------------------------------------------------------
KeySensor::KeySensor():
ChildNode(),
enabled(true)
{
	setType("KeySensor");
}

//------------------------------------------------------------------------------
KeySensor::~KeySensor()
{
	KeyboardInput::getInstance().unregisterCallback(this);
}

//------------------------------------------------------------------------------
void KeySensor::initialise()
{
	KeyboardInput::getInstance().registerCallback(this);
}

//------------------------------------------------------------------------------
void KeySensor::setAttribute(const string& attribute, EventValue& val)
{
	if(attribute == "enabled")
	{
		StringParser parse(val);

		if(parse.getNumBools() > 0)
			enabled = parse.getBool();
	}
}

//------------------------------------------------------------------------------
void KeySensor::keyPressed(const std::string& key)
{
	if(enabled)
	{
		cout << "KeySensor: " << key << endl;
		sendEvent("isActive", string("true"));
		sendEvent("keyPress", key);
	}
}

//------------------------------------------------------------------------------
void KeySensor::keyReleased(const std::string& key)
{
	if(enabled)
	{
		sendEvent("isActive", string("false"));
		sendEvent("keyRelease", key);
	}
}

//------------------------------------------------------------------------------
void KeySensor::altKeyChanged(bool val)
{
	if(enabled)
	{
		//Do I need to send an isActive event?
		sendEvent("altKey", (val ? string("true") : string("false")));
	}
}

//------------------------------------------------------------------------------
void KeySensor::ctrlKeyChanged(bool val)
{
	if(enabled)
	{
		//Do I need to send an isActive event?
		sendEvent("controlKey", (val ? string("true") : string("false")));
	}
}

//------------------------------------------------------------------------------
void KeySensor::shiftKeyChanged(bool val)
{
	if(enabled)
	{
		//Do I need to send an isActive event?
		sendEvent("shiftKey", (val ? string("true") : string("false")));
	}
}

//------------------------------------------------------------------------------
void KeySensor::actionKeyPressed(KeyboardCallback::ActionKey key)
{
	if(enabled)
	{
		sendEvent("isActive", string("true"));
		sendEvent("actionKeyPress", (int)key);
	}
}

//------------------------------------------------------------------------------
void KeySensor::actionKeyReleased(KeyboardCallback::ActionKey key)
{
	if(enabled)
	{
		sendEvent("isActive", string("false"));
		sendEvent("actionKeyRelease", (int)key);
	}
}

}
