//	KeySensor.h - Class representing the KeySensor node.
//  ----------------------------------------------------------------------------
//	This file is part of 'Heilan Browser', an X3D browser specialised for audio
//	work.
//	Copyright (C) 2005-2009  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#ifndef KEYSENSOR_H_
#define KEYSENSOR_H_

#include "AbstractNode.h"
#include "KeyboardInput.h"

namespace HeilanX3D
{

///	Class representing the KeySensor node.
class KeySensor : public ChildNode,
				  public KeyboardCallback
{
  public:
	///	Constructor.
	KeySensor();
	///	Destructor.
	~KeySensor();

	///	Used to register us with KeyboardInput.
	void initialise();

	///	Over-ridden to do nothing.
	void graphicDoStuff() {};
	///	Over-ridden to do nothing.
	void setBlockSize(int newSize) {};

	///	Sets some values.
	void setAttribute(const std::string& attribute, EventValue& val);

	///	Called when a key is pressed.
	void keyPressed(const std::string& key);
	///	Called when a key is released.
	void keyReleased(const std::string& key);
	///	Called when the alt key changes state.
	void altKeyChanged(bool val);
	///	Called when the ctrl key changes state.
	void ctrlKeyChanged(bool val);
	///	Called when the shift key changes state.
	void shiftKeyChanged(bool val);
	///	Called when an action key is pressed.
	void actionKeyPressed(KeyboardCallback::ActionKey key);
	///	Called when an action key is released.
	void actionKeyReleased(KeyboardCallback::ActionKey key);

	static NodeConstructor<KeySensor> constructor;
  private:
	///	Whether this node is enabled or not.
	bool enabled;
};

}

#endif
