//	LineProperties.cpp - Class to change the properties of lines.
//  ----------------------------------------------------------------------------
//	This file is part of 'Heilan Browser', an X3D browser specialised for audio
//	work.
//	Copyright (C) 2005-2009  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#include "LineProperties.h"
#include "StringParser.h"

#include "GLHeaders.h"

using namespace std;

namespace HeilanX3D
{

NodeConstructor<LineProperties> LineProperties::constructor;

//------------------------------------------------------------------------------
LineProperties::LineProperties():
ChildNode(),
applied(true),
linetype(1),
linewidthScaleFactor(0)
{
	setType("LineProperties");
	setVisual(true);
}

//------------------------------------------------------------------------------
LineProperties::~LineProperties()
{
	
}

//------------------------------------------------------------------------------
void LineProperties::graphicDoStuff()
{
	if(applied)
	{
		switch(linetype)
		{
			case 2:
				glLineStipple(1, 0xF0F0);
				break;
			case 3:
				glLineStipple(1, 0xAAAA);
				break;
			case 4:
				glLineStipple(1, 0xF5EB);
				break;
			case 5:
				glLineStipple(1, 0xF57A);
				break;
			default:
				glLineStipple(1, 0xFFFF);
			break;
		}
		if(linewidthScaleFactor <= 0)
			glLineWidth(1.0f);
		else
			glLineWidth(linewidthScaleFactor);
	}
}

//------------------------------------------------------------------------------
void LineProperties::setAttribute(const string& attribute, EventValue& val)
{
	if(attribute == "applied")
	{
		StringParser parse(val);

		if(parse.getNumBools() > 0)
		{
			if(parse.getBool())
				applied = true;
			else
				applied = false;
		}
	}
	else if(attribute == "linetype")
		linetype = val;
	else if(attribute == "linewidthScaleFactor")
		linewidthScaleFactor = val;
}

}
