//	LineProperties.h - Class to change the properties of lines.
//  ----------------------------------------------------------------------------
//	This file is part of 'Heilan Browser', an X3D browser specialised for audio
//	work.
//	Copyright (C) 2005-2009  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#ifndef LINEPROPERTIES_H_
#define LINEPROPERTIES_H_

#include "AbstractNode.h"
#include "ThreeFloats.h"

namespace HeilanX3D
{

///	Class to change the properties of lines.
class LineProperties : public ChildNode
{
  public:
	///	Constructor.
	LineProperties();
	///	Destructor.
	~LineProperties();

	///	Sets the line properties.
	void graphicDoStuff();

	///	Not used, overridden to do nothing.
	void setBlockSize(int newSize) {};

	///	Used to set the material's attributes.
	void setAttribute(const std::string& attribute, EventValue& val);

	static NodeConstructor<LineProperties> constructor;
  private:
	///	Whether the properties should currently be applied or not.
	bool applied;
	///	The stipple pattern of the lines drawn.
	int linetype;
	///	How much the line width should be scaled by.
	float linewidthScaleFactor;
};

}

#endif
