//	Material.cpp - Class representing a material node.
//  ----------------------------------------------------------------------------
//	This file is part of 'Heilan Browser', an X3D browser specialised for audio
//	work.
//	Copyright (C) 2005-2009  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#include "Material.h"
#include "StringParser.h"
#include "AttributeHelper.h"

#include "GLHeaders.h"
#include <iostream>

using namespace std;

namespace HeilanX3D
{

NodeConstructor<Material> Material::constructor;

//------------------------------------------------------------------------------
Material::Material():
ChildNode(),
ambientIntensity(0.2f),
diffuseColor(0.8f),
emissiveColor(0.0f),
shininess(0.2f),
specularColor(0.0f),
transparency(0.0f)
{
	setType("Material");
	setVisual(true);
}

//------------------------------------------------------------------------------
Material::~Material()
{
	
}

//------------------------------------------------------------------------------
void Material::graphicDoStuff()
{
	GLfloat diffuse[4];
	GLfloat ambient[4];
	GLfloat emissive[4];
	GLfloat specular[4];

	diffuse[0] = diffuseColor.x;
	diffuse[1] = diffuseColor.y;
	diffuse[2] = diffuseColor.z;
	diffuse[3] = 1.0f;
	ambient[0] = ambientIntensity * diffuseColor.x;
	ambient[1] = ambientIntensity * diffuseColor.y;
	ambient[2] = ambientIntensity * diffuseColor.z;
	ambient[3] = 1.0f;
	emissive[0] = emissiveColor.x;
	emissive[1] = emissiveColor.y;
	emissive[2] = emissiveColor.z;
	emissive[3] = 1.0f;
	specular[0] = specularColor.x;
	specular[1] = specularColor.y;
	specular[2] = specularColor.z;
	specular[3] = 1.0f;

	glMaterialfv(GL_FRONT, GL_DIFFUSE, diffuse);
	glMaterialfv(GL_FRONT, GL_AMBIENT, ambient);
	glMaterialfv(GL_FRONT, GL_EMISSION, emissive);
	glMaterialf(GL_FRONT, GL_SHININESS, shininess); 
	glMaterialfv(GL_FRONT, GL_SPECULAR, specular);

	if(transparency > 0.0f)
	{
		glDisable(GL_LIGHTING);
		glColor4f(1.0f, 1.0f, 1.0f, (1.0f-transparency));
	}
}

//------------------------------------------------------------------------------
void Material::setAttribute(const string& attribute, EventValue& val)
{
	string realAtt = AttributeHelperBase::getAddress(attribute);

	if(realAtt == "diffuseColor")
	{
		StringParser parser(val);
		AttributeHelper<ThreeFloats> helper(&diffuseColor);

		if(helper.isValid(attribute))
			helper.convert(attribute, val);
		else if(parser.getNumThreeFloats() > 0)
			diffuseColor = parser.getThreeFloats();
	}
	else if(realAtt == "emissiveColor")
	{
		StringParser parser(val);
		AttributeHelper<ThreeFloats> helper(&emissiveColor);

		if(helper.isValid(attribute))
			helper.convert(attribute, val);
		else if(parser.getNumThreeFloats() > 0)
			emissiveColor = parser.getThreeFloats();
	}
	else if(realAtt == "specularColor")
	{
		StringParser parser(val);
		AttributeHelper<ThreeFloats> helper(&specularColor);

		if(helper.isValid(attribute))
			helper.convert(attribute, val);
		else if(parser.getNumThreeFloats() > 0)
			specularColor = parser.getThreeFloats();
	}
	else if(attribute == "ambientIntensity")
		ambientIntensity = val;
	else if(attribute == "shininess")
		shininess = val;
	else if(attribute == "transparency")
		transparency = val;
}

}
