//	Material.h - Class representing a material node.
//  ----------------------------------------------------------------------------
//	This file is part of 'Heilan Browser', an X3D browser specialised for audio
//	work.
//	Copyright (C) 2005-2009  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#ifndef MATERIAL_H_
#define MATERIAL_H_

#include "AbstractNode.h"
#include "ThreeFloats.h"

namespace HeilanX3D
{

///	Class representing an X3D material node.
/*!
	\todo Transparency?
 */
class Material : public ChildNode
{
  public:
	///	Constructor.
	Material();
	///	Destructor.
	~Material();

	///	Draws the Material.
	void graphicDoStuff();
	///	Not used, overridden to do nothing.
	/*AmbisonicData *audioDoStuff(int numSamples,
								const ThreeFloats& pos) {return 0;};*/

	///	Not used, overridden to do nothing.
	void setBlockSize(int newSize) {};

	///	Used to set the material's attributes.
	void setAttribute(const std::string& attribute, EventValue& val);

	static NodeConstructor<Material> constructor;
  private:
	///	The material's ambient intensity.
	float ambientIntensity;
	///	The material's diffuse colour.
	ThreeFloats diffuseColor;
	///	The material's emissive colour.
	ThreeFloats emissiveColor;
	///	The material's shininess.
	float shininess;
	///	The material's specular colour.
	ThreeFloats specularColor;
	///	The material's transparency.
	float transparency;
};

}

#endif
