//	MultiTexture.cpp - Class representing the MultiTexture parent node.
//  ----------------------------------------------------------------------------
//	This file is part of 'Heilan Browser', an X3D browser specialised for audio
//	work.
//	Copyright (C) 2005-2009  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#include "MultiTexture.h"
#include "StringParser.h"
#include "AttributeHelper.h"

#include "GLHeaders.h"

using namespace std;

namespace HeilanX3D
{

NodeConstructor<MultiTexture> MultiTexture::constructor;

//------------------------------------------------------------------------------
MultiTexture::MultiTexture():
alpha(1.0f),
color(1.0f, 1.0f, 1.0f)
{
	setType("MultiTexture");
	setVisual(true);
}

//------------------------------------------------------------------------------
MultiTexture::~MultiTexture()
{
	
}

//------------------------------------------------------------------------------
void MultiTexture::graphicDoStuff()
{
	AbstractNode *tempNode = 0;

	//We assume every child node is a texture node.

	//--1st texture--
	tempNode = getChild(0);
	if(tempNode)
	{
		glActiveTextureARB(GL_TEXTURE0_ARB);
		glEnable(GL_TEXTURE_2D);

		glTexEnvi(GL_TEXTURE_ENV, GL_COMBINE_RGB_ARB, GL_REPLACE);

		if(tempNode->isVisual())
			tempNode->graphicDoStuff();
	}

	//--2nd texture--
	tempNode = getChild(1);
	if(tempNode)
	{
		glActiveTextureARB(GL_TEXTURE1_ARB);
		glEnable(GL_TEXTURE_2D);

		//For now, we ignore the mode parameter, and just do a MODULATE
		//operation.
		glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);

		if(tempNode->isVisual())
			tempNode->graphicDoStuff();
	}
}

//------------------------------------------------------------------------------
void MultiTexture::setAttribute(const string& attribute, EventValue& val)
{
	string realAtt = AttributeHelperBase::getAddress(attribute);

	if(realAtt == "alpha")
		alpha = val;
	else if(realAtt == "color")
	{
		StringParser parser(val);
		AttributeHelper<ThreeFloats> helper(&color);

		if(helper.isValid(attribute))
			helper.convert(attribute, val);
		else if(parser.getNumNumbers() > 0)
			color = parser.getThreeFloats();
	}
	else if(realAtt == "function")
		function = (string)val;
	else if(realAtt == "mode")
		mode = (string)val;
	else if(realAtt == "source")
		source = (string)val;
}

//------------------------------------------------------------------------------
void MultiTexture::postChildDraw()
{
	
}

//------------------------------------------------------------------------------
void MultiTexture::childrenLoaded()
{
	
}

}
