//	MultiTexture.h - Class representing the MultiTexture parent node.
//  ----------------------------------------------------------------------------
//	This file is part of 'Heilan Browser', an X3D browser specialised for audio
//	work.
//	Copyright (C) 2005-2009  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#ifndef MULTITEXTURE_H_
#define MULTITEXTURE_H_

#include "ParentNode.h"

namespace HeilanX3D
{

///	Class representing the MultiTexture parent node.
/*!
	For (coding) simplicity, we limit the number of textures used at any one
	time to 2.  As I understand it, the Interchange profile requires a minimum
	of 1 texture to be displayed when using the MultiTexturing nodes, so this is
	perfectly okay.

	\todo Only the MODULATE mode is supported at the moment, add support for
	all the other modes.
 */
class MultiTexture : public ParentNode
{
  public:
	///	Constructor.
	MultiTexture();
	///	Destructor.
	~MultiTexture();

	///	Used to set various OpenGL things.
	/*!
		We override this method because we need to set certain OpenGL state
		variables before we call our childrens' graphicDoStuffs.  Normally
		there'd be no reason for a ParentNode to override graphicDoStuff().
	 */
	void graphicDoStuff();

	///	Used to set the various attributes of the texture.
	void setAttribute(const std::string& attribute, EventValue& val);

	static NodeConstructor<MultiTexture> constructor;
  private:
	///	Called after the node has finished drawing it's children.
	/*!
		This is where we do our texture setup.
	 */
	void postChildDraw();
	///	Implemented to do nothing.
	void audioBlockSizeChanged(int newSize) {};
	///	Used to look at our child texture nodes' current state.
	void childrenLoaded();

	///	Alpha blend value.
	float alpha;
	///	Colour value.
	ThreeFloats color;
	///	Function applied to the textures.
	std::string function;
	///	Mode of texture blending.
	std::string mode;
	///	Colour source (?).
	std::string source;
};

}

#endif
