//	MultiTextureCoordinate.h - Class representing the MultiTextureCoordinate
//							   parent node.
//  ----------------------------------------------------------------------------
//	This file is part of 'Heilan Browser', an X3D browser specialised for audio
//	work.
//	Copyright (C) 2005-2009  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#ifndef MULTITEXTURECOORDINATE_H_
#define MULTITEXTURECOORDINATE_H_

#include "ParentNode.h"
#include "TextureCoordinate.h"

namespace HeilanX3D
{

///	Class representing the MultiTextureCoordinate parent node.
class MultiTextureCoordinate : public ParentNode
{
  public:
	///	Constructor.
	MultiTextureCoordinate();
	///	Destructor.
	~MultiTextureCoordinate();

	///	Implemented to do nothing.
	void setAttribute(const std::string& attribute, EventValue& val) {};

	///	Returns the number of child TextureCoordinates.
	int getNumChildTexCoord() const {return childTex.size();};
	///	Returns the indexed child TextureCoordinate.
	TextureCoordinate *getChildTexCoord(int index) const {return childTex[index];};

	static NodeConstructor<MultiTextureCoordinate> constructor;
  private:
	///	Implemented to do nothing.
	void audioBlockSizeChanged(int newSize) {};

	///	Used to get the coordinates of the child TextureCoordinates.
	void childrenLoaded();

	///	Our copies of the child TextureCoordinates.
	std::vector<TextureCoordinate *> childTex;
};

}

#endif
