//	NavigationInfo.cpp - Node specifying how the user may navigate the scene.
//  ----------------------------------------------------------------------------
//	This file is part of 'Heilan Browser', an X3D browser specialised for audio
//	work.
//	Copyright (C) 2005-2009  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#include "NavigationInfo.h"
#include "StringParser.h"
#include "AttributeHelper.h"
#include "Lighting.h"

using namespace std;

namespace HeilanX3D
{

NodeConstructor<NavigationInfo> NavigationInfo::constructor;

//------------------------------------------------------------------------------
NavigationInfo::NavigationInfo():
BindableNode(),
headlight(true),
speed(1.0f),
transitionType("LINEAR"),
visibilityLimit(0.0f)
{
	setType("NavigationInfo");
}

//------------------------------------------------------------------------------
NavigationInfo::~NavigationInfo()
{
	//Unregister with the BindableNodeSingleton.
	BindableNodeSingleton::getInstance()->unregisterNode(this);
}

//------------------------------------------------------------------------------
void NavigationInfo::initialise()
{
	//Register with the BindableNodeSingleton.
	BindableNodeSingleton::getInstance()->registerNode(this);
}

//------------------------------------------------------------------------------
void NavigationInfo::setAttribute(const string& attribute, EventValue& val)
{
	string realAtt = AttributeHelperBase::getAddress(attribute);

	if(realAtt == "type")
		navTypes = (string)val;
	else if(realAtt == "avatarSize")
	{
		StringParser parser(val);
		AttributeHelper<ThreeFloats> helper(&avatarSize);

		if(helper.isValid(attribute))
			helper.convert(attribute, val);
		else if(parser.getNumThreeFloats() > 0)
			avatarSize = parser.getThreeFloats();
	}
	else if(realAtt == "headlight")
	{
		StringParser parser(val);

		if(parser.getNumBools())
		{
			headlight = parser.getBool();

			Lighting::getInstance().switchHeadlight(headlight);
		}
	}
	else if(realAtt == "speed")
		speed = val;
	else if(realAtt == "transitionType")
		transitionType = (string)val;
	else if(realAtt == "visibilityLimit")
		visibilityLimit = val;
}

}
