//	NavigationInfo.h - Node specifying how the user may navigate the scene.
//  ----------------------------------------------------------------------------
//	This file is part of 'Heilan Browser', an X3D browser specialised for audio
//	work.
//	Copyright (C) 2005-2009  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#ifndef NAVIGATIONINFO_H_
#define NAVIGATIONINFO_H_

#include "BindableNode.h"

namespace HeilanX3D
{

///	Node specifying how the user may navigate the scene.
/*!
	\todo Support all the attributes NavigationInfo is supposed to have.
 */
class NavigationInfo : public BindableNode
{
  public:
	///	Constructor.
	NavigationInfo();
	///	Destructor.
	~NavigationInfo();

	///	Initialises various bits and pieces.
	void initialise();

	///	Overridden to do nothing.
	void graphicDoStuff() {};
	///	Overridden to do nothing.
	/*AmbisonicData *audioDoStuff(int numSamples,
								const ThreeFloats& pos) {return 0;};*/
	///	Overridden to do nothing.
	void setBlockSize(int newSize) {};

	///	Called to set the value of the named attribute.
	void setAttribute(const std::string& attribute, EventValue& val);

	///	Returns the available navigation types for this scene.
	const std::string& getTypes() const {return navTypes;};

	static NodeConstructor<NavigationInfo> constructor;
  private:
	///	The navigation types allowed for this scene.
	std::string navTypes;

	///	Size of the user's avatar.
	ThreeFloats avatarSize;
	///	Whether the headlight should be on by default or not.
	bool headlight;
	///	How fast the viewer moves through the scene.
	float speed;
	///	How the camera should transition between Viewpoints.
	std::string transitionType;
	///	How far the camera can see.
	float visibilityLimit;
};

}

#endif
