//	OrientationInterpolator.cpp - Node used to interpolate between orientations.
//  ----------------------------------------------------------------------------
//	This file is part of 'Heilan Browser', an X3D browser specialised for audio
//	work.
//	Copyright (C) 2005-2009  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#include "OrientationInterpolator.h"
#include "StringParser.h"

#include <iostream>
#include <sstream>

using namespace std;

namespace HeilanX3D
{

NodeConstructor<OrientationInterpolator> OrientationInterpolator::constructor;

//------------------------------------------------------------------------------
OrientationInterpolator::OrientationInterpolator():
ChildNode()
{
	setType("OrientationInterpolator");
}

//------------------------------------------------------------------------------
OrientationInterpolator::~OrientationInterpolator()
{
	
}

//------------------------------------------------------------------------------
void OrientationInterpolator::setAttribute(const string& attribute,
										   EventValue& val)
{
	unsigned int i;
	float tempf, tempf2;
	OrientationKey tempKey;
	FourFloats tempRot;
	stringstream tempConverter;

	if(attribute == "key")
	{
		StringParser parser(val);

		//This is to handle the (hopefully) rare case that X3DFileLoader would
		//read keyValue before it reads key.
		if(keys.size() <= 0)
		{
			for(i=0;i<(unsigned int)parser.getNumNumbers();++i)
			{
				tempKey.key = parser.getFloat(i);
				tempKey.orientation = tempRot;

				keys.push_back(tempKey);
			}
		}
		else //keys is already populated with PositionKeys.
		{
			for(i=0;i<keys.size();++i)
				keys[i].key = parser.getFloat(i);
		}
	}
	else if(attribute == "keyValue")
	{
		StringParser parser(val);

		//This is to handle the (hopefully) rare case that X3DFileLoader would
		//read keyValue before it reads key.
		if(keys.size() <= 0)
		{
			for(i=0;i<(unsigned int)parser.getNumFourFloats();++i)
			{
				tempKey.key = 0.0f;
				tempKey.orientation = parser.getFourFloats(i);

				keys.push_back(tempKey);
			}
		}
		else //keys is already populated with PositionKeys.
		{
			//Just to make sure...
			if((unsigned int)parser.getNumFourFloats() == keys.size())
			{
				for(i=0;i<keys.size();++i)
					keys[i].orientation = parser.getFourFloats(i);
			}
			else
			{
				cout << "Error: Trying to intialise OrientationInterpolator with ";
				cout << "a different number of key-keyValue pairs." << endl;
			}
		}
	}
	else if(attribute == "set_fraction")
	{
		tempf = val;

		//Evaluate the interpolator at fraction tempf.
		for(i=0;i<(keys.size()-1);++i)
		{
			if((tempf >= keys[i].key) && (tempf < keys[i+1].key))
			{
				tempf2 = keys[i+1].key - keys[i].key;
				tempf2 = (tempf-keys[i].key)/tempf2;

				tempRot.x = (keys[i].orientation.x * (1.0f-tempf2)) +
							(keys[i+1].orientation.x * tempf2);
				tempRot.y = (keys[i].orientation.y * (1.0f-tempf2)) +
							(keys[i+1].orientation.y * tempf2);
				tempRot.z = (keys[i].orientation.z * (1.0f-tempf2)) +
							(keys[i+1].orientation.z * tempf2);
				tempRot.r = (keys[i].orientation.r * (1.0f-tempf2)) +
							(keys[i+1].orientation.r * tempf2);

				/*tempConverter << tempRot.x << " ";
				tempConverter << tempRot.y << " ";
				tempConverter << tempRot.z << " ";
				tempConverter << tempRot.r;*/
				tempConverter << tempRot.r << " ";
				tempConverter << tempRot.x << " ";
				tempConverter << tempRot.y << " ";
				tempConverter << tempRot.z;
				sendEvent("value_changed", tempConverter.str());
				break;
			}
		}
	}
}

}
