//	PixelTexture.cpp - Class representing a basic texture.
//  ----------------------------------------------------------------------------
//	This file is part of 'Heilan Browser', an X3D browser specialised for audio
//	work.
//	Copyright (C) 2005-2009  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#include "PixelTexture.h"
#include "StringParser.h"
#include "AttributeHelper.h"

#include <iostream>

using namespace std;

namespace HeilanX3D
{

NodeConstructor<PixelTexture> PixelTexture::constructor;

//------------------------------------------------------------------------------
PixelTexture::PixelTexture():
ChildNode(),
repeatS(true),
repeatT(true)
{
	setType("PixelTexture");
	setVisual(true);
}

//------------------------------------------------------------------------------
PixelTexture::~PixelTexture()
{
	
}

//------------------------------------------------------------------------------
void PixelTexture::graphicDoStuff()
{
	glBindTexture(GL_TEXTURE_2D, texture);
}

//------------------------------------------------------------------------------
void PixelTexture::setAttribute(const string& attribute, EventValue& val)
{
	int i;
	string realAtt = AttributeHelperBase::getAddress(attribute);

	if(realAtt == "image")
	{
		StringParser parser(val);
		AttributeHelper< vector<int> > helper(&image);

		critSec.lock();
		{
			if(helper.isValid(attribute))
				helper.convert(attribute, val);
			else
			{
				if(image.size() > 0)
					image.clear();
	
				for(i=0;i<parser.getNumNumbers();++i)
					image.push_back(parser.getInt(i));
			}

			if(pixelTexture.loadImage(image))
			{
				glGenTextures(1, &texture);

				glBindTexture(GL_TEXTURE_2D, texture);

				switch(pixelTexture.getNumComponents())
				{
					case 1:
						glTexImage2D(GL_TEXTURE_2D,
									 0,
									 pixelTexture.getNumComponents(),
									 pixelTexture.getWidth(),
									 pixelTexture.getHeight(),
									 0,
									 GL_LUMINANCE,
									 GL_UNSIGNED_BYTE,
									 pixelTexture.getPixels());
						break;
					case 3:
						glTexImage2D(GL_TEXTURE_2D,
									 0,
									 pixelTexture.getNumComponents(),
									 pixelTexture.getWidth(),
									 pixelTexture.getHeight(),
									 0,
									 GL_RGB,
									 GL_UNSIGNED_BYTE,
									 pixelTexture.getPixels());
						break;
					case 4:
						glTexImage2D(GL_TEXTURE_2D,
									 0,
									 pixelTexture.getNumComponents(),
									 pixelTexture.getWidth(),
									 pixelTexture.getHeight(),
									 0,
									 GL_RGBA,
									 GL_UNSIGNED_BYTE,
									 pixelTexture.getPixels());
						break;
				}

				/*glTexParameteri(GL_TEXTURE_2D,
								GL_TEXTURE_MIN_FILTER,
								GL_LINEAR);
				glTexParameteri(GL_TEXTURE_2D,
								GL_TEXTURE_MAG_FILTER,
								GL_LINEAR);*/
				glTexParameteri(GL_TEXTURE_2D,
								GL_TEXTURE_MIN_FILTER,
								GL_NEAREST);
				glTexParameteri(GL_TEXTURE_2D,
								GL_TEXTURE_MAG_FILTER,
								GL_NEAREST);
				if(repeatS)
				{
					glTexParameteri(GL_TEXTURE_2D,
									GL_TEXTURE_WRAP_S,
									GL_REPEAT);
				}
				else
				{
					glTexParameteri(GL_TEXTURE_2D,
									GL_TEXTURE_WRAP_S,
									GL_CLAMP_TO_EDGE);
				}
				if(repeatT)
				{
					glTexParameteri(GL_TEXTURE_2D,
									GL_TEXTURE_WRAP_T,
									GL_REPEAT);
				}
				else
				{
					glTexParameteri(GL_TEXTURE_2D,
									GL_TEXTURE_WRAP_T,
									GL_CLAMP_TO_EDGE);
				}

				glBindTexture(GL_TEXTURE_2D, 0);
			}
		}
		critSec.unlock();
	}
	else if(attribute == "repeatS")
	{
		StringParser parse(val);

		if(parse.getNumBools() > 0)
		{
			repeatS = parse.getBool();

			if(repeatS)
			{
				glBindTexture(GL_TEXTURE_2D, texture);
				if(repeatS)
					glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
				else
					glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
				glBindTexture(GL_TEXTURE_2D, 0);
			}
		}
	}
	else if(attribute == "repeatT")
	{
		StringParser parse(val);

		if(parse.getNumBools() > 0)
		{
			repeatT = parse.getBool();

			if(repeatT)
			{
				glBindTexture(GL_TEXTURE_2D, texture);
				if(repeatT)
					glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
				else
					glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
				glBindTexture(GL_TEXTURE_2D, 0);
			}
		}
	}
}

}
