//	PixelTexture.h - Class representing a texture made up of user-specified
//					 pixels.
//  ----------------------------------------------------------------------------
//	This file is part of 'Heilan Browser', an X3D browser specialised for audio
//	work.
//	Copyright (C) 2005-2009  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#ifndef PIXELTEXTURE_H_
#define PIXELTEXTURE_H_

#include "AbstractNode.h"
#include "SFImage.h"

#include "GLHeaders.h"
#include <vector>

namespace HeilanX3D
{

///	Class representing a texture made up of user-specified pixels.
class PixelTexture : public ChildNode
{
  public:
	///	Constructor.
	PixelTexture();
	///	Destructor.
	~PixelTexture();

	///	Draws the PixelTexture.
	void graphicDoStuff();
	///	Not used, overridden to do nothing.
	/*AmbisonicData *audioDoStuff(int numSamples,
								const ThreeFloats& pos) {return 0;};*/

	///	Not used, overridden to do nothing.
	void setBlockSize(int newSize) {};

	///	Used to set the material's attributes.
	void setAttribute(const std::string& attribute, EventValue& val);

	static NodeConstructor<PixelTexture> constructor;
  private:
	///	The SFImage that makes up the texture.
	SFImage pixelTexture;
	///	Whether the texture should repeat in the S direction.
	bool repeatS;
	///	Whether the texture should repeat in the T direction.
	bool repeatT;

	///	Our internal representation of the pixel data.
	std::vector<int> image;

	///	The texture we use.
	GLuint texture;

	///	Critical section used to guard the pixel data while we're drawing.
	CriticalSection critSec;
};

}

#endif
