//	PointLight.h - Class representing a point light.
//  ----------------------------------------------------------------------------
//	This file is part of 'Heilan Browser', an X3D browser specialised for audio
//	work.
//	Copyright (C) 2005-2009  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#ifndef POINTLIGHT_H_
#define POINTLIGHT_H_

#include "AbstractNode.h"

namespace HeilanX3D
{

///	Class representing a point light.
/*!
	\sa DirectionalLight

	\todo Switching global to true will only work 100% correctly when it's the
	last attribute in the list.
	\todo Radius is not taken into account.
 */
class PointLight : public ChildNode
{
  public:
	///	Constructor.
	PointLight();
	///	Destructor.
	~PointLight();

	///	Draws the PointLight.
	void graphicDoStuff();

	///	Not used, overridden to do nothing.
	void setBlockSize(int newSize) {};

	///	Used to set the attributes of the light.
	void setAttribute(const std::string& attribute, EventValue& val);

	static NodeConstructor<PointLight> constructor;
  private:
	///	The ambient intensity of the light.
	float ambientIntensity;
	///	The attenuation of the light.
	ThreeFloats attenuation;
	///	The colour of the light.
	ThreeFloats color;
	///	Whether the light applies globally or not.
	bool global;
	///	The intensity of the light.
	float intensity;
	///	The location of the light.
	ThreeFloats location;
	///	Whether the light is on or not.
	bool on;
	///	The radius of the light.
	float radius;
};

}

#endif
