//	PointSet.cpp - Class representing an X3D point set.
//  ----------------------------------------------------------------------------
//	This file is part of 'Heilan Browser', an X3D browser specialised for audio
//	work.
//	Copyright (C) 2005-2009  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#include "PointSet.h"
#include "StringParser.h"
#include "AttributeHelper.h"

using namespace std;

namespace HeilanX3D
{

NodeConstructor<PointSet> PointSet::constructor;

//------------------------------------------------------------------------------
PointSet::PointSet():
ParentNode(),
coordinates(0),
colors(0),
colorsRGBA(0)
{
	setType("PointSet");
	setVisual(true);
}

//------------------------------------------------------------------------------
PointSet::~PointSet()
{
	
}

//------------------------------------------------------------------------------
void PointSet::setAttribute(const string& attribute, EventValue& val)
{
	
}

//------------------------------------------------------------------------------
void PointSet::postChildDraw()
{
	unsigned int i; //Coordinates iterator.

	if(coordinates)
	{
		//Re-load if necessary.
		if(coordinates->getHasChanged())
			childrenLoaded();

		//PointSets are unlit.
		glDisable(GL_LIGHTING);

		for(i=0;i<drawingCoords.size();++i)
		{
			glBegin(GL_POINTS);
			{
				//Set colour.
				if(i<drawingColours.size())
				{
					glColor4f(drawingColours[i].x,
							  drawingColours[i].y,
							  drawingColours[i].z,
							  drawingColours[i].r);
				}

				//Draw the vertex.
				glVertex3f(drawingCoords[i].x,
					   	   drawingCoords[i].y,
					   	   drawingCoords[i].z);
			}
			glEnd();
		}

		glEnable(GL_LIGHTING);
	}
}

//------------------------------------------------------------------------------
void PointSet::childrenLoaded()
{
	unsigned int i, j;

	for(i=0;i<(unsigned)getNumChildren();++i)
	{
		if(getChild(i)->getType() == "Coordinate")
		{
			coordinates = dynamic_cast<Coordinate *>(getChild(i));

			const vector<ThreeFloats>& tempPoints = coordinates->getPoints();

			drawingCoords.clear();

			for(j=0;j<tempPoints.size();++j)
				drawingCoords.push_back(tempPoints[j]);
		}
		else if(getChild(i)->getType() == "Color")
		{
			colors = dynamic_cast<Color *>(getChild(i));

			const vector<ThreeFloats>& tempPoints = colors->getColours();

			drawingColours.clear();

			for(j=0;j<tempPoints.size();++j)
			{
				FourFloats tempf(tempPoints[j].x,
								 tempPoints[j].y,
								 tempPoints[j].z,
								 1.0f);

				drawingColours.push_back(tempf);
			}
		}
		else if(getChild(i)->getType() == "ColorRGBA")
		{
			colorsRGBA = dynamic_cast<ColorRGBA *>(getChild(i));

			const vector<FourFloats>& tempPoints = colorsRGBA->getColours();

			drawingColours.clear();

			for(j=0;j<tempPoints.size();++j)
				drawingColours.push_back(tempPoints[j]);
		}
	}
}

}
