//	PositionInterpolator.cpp - Node used to interpolate between positions.
//  ----------------------------------------------------------------------------
//	This file is part of 'Heilan Browser', an X3D browser specialised for audio
//	work.
//	Copyright (C) 2005-2009  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#include "PositionInterpolator.h"
#include "StringParser.h"

#include <iostream>
#include <sstream>

using namespace std;

namespace HeilanX3D
{

NodeConstructor<PositionInterpolator> PositionInterpolator::constructor;

//------------------------------------------------------------------------------
PositionInterpolator::PositionInterpolator():
ChildNode()
{
	setType("PositionInterpolator");
}

//------------------------------------------------------------------------------
PositionInterpolator::~PositionInterpolator()
{
	
}

//------------------------------------------------------------------------------
void PositionInterpolator::setAttribute(const string& attribute,
										EventValue& val)
{
	unsigned int i;
	float tempf, tempf2;
	PositionKey tempKey;
	ThreeFloats tempPos;
	stringstream tempConverter;

	if(attribute == "key")
	{
		StringParser parser(val);

		//This is to handle the (hopefully) rare case that X3DFileLoader would
		//read keyValue before it reads key.
		if(keys.size() <= 0)
		{
			for(i=0;i<(unsigned int)parser.getNumNumbers();++i)
			{
				tempKey.key = parser.getFloat(i);
				tempKey.position = tempPos;

				keys.push_back(tempKey);
			}
		}
		else //keys is already populated with PositionKeys.
		{
			for(i=0;i<keys.size();++i)
				keys[i].key = parser.getFloat(i);
		}
	}
	else if(attribute == "keyValue")
	{
		StringParser parser(val);

		//This is to handle the (hopefully) rare case that X3DFileLoader would
		//read keyValue before it reads key.
		if(keys.size() <= 0)
		{
			for(i=0;i<(unsigned int)parser.getNumThreeFloats();++i)
			{
				tempKey.key = 0.0f;
				tempKey.position = parser.getThreeFloats(i);

				keys.push_back(tempKey);
			}
		}
		else //keys is already populated with PositionKeys.
		{
			//Just to make sure...
			if((unsigned int)parser.getNumThreeFloats() == keys.size())
			{
				for(i=0;i<keys.size();++i)
					keys[i].position = parser.getThreeFloats(i);
			}
			else
			{
				cout << "Error: Trying to intialise PositionInterpolator with ";
				cout << "a different number of key-keyValue pairs." << endl;
			}
		}
	}
	else if(attribute == "set_fraction")
	{
		tempf = val;

		//Evaluate the interpolator at fraction tempf.
		for(i=0;i<(keys.size()-1);++i)
		{
			if((tempf >= keys[i].key) && (tempf < keys[i+1].key))
			{
				tempf2 = keys[i+1].key - keys[i].key;
				tempf2 = (tempf-keys[i].key)/tempf2;

				tempPos.x = (keys[i].position.x * (1.0f-tempf2)) +
							(keys[i+1].position.x * tempf2);
				tempPos.y = (keys[i].position.y * (1.0f-tempf2)) +
							(keys[i+1].position.y * tempf2);
				tempPos.z = (keys[i].position.z * (1.0f-tempf2)) +
							(keys[i+1].position.z * tempf2);

				tempConverter << tempPos.x << " ";
				tempConverter << tempPos.y << " ";
				tempConverter << tempPos.z;
				sendEvent("value_changed", tempConverter.str());
				break;
			}
		}
	}
}

}
