//	ProgramShader.cpp - Class representing the ProgramShader parent node.
//  ----------------------------------------------------------------------------
//	This file is part of 'Heilan Browser', an X3D browser specialised for audio
//	work.
//	Copyright (C) 2005-2009  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#include "ProgramShader.h"
#include "ShaderProgram.h"

#include "GLHeaders.h"
#include <iostream>

using namespace std;

namespace HeilanX3D
{

NodeConstructor<ProgramShader> ProgramShader::constructor;

//------------------------------------------------------------------------------
ProgramShader::ProgramShader():
ParentNode(),
vertexShader(0),
fragmentShader(0),
vertex(0),
fragment(0),
program(0)
{
	setType("ProgramShader");
	setVisual(true);
}

//------------------------------------------------------------------------------
ProgramShader::~ProgramShader()
{
	
}

//------------------------------------------------------------------------------
void ProgramShader::setAttribute(const string& attribute, EventValue& val)
{
	if(attribute == "language")
	{
		language = (string)val;
		if(language != "GLSL")
			cout << "ProgramShader Error: Heilan only supports GLSL shaders." << endl;
	}
}

//------------------------------------------------------------------------------
void ProgramShader::postChildDraw()
{
	if(program)
		glUseProgramObjectARB(program);
}

//------------------------------------------------------------------------------
void ProgramShader::childrenLoaded()
{
	int i;
	ShaderProgram *tempNode = 0;

	//Don't bother loading anything if the shader language requested isn't GLSL.
	if(language != "GLSL")
		return;

	for(i=0;i<getNumChildren();++i)
	{
		tempNode = dynamic_cast<ShaderProgram *>(getChild(i));

		if(tempNode)
		{
			if((!vertexShader) && (tempNode->getShaderType() == "VERTEX"))
				vertexShader = tempNode;
			else if((!fragmentShader) && (tempNode->getShaderType() == "FRAGMENT"))
				fragmentShader = tempNode;

			//Quit once we've got both shaders.
			if(vertexShader && fragmentShader)
				break;
		}
	}

	//Load the shaders into program.
	if(vertexShader && fragmentShader)
	{
		vertex = glCreateShaderObjectARB(GL_VERTEX_SHADER_ARB);
		fragment = glCreateShaderObjectARB(GL_FRAGMENT_SHADER_ARB);

		HeilanGL::setupShader(vertex, vertexShader->getUrl());
		HeilanGL::setupShader(fragment, fragmentShader->getUrl());

		glCompileShaderARB(vertex);
		glCompileShaderARB(fragment);

		cout << vertexShader->getUrl() << " InfoLog:" << endl;
		HeilanGL::displayInfoLog(vertex);
		cout << fragmentShader->getUrl() << " InfoLog:" << endl;
		HeilanGL::displayInfoLog(fragment);

		program = glCreateProgramObjectARB();
		glAttachObjectARB(program, vertex);
		glAttachObjectARB(program, fragment);

		glLinkProgramARB(program);

		cout << "Shader program InfoLog:" << endl;
		HeilanGL::displayInfoLog(program);
	}
	else if(vertexShader)
		cout << "ProgramShader Error: Missing fragment shader." << endl;
	else if(fragmentShader)
		cout << "ProgramShader Error: Missing vertex shader." << endl;
}

}
