//	ProgramShader.h - Class representing the ProgramShader parent node.
//  ----------------------------------------------------------------------------
//	This file is part of 'Heilan Browser', an X3D browser specialised for audio
//	work.
//	Copyright (C) 2005-2009  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#ifndef PROGRAMSHADER_H_
#define PROGRAMSHADER_H_

#include "ParentNode.h"
#include "GLHeaders.h"

namespace HeilanX3D
{

class ShaderProgram;

///	Class representing the ProgramShader parent node.
/*!
	We just implement GLSL shaders, since we're using OpenGL, and that's the
	only shader language I've used to date.

	Note that the actual shader program will not be loaded if there is only a
	vertex shader child (or vice-versa) - it will only be loaded if this node
	has a vertex *AND* a fragment shader.

	We probably won't support the full X3D Shaders component, but the
	ProgramShader and ShaderProgram (sigh... honestly, what a ridiculous naming
	scheme...) nodes should provide a lot of the functionality you can get from
	shaders.

	\todo Implement the various shader attribute nodes.
 */
class ProgramShader : public ParentNode
{
  public:
	///	Constructor.
	ProgramShader();
	///	Destructor.
	~ProgramShader();

	///	Where the various attributes get set.
	void setAttribute(const std::string& attribute, EventValue& val);

	static NodeConstructor<ProgramShader> constructor;
  private:
	///	We override this so we can activate the shader program.
	void postChildDraw();

	///	Used to load the vertex and fragment shaders.
	void childrenLoaded();

	///	Implemented to do nothing.
	void audioBlockSizeChanged(int newSize) {};

	///	The language attribute.
	std::string language;

	///	Our pointer to the vertex shader.
	ShaderProgram *vertexShader;
	///	Our pointer to the fragment shader.
	ShaderProgram *fragmentShader;

	///	Our actual vertex shader.
	GLhandleARB vertex;
	///	Our actual fragment shader.
	GLhandleARB fragment;
	///	Program object.
	GLhandleARB program;
};

}

#endif
