//	ProximitySensor.h - Class representing an X3D ProximitySensor.
//  ----------------------------------------------------------------------------
//	This file is part of 'Heilan Browser', an X3D browser specialised for audio
//	work.
//	Copyright (C) 2005-2009  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#ifndef PROXIMITYSENSOR_H_
#define PROXIMITYSENSOR_H_

#include "AbstractNode.h"
#include "Quaternion.h"

#include "GLHeaders.h"

namespace HeilanX3D
{

///	Class representing an X3D box node.
/*!
	\todo Make sure ProximitySensor works as a child of a Transform node.
 */
class ProximitySensor : public ChildNode
{
  public:
	///	Constructor.
	ProximitySensor();
	///	Destructor.
	~ProximitySensor();

	///	This is where we test for proximity.
	void graphicDoStuff();

	///	All our time calculations take place here.
	AmbisonicData *getMonoAudio(int numSamples);
	///	Not used, overridden to do nothing.
	void setBlockSize(int newSize) {};
	///	We need the samplerate to calculate the current time.
	void setSamplerate(float val);

	///	Used to set the size and solidity of the box.
	void setAttribute(const std::string& attribute, EventValue& val);

	static NodeConstructor<ProximitySensor> constructor;
  private:
	///	The ProximitySensor's position.
	ThreeFloats center;
	///	Whether the sensor's enabled or not.
	bool enabled;
	///	The size of the sensor's bounding box (width, height, depth).
	ThreeFloats size;

	///	Whether the Camera's currently in the sensor's bounding box.
	bool active;
	///	Our copy of the current time.
	double currentTime;
	///	The time of the last graphicDoStuff() call.
	double lastTime;
	///	The Camera's last position.
	ThreeFloats lastCamPos;
	///	The Camera's last orientation.
	Quaternion lastCamRot;

	///	The current samplerate.
	double samplerate;
	///	1/samplerate - simple optimisation.
	double invSamplerate;
};

}

#endif
