//	Scene.h - Class representing the Scene base node in all X3D files.
//  ----------------------------------------------------------------------------
//	This file is part of 'Heilan Browser', an X3D browser specialised for audio
//	work.
//	Copyright (C) 2005-2009  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#ifndef SCENE_H_
#define SCENE_H_

#include "ParentNode.h"

namespace HeilanX3D
{

///	Class representing the Scene base node in all X3D files.
/*!
	Although the X3D spec doesn't actually define Scene as a separate node, it
	suits our purposes to give it a class of it's own, which we'll use to
	manipulate the X3D scene.
 */
class Scene : public ParentNode
{
  public:
	///	Constructor.
	Scene();
	///	Destructor.
	~Scene();

	///	Implemented to do nothing.
	void setAttribute(const std::string& attribute, EventValue& val) {};

	static NodeConstructor<Scene> constructor;
  private:
	///	Implemented to do nothing.
	void audioBlockSizeChanged(int newSize) {};
};

}

#endif
