//	ShaderProgram.h - Class representing shader code.
//  ----------------------------------------------------------------------------
//	This file is part of 'Heilan Browser', an X3D browser specialised for audio
//	work.
//	Copyright (C) 2005-2009  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#ifndef SHADERPROGRAM_H_
#define SHADERPROGRAM_H_

#include "AbstractNode.h"

namespace HeilanX3D
{

///	Class representing shader code.
class ShaderProgram : public ChildNode
{
  public:
	///	Constructor.
	ShaderProgram();
	///	Destructor.
	~ShaderProgram();

	///	Overridden to do nothing.
	void graphicDoStuff() {};

	///	Not used, overridden to do nothing.
	void setBlockSize(int newSize) {};

	///	Used to set the url and type of the shader.
	void setAttribute(const std::string& attribute, EventValue& val);

	///	Returns the url to the shader's code.
	std::string getUrl() const {return url;};
	///	Returns the type of the shader.
	std::string getShaderType() const {return type;};

	static NodeConstructor<ShaderProgram> constructor;
  private:
	///	The url to the shader's code.
	std::string url;
	///	The type of the shader (VERTEX or FRAGMENT).
	std::string type;
};

}

#endif
