//	Shape.cpp - Class representing the Shape parent node.
//  ----------------------------------------------------------------------------
//	This file is part of 'Heilan Browser', an X3D browser specialised for audio
//	work.
//	Copyright (C) 2005-2009  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#include "Shape.h"
#include "GLHeaders.h"

namespace HeilanX3D
{

NodeConstructor<Shape> Shape::constructor;

//------------------------------------------------------------------------------
Shape::Shape():
ParentNode()
{
	setType("Shape");
	setVisual(true);
}

//------------------------------------------------------------------------------
Shape::~Shape()
{
	
}

//------------------------------------------------------------------------------
void Shape::preChildDraw()
{
	glMatrixMode(GL_MODELVIEW);
	glPushMatrix();

	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	glEnable(GL_BLEND);
}

//------------------------------------------------------------------------------
void Shape::postChildDraw()
{
	const GLfloat diffuse[]={1.0f, 1.0f, 1.0f, 1.0f};
	const GLfloat ambient[]={0.0f, 0.0f, 0.0f, 1.0f};
	const GLfloat emissive[]={0.0f, 0.0f, 0.0f, 1.0f};
	const GLfloat specular[]={0.5f, 0.5f, 0.5f, 1.0f};
	const GLfloat shininess = 0.0f;

	//Not sure if there's a better way to do this?
	glActiveTextureARB(GL_TEXTURE1_ARB);
	glBindTexture(GL_TEXTURE_2D, 0);
	glActiveTextureARB(GL_TEXTURE0_ARB);
	glBindTexture(GL_TEXTURE_2D, 0);
	glColor4f(1.0f, 1.0f, 1.0f, 1.0f);
	glMaterialfv(GL_FRONT, GL_DIFFUSE, diffuse);
	glMaterialfv(GL_FRONT, GL_AMBIENT, ambient);
	glMaterialfv(GL_FRONT, GL_EMISSION, emissive);
	glMaterialf(GL_FRONT, GL_SHININESS, shininess); 
	glMaterialfv(GL_FRONT, GL_SPECULAR, specular);

	glDisable(GL_BLEND);

	//For the LineProperties node.
	glDisable(GL_LINE_STIPPLE);

	glMatrixMode(GL_MODELVIEW);
	glPopMatrix();

	if(glUseProgramObjectARB)
		glUseProgramObjectARB(0);
}

}
