//	Shape.h - Class representing the Shape parent node.
//  ----------------------------------------------------------------------------
//	This file is part of 'Heilan Browser', an X3D browser specialised for audio
//	work.
//	Copyright (C) 2005-2009  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#ifndef SHAPE_H_
#define SHAPE_H_

#include "ParentNode.h"

namespace HeilanX3D
{

///	Class representing the Shape base node in all X3D files.
class Shape : public ParentNode
{
  public:
	///	Constructor.
	Shape();
	///	Destructor.
	~Shape();

	///	Implemented to do nothing.
	/*!
		We ignore the bboxCenter and bboxSize attributes, because we won't use
		them.
	 */
	void setAttribute(const std::string& attribute, EventValue& val) {};

	static NodeConstructor<Shape> constructor;
  private:
	///	We override this so we can save and restore the current OpenGL matrix.
	void preChildDraw();
	///	We override this so we can save and restore the current OpenGL matrix.
	void postChildDraw();

	///	Implemented to do nothing.
	void audioBlockSizeChanged(int newSize) {};
};

}

#endif
