//	Sphere.cpp - Class representing an X3D sphere node.
//  ----------------------------------------------------------------------------
//	This file is part of 'Heilan Browser', an X3D browser specialised for audio
//	work.
//	Copyright (C) 2005-2009  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#include "Sphere.h"
#include "StringParser.h"

using namespace std;

namespace HeilanX3D
{

NodeConstructor<Sphere> Sphere::constructor;

//------------------------------------------------------------------------------
Sphere::Sphere():
radius(1.0f),
solid(true)
{
	setType("Sphere");
	setVisual(true);
}

//------------------------------------------------------------------------------
Sphere::~Sphere()
{
	
}

//------------------------------------------------------------------------------
void Sphere::initialise()
{
	int x, y;

	//Generate the display list.
	displayList = glGenLists(1);
	glNewList(displayList, GL_COMPILE);
	{
		glBegin(GL_QUADS);
		{
			//Main sphere.
			for(y=0;y<NumSections;++y)
			{
				//for(x=0;x<(NumSections-2);++x)
				for(x=1;x<(NumSections-2);++x)
				{
					calcVertex(x, y);
					calcVertex(x+1, y);
					calcVertex(x+1, y+1);
					if(x < (NumSections-2))
						calcVertex(x, y+1);
					else
						calcVertex(x+1, y+1);
				}
			}
		}
		glEnd();

		glBegin(GL_TRIANGLES);
		{
			ThreeFloats verts1[3];

			//Top of the sphere.
			for(x=0;x<NumSections;++x)
			{
				verts1[0].x = 0.0f;
				verts1[0].y = 1.0f;
				verts1[0].z = 0.0f;
				verts1[1] = getVertex((NumSections-1), x);
				if(x < (NumSections-1))
					verts1[2] = getVertex((NumSections-1), x+1);
				else
					verts1[2] = getVertex((NumSections-1), 0);

				glNormal3f(0.0f, 1.0f, 0.0f);
				glMultiTexCoord2fARB(GL_TEXTURE0_ARB,
									 (((float)(NumSections-x)+0.5f)/(float)NumSections),
							 		 1.0f/(float)NumSections);
				glMultiTexCoord2fARB(GL_TEXTURE1_ARB,
									 (((float)(NumSections-x)+0.5f)/(float)NumSections),
							 		 1.0f/(float)NumSections);
				glVertex3f(verts1[0].x, verts1[0].y, verts1[0].z);

				glMultiTexCoord2fARB(GL_TEXTURE0_ARB,
									 ((float)(NumSections-x)/(float)NumSections),
							 		 0.0f);
				glMultiTexCoord2fARB(GL_TEXTURE1_ARB,
									 ((float)(NumSections-x)/(float)NumSections),
							 		 0.0f);
				glVertex3f(verts1[1].x, verts1[1].y, verts1[1].z);

				glMultiTexCoord2fARB(GL_TEXTURE0_ARB,
									 ((float)((NumSections-x)+1)/(float)NumSections),
							 		 0.0f);
				glMultiTexCoord2fARB(GL_TEXTURE1_ARB,
									 ((float)((NumSections-x)+1)/(float)NumSections),
							 		 0.0f);
				glVertex3f(verts1[2].x, verts1[2].y, verts1[2].z);
			}

			//Bottom of the sphere.
			for(x=0;x<NumSections;++x)
			{
				verts1[0].x = 0.0f;
				verts1[0].y = -1.0f;
				verts1[0].z = 0.0f;
				verts1[1] = getVertex(2, x);
				if(x < (NumSections-1))
					verts1[2] = getVertex(2, x+1);
				else
					verts1[2] = getVertex(2, 0);

				glNormal3f(0.0f, -1.0f, 0.0f);
				glMultiTexCoord2fARB(GL_TEXTURE0_ARB,
									 (((float)(NumSections-x)+0.5f)/(float)NumSections),
							 		 ((float)(NumSections-1)/(float)NumSections));
				glMultiTexCoord2fARB(GL_TEXTURE1_ARB,
									 (((float)(NumSections-x)+0.5f)/(float)NumSections),
							 		 ((float)(NumSections-1)/(float)NumSections));
				glVertex3f(verts1[0].x, verts1[0].y, verts1[0].z);

				glMultiTexCoord2fARB(GL_TEXTURE0_ARB,
									 ((float)(NumSections-x)/(float)NumSections),
							 		 1.0f);
				glMultiTexCoord2fARB(GL_TEXTURE1_ARB,
									 ((float)(NumSections-x)/(float)NumSections),
							 		 1.0f);
				glVertex3f(verts1[1].x, verts1[1].y, verts1[1].z);

				glMultiTexCoord2fARB(GL_TEXTURE0_ARB,
									 ((float)((NumSections-x)+1)/(float)NumSections),
							 		 1.0f);
				glMultiTexCoord2fARB(GL_TEXTURE1_ARB,
									 ((float)((NumSections-x)+1)/(float)NumSections),
							 		 1.0f);
				glVertex3f(verts1[2].x, verts1[2].y, verts1[2].z);
			}
		}
		glEnd();
	}
	glEndList();
}

//------------------------------------------------------------------------------
void Sphere::graphicDoStuff()
{
	glMatrixMode(GL_MODELVIEW);
	glPushMatrix();

	glScalef(radius, radius, radius);

	glCallList(displayList);

	glPopMatrix();
}

//------------------------------------------------------------------------------
void Sphere::setAttribute(const string& attribute, EventValue& val)
{
	StringParser parser(val);

	if(attribute == "radius")
		radius = val;
	else if(attribute == "solid")
	{
		if(parser.getNumBools() > 0)
			solid = parser.getBool();
	}
}

//------------------------------------------------------------------------------
void Sphere::calcVertex(int x, int y)
{
	ThreeFloats verts1[4];
	ThreeFloats verts2[4];
	ThreeFloats verts3[4];
	ThreeFloats verts4[4];
	ThreeFloats n1;
	ThreeFloats n2;
	ThreeFloats n3;
	ThreeFloats n4;
	ThreeFloats n5;

	//Current quad.
	verts1[0] = getVertex(x+1, y+1);
	verts1[1] = getVertex(x, y+1);
	verts1[2] = getVertex(x, y);
	verts1[3] = getVertex(x+1, y);

	//Next quad.
	verts2[0] = getVertex(x+2, y+1);
	verts2[1] = getVertex(x+1, y+1);
	verts2[2] = getVertex(x+1, y);
	verts2[3] = getVertex(x+2, y);

	//Quad above.
	verts3[0] = getVertex(x+1, y+2);
	verts3[1] = getVertex(x, y+2);
	verts3[2] = getVertex(x, y+1);
	verts3[3] = getVertex(x+1, y+1);

	//Quad above and along.
	verts3[0] = getVertex(x+2, y+2);
	verts3[1] = getVertex(x+1, y+2);
	verts3[2] = getVertex(x+1, y+1);
	verts3[3] = getVertex(x+2, y+1);

	n1 = calcNormal2(verts1[0], verts1[1], verts1[2]);
	n2 = calcNormal2(verts2[0], verts2[1], verts2[2]);
	n3 = calcNormal2(verts3[0], verts3[1], verts3[2]);
	n4 = calcNormal2(verts4[0], verts4[1], verts4[2]);
	n5 = calcSmooth(n1, n2, n3, n4);

	glNormal3f(n5.x, n5.y, n5.z);
	/*glTexCoord2f(((float)(NumSections-(y+1))/(float)NumSections),
				 ((float)(NumSections-x)/(float)NumSections));*/
	glMultiTexCoord2fARB(GL_TEXTURE0_ARB,
						 ((float)(NumSections-(y+1))/(float)NumSections),
				 		 ((float)(NumSections-x)/(float)NumSections));
	glMultiTexCoord2fARB(GL_TEXTURE1_ARB,
						 ((float)(NumSections-(y+1))/(float)NumSections),
				 		 ((float)(NumSections-x)/(float)NumSections));
	glVertex3f(verts1[0].x, verts1[0].y, verts1[0].z);
}

}
