//	SpotLight.cpp - Class representing a spot light.
//  ----------------------------------------------------------------------------
//	This file is part of 'Heilan Browser', an X3D browser specialised for audio
//	work.
//	Copyright (C) 2005-2009  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#include "SpotLight.h"
#include "StringParser.h"
#include "AttributeHelper.h"
#include "Lighting.h"

using namespace std;

namespace HeilanX3D
{

NodeConstructor<SpotLight> SpotLight::constructor;

//------------------------------------------------------------------------------
SpotLight::SpotLight():
ambientIntensity(0.0f),
attenuation(1.0f, 0.0f, 0.0f),
beamWidth(90.0f),
color(1.0f, 1.0f, 1.0f),
cutOffAngle(45.0f),
direction(0.0f, 0.0f, -1.0f),
global(false),
intensity(1.0f),
location(0.0f, 0.0f, 0.0f),
on(true),
radius(100.0f)
{
	setType("SpotLight");
	setVisual(true);
	setShouldBeStart(true);
}

//------------------------------------------------------------------------------
SpotLight::~SpotLight()
{
	
}

//------------------------------------------------------------------------------
void SpotLight::graphicDoStuff()
{
	if(on && !global)
	{
		ThreeFloats colour = color * intensity;

		Lighting::getInstance().addSpotLight(location,
											 attenuation,
											 colour,
											 ambientIntensity,
											 beamWidth,
											 cutOffAngle,
											 direction);
	}
}

//------------------------------------------------------------------------------
void SpotLight::setAttribute(const string& attribute, EventValue& val)
{
	string realAtt = AttributeHelperBase::getAddress(attribute);

	if(realAtt == "ambientIntensity")
		ambientIntensity = val;
	else if(realAtt == "attenuation")
	{
		StringParser parser(val);
		AttributeHelper<ThreeFloats> helper(&attenuation);

		if(helper.isValid(attribute))
			helper.convert(attribute, val);
		else if(parser.getNumNumbers() > 0)
			attenuation = parser.getThreeFloats();
	}
	else if(realAtt == "beamWidth")
	{
		beamWidth = val;
		beamWidth /= 1.570796327f;
		beamWidth *= 90.0f;
	}
	else if(realAtt == "color")
	{
		StringParser parser(val);
		AttributeHelper<ThreeFloats> helper(&color);

		if(helper.isValid(attribute))
			helper.convert(attribute, val);
		else if(parser.getNumNumbers() > 0)
			color = parser.getThreeFloats();
	}
	else if(realAtt == "cutOffAngle")
	{
		cutOffAngle = val;
		cutOffAngle /= 1.570796327f;
		cutOffAngle *= 90.0f;
	}
	else if(realAtt == "direction")
	{
		StringParser parser(val);
		AttributeHelper<ThreeFloats> helper(&direction);

		if(helper.isValid(attribute))
			helper.convert(attribute, val);
		else if(parser.getNumNumbers() > 0)
			direction = parser.getThreeFloats();
	}
	else if(realAtt == "global")
	{
		StringParser parse(val);

		if(parse.getNumBools() > 0)
		{
			global = parse.getBool();

			if(global)
			{
				ThreeFloats colour = color * intensity;

				Lighting::getInstance().addGlobalSpotLight(location,
														   attenuation,
														   colour,
														   ambientIntensity,
														   beamWidth,
														   cutOffAngle,
														   direction);
			}
		}
	}
	else if(realAtt == "intensity")
		intensity = val;
	else if(realAtt == "location")
	{
		StringParser parser(val);
		AttributeHelper<ThreeFloats> helper(&location);

		if(helper.isValid(attribute))
			helper.convert(attribute, val);
		else if(parser.getNumNumbers() > 0)
			location = parser.getThreeFloats();
	}
	else if(realAtt == "on")
	{
		StringParser parse(val);

		if(parse.getNumBools() > 0)
			on = parse.getBool();
	}
	else if(realAtt == "radius")
		radius = val;
}
	
}
