//	StringSensor.cpp - Class representing the StringSensor node.
//  ----------------------------------------------------------------------------
//	This file is part of 'Heilan Browser', an X3D browser specialised for audio
//	work.
//	Copyright (C) 2005-2009  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#include "StringSensor.h"
#include "StringParser.h"

#include <sstream>

using namespace std;

namespace HeilanX3D
{

NodeConstructor<StringSensor> StringSensor::constructor;

//------------------------------------------------------------------------------
StringSensor::StringSensor():
ChildNode(),
deletionAllowed(true),
enabled(true),
isTyping(false)
{
	setType("StringSensor");
}

//------------------------------------------------------------------------------
StringSensor::~StringSensor()
{
	KeyboardInput::getInstance().unregisterCallback(this);
}

//------------------------------------------------------------------------------
void StringSensor::initialise()
{
	KeyboardInput::getInstance().registerCallback(this);
}

//------------------------------------------------------------------------------
void StringSensor::setAttribute(const string& attribute, EventValue& val)
{
	if(attribute == "deletionAllowed")
	{
		StringParser parse(val);

		if(parse.getNumBools() > 0)
			deletionAllowed = parse.getBool();
	}
	else if(attribute == "enabled")
	{
		StringParser parse(val);

		if(parse.getNumBools() > 0)
			enabled = parse.getBool();
	}
}

//------------------------------------------------------------------------------
void StringSensor::keyPressed(const std::string& key)
{
	
}

//------------------------------------------------------------------------------
void StringSensor::keyReleased(const std::string& key)
{
	if(enabled)
	{
		if(!isTyping)
		{
			isTyping = true;
			sendEvent("isActive", "true");
		}

		enteredText += key;
		sendEvent("enteredText", enteredText);
	}
}

//------------------------------------------------------------------------------
void StringSensor::altKeyChanged(bool val)
{
	
}

//------------------------------------------------------------------------------
void StringSensor::ctrlKeyChanged(bool val)
{
	
}

//------------------------------------------------------------------------------
void StringSensor::shiftKeyChanged(bool val)
{
	
}

//------------------------------------------------------------------------------
void StringSensor::actionKeyPressed(KeyboardCallback::ActionKey key)
{
	
}

//------------------------------------------------------------------------------
void StringSensor::actionKeyReleased(KeyboardCallback::ActionKey key)
{
	if(enabled)
	{
		if(isTyping)
		{
			if((key == KeyboardCallback::Backspace) && deletionAllowed)
			{
				enteredText = enteredText.substr(0, enteredText.size()-1);
				sendEvent("enteredText", enteredText);
			}
			else if(key == KeyboardCallback::Return)
			{
				finalText = enteredText;
				sendEvent("finalText", enteredText);
				sendEvent("isActive", "FALSE");
				isTyping = false;
				enteredText = "";
			}
		}
	}
}

}
