//	Switch.cpp - Class representing the Switch parent node.
//  ----------------------------------------------------------------------------
//	This file is part of 'Heilan Browser', an X3D browser specialised for audio
//	work.
//	Copyright (C) 2005-2009  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#include "Switch.h"
#include "StringParser.h"

#include "GLHeaders.h"

using namespace std;

namespace HeilanX3D
{

NodeConstructor<Switch> Switch::constructor;

//------------------------------------------------------------------------------
Switch::Switch():
ParentNode(),
whichChoice(-1)
{
	setType("Switch");
	setVisual(true);
	setAural(true);
}

//------------------------------------------------------------------------------
Switch::~Switch()
{
	
}

//------------------------------------------------------------------------------
void Switch::graphicDoStuff()
{
	const GLfloat diffuse[]={1.0f, 1.0f, 1.0f};
	const GLfloat ambient[]={0.0f, 0.0f, 0.0f};
	const GLfloat emissive[]={0.0f, 0.0f, 0.0f};
	const GLfloat specular[]={0.5f, 0.5f, 0.5f};
	const GLfloat shininess = 0.0f;

	glMatrixMode(GL_MODELVIEW);
	glPushMatrix();


	if((whichChoice > -1) && (whichChoice < getNumChildren()))
		getChild(whichChoice)->graphicDoStuff();


	//Not sure if there's a better way to do this?
	glBindTexture(GL_TEXTURE_2D, 0);
	glColor4f(1.0f, 1.0f, 1.0f, 1.0f);
	glMaterialfv(GL_FRONT, GL_DIFFUSE, diffuse);
	glMaterialfv(GL_FRONT, GL_AMBIENT, ambient);
	glMaterialfv(GL_FRONT, GL_EMISSION, emissive);
	glMaterialf(GL_FRONT, GL_SHININESS, shininess); 
	glMaterialfv(GL_FRONT, GL_SPECULAR, specular);

	glMatrixMode(GL_MODELVIEW);
	glPopMatrix();
}

//------------------------------------------------------------------------------
AmbisonicData *Switch::getAmbisonicAudio(int numSamples, const ThreeFloats& pos)
{
	AmbisonicData *retval = 0;

	if((whichChoice > -1) && (whichChoice < getNumChildren()))
		retval = getChild(whichChoice)->getAmbisonicAudio(numSamples, pos);

	return retval;
}

//------------------------------------------------------------------------------
void Switch::setAttribute(const string& attribute, EventValue& val)
{
	if(attribute == "whichChoice")
		whichChoice = val;
}

}
