//	Switch.h - Class representing the Switch parent node.
//  ----------------------------------------------------------------------------
//	This file is part of 'Heilan Browser', an X3D browser specialised for audio
//	work.
//	Copyright (C) 2005-2009  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#ifndef SWITCH_H_
#define SWITCH_H_

#include "ParentNode.h"

namespace HeilanX3D
{

///	Class representing the Switch parent node.
class Switch : public ParentNode
{
  public:
	///	Constructor.
	Switch();
	///	Destructor.
	~Switch();

	///	Overridden to select which child to display.
	void graphicDoStuff();
	///	Overridden to select which child to play.
	AmbisonicData *getAmbisonicAudio(int numSamples, const ThreeFloats& pos);

	///	Where we react to whichChoice changing.
	void setAttribute(const std::string& attribute, EventValue& val);

	static NodeConstructor<Switch> constructor;
  private:
	///	Implemented to do nothing.
	void audioBlockSizeChanged(int newSize) {};

	///	Which child is currently displayed/played.
	int whichChoice;
};

}

#endif
