//	TextureCoordinate.cpp - Class representing a collection of texture coordinates.
//  ----------------------------------------------------------------------------
//	This file is part of 'Heilan Browser', an X3D browser specialised for audio
//	work.
//	Copyright (C) 2005-2009  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#include "TextureCoordinate.h"
#include "StringParser.h"
#include "AttributeHelper.h"

#include <iostream>

using namespace std;

namespace HeilanX3D
{
	
NodeConstructor<TextureCoordinate> TextureCoordinate::constructor;

//------------------------------------------------------------------------------
TextureCoordinate::TextureCoordinate():
ChildNode(),
valueChanged(false)
{
	setType("TextureCoordinate");
}

//------------------------------------------------------------------------------
TextureCoordinate::~TextureCoordinate()
{
	
}

//------------------------------------------------------------------------------
void TextureCoordinate::setAttribute(const string& attribute, EventValue& val)
{
	int i;
	string realAtt = AttributeHelperBase::getAddress(attribute);

	if(realAtt == "point")
	{
		StringParser parser(val);
		AttributeHelper< vector<TwoFloats> > helper(&coords);

		if(helper.isValid(attribute))
		{
			helper.convert(attribute, val);
			valueChanged = true;
		}
		else
		{
			if(coords.size() > 0)
				coords.clear();
	
			for(i=0;i<parser.getNumTwoFloats();++i)
				coords.push_back(parser.getTwoFloats(i));
	
			valueChanged = true;
		}
	}
}

//------------------------------------------------------------------------------
bool TextureCoordinate::getHasChanged()
{
	bool retval = valueChanged;

	if(valueChanged)
		valueChanged = false;

	return retval;
}

}
