//	TextureCoordinateGenerator.cpp - Class representing a texture coordinate
//								   	 generator.
//  ----------------------------------------------------------------------------
//	This file is part of 'Heilan Browser', an X3D browser specialised for audio
//	work.
//	Copyright (C) 2005-2009  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#include "TextureCoordinateGenerator.h"
#include "StringParser.h"
#include "AttributeHelper.h"

#include "GLHeaders.h"

using namespace std;

namespace HeilanX3D
{

NodeConstructor<TextureCoordinateGenerator> TextureCoordinateGenerator::constructor;

//------------------------------------------------------------------------------
TextureCoordinateGenerator::TextureCoordinateGenerator():
ChildNode(),
mode("SPHERE")
{
	setType("TextureCoordinateGenerator");
	setVisual(true);
}

//------------------------------------------------------------------------------
TextureCoordinateGenerator::~TextureCoordinateGenerator()
{
	
}

//------------------------------------------------------------------------------
void TextureCoordinateGenerator::graphicDoStuff()
{
	if(mode == "SPHERE")
	{
		glTexGeni(GL_S, GL_TEXTURE_GEN_MODE, GL_SPHERE_MAP);
		glTexGeni(GL_T, GL_TEXTURE_GEN_MODE, GL_SPHERE_MAP);
		glTexGeni(GL_R, GL_TEXTURE_GEN_MODE, GL_SPHERE_MAP);
		glTexGeni(GL_Q, GL_TEXTURE_GEN_MODE, GL_SPHERE_MAP);
	}
	else if(mode == "COORD")
	{
		glTexGeni(GL_S, GL_TEXTURE_GEN_MODE, GL_OBJECT_LINEAR);
		glTexGeni(GL_T, GL_TEXTURE_GEN_MODE, GL_OBJECT_LINEAR);
		glTexGeni(GL_R, GL_TEXTURE_GEN_MODE, GL_OBJECT_LINEAR);
		glTexGeni(GL_Q, GL_TEXTURE_GEN_MODE, GL_OBJECT_LINEAR);
	}
	else if(mode == "COORD-EYE")
	{
		glTexGeni(GL_S, GL_TEXTURE_GEN_MODE, GL_EYE_LINEAR);
		glTexGeni(GL_T, GL_TEXTURE_GEN_MODE, GL_EYE_LINEAR);
		glTexGeni(GL_R, GL_TEXTURE_GEN_MODE, GL_EYE_LINEAR);
		glTexGeni(GL_Q, GL_TEXTURE_GEN_MODE, GL_EYE_LINEAR);
	}
}

//------------------------------------------------------------------------------
void TextureCoordinateGenerator::setAttribute(const string& attribute,
											  EventValue& val)
{
	string realAtt = AttributeHelperBase::getAddress(attribute);

	if(realAtt == "mode")
		mode = (string)val;
	else if(realAtt == "parameter")
	{
		StringParser parser(val);
		AttributeHelper< vector<float> > helper(&parameter);

		if(helper.isValid(attribute))
		{
			parameter.clear();
			helper.convert(attribute, val);
		}
		else if(parser.getNumNumbers() > 0)
		{
			parameter.clear();
			parameter.push_back(parser.getFloat());
		}
	}
}

}
