//	TextureTransform.cpp - Class representing a texture transformation.
//  ----------------------------------------------------------------------------
//	This file is part of 'Heilan Browser', an X3D browser specialised for audio
//	work.
//	Copyright (C) 2005-2009  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#include "TextureTransform.h"
#include "StringParser.h"
#include "AttributeHelper.h"

#include "GLHeaders.h"

using namespace std;

namespace HeilanX3D
{
	
NodeConstructor<TextureTransform> TextureTransform::constructor;

//------------------------------------------------------------------------------
TextureTransform::TextureTransform():
rotation(0.0f)
{
	setType("TextureTransform");
	setVisual(true);

	scale.x = 1.0f;
	scale.y = 1.0f;
}

//------------------------------------------------------------------------------
TextureTransform::~TextureTransform()
{
	
}

//------------------------------------------------------------------------------
void TextureTransform::graphicDoStuff()
{
	//Setup matrix.
	glMatrixMode(GL_TEXTURE);
	glPushMatrix();

	//Now perform various translations etc.

	//Apply the rotation.
	glRotatef(rotation, 1.0f, 1.0f, 1.0f);

	//Apply the translation, offset from center.
	glTranslatef(center.x+translation.x,
				 center.y+translation.y,
				 0.0f);

	//Now Apply our scale operation.
	glScalef(scale.x, scale.y, 1.0f);

	//Just in case...
	glMatrixMode(GL_MODELVIEW);
}

//------------------------------------------------------------------------------
void TextureTransform::setAttribute(const string& attribute, EventValue& val)
{
	string realAtt = AttributeHelperBase::getAddress(attribute);

	if(realAtt == "center")
	{
		StringParser parser(val);
		AttributeHelper<TwoFloats> helper(&center);

		if(helper.isValid(attribute))
			helper.convert(attribute, val);
		else
		{
			if(parser.getNumTwoFloats() > 0)
				center = parser.getTwoFloats();
		}
	}
	else if(realAtt == "rotation")
		rotation = val;
	else if(realAtt == "scale")
	{
		StringParser parser(val);
		AttributeHelper<TwoFloats> helper(&scale);

		if(helper.isValid(attribute))
			helper.convert(attribute, val);
		else
		{
			if(parser.getNumTwoFloats() > 0)
				scale = parser.getTwoFloats();
		}
	}
	else if(realAtt == "translation")
	{
		StringParser parser(val);
		AttributeHelper<TwoFloats> helper(&translation);

		if(helper.isValid(attribute))
			helper.convert(attribute, val);
		else
		{
			if(parser.getNumTwoFloats() > 0)
				translation = parser.getTwoFloats();
		}
	}
}

}
