//	TextureTransform.h - Class representing a texture transformation.
//  ----------------------------------------------------------------------------
//	This file is part of 'Heilan Browser', an X3D browser specialised for audio
//	work.
//	Copyright (C) 2005-2009  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#ifndef TEXTURETRANSFORM_H_
#define TEXTURETRANSFORM_H_

#include "AbstractNode.h"
#include "TwoFloats.h"

namespace HeilanX3D
{

///	Class representing a basic texture.
class TextureTransform : public ChildNode
{
  public:
	///	Constructor.
	TextureTransform();
	///	Destructor.
	~TextureTransform();

	///	Draws the TextureTransform.
	void graphicDoStuff();
	///	Not used, overridden to do nothing.
	/*AmbisonicData *audioDoStuff(int numSamples,
								const ThreeFloats& pos) {return 0;};*/

	///	Not used, overridden to do nothing.
	void setBlockSize(int newSize) {};

	///	Used to set the material's attributes.
	void setAttribute(const std::string& attribute, EventValue& val);

	static NodeConstructor<TextureTransform> constructor;
  private:
	///	The centre point for any transformations.
	TwoFloats center;
	///	The angle for rotating the texture.
	float rotation;
	///	For scaling the texture.
	TwoFloats scale;
	///	For translating (moving) the texture.
	TwoFloats translation;
};

}

#endif
