//	TimeSensor.h - Node used to output time-based events.
//  ----------------------------------------------------------------------------
//	This file is part of 'Heilan Browser', an X3D browser specialised for audio
//	work.
//	Copyright (C) 2005-2009  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#ifndef TIMESENSOR_H_
#define TIMESENSOR_H_

#include "AbstractNode.h"

namespace HeilanX3D
{

///	Node used to output time-based events.
/*!
	\todo Test that all the various events are output correctly.
 */
class TimeSensor : public ChildNode
{
  public:
	///	Constructor.
	TimeSensor();
	///	Destructor.
	~TimeSensor();

	///	Overridden to do nothing.
	void graphicDoStuff() {};
	///	All our time calculations take place here.
	//AmbisonicData *audioDoStuff(int numSamples, const ThreeFloats& pos);
	AmbisonicData *getMonoAudio(int numSamples);

	///	Overridden to do nothing.
	void setBlockSize(int newSize) {};
	///	We need the samplerate to calculate the current time.
	void setSamplerate(float val);

	///	Called to set the value of the named attribute.
	void setAttribute(const std::string& attribute, EventValue& val);

	static NodeConstructor<TimeSensor> constructor;
  private:
	///	The current samplerate.
	float samplerate;

	///	The current cycle interval (measured in seconds).
	double cycleInterval;
	///	If the sensor should currently be enabled.
	bool enabled;
	///	If the sensor should be looping or not.
	bool loop;
	///	The time at which the sensor should pause operation.
	double pauseTime;
	///	The time at which the sensor should resume operation after pausing.
	double resumeTime;
	///	The time at which the sensor should start operation.
	double startTime;
	///	The time at which the sensor should stop operation.
	double stopTime;

	//The following don't strictly need to be class members, but I kept them
	//here for completeness.
	///	Generated at the start of each cycle.
	double cycleTime;
	///	Generated as long as the sensor is active.
	/*!
		Contains the total number of seconds the sensor has been active for
		(i.e. it doesn't reset after pausing and resuming).
	 */
	double elapsedTime;
	///	Generated as long as the sensor is active.
	/*!
		Contains the current position within the sensor's cycle.
	 */
	float fractionChanged;
	///	Generated when the sensor is activated.
	bool isActive;
	///	Generated when the sensor is paused.
	bool isPaused;
	///	Generated as long as the sensor is active.
	/*!
		Contains the current time.
	 */
	double time;

	///	1/samplerate - simple optimisation.
	double invSamplerate;
	///	Current cycle.
	int currentCycle;
	///	The time of the last cycle event.
	double lastCycle;
	///	So we can tell if we've passed the end of a non-looping cycle.
	bool cyclesFinished;
	///	Counter used to determine when to send the continuous signals.
	/*!
		i.e. elapsedTime, fractionChanged, time.
	 */
	int contCounter;
};

}

#endif
