//	TimeTrigger.h - Class representing the TimeTrigger node.
//  ----------------------------------------------------------------------------
//	This file is part of 'Heilan Browser', an X3D browser specialised for audio
//	work.
//	Copyright (C) 2005-2009  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#ifndef TIMETRIGGER_H_
#define TIMETRIGGER_H_

#include "AbstractNode.h"

namespace HeilanX3D
{

///	Class representing the TimeTrigger node.
class TimeTrigger : public ChildNode
{
  public:
	///	Constructor.
	TimeTrigger();
	///	Destructor.
	~TimeTrigger();

	///	Over-ridden to do nothing.
	void graphicDoStuff() {};

	///	Used to determine the time.
	AmbisonicData *getMonoAudio(int numSamples);
	///	Over-ridden to do nothing.
	void setBlockSize(int newSize) {};
	///	We need to know the samplerate to calculate the time.
	void setSamplerate(float val);

	///	Where the main work this class does happens.
	void setAttribute(const std::string& attribute, EventValue& val);

	static NodeConstructor<TimeTrigger> constructor;
  private:
	///	The current time.
	double time;
	///	The inverse of the samplerate.
	double invSamplerate;
};

}

#endif
