//	Transform.h - Class representing the Transform parent node.
//  ----------------------------------------------------------------------------
//	This file is part of 'Heilan Browser', an X3D browser specialised for audio
//	work.
//	Copyright (C) 2005-2009  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#ifndef TRANSFORM_H_
#define TRANSFORM_H_

#include "ParentNode.h"
#include "Quaternion.h"

namespace HeilanX3D
{

///	Class representing the Transform parent node.
class Transform : public ParentNode
{
  public:
	///	Constructor.
	Transform();
	///	Destructor.
	~Transform();

	///	Updates the position for any child nodes.
	AmbisonicData *getAmbisonicAudio(int numSamples, const ThreeFloats& newPos);

	///	Used to set the transform's attributes..
	void setAttribute(const std::string& attribute, EventValue& val);

	static NodeConstructor<Transform> constructor;
  private:
	///	We override this so we can save and restore the current OpenGL matrix.
	void preChildDraw();
	///	We override this so we can save and restore the current OpenGL matrix.
	void postChildDraw();
	///	We override this so we can update our position for our children.
	//AmbisonicData *preChildAudio(int numSamples, const ThreeFloats& newPos);

	///	Implemented to do nothing.
	void audioBlockSizeChanged(int newSize) {};

	///	The centre of the transform operation.
	ThreeFloats center;
	///	The rotation of the transform.
	Quaternion rotation;
	///	The scale of the transform.
	ThreeFloats scale;
	///	The orientation of the scale operation.
	Quaternion scaleOrientation;
	///	The translation of the transform.
	ThreeFloats translation;
	///	The centre of the bounding box.
	ThreeFloats bboxCenter;
	///	The size of the bounding box.
	ThreeFloats bboxSize;
};

}

#endif
