//	Viewpoint.cpp - Viewpoint node.
//  ----------------------------------------------------------------------------
//	This file is part of 'Heilan Browser', an X3D browser specialised for audio
//	work.
//	Copyright (C) 2005-2009  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#include "Viewpoint.h"
#include "StringParser.h"
#include "AttributeHelper.h"

using namespace std;

namespace HeilanX3D
{

NodeConstructor<Viewpoint> Viewpoint::constructor;

//------------------------------------------------------------------------------
Viewpoint::Viewpoint():
BindableNode(),
timeToGetHere(4.0f),
pauseTime(0.0f)
{
	setType("Viewpoint");
	setVisual(false);
	setAural(false);
}

//------------------------------------------------------------------------------
Viewpoint::~Viewpoint()
{
	BindableNodeSingleton::getInstance()->unregisterNode(this);
}

//------------------------------------------------------------------------------
void Viewpoint::initialise()
{
	//Register with the BindableNodeSingleton.
	BindableNodeSingleton::getInstance()->registerNode(this);
}

//------------------------------------------------------------------------------
void Viewpoint::setAttribute(const string& attribute, EventValue& val)
{
	string realAtt = AttributeHelperBase::getAddress(attribute);

	if(realAtt == "position")
	{
		StringParser parser(val);
		AttributeHelper<ThreeFloats> helper(&position);

		if(helper.isValid(attribute))
			helper.convert(attribute, val);
		else if(parser.getNumThreeFloats() > 0)
			position = parser.getThreeFloats();
	}
	else if(realAtt == "orientation")
	{
		FourFloats tempf;
		ThreeFloats temp3f;
		StringParser parser(val);
		AttributeHelper<FourFloats> helper(&tempf);

		if(helper.isValid(attribute))
		{
			helper.convert(attribute, val);
			temp3f = ThreeFloats(tempf.r, tempf.x, tempf.y);
			rotation.setFromAxisAngle(tempf.z*0.017453292519943295769236907684886,
									  temp3f);
		}
		else if(parser.getNumFourFloats() > 0)
		{
			tempf = parser.getFourFloats();
			temp3f = ThreeFloats(tempf.r, tempf.x, tempf.y);
			rotation.setFromAxisAngle(tempf.z*0.017453292519943295769236907684886,
									  temp3f);
		}
	}
}

//------------------------------------------------------------------------------
void Viewpoint::setMetadata(const std::string& name, const std::string& value)
{
	if(name == "heilan_timeToGetHere")
	{
		StringParser parser(value);

		if(parser.getNumNumbers() > 0)
			timeToGetHere = parser.getFloat();
	}
	else if(name == "heilan_pauseTime")
	{
		StringParser parser(value);

		if(parser.getNumNumbers() > 0)
			pauseTime = parser.getFloat();
	}
}

}
