//	Viewpoint.h - Viewpoint node.
//  ----------------------------------------------------------------------------
//	This file is part of 'Heilan Browser', an X3D browser specialised for audio
//	work.
//	Copyright (C) 2005-2009  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#ifndef VIEWPOINT_H_
#define VIEWPOINT_H_

#include "BindableNode.h"

namespace HeilanX3D
{

///	Viewpoint node.
class dllStuff Viewpoint : public BindableNode
{
  public:
	///	Constructor.
	Viewpoint();
	///	Destructor.
	~Viewpoint();

	///	Initialises various bits and pieces.
	void initialise();

	///	Overridden to do nothing.
	void graphicDoStuff() {};
	///	Overridden to do nothing.
	/*AmbisonicData *audioDoStuff(int numSamples,
								const ThreeFloats& pos) {return 0;};*/
	///	Overridden to do nothing.
	void setBlockSize(int newSize) {};

	///	Called to set the value of the named attribute.
	void setAttribute(const std::string& attribute, EventValue& val);

	///	Used to set the timeToGetHere and pauseTime variables.
	void setMetadata(const std::string& name, const std::string& value);

	///	Returns this viewpoint's position.
	const ThreeFloats& getPosition() const {return position;};
	///	Returns this viewpoint's rotation.
	const Quaternion& getRotation() const {return rotation;};

	///	Returns how long it should take the camera to move from the previous viewpoint to this one.
	float getTimeToGetHere() const {return timeToGetHere;};
	///	Returns how long the camera should pause here before it moves onto the next viewpoint.
	float getPauseTime() const {return pauseTime;};

	static NodeConstructor<Viewpoint> constructor;
  private:
	///	This viewpoint's position.
	ThreeFloats position;
	///	This viewpoint's rotation.
	Quaternion rotation;

	///	The time it should take to move from the previous viewpoint to this one.
	/*!
		In seconds.
	 */
	float timeToGetHere;
	///	How long the camera should pause here before it moves onto the next viewpoint.
	/*!
		In seconds.
	 */
	float pauseTime;
};

}

#endif
