//	Viewport.cpp - Class representing the Viewport parent node.
//  ----------------------------------------------------------------------------
//	This file is part of 'Heilan Browser', an X3D browser specialised for audio
//	work.
//	Copyright (C) 2005-2009  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#include "Viewport.h"
#include "StringParser.h"
#include "AttributeHelper.h"
#include "Camera.h"

#include "GLHeaders.h"
#include <iostream>

using namespace std;

namespace HeilanX3D
{

NodeConstructor<Viewport> Viewport::constructor;

//------------------------------------------------------------------------------
Viewport::Viewport():
ParentNode()
{
	setType("Viewport");
	setVisual(true);
	setAural(true); //?
}

//------------------------------------------------------------------------------
Viewport::~Viewport()
{
	
}

//------------------------------------------------------------------------------
void Viewport::setAttribute(const string& attribute, EventValue& val)
{
	string realAtt = AttributeHelperBase::getAddress(attribute);

	if(realAtt == "clipBoundary")
	{
		StringParser parser(val);
		AttributeHelper<FourFloats> helper(&clipBoundary);

		if(helper.isValid(attribute))
			helper.convert(attribute, val);
		else if(parser.getNumFourFloats() > 0)
			clipBoundary = parser.getFourFloats();
	}
}

//------------------------------------------------------------------------------
void Viewport::preChildDraw()
{
	int x, y, width, height;

	x = (int)((clipBoundary.r*0.01f) *
			  (float)Camera::getInstance()->getViewWidth());
	y = (int)((clipBoundary.y*0.01f) *
			  (float)Camera::getInstance()->getViewHeight());
	width = (int)((clipBoundary.x*0.01f) *
				  (float)Camera::getInstance()->getViewWidth());
	width -= x;
	height = (int)((clipBoundary.z*0.01f) *
				   (float)Camera::getInstance()->getViewHeight());
	height -= y;

	glGetIntegerv(GL_VIEWPORT, prevView);
	glViewport(x, y, width, height);
}

//------------------------------------------------------------------------------
void Viewport::postChildDraw()
{
	glViewport(prevView[0], prevView[1], prevView[2], prevView[3]);
}

}
