//	Viewport.h - Class representing the Viewport parent node.
//  ----------------------------------------------------------------------------
//	This file is part of 'Heilan Browser', an X3D browser specialised for audio
//	work.
//	Copyright (C) 2005-2009  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#ifndef VIEWPORT_H_
#define VIEWPORT_H_

#include "ParentNode.h"
#include "GLHeaders.h"

namespace HeilanX3D
{

///	Class representing the Viewport parent node.
class Viewport : public ParentNode
{
  public:
	///	Constructor.
	Viewport();
	///	Destructor.
	~Viewport();

	///	So we can set clipBoundary.
	void setAttribute(const std::string& attribute, EventValue& val);

	static NodeConstructor<Viewport> constructor;
  private:
	///	To setup the viewport.
	void preChildDraw();
	///	To take down the viewport.
	void postChildDraw();

	///	Implemented to do nothing.
	void audioBlockSizeChanged(int newSize) {};

	///	The clipping boundary to use.
	FourFloats clipBoundary;

	///	Stores the previous viewport boundary so we can restore it in postChildDraw().
	GLint prevView[4];
};

}

#endif
