//	AudioThread.cpp - Thread class used to parallelise audio for individual
//					  nodes.
//  ----------------------------------------------------------------------------
//	This file is part of 'Heilan Browser', an X3D browser specialised for audio
//	work.
//	Copyright (C) 2005-2009  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#include "AudioThread.h"
#include "ParentNode.h"
#include "SDL.h"

namespace HeilanX3D
{

//------------------------------------------------------------------------------
AudioThread::AudioThread(AbstractNode *parallelNode, ParentNode *parentNode):
Thread(),
node(parallelNode),
parent(parentNode),
timeToStop(false),
numSamples(0)
{
	startThread(this);
}

//------------------------------------------------------------------------------
AudioThread::~AudioThread()
{
	timeToStop = true;
}

//------------------------------------------------------------------------------
bool AudioThread::run()
{
	//Wait until we've got some audio to process.
	semaphore.wait();
	while(!timeToStop)
	{
		//Guard audio operations.
		critSec.lock();

		//Do the actual work.
		//parent->parallelAudioReady(node->audioDoStuff(numSamples, position));
		parent->parallelAudioReady(node->getAmbisonicAudio(numSamples, position));

		critSec.unlock();

		//At the end of the loop in case the reason waiter.signal() was called
		//is that timeToStop's been set true.
		semaphore.wait();
	}

	return true;
}

//------------------------------------------------------------------------------
void AudioThread::getAudioBlock(int numSamps, const ThreeFloats& pos)
{
	critSec.lock();

	//Set numSamples and position.
	numSamples = numSamps;
	position = pos;
	//Tell waiter to stop waiting.
	//waiter.signal();
	semaphore.unlock();

	critSec.unlock();
}

}
