//	FileFinder.cpp - Singleton class to use when looking for a file.
//  ----------------------------------------------------------------------------
//	This file is part of 'Heilan Browser', an X3D browser specialised for audio
//	work.
//	Copyright (C) 2005-2009  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#include "FileFinder.h"
#include "MFString.h"

#include <sstream>
#include <iostream>

#ifdef WIN32
#include <shlobj.h> //Do I need both of these?
#include <direct.h>
#else
#include <sys/stat.h>
#endif

using namespace std;

namespace HeilanX3D
{

//------------------------------------------------------------------------------
FileFinder& FileFinder::getInstance()
{
	static FileFinder retval;

	return retval;
}

//------------------------------------------------------------------------------
string FileFinder::findFile(const string& path, bool storePath)
{
	int j;
	bool existsTest = false;
	string retval;
	string tempstr, tempstr3;
	unsigned int i;
	unsigned int sepPos;
	stringstream tempstr2;

	MFString tempMF(path);

	critSec.lock();
	{
		for(j=0;j<tempMF.getNumStrings();++j)
		{
			if(exists(tempMF.getString(j)))
			{
				retval = tempMF.getString(j);

				if(storePath)
				{
					sepPos = retval.find_last_of(separator);

					if(sepPos != string::npos)
						paths.push_back(retval.substr(0, sepPos));
				}
				break;
			}
			else
			{
				tempstr3 = tempMF.getString(j);

				sepPos = tempstr3.find_last_of(separator);

				if(sepPos != string::npos)
					tempstr = tempstr3.substr(0, sepPos);
				else
					tempstr = tempstr3;

				for(i=0;i<paths.size();++i)
				{
					tempstr2.str("");
					tempstr2 << paths[i];
					tempstr2 << separator << tempstr;

					if(exists(tempstr2.str()))
					{
						retval = tempstr2.str();
						existsTest = true;
						cout << "FileFinder: Using " << tempstr2.str() << " as the absolute path." << endl;
						break;
					}
				}
				if(existsTest)
					break;
			}
		}
	}
	critSec.unlock();

	return retval;
}

//------------------------------------------------------------------------------
void FileFinder::clearStoredPaths()
{
	critSec.lock();
	{
		paths.clear();
	}
	critSec.unlock();
}

//------------------------------------------------------------------------------
FileFinder::~FileFinder()
{
	
}

//------------------------------------------------------------------------------
FileFinder::FileFinder()
{
#ifdef WIN32
	separator = '\\';
#else
	separator = '/';
#endif
}

//------------------------------------------------------------------------------
bool FileFinder::exists(const string& path)
{
	bool retval = false;

#ifdef WIN32
	DWORD dwAttr;

	dwAttr = GetFileAttributes(path.c_str());
	if(dwAttr == 0xffffffff) //i.e. it's not there.
		cout << "FileFinder: Could not find " << path << endl;
	else
		retval = true;
#else
	struct stat fileInfo;

	if(stat(path.c_str(), &fileInfo) != 0)
		cout << "FileFinder: Could not find " << path << endl;
	else
		retval = true;
#endif

	return retval;
}

}
