//	NodeFactory.cpp - Singleton class used to construct nodes from the text in
//					  an X3D file.
//  ----------------------------------------------------------------------------
//	This file is part of 'Heilan Browser', an X3D browser specialised for audio
//	work.
//	Copyright (C) 2005-2009  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#include "NodeFactory.h"
#include "AbstractNode.h"
#include <iostream>
#include <cassert>

using namespace std;

namespace HeilanX3D
{

//------------------------------------------------------------------------------
NodeFactory& NodeFactory::getInstance()
{
	static NodeFactory instance;

	return instance;
}

//------------------------------------------------------------------------------
AbstractNode *NodeFactory::createNode(const std::string& text) const
{
	NodeConstructors::const_iterator it;

	it = nodes.find(text);

	if(it != nodes.end())
		return it->second->createInstance();
	else
		return 0;
}

//------------------------------------------------------------------------------
void NodeFactory::registerNode(NodeConstructorBase *constructor)
{
	pair<NodeConstructors::iterator, bool> it;

	//Make sure nodes registered always have a valid type name.
	assert(constructor->getType() != string(""));

	it = nodes.insert(make_pair(constructor->getType(), constructor));
	if(!it.second)
		cout << "Could not insert node " << constructor->getType() << endl;
}

//------------------------------------------------------------------------------
void NodeFactory::unregisterAll()
{
	nodes.clear();
}

//------------------------------------------------------------------------------
void NodeFactory::printNodes() const
{
	NodeConstructors::const_iterator it;

	for(it=nodes.begin();it!=nodes.end();++it)
		cout << it->second->getType() << endl;
}

//------------------------------------------------------------------------------
NodeFactory::NodeFactory()
{
	
}

//------------------------------------------------------------------------------
NodeFactory::~NodeFactory()
{
	
}

}
