//	NodeFactory.h - Singleton class used to construct nodes from the text in an
//					X3D file.
//  ----------------------------------------------------------------------------
//	This file is part of 'Heilan Browser', an X3D browser specialised for audio
//	work.
//	Copyright (C) 2005-2009  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#ifndef NODEFACTORY_H_
#define NODEFACTORY_H_

//Stupid MSVC v6 warning about the STL...
#ifdef WIN32
#pragma warning(disable:4786)
#endif

#include "DllStuff.h"

#include <string>
#include <map>

namespace HeilanX3D
{

class AbstractNode;
class NodeConstructorBase;

///	Singleton class used to construct nodes from the text in an X3D file.
class dllStuff NodeFactory
{
  public:
	///	Returns the sole instance of NodeFactory.
	static NodeFactory& getInstance();

	///	Constructs a new AbstractNode subclass according to text.
	/*!
  		\param text The name of the node as it appears in the X3D file.

		\return 0 if it fails.
	 */
	AbstractNode *createNode(const std::string& text) const;

	///	Registers a NodeConstructor with the factory.
	void registerNode(NodeConstructorBase *constructor);
	///	Unregisters all NodeConstructors.
	void unregisterAll();

	///	Outputs the types of all the registered nodes to the command line.
	void printNodes() const;
	///	Returns the number of registered node types.
	int getNumNodes() const {return nodes.size();};

	//Stupid fix for MSVC v6.
#ifdef WIN32
	///	Destructor.
	~NodeFactory();
#endif
  private:
	///	Constructor.
	NodeFactory();
#ifndef WIN32
	///	Destructor.
	~NodeFactory();
#endif

	typedef std::map<std::string, NodeConstructorBase *> NodeConstructors;
	///	Holds all our currently-available node types.
	NodeConstructors nodes;
};

}

#endif
