//	OscRecorder.h - Singleton class used to record OSC messages.
//  ----------------------------------------------------------------------------
//	This file is part of 'Heilan Browser', an X3D browser specialised for audio
//	work.
//	Copyright (C) 2005-2009  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#ifndef OSCRECORDER_H_
#define OSCRECORDER_H_

#include "OSCMessage.h"

#include <string>
#include <fstream>

///	Singleton class used to record OSC messages.
class OscRecorder
{
  public:
	///	Returns the sole instance of OscRecorder.
	static OscRecorder *getInstance();

	///	Writes the message passed in to a file.
	/*!
		\param address The OSC address of the node sent to.
		\param param The name of the parameter to be sent to.
		\param value The value to be sent to the address and param.
	 */
	void writeMessage(const std::string& address,
					  const std::string& param,
					  const std::string& value);

	///	Used to keep track of what sample we're at in the audio.
	/*!
		\param val The amount to add to sampleVal - this should be the number
		of samples in the current audio block.
	 */
	void updateSampleVal(unsigned int val);

	///	Sets the file to be written to.
	/*!
		The class will not write anything unless this is first called.
	 */
	void setFile(const std::string& file);

#ifdef WIN32
	///	Stupid fix for MSVC v6.
	~OscRecorder();
#endif
  private:
	///	Constructor.
	OscRecorder();
#ifndef WIN32
	///	Stupid fix for MSVC v6.
	~OscRecorder();
#endif

	///	The file to write to.
	std::string filepath;
	///	The file stream to write with.
	std::ofstream outFile;

	///	What sample we're at in the audio thread.
	unsigned int sampleVal;
};

#endif
