//	SceneConstructor.h - Class to handle certain scene construction operations.
//  ----------------------------------------------------------------------------
//	This file is part of 'Heilan Browser', an X3D browser specialised for audio
//	work.
//	Copyright (C) 2005-2009  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#ifndef SCENECONSTRUCTOR_H_
#define SCENECONSTRUCTOR_H_

//Stupid MSVC v6 warning about the STL...
#ifdef WIN32
#pragma warning(disable:4786)
#endif

#include <stack>

namespace HeilanX3D
{

class ParentNode;

///	Class to handle certain scene construction operations.
/*!
	The main purpose of this class is to allow children nodes to be added to
	their parents in a simple fashion, the point being that in the
	sceneConstructionEnter() method, ParentNodes call
	SceneConstructor::pushParent(), and then in sceneConstructionLeave(), they
	call SceneConstructor::popParent().  The X3DFileLoader loader class will
	then call SceneConstructor::getCurrentParent() to determine the current
	(topmost) parent node to add child nodes to.
 */
class SceneConstructor
{
  public:
	///	Constructor.
	SceneConstructor();
	///	Destructor.
	~SceneConstructor();

	///	Call this to push node onto the parentNode stack.
	void pushParent(ParentNode *node);
	///	Call this to pop the topmost node from the parentNode stack.
	void popParent();

	///	Called by X3DFileLoader to set the root node of the scene.
	void setRoot(ParentNode *node);

	///	X3DFileLoader calls this to determine which ParentNode to add children to.
	ParentNode *getCurrentParent() const;
	///	Returns the root node of the scene.
	ParentNode *getRoot() const {return rootNode;};
  private:
	///	The stack of parent nodes we keep.
	std::stack<ParentNode *> parentNodes;
	///	Holds the root node of the scene.
	ParentNode *rootNode;
};

}

#endif
