//	ExamineNavigation.h - 'Examine'-type navigation class.
//  ----------------------------------------------------------------------------
//	This file is part of 'Heilan Browser', an X3D browser specialised for audio
//	work.
//	Copyright (C) 2005-2009  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#ifndef EXAMINENAVIGATION_H_
#define EXAMINENAVIGATION_H_

#include "Navigation.h"

namespace HeilanX3D
{

///	'Examine'-type navigation class.
class ExamineNavigation : public Navigation
{
  public:
	///	Constructor.
	ExamineNavigation();
	///	Destructor.
	~ExamineNavigation();

	///	Called on a mouse button down event.
	void mouseDown(int button, int x, int y);
	///	Called on a mouse button move event.
	void mouseMove(int x, int y);
	///	Called on a mouse button up event.
	void mouseUp(int button, int x, int y);
	///	Called on a mouse wheel up event.
	void mouseWheelUp();
	///	Called on a mouse wheel down event.
	void mouseWheelDown();

	///	Called on a key down event.
	void keyDown(SDLKey key);
	///	Called on a key up event.
	void keyUp(SDLKey key);

	///	Used to make sure we update our position when the Viewpoint's position changes.
	void screenRefresh();

	///	Returns the current position.
	ThreeFloats getPosition() const;
	///	Returns the current rotation.
	Quaternion getRotation() const;

	static NavigationConstructor<ExamineNavigation> constructor;
  private:
	///	Helper method to update the Camera's position correctly.
	void updateCamera();

	///	Whether or not the left mouse button is currently down.
	bool leftIsDown;
	///	Whether or not the middle mouse button is currently down.
	bool middleIsDown;
	///	Whether or not the right mouse button is currently down.
	bool rightIsDown;

	///	The last x-position of the mouse.
	int lastX;
	///	The last y-position of the mouse.
	int lastY;

	///	Current x-axis rotation.
	float xRot;
	///	Current y-axis rotation.
	float yRot;
	///	Current distance to the point.
	float distance;
	///	Current panning, x.
	float xPan;
	///	Current panning, y.
	float yPan;

	///	True if we've just been constructed.
	bool start;

	///	Used to update our position etc. according to the bound Viewpoint's.
	ThreeFloats viewpointPos;
	///	Used to update our position etc. according to the bound Viewpoint's.
	Quaternion viewpointRot;
};

}

#endif
