//	Navigation.cpp - Base class for all classes controlling how the camera
//					 moves.
//  ----------------------------------------------------------------------------
//	This file is part of 'Heilan Browser', an X3D browser specialised for audio
//	work.
//	Copyright (C) 2005-2009  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#include "Navigation.h"
#include <cassert>

using namespace std;

namespace HeilanX3D
{

//------------------------------------------------------------------------------
//------------------------------------------------------------------------------
Navigation::Navigation():
offline(false)
{
	
}

//------------------------------------------------------------------------------
NavigationFactory *NavigationFactory::getInstance()
{
	static NavigationFactory retval;

	return &retval;
}

//------------------------------------------------------------------------------
Navigation *NavigationFactory::createNav(const std::string& text) const
{
	map<string, NavigationConstructorBase *>::const_iterator it;

	it = navTypes.find(text);

	if(it != navTypes.end())
		return it->second->createInstance();
	else
		return 0;
}

//------------------------------------------------------------------------------
void NavigationFactory::registerNavigation(const std::string text,
										   NavigationConstructorBase *constructor)
{
	map<string, NavigationConstructorBase *>::const_iterator it;

	it = navTypes.find(text);

	//Make sure we're not trying to load two Navigations with the same name.
	assert(it == navTypes.end());

	navTypes.insert(make_pair(text, constructor));
}

//------------------------------------------------------------------------------
NavigationFactory::NavigationFactory()
{
	
}

//------------------------------------------------------------------------------
NavigationFactory::~NavigationFactory()
{
	
}

}
