//	CustomSpeakerConfig.cpp - Loads a speaker configuration from a file.
//  ----------------------------------------------------------------------------
//	This file is part of 'Heilan Browser', an X3D browser specialised for audio
//	work.
//	Copyright (C) 2005-2009  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#include "CustomSpeakerConfig.h"
#include "tinyxml.h"
#include <cmath>
#include <iostream>

using namespace std;

//------------------------------------------------------------------------------
CustomSpeakerConfig::CustomSpeakerConfig(const std::string& filepath):
SpeakerConfig()
{
	//float h = 1.0f;
	//float h2;
	double theta, phi;
	AmbisonicData tempAmbi;
	vector<AmbisonicData> tempVec;
	TiXmlElement *tempElement;
	TiXmlDocument doc(filepath);
	doc.LoadFile();
	TiXmlElement *rootElement = doc.FirstChildElement("HeilanSpeakerConfiguration");

	if(rootElement)
	{
		int numSpeakers = 0;
		//Angle notation.
		for(tempElement=rootElement->FirstChildElement("Speaker");
			tempElement;
			tempElement=tempElement->NextSiblingElement())
		{
			++numSpeakers;
		}
		for(tempElement=rootElement->FirstChildElement("Speaker");
			tempElement;
			tempElement=tempElement->NextSiblingElement())
		{
			tempElement->Attribute("hAngle", &theta);
			tempElement->Attribute("vAngle", &phi);

			theta = theta*(3.141592653f/180.0f);
			phi = phi*(3.141592653f/180.0f);

			tempAmbi.X = 0.707f * 0.707f * cosf(theta) * cosf(phi);
			tempAmbi.Y = 0.707f * 0.707f * sinf(theta) * cosf(phi);
			tempAmbi.Z = 0.707f * 0.707f * sinf(phi) * cosf(theta); //* cos(theta)?
			/*tempAmbi.X = 0.5f * cosf(theta) * cosf(phi);
			tempAmbi.Y = 0.5f * sinf(theta) * cosf(phi);
			tempAmbi.Z = 0.5f * sinf(phi) * cosf(theta); //* cos(theta)?*/
			tempAmbi.W = (0.707f/((float)numSpeakers*0.5f));
			cout << "W: " << tempAmbi.W;
			cout << " X: " << tempAmbi.X;
			cout << " Y: " << tempAmbi.Y;
			cout << " Z: " << tempAmbi.Z << endl;

			tempVec.push_back(tempAmbi);
		}

		//Ambisonic component notation.
		for(tempElement=rootElement->FirstChildElement("SpeakerComponent");
			tempElement;
			tempElement=tempElement->NextSiblingElement())
		{
			double w, x ,y, z;

			tempElement->Attribute("W", &w);
			tempElement->Attribute("X", &x);
			tempElement->Attribute("Y", &y);
			tempElement->Attribute("Z", &z);
			tempAmbi.W = w;
			tempAmbi.X = x;
			tempAmbi.Y = y;
			tempAmbi.Z = z;

			cout << "X: " << tempAmbi.X;
			cout << " Y: " << tempAmbi.Y;
			cout << " Z: " << tempAmbi.Z;
			cout << " W: " << tempAmbi.W << endl;

			tempVec.push_back(tempAmbi);
		}

		setDecodingMatrix(tempVec);
	}
}
