//	LibSndFileLoader.cpp - libsndfile-based sound file loader.
//  ----------------------------------------------------------------------------
//	This file is part of 'Heilan Browser', an X3D browser specialised for audio
//	work.
//	Copyright (C) 2005-2009  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#include "LibSndFileLoader.h"
#include "FileFinder.h"

#include <iostream>

using namespace std;

namespace HeilanX3D
{
	
SFLoaderConstructor<LibSndFileLoader> LibSndFileLoader::constructor;

//------------------------------------------------------------------------------
LibSndFileLoader::LibSndFileLoader(const string& filePath):
SoundFileLoader(filePath),
file(0),
readPosition(0),
tempBlock(0)
{
	string tempPath;

	fileInfo.format = 0;

	tempPath = FileFinder::getInstance().findFile(filePath);
	if(tempPath != "")
	{
		file = sf_open(tempPath.c_str(), SFM_READ, &fileInfo);
		if(!file)
			cout << "libsndfile: Couldn't load sound file: " << tempPath << endl;
		else
			cout << "libsndfile: Loaded file: " << tempPath << endl;
	}
	else
		cout << "LibSndFileLoader: " << filePath << " does not seem to exist?" << endl;

	widthChanged();
}

//------------------------------------------------------------------------------
LibSndFileLoader::~LibSndFileLoader()
{
	if(file)
		sf_close(file);
	if(tempBlock)
		delete [] tempBlock;
}

//------------------------------------------------------------------------------
void LibSndFileLoader::seekg(int position)
{
	if(file)
	{
		if(position > fileInfo.frames)
			position = 0;
		readPosition = position;
	}
}

//------------------------------------------------------------------------------
SoundFileLoader& LibSndFileLoader::operator>>(float *buffer)
{
	int i;
	int result, tempResult;
	int j = 0;
	int blockSize = width();

	if(file)
	{
		//Make sure we're at our current readPosition.
		sf_seek(file, readPosition, SEEK_SET);

		//Read audio from disk into the current block.
		result = sf_readf_float(file, tempBlock, blockSize);
		for(i=0;i<blockSize;++i)
		{
			buffer[i] = tempBlock[j];
			j += fileInfo.channels;
		}

		//If we've gone past the end of the file, loop back to the start.
		tempResult = result;
		while(result < blockSize)
		{
			sf_seek(file, 0, SEEK_SET);
			//sf_readf_float(file, tempBlock, (blockSize-(result/fileInfo.channels)));
			tempResult = sf_readf_float(file, tempBlock, (blockSize-tempResult));
			j = 0;
			for(i=result;i<blockSize;++i)
			{
				buffer[i] = tempBlock[j];
				j += fileInfo.channels;
			}

			result += tempResult;
		}

		//Update readPosition.
		readPosition += blockSize;
		readPosition %= fileInfo.frames;
	}

	return *this;
}

//------------------------------------------------------------------------------
int LibSndFileLoader::getFileSize() const
{
	return fileInfo.frames;
}

//------------------------------------------------------------------------------
void LibSndFileLoader::widthChanged()
{
	if(tempBlock)
		delete [] tempBlock;

	tempBlock = new float[width() * fileInfo.channels];
}

}
