//	LibSndFileLoader.h - libsndfile-based sound file loader.
//  ----------------------------------------------------------------------------
//	This file is part of 'Heilan Browser', an X3D browser specialised for audio
//	work.
//	Copyright (C) 2005-2009  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#ifndef LIBSNDFILELOADER_H_
#define LIBSNDFILELOADER_H_

#include "SoundFileLoader.h"
#include <sndfile.h>

namespace HeilanX3D
{

///	libsndfile-based sound file loader.
class LibSndFileLoader : public SoundFileLoader
{
  public:
	///	Constructor.
	LibSndFileLoader(const std::string& filePath);
	///	Destructor.
	~LibSndFileLoader();

	///	Returns the current read position within the file.
	int tellg() const {return readPosition;};
	///	Sets the current read position within the file.
	void seekg(int position);

	///	Operator used to read from the file.
	SoundFileLoader& operator>>(float *buffer);

	///	Returns the number of samples in the 1st channel of the file.
	int getFileSize() const;

	static SFLoaderConstructor<LibSndFileLoader> constructor;
  private:
	///	Overridden so we can update tempBlock when the width changes.
	void widthChanged();

	///	Pointer to the sound file.
	SNDFILE *file;
	///	Information about the currently-loaded sound file.
	SF_INFO fileInfo;

	///	Current read position within the file.
	int readPosition;
	///	Temporary block of audio, used to convert from stereo (etc.) to mono data.
	float *tempBlock;
};

}

#endif
