//	SoundFileLoader.cpp - Base class for sound file loaders.
//  ----------------------------------------------------------------------------
//	This file is part of 'Heilan Browser', an X3D browser specialised for audio
//	work.
//	Copyright (C) 2005-2009  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#include "SoundFileLoader.h"
#include "FatalError.h"
#include <vector>
#include <iostream>
#include <algorithm>
#include <cctype>

using namespace std;

namespace HeilanX3D
{

//------------------------------------------------------------------------------
SoundFileLoader::SoundFileLoader(const string& filePath):
path(filePath),
currentWidth(1)
{
	
}

//------------------------------------------------------------------------------
SoundFileLoader::~SoundFileLoader()
{
	
}

//------------------------------------------------------------------------------
int SoundFileLoader::width(int newWidth)
{
	int retval;

	if(newWidth > -1)
	{
		currentWidth = newWidth;
		widthChanged();
	}
	retval = currentWidth;

	return retval;
}

//------------------------------------------------------------------------------
//------------------------------------------------------------------------------
SFLoaderFactory& SFLoaderFactory::getInstance()
{
	static SFLoaderFactory retval;

	return retval;
}

//------------------------------------------------------------------------------
void SFLoaderFactory::registerSFLoader(SFLoaderConstructorBase *constructor,
						  			   const string& fileTypes)
{
	vector<string> extensionArray;
	string tempstr;
	string tempext;
	unsigned int tempint;

	//Check if there are any space in fileTypes.
	if(fileTypes.find(' ', 0) == string::npos)
	{
		//No spaces: fileTypes contains only one extension.
		extensionArray.push_back(fileTypes);
	}
	else //Go through fileTypes, add each extension to extensionArray.
	{
		tempstr = fileTypes;
		while(tempstr.size() > 0)
		{
			//Get first extension in tempstr.
			tempint = tempstr.find_first_of(' ', 0);
			if((tempint != string::npos)&&(tempint > 0))
			{
				tempext = tempstr.substr(0, tempint);
				//Move tempstr on.
				tempstr = tempstr.substr(tempint+1, (tempstr.size()-(tempint+1)));
			}
			else
			{
				tempext = tempstr;
				//Move tempstr on.
				tempstr = "";
			}

			//Add tempext to extensionArray.
			extensionArray.push_back(tempext);
		}
	}

	//Add all the extensionArray extensions to loaders.
	for(unsigned int i=0;i<extensionArray.size();++i)
		loaders.insert(make_pair(extensionArray[i], constructor));
}

//------------------------------------------------------------------------------
SoundFileLoader *SFLoaderFactory::getFileLoader(const std::string& path) const
{
	unsigned int tempint;
	string tempstr;
	SoundFileLoader *retval;
	map<string, SFLoaderConstructorBase *>::const_iterator it;

	//Get path's extension.
	tempint = path.find_last_of('.', (path.size()-1));
	if((tempint != string::npos)&&(tempint < (path.size()-1)))
	{
		tempstr = path.substr((tempint+1), (path.size()-(tempint+1)));
		//Convert tempstr to lower case (all extensions will be stored in
		//lower case.
		transform(tempstr.begin(), tempstr.end(), tempstr.begin(), toLower);
		it = loaders.find(tempstr);
		if(it != loaders.end())
			retval = it->second->createInstance(path);
		else
		{
			cout << "Could not find a loader for: " << path << endl;
			retval = 0;
		}
	}
	else
	{
		cout << "Could not determine soundfile " << path;
		cout << "'s extension." << endl;
		retval = 0;
	}

	return retval;
}

//------------------------------------------------------------------------------
char SFLoaderFactory::toLower(char in)
{
	return tolower(in);
}

//------------------------------------------------------------------------------
SFLoaderFactory::SFLoaderFactory()
{
	
}

//------------------------------------------------------------------------------
SFLoaderFactory::~SFLoaderFactory()
{
	
}

}
