//	SpeakerConfig.cpp - Classes to handle the various speaker configurations.
//  ----------------------------------------------------------------------------
//	This file is part of 'Heilan Browser', an X3D browser specialised for audio
//	work.
//	Copyright (C) 2005-2009  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#include "SpeakerConfig.h"

using namespace std;

//------------------------------------------------------------------------------
//------------------------------------------------------------------------------
SpeakerConfig::SpeakerConfig()
{
	
}

//------------------------------------------------------------------------------
void SpeakerConfig::convertAmbi2Float(AmbisonicData *input,
						   			  float *output,
						   			  int numSamples) const
{
	int i;
	int j = 0;
	vector<AmbisonicData>::const_iterator it;

	for(i=0;i<numSamples;++i)
	{
		for(it=matrix.begin();it!=matrix.end();++it)
		{
			output[j] = input[i].X * it->X;
			output[j] += input[i].Y * it->Y;
			output[j] += input[i].Z * it->Z;
			output[j] += input[i].W * it->W;
			++j;
		}
	}
}

//------------------------------------------------------------------------------
void SpeakerConfig::setDecodingMatrix(const std::vector<AmbisonicData>& newMatrix)
{
	matrix = newMatrix;
}

//------------------------------------------------------------------------------
//------------------------------------------------------------------------------
MonoConfig::MonoConfig():
SpeakerConfig()
{
	vector<AmbisonicData> tempM;

	tempM.push_back(AmbisonicData(1.4142f, 0.0f, 0.0f, 0.0f));

	setDecodingMatrix(tempM);
}

//------------------------------------------------------------------------------
//------------------------------------------------------------------------------
StereoConfig::StereoConfig():
SpeakerConfig()
{
	vector<AmbisonicData> tempM;

	tempM.push_back(AmbisonicData(0.7071f, 0.0f, -0.5f, 0.0f));
	tempM.push_back(AmbisonicData(0.7071f, 0.0f, 0.5f, 0.0f));

	setDecodingMatrix(tempM);
}

//------------------------------------------------------------------------------
//------------------------------------------------------------------------------
SquareConfig::SquareConfig():
SpeakerConfig()
{
	vector<AmbisonicData> tempM;

	tempM.push_back(AmbisonicData(0.3536f, 0.1768f, 0.1768f, 0.0f));
	tempM.push_back(AmbisonicData(0.3536f, 0.1768f, -0.1768f, 0.0f));
	tempM.push_back(AmbisonicData(0.3536f, -0.1768f, -0.1768f, 0.0f));
	tempM.push_back(AmbisonicData(0.3536f, -0.1768f, 0.1768f, 0.0f));

	setDecodingMatrix(tempM);
}

//------------------------------------------------------------------------------
//------------------------------------------------------------------------------
PentagonConfig::PentagonConfig():
SpeakerConfig()
{
	vector<AmbisonicData> tempM;

	tempM.push_back(AmbisonicData(0.2828f, 0.1618f, 0.1176f, 0.0f));
	tempM.push_back(AmbisonicData(0.2828f, -0.0618f, 0.1902f, 0.0f));
	tempM.push_back(AmbisonicData(0.2828f, -0.2f, 0.0f, 0.0f));
	tempM.push_back(AmbisonicData(0.2828f, -0.0618f, 0.1902f, 0.0f));
	tempM.push_back(AmbisonicData(0.2828f, 0.1618f, -0.1176f, 0.0f));

	setDecodingMatrix(tempM);
}

//------------------------------------------------------------------------------
//------------------------------------------------------------------------------
HexagonConfig::HexagonConfig():
SpeakerConfig()
{
	vector<AmbisonicData> tempM;

	tempM.push_back(AmbisonicData(0.2357f, 0.1443f, 0.0833f, 0.0f));
	tempM.push_back(AmbisonicData(0.2357f, 0.0f, 0.1667f, 0.0f));
	tempM.push_back(AmbisonicData(0.2357f, -0.1443f, 0.0833f, 0.0f));
	tempM.push_back(AmbisonicData(0.2357f, -0.1443f, -0.0833f, 0.0f));
	tempM.push_back(AmbisonicData(0.2357f, 0.0f, -0.1667f, 0.0f));
	tempM.push_back(AmbisonicData(0.2357f, 0.1443f, -0.0833f, 0.0f));

	setDecodingMatrix(tempM);
}

//------------------------------------------------------------------------------
//------------------------------------------------------------------------------
Octagon1Config::Octagon1Config():
SpeakerConfig()
{
	vector<AmbisonicData> tempM;

	tempM.push_back(AmbisonicData(0.1768f, 0.1155f, 0.0478f, 0.0f));
	tempM.push_back(AmbisonicData(0.1768f, 0.0478f, 0.1155f, 0.0f));
	tempM.push_back(AmbisonicData(0.1768f, -0.0478f, 0.1155f, 0.0f));
	tempM.push_back(AmbisonicData(0.1768f, -0.1155f, 0.0478f, 0.0f));
	tempM.push_back(AmbisonicData(0.1768f, -0.1155f, -0.0478f, 0.0f));
	tempM.push_back(AmbisonicData(0.1768f, -0.0478f, -0.1155f, 0.0f));
	tempM.push_back(AmbisonicData(0.1768f, 0.0478f, -0.1155f, 0.0f));
	tempM.push_back(AmbisonicData(0.1768f, 0.1155f, -0.0478f, 0.0f));

	setDecodingMatrix(tempM);
}

//------------------------------------------------------------------------------
//------------------------------------------------------------------------------
Octagon2Config::Octagon2Config():
SpeakerConfig()
{
	vector<AmbisonicData> tempM;

	tempM.push_back(AmbisonicData(0.1768f, 0.1250f, 0.0f, 0.0f));
	tempM.push_back(AmbisonicData(0.1768f, 0.0884f, 0.0884f, 0.0f));
	tempM.push_back(AmbisonicData(0.1768f, 0.0f, 0.1250f, 0.0f));
	tempM.push_back(AmbisonicData(0.1768f, -0.0884f, 0.0884f, 0.0f));
	tempM.push_back(AmbisonicData(0.1768f, -0.1250f, 0.0f, 0.0f));
	tempM.push_back(AmbisonicData(0.1768f, -0.0884f, -0.0884f, 0.0f));
	tempM.push_back(AmbisonicData(0.1768f, 0.0f, -0.1250f, 0.0f));
	tempM.push_back(AmbisonicData(0.1768f, 0.0884f, -0.0884f, 0.0f));

	setDecodingMatrix(tempM);
}

//------------------------------------------------------------------------------
//------------------------------------------------------------------------------
Surround1Config::Surround1Config():
SpeakerConfig()
{
	vector<AmbisonicData> tempM;

	/*tempM.push_back(AmbisonicData(0.1690f, 0.0797f, 0.0891f, 0.0f));
	tempM.push_back(AmbisonicData(0.1635f, 0.0923f, 0.0f, 0.0f));
	tempM.push_back(AmbisonicData(0.1690f, 0.0797f, -0.0891f, 0.0f));
	tempM.push_back(AmbisonicData(0.4563f, -0.1259f, 0.1543f, 0.0f));
	tempM.push_back(AmbisonicData(0.4563f, -0.1259f, -0.1543f, 0.0f));*/
	tempM.push_back(AmbisonicData(0.1690f, 0.0797f, -0.0891f, 0.0f));
	tempM.push_back(AmbisonicData(0.1635f, 0.0923f, 0.0f, 0.0f));
	tempM.push_back(AmbisonicData(0.1690f, 0.0797f, 0.0891f, 0.0f));
	tempM.push_back(AmbisonicData(0.4563f, -0.1259f, -0.1543f, 0.0f));
	tempM.push_back(AmbisonicData(0.4563f, -0.1259f, 0.1543f, 0.0f));

	setDecodingMatrix(tempM);
}

//------------------------------------------------------------------------------
//------------------------------------------------------------------------------
CubeConfig::CubeConfig():
SpeakerConfig()
{
	vector<AmbisonicData> tempM;

	tempM.push_back(AmbisonicData(0.1768f, 0.0722f, 0.0722f, -0.0722f));
	tempM.push_back(AmbisonicData(0.1768f, 0.0722f, -0.0722f, -0.0722f));
	tempM.push_back(AmbisonicData(0.1768f, -0.0722f, -0.0722f, -0.0722f));
	tempM.push_back(AmbisonicData(0.1768f, -0.0722f, 0.0722f, -0.0722f));
	tempM.push_back(AmbisonicData(0.1768f, 0.0722f, 0.0722f, 0.0722f));
	tempM.push_back(AmbisonicData(0.1768f, 0.0722f, -0.0722f, 0.0722f));
	tempM.push_back(AmbisonicData(0.1768f, -0.0722f, -0.0722f, 0.0722f));
	tempM.push_back(AmbisonicData(0.1768f, -0.0722f, 0.0722f, 0.0722f));

	setDecodingMatrix(tempM);
}

//------------------------------------------------------------------------------
//------------------------------------------------------------------------------
Dodecahedron1Config::Dodecahedron1Config():
SpeakerConfig()
{
	vector<AmbisonicData> tempM;

	tempM.push_back(AmbisonicData(0.1179f, 0.0f, 0.0f, 0.25f));
	tempM.push_back(AmbisonicData(0.1179f, 0.0f, 0.0f, -0.25f));
	tempM.push_back(AmbisonicData(0.1179f, 0.1809f, 0.1314f, 0.1118f));
	tempM.push_back(AmbisonicData(0.1179f, -0.1809f, -0.1314f, -0.1118f));
	tempM.push_back(AmbisonicData(0.1179f, 0.1809f, -0.1314f, 0.1118f));
	tempM.push_back(AmbisonicData(0.1179f, -0.1809f, 0.1314f, -0.1118f));
	tempM.push_back(AmbisonicData(0.1179f, -0.0691f, 0.2127f, 0.1118f));
	tempM.push_back(AmbisonicData(0.1179f, 0.0691f, -0.2127f, -0.1118f));
	tempM.push_back(AmbisonicData(0.1179f, -0.0691f, -0.2127f, 0.1118f));
	tempM.push_back(AmbisonicData(0.1179f, 0.0691f, 0.2127f, -0.1118f));
	tempM.push_back(AmbisonicData(0.1179f, -0.2236f, 0.0f, 0.1118f));
	tempM.push_back(AmbisonicData(0.1179f, 0.2236f, 0.0f, -0.1118f));

	setDecodingMatrix(tempM);
}

//------------------------------------------------------------------------------
//------------------------------------------------------------------------------
Dodecahedron2Config::Dodecahedron2Config():
SpeakerConfig()
{
	vector<AmbisonicData> tempM;

	tempM.push_back(AmbisonicData(0.1179f, 0.0833f, 0.0f, 0.0f));
	tempM.push_back(AmbisonicData(0.1179f, -0.0833f, 0.0f, 0.0f));
	tempM.push_back(AmbisonicData(0.1179f, 0.0373f, 0.0f, -0.0745f));
	tempM.push_back(AmbisonicData(0.1179f, -0.0373f, 0.0f, 0.0745f));
	tempM.push_back(AmbisonicData(0.1179f, 0.0373f, 0.0709f, -0.0230f));
	tempM.push_back(AmbisonicData(0.1179f, -0.0373f, -0.0709f, 0.0230f));
	tempM.push_back(AmbisonicData(0.1179f, 0.0373f, -0.0709f, -0.0230f));
	tempM.push_back(AmbisonicData(0.1179f, -0.0373f, 0.0709f, 0.0230f));
	tempM.push_back(AmbisonicData(0.1179f, 0.0373f, 0.0438f, 0.0603f));
	tempM.push_back(AmbisonicData(0.1179f, -0.0373f, -0.0438f, -0.0603f));
	tempM.push_back(AmbisonicData(0.1179f, 0.0373f, -0.0438f, 0.0603f));
	tempM.push_back(AmbisonicData(0.1179f, -0.0373f, 0.0438f, -0.0603f));

	setDecodingMatrix(tempM);
}

//------------------------------------------------------------------------------
//------------------------------------------------------------------------------
ComponentsConfig::ComponentsConfig():
SpeakerConfig()
{
	vector<AmbisonicData> tempM;

	tempM.push_back(AmbisonicData(1.0f, 0.0f, 0.0f, 0.0f));
	tempM.push_back(AmbisonicData(0.0f, 1.0f, 0.0f, 0.0f));
	tempM.push_back(AmbisonicData(0.0f, 0.0f, 1.0f, 0.0f));
	tempM.push_back(AmbisonicData(0.0f, 0.0f, 0.0f, 1.0f));

	setDecodingMatrix(tempM);
}
