//	VorbisFileLoader.cpp - libsndfile-based sound file loader.
//  ----------------------------------------------------------------------------
//	This file is part of 'Heilan Browser', an X3D browser specialised for audio
//	work.
//	Copyright (C) 2005-2009  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#ifdef VORBISFILE

#include "VorbisFileLoader.h"
#include "FileFinder.h"

#include <vorbis/codec.h>
#include <iostream>

using namespace std;

namespace HeilanX3D
{
	
SFLoaderConstructor<VorbisFileLoader> VorbisFileLoader::constructor;

//------------------------------------------------------------------------------
VorbisFileLoader::VorbisFileLoader(const std::string& filePath):
SoundFileLoader(filePath),
readPosition(0),
fileSuccess(false)
{
	string tempPath;

	tempPath = FileFinder::getInstance().findFile(filePath);
	if(tempPath != "")
	{
		file = fopen(tempPath.c_str(), "rb");
		if(!file)
			cout << "vorbisfile: fopen could not open file: " << tempPath << endl;
		else
		{
			if(ov_open(file, &vorbisFile, NULL, 0) < 0)
				cout << "vorbisfile: Could not open file: " << tempPath << endl;
			else
			{
				if(ov_streams(&vorbisFile) > 0)
				{
					fileInfo = ov_info(&vorbisFile, 0);
					fileSuccess = true;
					cout << "vorbisfile: Loaded file: " << tempPath << endl;
				}
				else
				{
					ov_clear(&vorbisFile);
					cout << "vorbisfile: There are no streams in this file?" << endl;
				}
			}
		}
	}
	else
		cout << "VorbisFileLoader: " << filePath << " does not seem to exist?" << endl;
}

//------------------------------------------------------------------------------
VorbisFileLoader::~VorbisFileLoader()
{
	if(fileSuccess)
		ov_clear(&vorbisFile);
}

//------------------------------------------------------------------------------
void VorbisFileLoader::seekg(int position)
{
	if(fileSuccess)
	{
		if(position > ov_pcm_total(&vorbisFile, 0))
			position = 0;
		readPosition = position;
	}
}

//------------------------------------------------------------------------------
SoundFileLoader& VorbisFileLoader::operator>>(float *buffer)
{
	int i;
	int result;
	int tempint;
	float **tempBuf;
	int currentBitstream = 0;
	int blockSize = width();
	int samplesRead = 0;

	if(fileSuccess)
	{
		//Make sure we're at the correct point to read from.
		ov_pcm_seek(&vorbisFile, readPosition);

		//Read audio from disk into the current block.
		while(samplesRead < blockSize)
		{
			result = ov_read_float(&vorbisFile,
								   &tempBuf,
								   (blockSize-samplesRead),
								   &currentBitstream);
			if((currentBitstream > 0)||(result <= 0))
			{
				//We've either gone past the end of our bitstream, or there are
				//no other bitstreams in the file, and we've reached the end of
				//the file.
				readPosition = 0;
				ov_pcm_seek(&vorbisFile, readPosition);
			}
			else if(result <= blockSize)
			{
				//Copy the audio data to the output buffer passed in to this
				//method.
				tempint = samplesRead;
				for(i=0;i<result;++i)
				{
					if(samplesRead >= blockSize)
						break;
					buffer[tempint+i] = tempBuf[0][i];
					++samplesRead;
					++readPosition;
				}
			}
		}
	}

	return *this;
}

//------------------------------------------------------------------------------
int VorbisFileLoader::getFileSize() const
{
	return ov_pcm_total(const_cast<OggVorbis_File *>(&vorbisFile), 0);
}

}

#endif
