//	VorbisFileLoader.h - vorbisfile-based sound file loader.
//  ----------------------------------------------------------------------------
//	This file is part of 'Heilan Browser', an X3D browser specialised for audio
//	work.
//	Copyright (C) 2005-2009  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

//Only compile in ogg support if the user requested it in the configure script.
#ifdef VORBISFILE

#ifndef VORBISFILELOADER_H_
#define VORBISFILELOADER_H_

#include "SoundFileLoader.h"
#include <vorbis/vorbisfile.h>

namespace HeilanX3D
{

///	vorbisfile-based sound file loader.
class VorbisFileLoader : public SoundFileLoader
{
  public:
	///	Constructor.
	VorbisFileLoader(const std::string& filePath);
	///	Destructor.
	~VorbisFileLoader();

	///	Returns the current read position within the file.
	int tellg() const {return readPosition;};
	///	Sets the current read position within the file.
	void seekg(int position);

	///	Operator used to read from the file.
	SoundFileLoader& operator>>(float *buffer);

	///	Returns the number of samples in the 1st channel of the file.
	int getFileSize() const;

	static SFLoaderConstructor<VorbisFileLoader> constructor;
  private:
	///	Reference to the sound file.
	OggVorbis_File vorbisFile;
	///	The actual file.
	FILE *file;
	///	The file info.
	vorbis_info *fileInfo;

	///	Current read position within the file.
	int readPosition;

	///	True if the file loaded successfully.
	bool fileSuccess;
};

}

#endif

#endif
